package com.qhiot.core.sql.generator;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 * Twitter-SnowflakeIDGenerator, 分布式 64位 ID 算法<br>
 * <p>
 * 使用 41 bit来存放时间，精确到毫秒，可以使用到 2082 年<br>
 * 使用 10 bit来存放逻辑节点ID，节点ID 范围为 [0, 1023]<br>
 * 使用 12 bit来存放自增长ID，每个节点每毫秒最多可以生成 4096 个ID [0, 4095]<br>
 * 剩余最左侧 1 bit 未使用
 *
 * <pre>
 * ID 格式 =>
 * +-----------+----------+----------+
 * | timestamp | datanode | sequence |
 * +-----------+----------+----------+
 * |   41bit   |   10bit  |  12bit   |
 * +-----------+----------+----------+
 * </pre>
 *
 * @author 金建强(ptma @ 163.com)
 * @version 2016-05-25 13:01
 */
public class SnowflakeIDGenerator {

    public static final int NODE_SHIFT = 10;
    public static final int SEQ_SHIFT = 12;

    public static final short MAX_NODE = 1024;
    public static final short MAX_SEQUENCE = 4096;
    private static final Map<Integer, SnowflakeIDGenerator> INSTANCE_MAP = new ConcurrentHashMap<Integer, SnowflakeIDGenerator>();
    private long sequence;
    private long referenceTime;
    private int node;

    private SnowflakeIDGenerator(int node) {
        if (node < 0 || node > MAX_NODE) {
            throw new IllegalArgumentException(String.format("node must be between %s and %s", 0, MAX_NODE));
        }
        this.node = node;
    }

    public synchronized static SnowflakeIDGenerator getInstance() {
        return getInstance(0);
    }

    /**
     * 单机环境下，应该保证用相同的 node 在集群环境中，不同的机器必须提供不同的 node 来保证生成的ID的唯一性
     *
     * @param node 唯一标识号
     */
    public synchronized static SnowflakeIDGenerator getInstance(int node) {
        SnowflakeIDGenerator snowflake;
        if (!INSTANCE_MAP.containsKey(node)) {
            snowflake = new SnowflakeIDGenerator(node);
            INSTANCE_MAP.put(node, snowflake);
        } else {
            snowflake = INSTANCE_MAP.get(node);
        }
        return snowflake;
    }

    /**
     * 给定一个时间，算出当时的ID(node=0, sequence=0)
     *
     * @param specifiedTimestamp
     * @return
     */
    public static long specifiedId(long specifiedTimestamp) {
        return specifiedTimestamp << NODE_SHIFT << SEQ_SHIFT | 0 << SEQ_SHIFT | 0;
    }

    /**
     * 从ID中提取时间戳毫秒值
     *
     * @param id 已生成的ID值
     * @return 时间戳毫秒值
     */
    public static long getTimestamp(long id) {
        return id >> 22;
    }

    /**
     * 从ID中提取节点值
     *
     * @param id 已生成的ID值
     * @return 节点值
     */
    public static long getNode(long id) {
        // ‭0x3FF000 = 00111111 11110000 00000000‬
        return (id & 0x3FF000) >> 12;
    }

    /**
     * 从ID中提取序列值
     *
     * @param id 已生成的ID值
     * @return 序列值
     */
    public static long getSequence(long id) {
        // 0x0FFF = 00001111 11111111
        return id & 0x0FFF;
    }

    /**
     * 生成一个时间上有序的 64 位整型唯一ID. 同一毫秒内的按子序列自增长.
     *
     * @return 64-bit Integer.
     */
    public synchronized long next() {
        long currentTimestamp = System.currentTimeMillis();
        if (currentTimestamp < referenceTime) {
            String message = "Last referenceTime %s is after reference time %s";
            throw new RuntimeException(String.format(message, referenceTime, currentTimestamp));
        } else if (referenceTime == currentTimestamp) {
            sequence = (sequence + 1) % MAX_SEQUENCE;
            if (sequence == 0) {
                currentTimestamp = tilNextMillis(referenceTime);
            }
        } else {
            sequence = 0;
        }
        referenceTime = currentTimestamp;
        return currentTimestamp << NODE_SHIFT << SEQ_SHIFT | node << SEQ_SHIFT | sequence;
    }

    private long tilNextMillis(long lastTimestamp) {
        long timestamp = System.currentTimeMillis();
        while (timestamp <= lastTimestamp) {
            timestamp = System.currentTimeMillis();
        }
        return timestamp;
    }

}
