package com.qhiot.core.sql.mybatis.plugin;

import com.qhiot.core.sql.mybatis.dialect.DialectSQL;
import org.apache.ibatis.builder.StaticSqlSource;
import org.apache.ibatis.executor.Executor;
import org.apache.ibatis.logging.Log;
import org.apache.ibatis.mapping.BoundSql;
import org.apache.ibatis.mapping.MappedStatement;
import org.apache.ibatis.mapping.SqlCommandType;
import org.apache.ibatis.plugin.*;
import org.apache.ibatis.scripting.defaults.DefaultParameterHandler;
import org.apache.ibatis.session.Configuration;
import org.apache.ibatis.session.ResultHandler;
import org.apache.ibatis.session.RowBounds;
import org.apache.ibatis.transaction.Transaction;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;
import java.util.Properties;

/**
 * 索引检测拦截器 <br>
 * <br>
 * 在开发和测试环境中中使用, 实际运行环境中不推荐加载本插件
 *
 * @author 金建强(ptma @ 163.com)
 * @version 2014-8-20 10:24:42
 */
@Intercepts({@Signature(type = Executor.class, method = "query", args = {MappedStatement.class, Object.class,
        RowBounds.class, ResultHandler.class})})
public class IndexingInterceptor implements Interceptor {

    private static final Logger LOGGER = LoggerFactory.getLogger(IndexingInterceptor.class);

    private volatile DialectSQL.DialectDB dialect;

    /**
     * 获取数据库连接
     * <p>
     *
     * @param transaction
     * @return java.sql.Connection
     * @throws SQLException
     */
    protected Connection getConnection(Transaction transaction) throws SQLException {
        return transaction.getConnection();
    }

    protected void printWarn(Configuration configuration, MappedStatement mappedStatement, BoundSql boundSql,
                             Connection connection, Object parameter) throws Exception {
        PreparedStatement stmt = null;
        ResultSet rs = null;
        try {
            if (dialect == null) {
                dialect = DialectSQL.whichDB(connection.getMetaData());
            }
            StringBuilder explain = new StringBuilder("EXPLAIN ");
            explain.append(boundSql.getSql());
            String sqlExplain = explain.toString();
            StaticSqlSource sqlsource = new StaticSqlSource(configuration, sqlExplain, boundSql.getParameterMappings());
            MappedStatement.Builder builder = new MappedStatement.Builder(configuration, mappedStatement.getId(),
                    sqlsource, SqlCommandType.SELECT);
            builder.resultMaps(mappedStatement.getResultMaps()).resultSetType(mappedStatement.getResultSetType()).statementType(mappedStatement.getStatementType());
            MappedStatement queryStatement = builder.build();
            DefaultParameterHandler handler = new DefaultParameterHandler(queryStatement, parameter, boundSql);
            stmt = connection.prepareStatement(sqlExplain);
            handler.setParameters(stmt);
            rs = stmt.executeQuery();
            int count = 0;
            Log logger = queryStatement.getStatementLog();
            while (rs.next()) {
                count++;
                String type = rs.getString("type");

                if ("ALL".equals(type) && logger.isDebugEnabled()) {
                    logger.debug("使用了全表扫描的方式!");
                }
                if (rs.getString("key") == null && logger.isDebugEnabled()) {
                    logger.debug("没有使用索引, 可能存在性能问题!");
                }
                int rows = rs.getInt("rows");
                if (rows > 500 && logger.isDebugEnabled()) {
                    logger.debug("影响行数为 " + rows + ", 可能存在性能问题!");
                }
                String extra = rs.getString("Extra");
                if (extra != null && extra.contains("Using temporary") && logger.isDebugEnabled()) {
                    logger.debug("使用临时表, 可能存在性能问题!");
                }
                if (extra != null && extra.contains("Using filesort") && logger.isDebugEnabled()) {
                    logger.debug("使用文件排序, 可能存在性能问题!");
                }
            }
            if (count > 1 && logger.isDebugEnabled()) {
                logger.debug("当前SQL语句中用了子查询或者连接查询, 某些情况下使用n+1查询可提升查询性能!");
            }

        } catch (Exception e) {
            LOGGER.warn(e.getMessage(), e);
        } finally {
            if (rs != null) {
                rs.close();
            }
            if (stmt != null) {
                stmt.close();
            }
        }
    }

    @Override
    public Object intercept(Invocation invocation) throws Exception {
        Object result = invocation.proceed();

        // 返回结果是空, explain没有意义
        if (result instanceof List) {
            List<?> re = (List<?>) result;
            if (re.isEmpty()) {
                return result;
            }
        }
        // 不是mysql直接返回
        if (dialect != null && dialect != DialectSQL.DialectDB.MYSQL) {
            return result;
        }

        Object parameter = invocation.getArgs()[1];
        MappedStatement mappedStatement = (MappedStatement) invocation.getArgs()[0];
        Connection connection = null;
        try {

            Configuration configuration = mappedStatement.getConfiguration();
            BoundSql boundSql = mappedStatement.getBoundSql(parameter);
            Executor exe = (Executor) invocation.getTarget();
            connection = getConnection(exe.getTransaction());
            printWarn(configuration, mappedStatement, boundSql, connection, parameter);
        } catch (Exception e) {
            mappedStatement.getStatementLog().error("索引拦截监控出错", e);
        } finally {
            if (connection != null) {
                connection.close();
            }
        }
        return result;

    }

    @Override
    public Object plugin(Object target) {
        return Plugin.wrap(target, this);
    }

    @Override
    public void setProperties(Properties properties) {
    }
}
