package com.qhiot.core.sql.mybatis.plugin;

import com.qhiot.core.sql.annotation.Key;
import com.qhiot.core.sql.annotation.KeyType;
import com.qhiot.core.sql.generator.SnowflakeID;
import com.qhiot.core.util.ReflectUtil;
import com.qhiot.core.util.StringUtil;
import org.apache.ibatis.executor.Executor;
import org.apache.ibatis.mapping.MappedStatement;
import org.apache.ibatis.mapping.SqlCommandType;
import org.apache.ibatis.plugin.*;
import org.apache.ibatis.transaction.Transaction;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.lang.reflect.Field;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.*;
import java.util.Map.Entry;

/**
 * 主键生成拦截器 <br>
 * 拦截 MyBatis insert 语句, 对参数进行预处理, 当实体的属性被 @Key 标注的主键为空时， 生成一个新的值
 *
 * @author 金建强(ptma @ 163.com)
 * @version 2014-8-20 10:24:42
 */
@Intercepts({@Signature(method = "update", type = Executor.class, args = {MappedStatement.class, Object.class})})
public class KeyInterceptor implements Interceptor {

    private static final Logger LOGGER = LoggerFactory.getLogger(KeyInterceptor.class);

    private SnowflakeID snowflakeID;

    /**
     * 拦截后要执行的方法
     */
    @Override
    @SuppressWarnings({"unchecked"})
    public Object intercept(Invocation invocation) throws Exception {
        Object[] queryArgs = invocation.getArgs();
        Object parameter = queryArgs[1];
        MappedStatement ms = (MappedStatement) queryArgs[0];

        // 是否为 INSERT 语句， 并且参数不为空
        if (ms.getSqlCommandType() == SqlCommandType.INSERT && parameter != null) {
            if (parameter instanceof Map) {
                Map parameterMap = (Map) parameter;
                Iterator<Entry> iterator = parameterMap.entrySet().iterator();
                while (iterator.hasNext()) {
                    processParam(invocation, iterator.next().getValue());
                }
            } else {
                processParam(invocation, parameter);
            }
        }
        return invocation.proceed();
    }

    /**
     * 参数处理, 识别主键注解并注入响应值
     *
     * @param paramObj
     * @throws Exception
     */
    private void processParam(Invocation invocation, Object paramObj) throws Exception {
        if (paramObj == null) {
            return;
        }
        List<Field> fields = ReflectUtil.getFields(paramObj.getClass());
        for (Field f : fields) {
            if (!f.isAnnotationPresent(Key.class)) {
                continue;
            }
            Object fieldValue = ReflectUtil.getFieldValue(paramObj, f.getName());
            if (!StringUtil.isEmpty(fieldValue)) {
                continue;
            }
            Key keyAnn = f.getAnnotation(Key.class);
            if (keyAnn.type() == KeyType.UUID) {
                if ((f.getType() == String.class)) {
                    String uuid = UUID.randomUUID().toString().replace("-", "");
                    ReflectUtil.setFieldValue(f, paramObj, uuid);
                } else {
                    throw new RuntimeException("UUID 主键必须是 String 类型.");
                }
            } else if (keyAnn.type() == KeyType.SNOWFLAKE) {
                if ((f.getType() == String.class || f.getType() == Long.class)) {
                    long newPkValue = snowflakeID.next();
                    if (f.getType() == String.class) {
                        ReflectUtil.setFieldValue(f, paramObj, String.valueOf(newPkValue));
                    } else {
                        ReflectUtil.setFieldValue(f, paramObj, newPkValue);
                    }
                } else {
                    throw new RuntimeException("SNOWFLAKE Key 必须是 String 或 Long 类型.");
                }
            } else if (keyAnn.type() == KeyType.SQL) {
                Executor exe = (Executor) invocation.getTarget();
                Connection connection = getConnection(exe.getTransaction());
                try {
                    if (f.getType() == String.class) {
                        String sqlValue = executeStringSql(connection, keyAnn.sql());
                        ReflectUtil.setFieldValue(f, paramObj, sqlValue);
                    } else if (f.getType() == Long.class) {
                        Long sqlValue = executeLongSql(connection, keyAnn.sql());
                        ReflectUtil.setFieldValue(f, paramObj, sqlValue);
                    } else if (f.getType() == Integer.class) {
                        Integer sqlValue = executeIntSql(connection, keyAnn.sql());
                        ReflectUtil.setFieldValue(f, paramObj, sqlValue);
                    } else {
                        throw new RuntimeException("SQL Key 主键必须是 String、 Integer 或 Long 类型.");
                    }
                } finally {
                    if (connection != null) {
                        connection.close();
                    }
                }
            }
        }
    }

    /**
     * 拦截器对应的封装原始对象的方法
     */
    @Override
    public Object plugin(Object target) {
        return Plugin.wrap(target, this);
    }

    @Override
    public void setProperties(Properties properties) {

    }

    protected Connection getConnection(Transaction transaction) throws SQLException {
        return transaction.getConnection();
    }

    protected Long executeLongSql(Connection connection, String sql) throws Exception {
        PreparedStatement stmt = null;
        ResultSet rs = null;
        Long result = null;
        try {
            stmt = connection.prepareStatement(sql);
            rs = stmt.executeQuery();
            int count = 0;
            while (rs.next()) {
                count++;
                result = rs.getLong(1);
            }
            if (count != 1) {
                throw new RuntimeException("返回结果与预期不符, 预期返回结果数为1, 实际返回结果数为" + count);
            }
        } catch (Exception e) {
            LOGGER.warn(e.getMessage(), e);
        } finally {
            if (rs != null) {
                rs.close();
            }
            if (stmt != null) {
                stmt.close();
            }
        }
        return result;
    }

    protected String executeStringSql(Connection connection, String sql) throws Exception {
        PreparedStatement stmt = null;
        ResultSet rs = null;
        String result = null;
        try {
            stmt = connection.prepareStatement(sql);
            rs = stmt.executeQuery();
            int count = 0;
            while (rs.next()) {
                count++;
                result = rs.getString(1);
            }
            if (count != 1) {
                throw new RuntimeException("返回结果与预期不符, 预期返回结果数为1, 实际返回结果数为" + count);
            }
        } catch (Exception e) {
            LOGGER.warn(e.getMessage(), e);
        } finally {
            if (rs != null) {
                rs.close();
            }
            if (stmt != null) {
                stmt.close();
            }
        }
        return result;
    }

    protected Integer executeIntSql(Connection connection, String sql) throws Exception {
        PreparedStatement stmt = null;
        ResultSet rs = null;
        Integer result = null;
        try {
            stmt = connection.prepareStatement(sql);
            rs = stmt.executeQuery();
            int count = 0;
            while (rs.next()) {
                count++;
                result = rs.getInt(1);
            }
            if (count != 1) {
                throw new RuntimeException("返回结果与预期不符, 预期返回结果数为1, 实际返回结果数为" + count);
            }
        } catch (Exception e) {
            LOGGER.warn(e.getMessage(), e);
        } finally {
            if (rs != null) {
                rs.close();
            }
            if (stmt != null) {
                stmt.close();
            }
        }
        return result;
    }

    public void setSnowflakeID(SnowflakeID snowflakeID) {
        this.snowflakeID = snowflakeID;
    }
}
