package com.qhiot.core.sql.mybatis.plugin;

import com.qhiot.core.bean.Pager;
import com.qhiot.core.sql.mybatis.dialect.Dialect;
import org.apache.ibatis.executor.Executor;
import org.apache.ibatis.mapping.*;
import org.apache.ibatis.plugin.*;
import org.apache.ibatis.reflection.DefaultReflectorFactory;
import org.apache.ibatis.reflection.MetaObject;
import org.apache.ibatis.reflection.ReflectorFactory;
import org.apache.ibatis.reflection.factory.DefaultObjectFactory;
import org.apache.ibatis.reflection.factory.ObjectFactory;
import org.apache.ibatis.reflection.wrapper.DefaultObjectWrapperFactory;
import org.apache.ibatis.reflection.wrapper.ObjectWrapperFactory;
import org.apache.ibatis.session.ResultHandler;
import org.apache.ibatis.session.RowBounds;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

@Intercepts({@Signature(method = "query", type = Executor.class, args = {MappedStatement.class, Object.class,
        RowBounds.class, ResultHandler.class})})
@SuppressWarnings("rawtypes")
public class PageInterceptor implements Interceptor {

    public static final ObjectFactory DEFAULT_OBJECT_FACTORY = new DefaultObjectFactory();
    public static final ObjectWrapperFactory DEFAULT_OBJECT_WRAPPER_FACTORY = new DefaultObjectWrapperFactory();
    public static final ReflectorFactory DEFAULT_REFLECTOR_FACTORY = new DefaultReflectorFactory();
    private static final Logger LOGGER = LoggerFactory.getLogger("Init.Core");
    private static final List<ResultMapping> EMPTY_RESULTMAPPING = new ArrayList<ResultMapping>(0);
    private static final String BOUND_SQL = "sqlSource.boundSql.sql";
    private static final ThreadLocal<Pager> LOCAL_PAGE = new ThreadLocal<Pager>();
    private Dialect dialect;
    // RowBounds参数offset作为PageNum使用 - 默认不使用
    private boolean offsetAsPageNum = false;
    // RowBounds是否进行count查询 - 默认不查询
    private boolean rowBoundsWithCount = false;

    /**
     * 反射对象，增加对低版本Mybatis的支持
     */
    public static MetaObject forObject(Object object) {
        return MetaObject.forObject(object, DEFAULT_OBJECT_FACTORY, DEFAULT_OBJECT_WRAPPER_FACTORY,
                DEFAULT_REFLECTOR_FACTORY);
    }

    /**
     * 开始分页
     *
     * @param pageNum  页码
     * @param pageSize 页大小
     */
    public static void startPage(int pageNum, int pageSize, boolean countEnabled) {
        LOCAL_PAGE.set(new Pager(pageNum, pageSize, countEnabled));
    }

    /**
     * 获取分页参数
     *
     * @param rowBounds 分页参数
     * @return 分页数据
     */
    private Pager getPager(RowBounds rowBounds) {
        Pager pager = LOCAL_PAGE.get();
        // 移除本地变量
        LOCAL_PAGE.remove();

        if (pager == null) {
            if (offsetAsPageNum) {
                pager = new Pager(rowBounds.getOffset(), rowBounds.getLimit(), rowBoundsWithCount);
            } else {
                pager = new Pager(rowBounds, rowBoundsWithCount);
            }
        }
        return pager;
    }

    /**
     * 拦截后要执行的方法
     */
    @Override
    @SuppressWarnings({"unchecked"})
    public Object intercept(Invocation invocation) throws Exception {
        final Object[] args = invocation.getArgs();
        RowBounds rowBounds = (RowBounds) args[2];
        if (LOCAL_PAGE.get() == null && rowBounds == RowBounds.DEFAULT) {
            return invocation.proceed();
        } else {
            // 忽略RowBounds-否则会进行Mybatis自带的内存分页
            args[2] = RowBounds.DEFAULT;
            MappedStatement ms = (MappedStatement) args[0];
            Object parameterObject = args[1];
            BoundSql boundSql = ms.getBoundSql(parameterObject);

            // 分页信息
            Pager page = getPager(rowBounds);
            // 创建一个新的MappedStatement
            MappedStatement qs = newMappedStatement(ms, new BoundSqlSqlSource(boundSql));
            // 将参数中的MappedStatement替换为新的qs，防止并发异常
            args[0] = qs;
            MetaObject msObject = forObject(qs);
            String sql = (String) msObject.getValue(BOUND_SQL);
            // 简单的通过total的值来判断是否进行count查询
            if (page.isCountEnabled()) {
                // 求count - 重写sql
                msObject.setValue(BOUND_SQL, getCountSql(sql));
                // 查询总数
                Object result = invocation.proceed();
                // 设置总数
                page.setTotal((Integer) ((List) result).get(0));
            }
            // 分页sql - 重写sql
            msObject.setValue(BOUND_SQL, getPageSql(page, sql));
            // 恢复类型
            msObject.setValue("resultMaps", ms.getResultMaps());
            // 执行分页查询
            Object result = invocation.proceed();
            // 得到处理结果
            page.addAll((List) result);
            // 返回结果
            return page;
        }
    }

    /**
     * 由于MappedStatement是一个全局共享的对象，因而需要复制一个对象来进行操作，防止并发访问导致错误
     */
    private MappedStatement newMappedStatement(MappedStatement ms, SqlSource newSqlSource) {
        MappedStatement.Builder builder = new MappedStatement.Builder(ms.getConfiguration(), ms.getId(), newSqlSource,
                ms.getSqlCommandType());
        builder.resource(ms.getResource());
        builder.fetchSize(ms.getFetchSize());
        builder.statementType(ms.getStatementType());
        builder.keyGenerator(ms.getKeyGenerator());
        if (ms.getKeyProperties() != null && ms.getKeyProperties().length != 0) {
            StringBuilder keyProperties = new StringBuilder();
            for (String keyProperty : ms.getKeyProperties()) {
                keyProperties.append(keyProperty).append(",");
            }
            keyProperties.delete(keyProperties.length() - 1, keyProperties.length());
            builder.keyProperty(keyProperties.toString());
        }
        builder.timeout(ms.getTimeout());
        builder.parameterMap(ms.getParameterMap());
        // 由于resultMaps第一次需要返回int类型的结果，所以这里需要生成一个resultMap - 防止并发错误
        List<ResultMap> resultMaps = new ArrayList<ResultMap>();
        ResultMap resultMap = new ResultMap.Builder(ms.getConfiguration(), ms.getId(), Integer.class,
                EMPTY_RESULTMAPPING).build();
        resultMaps.add(resultMap);
        builder.resultMaps(resultMaps);
        builder.resultSetType(ms.getResultSetType());
        builder.cache(ms.getCache());
        builder.flushCacheRequired(ms.isFlushCacheRequired());
        builder.useCache(ms.isUseCache());

        return builder.build();
    }

    /**
     * 拦截器对应的封装原始对象的方法
     */
    @Override
    public Object plugin(Object target) {
        return Plugin.wrap(target, this);
    }

    /**
     * 设置注册拦截器时设定的属性
     */
    @Override
    public void setProperties(Properties properties) {
        String dialectClass = properties.getProperty("dialectClass");
        if (dialectClass == null || dialectClass.trim().length() == 0) {
            throw new RuntimeException("Property 'dialectClass' is missing!");
        }
        try {
            dialect = (Dialect) Class.forName(dialectClass).newInstance();
            LOGGER.info("注册数据库分页方言: '{}'", dialectClass);
        } catch (Exception e) {
            throw new RuntimeException("Cannot create dialect instance by dialectClass:" + dialectClass, e);
        }
        String offsetAsPageNum = properties.getProperty("offsetAsPageNum");
        if (offsetAsPageNum != null && offsetAsPageNum.trim().length() > 0) {
            this.offsetAsPageNum = Boolean.parseBoolean(offsetAsPageNum);
        }
        String rowBoundsWithCount = properties.getProperty("rowBoundsWithCount");
        if (rowBoundsWithCount != null && rowBoundsWithCount.trim().length() > 0) {
            this.rowBoundsWithCount = Boolean.parseBoolean(rowBoundsWithCount);
        }
    }

    /**
     * 根据 pager 对象获取对应的分页查询Sql语句
     *
     * @param pager 分页对象
     * @param sql   原sql语句
     * @return 分页 SQL 语句
     */
    private String getPageSql(Pager pager, String sql) {
        int offset = (pager.getPageNumber() - 1) * pager.getPageSize();
        int limit = pager.getPageSize();
        return dialect.getLimitString(sql, offset, limit);
    }

    /**
     * 根据原Sql语句获取对应的查询总记录数的Sql语句
     *
     * @param sql 原sql语句
     * @return 总数 SQL 语句
     */
    private String getCountSql(String sql) {
        return dialect.getCountSql(sql);
    }

    public static class BoundSqlSqlSource implements SqlSource {

        BoundSql boundSql;

        public BoundSqlSqlSource(BoundSql boundSql) {
            this.boundSql = boundSql;
        }

        @Override
        public BoundSql getBoundSql(Object parameterObject) {
            return boundSql;
        }
    }
}
