package com.qhiot.core.util;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.beanutils.PropertyUtils;
import org.apache.poi.POIXMLDocument;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.openxml4j.opc.OPCPackage;
import org.apache.poi.poifs.filesystem.POIFSFileSystem;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.jxls.common.Context;
import org.jxls.util.JxlsHelper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.StringUtils;

import javax.mail.internet.MimeUtility;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;
import java.lang.reflect.Field;
import java.net.URLEncoder;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.*;

public class ExcelUtil {

    private static final Logger LOGGER = LoggerFactory.getLogger(ExcelUtil.class);

    private static final String OPEN_XML =
            "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet";

    private static final String XLS = "application/vnd.ms-excel";

    private ExcelUtil() {

    }

    public static <T> List<T> importExcel(Class<T> clazz, File file) throws Exception {
        try {
            return importExcel(clazz, new FileInputStream(file));
        } catch (FileNotFoundException fe) {
            throw new Exception("文件不存在：" + fe.getMessage(), fe);
        }
    }

    public static <T> List<T> importExcel(Class<T> clazz, InputStream in) throws Exception {
        List<T> dist = new ArrayList<T>();
        int iRow = 0, iCol = 0;
        /**
         * 类反射得到调用方法
         */
        Map<String, String> propertymap = new HashMap<String, String>();
        try {
            Field[] filed = clazz.getDeclaredFields();
            for (int i = 0; i < filed.length; i++) {
                Field f = filed[i];
                ExcelHeader exa = f.getAnnotation(ExcelHeader.class);
                if (exa != null) {
                    String fieldname = f.getName();
                    propertymap.put(exa.value(), fieldname);
                }
            }
        } catch (Exception e) {
            throw new Exception(e.getMessage(), e);
        }
        /**
         * excel的解析开始
         */
        String titleString = "";
        Workbook book;
        try {
            boolean isXSSFWorkbook = true;
            if (!(in.markSupported())) {
                in = new PushbackInputStream(in, 8);
            }
            if (POIFSFileSystem.hasPOIFSHeader(in)) {
                book = new HSSFWorkbook(in);
                isXSSFWorkbook = false;
            } else if (POIXMLDocument.hasOOXMLHeader(in)) {
                book = new XSSFWorkbook(OPCPackage.open(in));
            } else {
                throw new Exception("错误的 Excel 文件格式！");
            }

            FormulaEvaluator evaluator = book.getCreationHelper().createFormulaEvaluator();
            Sheet sheet = book.getSheetAt(0);
            sheet.setForceFormulaRecalculation(true);
            Iterator<Row> row = sheet.rowIterator();

            /**
             * 标题解析
             */
            Row titleRow = row.next();
            int rowIndex = 0;
            Iterator<Cell> cellTitle = titleRow.cellIterator();
            Map<Integer, String> titlemap = new HashMap<Integer, String>();
            int colCount = 0;
            while (cellTitle.hasNext()) {
                Cell cell = cellTitle.next();
                boolean isMerge = isMergedRegion(sheet, rowIndex, cell.getColumnIndex());
                String titleValue;
                if (isMerge) {
                    titleValue = getMergedRegionValue(sheet, rowIndex, cell.getColumnIndex(), evaluator);
                } else {
                    titleValue = getCellValue(cell, evaluator);
                }

                titlemap.put(colCount, titleValue);
                colCount++;
            }
            /**
             * 解析内容行
             */
            iRow = 2;
            while (row.hasNext()) {
                Row rown = row.next();
                rowIndex++;
                T tObject = clazz.newInstance();
                iCol = 1;
                boolean emptyRow = true;// 用于控制忽略空行
                for (int i = 0; i < colCount; i++) {
                    Cell cell = rown.getCell(i);

                    if (cell == null) {
                        continue;
                    }
                    titleString = titlemap.get(i);
                    if (propertymap.containsKey(titleString)) {
                        String fieldName = propertymap.get(titleString);
                        String cellValue;
                        boolean isMerge = isMergedRegion(sheet, rowIndex, cell.getColumnIndex());
                        if (isMerge) {
                            cellValue = getMergedRegionValue(sheet, rowIndex, cell.getColumnIndex(), evaluator);
                        } else {
                            cellValue = getCellValue(cell, evaluator);
                        }
                        if (!StringUtils.isEmpty(cellValue) && cellValue.trim().length() > 0) {
                            emptyRow = false;
                        }
                        Class propertyClass = PropertyUtils.getPropertyType(tObject, fieldName);
                        if (propertyClass == Integer.class || propertyClass == int.class
                                || propertyClass == Short.class || propertyClass == short.class
                                || propertyClass == Long.class || propertyClass == long.class) {
                            if (StringUtil.isNumber(cellValue)) {
                                cellValue = numericCellStringValue(Double.parseDouble(cellValue));
                            } else if (!StringUtil.isLong(cellValue)) {
                                throw new RuntimeException("单元格内容与属性 " + fieldName + " 的类型不符");
                            }
                        }
                        BeanUtils.setProperty(tObject, fieldName, cellValue);
                    }
                    iCol++;
                }
                iRow++;
                if (!emptyRow) {
                    dist.add(tObject);
                }
            }
        } catch (Exception e) {
            if (iRow == 0 && iCol == 0) {
                throw new Exception("文件不存在：" + e.getMessage(), e);
            } else {
                String error = "解析Excel文件发生错误(第" + iRow + "行,第" + iCol + "列[" + titleString + "]:" + e.getMessage();
                throw new Exception(error, e);
            }
        } finally {
            if (in != null) {
                try {
                    in.close();
                } catch (IOException ie) {
                    LOGGER.trace(ie.getMessage(), ie);
                }
            }
        }
        return dist;
    }

    /**
     * 解析excel获取每行每列的值（存在合并单元格时，所有子格的值均为合并单元格的值）
     *
     * @param in
     * @return
     * @throws Exception
     */
    public static List<List<String>> importExcel(InputStream in) throws Exception {
        List<List<String>> dist = new ArrayList();
        int iRow = 0, iCol = 0;
        /**
         * excel的解析开始
         */
        Workbook book;
        try {
            boolean isXSSFWorkbook = true;
            if (!(in.markSupported())) {
                in = new PushbackInputStream(in, 8);
            }
            if (POIFSFileSystem.hasPOIFSHeader(in)) {
                book = new HSSFWorkbook(in);
                isXSSFWorkbook = false;
            } else if (POIXMLDocument.hasOOXMLHeader(in)) {
                book = new XSSFWorkbook(OPCPackage.open(in));
            } else {
                throw new Exception("错误的 Excel 文件格式！");
            }

            FormulaEvaluator evaluator = book.getCreationHelper().createFormulaEvaluator();
            Sheet sheet = book.getSheetAt(0);
            sheet.setForceFormulaRecalculation(true);
            Iterator<Row> row = sheet.rowIterator();

            /**
             * 解析内容行
             */
            while (row.hasNext()) {
                Row rown = row.next();
                Iterator<Cell> cellIterator = rown.cellIterator();
                List<String> rowlist = new ArrayList<>();

                while (cellIterator.hasNext()) {
                    Cell cell = cellIterator.next();
                    String cellValue;
                    boolean isMerge = isMergedRegion(sheet, rown.getRowNum(), cell.getColumnIndex());
                    if (isMerge) {
                        cellValue = getMergedRegionValue(sheet, rown.getRowNum(), cell.getColumnIndex(), evaluator);
                    } else {
                        cellValue = getCellValue(cell, evaluator);
                    }
                    rowlist.add(cellValue);
                }
                dist.add(rowlist);
                System.out.println(rowlist.toString());
            }

        } catch (Exception e) {
            if (iRow == 0 && iCol == 0) {
                throw new Exception("文件不存在：" + e.getMessage(), e);
            } else {
                String error = "解析Excel文件发生错误(第" + iRow + "行,第" + iCol + "列:" + e.getMessage();
                throw new Exception(error, e);
            }
        } finally {
            if (in != null) {
                try {
                    in.close();
                } catch (IOException ie) {
                    LOGGER.trace(ie.getMessage(), ie);
                }
            }
        }
        return dist;
    }

    private static String getCellValue(Cell cell, FormulaEvaluator evaluator) {
        String result = "";
        if (cell == null) {
            return result;
        }
        if (cell.getCellType() == Cell.CELL_TYPE_NUMERIC) {
            if (org.apache.poi.ss.usermodel.DateUtil.isCellDateFormatted(cell)) {
                double d = cell.getNumericCellValue();
                Date date = org.apache.poi.ss.usermodel.DateUtil.getJavaDate(d);
                SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd");
                result = formatter.format(date);
            } else {
                result = numericCellStringValue(cell.getNumericCellValue());
            }
        } else if (cell.getCellType() == Cell.CELL_TYPE_STRING) {
            RichTextString cellString = cell.getRichStringCellValue();
            cellString.clearFormatting();
            result = cellString.getString();
        } else if (cell.getCellType() == Cell.CELL_TYPE_ERROR) {
            result = "";
        } else if (cell.getCellType() == Cell.CELL_TYPE_FORMULA) {
            result = evaluator.evaluate(cell).formatAsString();
        } else {
            RichTextString cellString = cell.getRichStringCellValue();
            cellString.clearFormatting();
            result = cellString.getString();
        }
        return result;
    }

    private static String numericCellStringValue(double cellValue) {
        String result = String.valueOf(cellValue);

        // 如果读取的是科学计数法的格式，则转换为普通格式
        if (result.contains(".") && result.contains("E")) {
            DecimalFormat df = new DecimalFormat();
            try {
                result = df.parse(result).toString();
            } catch (Exception e) {
                LOGGER.trace(e.getMessage(), e);
            }
        }

        int index = result.indexOf(".");
        if (index != -1) {
            String s = result.substring(index + 1, result.length());// 获取小数部分的子字符串
            long in = Long.parseLong(s);// 将代表小数部分的字符串转化为整数
            if (in == 0) { // 如果小数部分等于0，则去掉小数部分和小数点
                result = String.valueOf(Math.round(cellValue));
            }
        }
        return result;
    }

    /**
     * 获取合并单元格的值
     */
    private static String getMergedRegionValue(Sheet sheet, int row, int column, FormulaEvaluator evaluator) {
        int sheetMergeCount = sheet.getNumMergedRegions();

        for (int i = 0; i < sheetMergeCount; i++) {
            CellRangeAddress ca = sheet.getMergedRegion(i);
            int firstColumn = ca.getFirstColumn();
            int lastColumn = ca.getLastColumn();
            int firstRow = ca.getFirstRow();
            int lastRow = ca.getLastRow();

            if (row >= firstRow && row <= lastRow) {

                if (column >= firstColumn && column <= lastColumn) {
                    Row fRow = sheet.getRow(firstRow);
                    Cell fCell = fRow.getCell(firstColumn);
                    return getCellValue(fCell, evaluator);
                }
            }
        }

        return null;
    }

    /**
     * 判断指定的单元格是否是合并单元格
     */
    private static boolean isMergedRegion(Sheet sheet, int row, int column) {
        int sheetMergeCount = sheet.getNumMergedRegions();
        for (int i = 0; i < sheetMergeCount; i++) {
            CellRangeAddress range = sheet.getMergedRegion(i);
            int firstColumn = range.getFirstColumn();
            int lastColumn = range.getLastColumn();
            int firstRow = range.getFirstRow();
            int lastRow = range.getLastRow();
            if (row >= firstRow && row <= lastRow) {
                if (column >= firstColumn && column <= lastColumn) {
                    return true;
                }
            }
        }
        return false;
    }

    /**
     * 根据模板导出Excel， 导出的Excel版本由模板的版本决定
     *
     * @param templateClassPath  模板的classPath
     * @param beanParams         excel模板中用到的变量 map
     * @param request            HttpServletRequest
     * @param response           HttpServletResponse
     * @param fileNameWithoutExt 不含扩展名的文件名
     * @throws IOException
     */
    public static void export(String templateClassPath, Map<String, Object> beanParams, HttpServletRequest request,
                              HttpServletResponse response, String fileNameWithoutExt) throws IOException {
        InputStream is = null;
        try {
            is = ExcelUtil.class.getResourceAsStream(templateClassPath);

            response.reset();
            response.setHeader("Pragma", "No-caches");
            response.setHeader("Cache-Control", "must-revalidate, no-transform");
            response.setDateHeader("Expires", 0L);
            response.setCharacterEncoding("UTF-8");

            String ext = FileUtil.extName(templateClassPath);
            if (StringUtil.isNotBlank(ext)) {
                ext = "." + ext;
            }
            if (".xlsx".equals(ext)) {
                response.setContentType(OPEN_XML);
            } else {
                response.setContentType(XLS);
            }
            String userAgent = request.getHeader("User-Agent");
            String encodedFilename;
            boolean isIE = (userAgent != null && userAgent.toLowerCase().contains("msie"));
            boolean isFirefox = (userAgent != null && userAgent.toLowerCase().contains("firefox"));
            if (isIE) {
                encodedFilename = URLEncoder.encode(fileNameWithoutExt, "UTF-8") + ext;
                response.setHeader("Content-Disposition", "attachment; filename=\"" + encodedFilename + "\"");
            } else if (isFirefox) {
                encodedFilename = fileNameWithoutExt + ext;
                response.setHeader("Content-Disposition", "attachment; filename*=UTF-8''" + encodedFilename);
            } else {
                encodedFilename = MimeUtility.encodeText(fileNameWithoutExt, "UTF8", "B") + ext;
                response.setHeader("Content-Disposition", "attachment; filename=" + encodedFilename);
            }
            OutputStream os = response.getOutputStream();
            Context context = new Context(beanParams);
            JxlsHelper.getInstance().processTemplate(is, os, context);

            is.close();
            os.flush();
            os.close();
        } catch (IOException e) {
            throw e;
        } finally {
            if (is != null) {
                is.close();
            }
        }
    }

    @Retention(RetentionPolicy.RUNTIME)
    @Target(ElementType.FIELD)
    public @interface ExcelHeader {

        /**
         * Excel导出时标题显示的文字，如果没有设置 Annotation 属性，将不会被导出和导入
         */
        String value();

        String group() default "";
    }
}
