package com.qhiot.core.util;

import javax.script.ScriptEngine;
import javax.script.ScriptEngineManager;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * 判断某个值是否在某个区间工具类，返回booblean值
 */
public class IntervalUtil {

    private static final String[] operatorList = {"[", "]", "(", ")", "<", ">", "≤", "≥"};

    /**
     * 判断data_value是否在interval区间范围内
     *
     * @param data_value 数值类型的
     * @param interval   正常的数学区间，包括无穷大等，如：(1,3)、>5%、(-∞,6]、(125%,135%)U(70%,80%)
     * @return true：表示data_value在区间interval范围内，false：表示data_value不在区间interval范围内
     */
    public static boolean isInTheInterval(String data_value, String interval) {
        //将区间和data_value转化为可计算的表达式
        try {
            String formula = getFormulaByAllInterval(data_value, interval, "||");
            ScriptEngine jse = new ScriptEngineManager().getEngineByName("JavaScript");
            //计算表达式
            return (Boolean) jse.eval(formula);
        } catch (Exception t) {
            return false;
        }
    }

    /**
     * 将所有阀值区间转化为公式：如
     * [75,80)   =》                   date_value < 80 && date_value >= 75
     * (125%,135%)U(70%,80%)   =》     (date_value < 1.35 && date_value > 1.25) || (date_value < 0.8 && date_value > 0.7)
     *
     * @param date_value
     * @param interval   形式如：(125%,135%)U(70%,80%)
     * @param connector  连接符 如：") || ("
     */
    private static String getFormulaByAllInterval(String date_value, String interval, String connector) throws Exception {
        StringBuffer buff = new StringBuffer();
        for (String limit : interval.split("U")) {//如：（125%,135%）U (70%,80%)
            buff.append("(").append(getFormulaByInterval(date_value, limit, " && ")).append(")").append(connector);
        }
        String allLimitInvel = buff.toString();
        int index = allLimitInvel.lastIndexOf(connector);
        allLimitInvel = allLimitInvel.substring(0, index);
        return allLimitInvel;
    }

    /**
     * 将整个阀值区间转化为公式：如
     * 145)      =》         date_value < 145
     * [75,80)   =》        date_value < 80 && date_value >= 75
     *
     * @param date_value
     * @param interval   形式如：145)、[75,80)
     * @param connector  连接符 如：&&
     */
    private static String getFormulaByInterval(String date_value, String interval, String connector) throws Exception {
        StringBuffer buff = new StringBuffer();
        String[] stringArray = interval.split(",");
        if (stringArray.length > 2) {
            throw new Exception("整个区间中只可拥有一个','");
        }
        for (String halfInterval : interval.split(",")) {//如：[75,80)、≥80
            buff.append(getFormulaByHalfInterval(halfInterval, date_value)).append(connector);
        }
        String limitInvel = buff.toString();
        int index = limitInvel.lastIndexOf(connector);
        limitInvel = limitInvel.substring(0, index);
        return limitInvel;
    }

    /**
     * 将半个阀值区间转化为公式：如
     * 145)      =》         date_value < 145
     * ≥80%      =》         date_value >= 0.8
     * [130      =》         date_value >= 130
     * <80%     =》         date_value < 0.8
     *
     * @param halfInterval 形式如：145)、≥80%、[130、<80%
     * @param date_value
     * @return date_value < 145
     */
    private static String getFormulaByHalfInterval(String halfInterval, String date_value) throws Exception {
        halfInterval = halfInterval.trim();
        if (halfInterval.contains("∞")) {//包含无穷大则不需要公式
            return "1 == 1";
        }
        StringBuffer formula = new StringBuffer();
        String data = "";
        String opera = "";
        if (halfInterval.matches("^([<>≤≥\\[\\(]{1}(-?\\d+.?\\d*\\%?))$")) {//表示判断方向（如>）在前面 如：≥80%
            opera = halfInterval.substring(0, 1);
            data = halfInterval.substring(1);
        } else {//[130、145)
            opera = halfInterval.substring(halfInterval.length() - 1);
            data = halfInterval.substring(0, halfInterval.length() - 1);
        }
        checkOperator(opera);
        double value = dealPercent(data);
        formula.append(date_value).append(" ").append(opera).append(" ").append(value);
        String a = formula.toString();
        //转化特定字符
        return a.replace("[", ">=").replace("(", ">").replace("]", "<=").replace(")", "<").replace("≤", "<=").replace("≥", ">=");
    }

    /**
     * 去除百分号，转为小数
     *
     * @param str 可能含百分号的数字
     * @return
     */
    private static double dealPercent(String str) {
        double d = 0.0;
        if (str.contains("%")) {
            str = str.substring(0, str.length() - 1);
            d = Double.parseDouble(str) / 100;
        } else {
            d = Double.parseDouble(str);
        }
        return d;
    }

    private static void checkOperator(String operator) throws Exception {
        if (!ArrayUtil.contains(operatorList, operator)) {
            throw new Exception("非法的运算符");
        }
    }

    /**
     * 判断一个字符串是否为一个标准区间
     *
     * @param interval
     * @return
     */
    public static boolean isAInterval(String interval) {
        boolean result = true;
        try {
            getFormulaByAllInterval("1", interval, "||");
        } catch (Exception e) {
            result = false;
        }
        return result;
    }

    /**
     * 判断一个字符串是否含有中文标点符号
     *
     * @param interval
     * @return
     */
    public static boolean isContainChinesePunctuation(String interval) {
        Pattern p = Pattern.compile("[\uFF01]|[\uFF0C-\uFF0E]|[\uFF1A-\uFF1B]|[\uFF1F]|[\uFF08-\uFF09]|[\u3001-\u3002]|[\u3010-\u3011]|[\u201C-\u201D]|[\u2013-\u2014]|[\u2018-\u2019]|[\u2026]|[\u3008-\u300F]|[\u3014-\u3015]");
        Matcher m = p.matcher(interval);
        return m.find();
    }

//    public static void main(String[] args) {
//
//        System.out.println(IntervalUtil.isInTheInterval("0.49", "[40%,80%)U(100%,120%]"));
//    }
}
