package com.qhiot.core.util;

import okhttp3.*;
import org.apache.http.NameValuePair;
import org.apache.http.entity.ContentType;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.ws.rs.core.HttpHeaders;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.URLEncoder;
import java.util.*;
import java.util.concurrent.TimeUnit;

/**
 * OkHTTP 请求工具类, 请设置 java 参数: -Djava.net.preferIPv4Stack=true
 *
 * @author 金建强(ptma @ 163.com)
 * @version 2016/6/15 15:05
 */
public class OkHttpUtil {

    private static final Logger LOGGER = LoggerFactory.getLogger(OkHttpUtil.class);

    private OkHttpUtil() {

    }

    private static OkHttpClient getClient() {
        return ClientHolder.INSTANCE;
    }

    /**
     * GET 请求
     *
     * @param url 请求 Url
     * @return 请求结果
     * @throws IOException
     */
    public static String get(String url) throws IOException {
        return get(url, null, null);
    }

    /**
     * GET 请求
     *
     * @param url    请求 Url
     * @param params 参数
     * @return 请求结果
     * @throws IOException
     */
    public static String get(String url, Map<String, Object> params) throws IOException {
        return get(url, params, null);
    }

    /**
     * POST 表单 (Content-Type: application/x-www-form-urlencoded; charset=UTF-8)
     *
     * @param url     请求 Url
     * @param params  Map with parameters/values
     * @param headers HTTP头, addHeader(“Accept-Encoding”, “gzip”) 会导致乱码, OkHttp会认为你要自己处理解压缩步骤
     * @return 请求结果
     * @throws IOException
     */
    public static String get(String url, Map<String, Object> params, Map<String, String> headers) throws IOException {
        OkHttpClient client = getClient();
        Request.Builder requestBuilder = new Request.Builder();
        requestBuilder.get().url(appendQueryParams(url, params));

        // headers
        if (headers == null) {
            headers = new HashMap<String, String>();
        }
        Iterator<Map.Entry<String, String>> iterator = headers.entrySet().iterator();
        while (iterator.hasNext()) {
            Map.Entry<String, String> entry = iterator.next();
            requestBuilder.header(entry.getKey(), entry.getValue());
        }

        Response response = client.newCall(requestBuilder.build()).execute();
        if (!response.isSuccessful()) {
            response.close();
            throw new IOException("Unexpected code " + response.code());
        }
        return response.body().string();
    }

    /**
     * GET 请求 (Accept:application/json)
     *
     * @param url 请求 Url
     * @return 请求结果
     * @throws IOException
     */
    public static String getJson(String url) throws IOException {
        return getJson(url, null, null);
    }

    /**
     * GET 请求 (Accept:application/json)
     *
     * @param url    请求 Url
     * @param params 参数
     * @return 请求结果
     * @throws IOException
     */
    public static String getJson(String url, Map<String, Object> params) throws IOException {
        return getJson(url, params, null);
    }

    /**
     * GET 请求 (Accept:application/json)
     *
     * @param url     请求 Url
     * @param params  参数
     * @param headers HTTP头, addHeader(“Accept-Encoding”, “gzip”) 会导致乱码, OkHttp会认为你要自己处理解压缩步骤
     * @return 请求结果
     * @throws IOException
     */
    public static String getJson(String url, Map<String, Object> params, Map<String, String> headers)
            throws IOException {
        if (headers == null) {
            headers = new HashMap<String, String>();
        }
        headers.put(HttpHeaders.ACCEPT, ContentType.APPLICATION_JSON.toString());
        return get(url, params, headers);
    }

    /**
     * POST 表单 (Content-Type: application/x-www-form-urlencoded; charset=UTF-8)
     *
     * @param url    请求 Url
     * @param params map with parameters/values
     * @return 请求结果
     * @throws IOException
     */
    public static String postForm(String url, Map<String, Object> params) throws IOException {
        return postForm(url, params, null);
    }

    /**
     * POST 表单 (Content-Type: application/x-www-form-urlencoded; charset=UTF-8)
     *
     * @param url     请求 Url
     * @param params  Map with parameters/values
     * @param headers HTTP头, addHeader(“Accept-Encoding”, “gzip”) 会导致乱码, OkHttp会认为你要自己处理解压缩步骤
     * @return 请求结果
     * @throws IOException
     */
    public static String postForm(String url, Map<String, Object> params, Map<String, String> headers)
            throws IOException {
        OkHttpClient client = getClient();

        Request.Builder requestBuilder = new Request.Builder();

        FormBody.Builder formBodyBuilder = new FormBody.Builder();
        if (params != null) {
            Iterator<Map.Entry<String, Object>> itr = params.entrySet().iterator();
            while (itr.hasNext()) {
                Map.Entry<String, Object> entry = itr.next();
                formBodyBuilder.add(entry.getKey(), entry.getValue().toString());
            }
        }

        requestBuilder.url(url).post(formBodyBuilder.build());


        // headers
        if (headers == null) {
            headers = new HashMap<String, String>();
        }
        Iterator<Map.Entry<String, String>> iterator = headers.entrySet().iterator();
        while (iterator.hasNext()) {
            Map.Entry<String, String> entry = iterator.next();
            requestBuilder.addHeader(entry.getKey(), entry.getValue());
        }

        Response response = client.newCall(requestBuilder.build()).execute();
        if (!response.isSuccessful()) {
            response.close();
            throw new IOException("Unexpected code " + response.code());
        }
        return response.body().string();
    }

    /**
     * POST 表单 (Content-Type: application/x-www-form-urlencoded; charset=UTF-8)
     *
     * @param url     请求 Url
     * @param params  Map with parameters/values
     * @param headers HTTP头, addHeader(“Accept-Encoding”, “gzip”) 会导致乱码, OkHttp会认为你要自己处理解压缩步骤
     * @return 请求结果
     * @throws IOException
     */
    public static void postFormAsync(String url, Map<String, Object> params, Map<String, String> headers, Callback callback)
            throws IOException {
        OkHttpClient client = getClient();

        Request.Builder requestBuilder = new Request.Builder();

        FormBody.Builder formBodyBuilder = new FormBody.Builder();
        if (params != null) {
            Iterator<Map.Entry<String, Object>> itr = params.entrySet().iterator();
            while (itr.hasNext()) {
                Map.Entry<String, Object> entry = itr.next();
                formBodyBuilder.add(entry.getKey(), entry.getValue().toString());
            }
        }

        requestBuilder.url(url).post(formBodyBuilder.build());


        // headers
        if (headers == null) {
            headers = new HashMap<String, String>();
        }
        Iterator<Map.Entry<String, String>> iterator = headers.entrySet().iterator();
        while (iterator.hasNext()) {
            Map.Entry<String, String> entry = iterator.next();
            requestBuilder.addHeader(entry.getKey(), entry.getValue());
        }
        if (callback == null) {
            callback = new Callback() {
                @Override
                public void onFailure(Call call, IOException e) {
                    LOGGER.error("http请求失败", e);
                }

                @Override
                public void onResponse(Call call, Response response) throws IOException {
                    LOGGER.error("http请求成功", response.message());
                }
            };
        }
        client.newCall(requestBuilder.build()).enqueue(callback);
    }

    /**
     * POST 请求
     *
     * @param url  请求 Url
     * @param body 请求正文
     * @return 请求结果
     * @throws IOException
     */
    public static String postBody(String url, String body) throws IOException {
        return postBody(url, body, null);
    }

    /**
     * POST 请求
     *
     * @param url     请求 Url
     * @param body    请求正文
     * @param headers HTTP头, addHeader(“Accept-Encoding”, “gzip”) 会导致乱码, OkHttp会认为你要自己处理解压缩步骤
     * @return 请求结果
     * @throws IOException
     */
    public static String postBody(String url, String body, Map<String, String> headers) throws IOException {
        OkHttpClient client = getClient();
        Request.Builder requestBuilder = new Request.Builder();

        // headers
        if (headers == null) {
            headers = new HashMap<String, String>();
        }
        Iterator<Map.Entry<String, String>> iterator = headers.entrySet().iterator();
        while (iterator.hasNext()) {
            Map.Entry<String, String> entry = iterator.next();
            requestBuilder.addHeader(entry.getKey(), entry.getValue());
        }

        MediaType MEDIA_TYPE_TEXT = MediaType.parse("text/plain; charset=UTF-8");
        requestBuilder.url(url).post(RequestBody.create(MEDIA_TYPE_TEXT, body));

        Response response = client.newCall(requestBuilder.build()).execute();
        if (!response.isSuccessful()) {
            response.close();
            throw new IOException("Unexpected code " + response.code() + ", " + response.body().string());
        }
        return response.body().string();
    }

    /**
     * POST 请求
     *
     * @param url     请求 Url
     * @param jsonObj 请求正文
     * @return 请求结果
     * @throws IOException
     */
    public static String postJson(String url, Object jsonObj) throws IOException {
        return postJson(url, jsonObj, null);
    }

    /**
     * POST 请求
     *
     * @param url     请求 Url
     * @param jsonObj 请求json对象
     * @param headers HTTP头, addHeader(“Accept-Encoding”, “gzip”) 会导致乱码, OkHttp会认为你要自己处理解压缩步骤
     * @return 请求结果
     * @throws IOException
     */
    public static String postJson(String url, Object jsonObj, Map<String, String> headers) throws IOException {
        OkHttpClient client = getClient();

        Request.Builder requestBuilder = new Request.Builder();

        // headers
        if (headers == null) {
            headers = new HashMap<String, String>();
        }
        if (!headers.containsKey(HttpHeaders.CONTENT_TYPE)) {
            headers.put(HttpHeaders.CONTENT_TYPE, ContentType.APPLICATION_JSON.toString());
        }
        Iterator<Map.Entry<String, String>> iterator = headers.entrySet().iterator();
        while (iterator.hasNext()) {
            Map.Entry<String, String> entry = iterator.next();
            requestBuilder.addHeader(entry.getKey(), entry.getValue());
        }

        MediaType MEDIA_TYPE_TEXT = MediaType.parse("application/json; charset=UTF-8");
        requestBuilder.url(url)
                .post(RequestBody.create(MEDIA_TYPE_TEXT, JsonUtil.toJson(jsonObj)));

        Response response = client.newCall(requestBuilder.build()).execute();
        if (!response.isSuccessful()) {
            response.close();
            throw new IOException("Unexpected code " + response.code());
        }
        return response.body().string();
    }

    /**
     * POST 请求
     *
     * @param url     请求 Url
     * @param jsonObj 请求json对象
     * @param headers HTTP头, addHeader(“Accept-Encoding”, “gzip”) 会导致乱码, OkHttp会认为你要自己处理解压缩步骤
     * @return 请求结果
     * @throws IOException
     */
    public static void postJsonAsync(String url, Object jsonObj, Map<String, String> headers, Callback callback) throws IOException {
        OkHttpClient client = getClient();

        Request.Builder requestBuilder = new Request.Builder();

        // headers
        if (headers == null) {
            headers = new HashMap<String, String>();
        }
        if (!headers.containsKey(HttpHeaders.CONTENT_TYPE)) {
            headers.put(HttpHeaders.CONTENT_TYPE, ContentType.APPLICATION_JSON.toString());
        }
        Iterator<Map.Entry<String, String>> iterator = headers.entrySet().iterator();
        while (iterator.hasNext()) {
            Map.Entry<String, String> entry = iterator.next();
            requestBuilder.addHeader(entry.getKey(), entry.getValue());
        }

        MediaType MEDIA_TYPE_TEXT = MediaType.parse("application/json; charset=UTF-8");
        requestBuilder.url(url)
                .post(RequestBody.create(MEDIA_TYPE_TEXT, JsonUtil.toJson(jsonObj)));

        if (callback == null) {
            callback = new Callback() {
                @Override
                public void onFailure(Call call, IOException e) {
                    LOGGER.error("http请求失败", e);
                }

                @Override
                public void onResponse(Call call, Response response) throws IOException {
                    LOGGER.error("http请求成功", response.message());
                }
            };
        }
        client.newCall(requestBuilder.build()).enqueue(callback);
    }

    /**
     * PUT 请求
     *
     * @param url 请求 Url
     * @return 请求结果
     * @throws IOException
     */
    public static String put(String url, String body, Map<String, String> headers) throws IOException {
        OkHttpClient client = getClient();

        Request.Builder requestBuilder = new Request.Builder();

        // headers
        if (headers == null) {
            headers = new HashMap<String, String>();
        }
        Iterator<Map.Entry<String, String>> iterator = headers.entrySet().iterator();
        while (iterator.hasNext()) {
            Map.Entry<String, String> entry = iterator.next();
            requestBuilder.addHeader(entry.getKey(), entry.getValue());
        }

        MediaType MEDIA_TYPE_TEXT = MediaType.parse("text/plain; charset=UTF-8");
        requestBuilder.url(url)
                .put(RequestBody.create(MEDIA_TYPE_TEXT, body));

        Response response = client.newCall(requestBuilder.build()).execute();
        if (!response.isSuccessful()) {
            response.close();
            throw new IOException("Unexpected code " + response.code());
        }
        return response.body().string();
    }

    /**
     * DELETE 请求
     *
     * @param url 请求 Url
     * @return 请求结果
     * @throws IOException
     */
    public static String delete(String url) throws IOException {
        return delete(url, null);
    }

    /**
     * DELETE 请求
     *
     * @param url     请求 Url
     * @param headers HTTP头, addHeader(“Accept-Encoding”, “gzip”) 会导致乱码, OkHttp会认为你要自己处理解压缩步骤
     * @return 请求结果
     * @throws IOException
     */
    public static String delete(String url, Map<String, String> headers) throws IOException {
        OkHttpClient client = getClient();

        Request.Builder requestBuilder = new Request.Builder();

        // headers
        if (headers == null) {
            headers = new HashMap<String, String>();
        }
        Iterator<Map.Entry<String, String>> iterator = headers.entrySet().iterator();
        while (iterator.hasNext()) {
            Map.Entry<String, String> entry = iterator.next();
            requestBuilder.addHeader(entry.getKey(), entry.getValue());
        }
        requestBuilder.url(url).delete();
        Response response = client.newCall(requestBuilder.build()).execute();
        if (!response.isSuccessful()) {
            response.close();
            throw new IOException("Unexpected code " + response.code());
        }
        return response.body().string();
    }

    /**
     * Post 上传
     *
     * @param url
     * @param params
     * @param files
     * @return
     */
    public static String postMultiPart(String url, Map<String, Object> params, Collection<FileEntry> files) throws IOException {
        return postMultiPart(url, params, files, null);
    }

    /**
     * Post 上传
     *
     * @param url
     * @param params
     * @param files
     * @param headers
     * @return
     */
    public static String postMultiPart(String url, Map<String, Object> params, Collection<FileEntry> files,
                                       Map<String, String> headers) throws IOException {
        OkHttpClient client = getClient();

        Request.Builder requestBuilder = new Request.Builder();
        // headers
        if (headers == null) {
            headers = new HashMap<String, String>();
        }
        Iterator<Map.Entry<String, String>> iterator = headers.entrySet().iterator();
        while (iterator.hasNext()) {
            Map.Entry<String, String> entry = iterator.next();
            requestBuilder.addHeader(entry.getKey(), entry.getValue());
        }

        MultipartBody.Builder bodyBuilder = new MultipartBody.Builder()
                .setType(MultipartBody.FORM);

        // parameters
        if (params != null) {
            List<NameValuePair> urlParameters = new ArrayList<NameValuePair>();
            for (Map.Entry<String, Object> entry : params.entrySet()) {
                if (entry.getValue() instanceof File) {
                    File f = (File) entry.getValue();
                    MediaType type = MediaType.parse("application/octet-stream");
                    RequestBody fileBody = RequestBody.create(type, f);
                    bodyBuilder.addFormDataPart(entry.getKey(), f.getName(), fileBody);
                    // identify param type by Key
                } else {
                    bodyBuilder.addFormDataPart(entry.getKey(), String.valueOf(entry.getValue()));
                }
            }
        }

        if (files != null) {
            for (FileEntry entry : files) {
                RequestBody fileBody = RequestBody.create(entry.getMediaType(), entry.getStreamBytes());
                bodyBuilder.addFormDataPart(entry.getName(), entry.getFileName(), fileBody);
            }
        }

        requestBuilder.url(url).post(bodyBuilder.build());
        Response response = client.newCall(requestBuilder.build()).execute();
        if (!response.isSuccessful()) {
            response.close();
            throw new IOException("Unexpected code " + response.code());
        }
        return response.body().string();
    }

    /**
     * WebService SOAP1.1 请求
     *
     * @param wsdl      WebService地址或 wsdl 地址
     * @param namespace 命名空间
     * @param method    方法名
     * @param params    参数, 为了保证参数有序, 建议使用 LinkedHashMap
     * @return 请求结果
     * @throws Exception
     */
    public static String soap11(String wsdl, String namespace, String method, Map<String, Object> params)
            throws Exception {
        // 拼接SOAP
        StringBuffer soapRequestData = new StringBuffer("");
        soapRequestData.append("<soapenv:Envelope xmlns:soapenv=\"http://schemas.xmlsoap.org/soap/envelope/\"");
        if (StringUtil.isNotEmpty(namespace)) {
            soapRequestData.append(StringUtil.format(" xmlns:ns=\"{}\"", namespace));
        }
        soapRequestData.append(">");
        soapRequestData.append("<soapenv:Header/>");
        soapRequestData.append("<soapenv:Body>");
        if (StringUtil.isEmpty(namespace)) {
            soapRequestData.append(StringUtil.format("<{}>", method));
        } else {
            soapRequestData.append(StringUtil.format("<ns:{}>", method));
        }

        // 拼接参数
        soapRequestData.append(getSoapMethodParam(params));

        if (StringUtil.isEmpty(namespace)) {
            soapRequestData.append(StringUtil.format("</{}>", method));
        } else {
            soapRequestData.append(StringUtil.format("</ns:{}>", method));
        }

        soapRequestData.append("</soapenv:Body></soapenv:Envelope>");

        // SOAP 1.1 HTTP 头
        Map<String, String> headers = new HashMap<String, String>();
        headers.put("Content-Type", "text/xml; charset=UTF-8");
        headers.put("SOAPAction", method);

        return postBody(wsdl, soapRequestData.toString(), headers);
    }

    /**
     * WebService SOAP1.2 请求
     *
     * @param wsdl      WebService地址或 wsdl 地址
     * @param namespace 命名空间
     * @param method    方法名
     * @param params    参数, 为了保证参数有序, 建议使用 LinkedHashMap
     * @return 请求结果
     * @throws Exception
     */
    public static String soap12(String wsdl, String namespace, String method, Map<String, Object> params)
            throws Exception {
        // 拼接SOAP
        StringBuffer soapRequestData = new StringBuffer("");
        soapRequestData.append("<soap:Envelope xmlns:soap=\"http://www.w3.org/2003/05/soap-envelope\"");
        if (StringUtil.isNotEmpty(namespace)) {
            soapRequestData.append(StringUtil.format(" xmlns:ws=\"{}\"", namespace));
        }
        soapRequestData.append(">");
        soapRequestData.append("<soap:Header/>");
        soapRequestData.append("<soap:Body>");
        if (StringUtil.isEmpty(namespace)) {
            soapRequestData.append(StringUtil.format("<{}>", method));
        } else {
            soapRequestData.append(StringUtil.format("<ws:{}>", method));
        }

        // 拼接参数
        soapRequestData.append(getSoapMethodParam(params));

        if (StringUtil.isEmpty(namespace)) {
            soapRequestData.append(StringUtil.format("</{}>", method));
        } else {
            soapRequestData.append(StringUtil.format("</ws:{}>", method));
        }

        soapRequestData.append("</soap:Body></soap:Envelope>");

        // SOAP 1.2 HTTP 头
        Map<String, String> headers = new HashMap<String, String>();
        headers.put(HttpHeaders.CONTENT_TYPE, "application/soap+xml;UTF-8");

        return postBody(wsdl, soapRequestData.toString(), headers);
    }

    /**
     * 下载文件
     *
     * @param url 请求 Url
     * @return
     * @throws IOException
     */
    public static InputStream download(String url) throws IOException {
        return download(url, null);
    }

    /**
     * 下载文件
     *
     * @param url    请求 Url
     * @param params 参数
     * @return
     * @throws IOException
     */
    public static InputStream download(String url, Map<String, Object> params) throws IOException {
        OkHttpClient client = getClient();

        Request.Builder requestBuilder = new Request.Builder();
        requestBuilder.url(appendQueryParams(url, params)).get();
        Response response = client.newCall(requestBuilder.build()).execute();
        if (!response.isSuccessful()) {
            response.close();
            throw new IOException("Unexpected code " + response.code());
        }
        return response.body().byteStream();
    }

    private static String getSoapMethodParam(Map<String, Object> params) {
        StringBuffer param = new StringBuffer("");
        if (params != null && !params.isEmpty()) {
            Iterator<Map.Entry<String, Object>> itr = params.entrySet().iterator();
            while (itr.hasNext()) {
                Map.Entry<String, Object> entry = itr.next();
                param.append(StringUtil.format("<{}>{}</{}>", entry.getKey(), entry.getValue(), entry.getKey()));
            }
        }
        return param.toString();
    }

    /**
     * 拼接参数到 Url
     *
     * @param url    请求 Url
     * @param params 参数
     * @return url 拼装后的 Url
     * @throws IOException
     */
    public static String appendQueryParams(String url, Map<String, Object> params) throws IOException {
        StringBuffer fullUrl = new StringBuffer(url);
        if (params != null) {
            boolean first = (fullUrl.indexOf("?") == -1);
            Iterator<Map.Entry<String, Object>> itr = params.entrySet().iterator();
            while (itr.hasNext()) {
                Map.Entry<String, Object> entry = itr.next();
                if (first) {
                    fullUrl.append("?");
                    first = false;
                } else {
                    fullUrl.append("&");
                }
                fullUrl.append(URLEncoder.encode(entry.getKey(), "UTF-8")).append("=");
                fullUrl.append(URLEncoder.encode(entry.getValue().toString(), "UTF-8"));
            }
        }

        return fullUrl.toString();
    }

    private static String streamToString(InputStream inputStream) throws IOException {
        ByteArrayOutputStream result = new ByteArrayOutputStream();
        byte[] buffer = new byte[1024];
        int length;
        while ((length = inputStream.read(buffer)) != -1) {
            result.write(buffer, 0, length);
        }
        return result.toString("UTF-8");
    }

    private static class ClientHolder {
        final static OkHttpClient INSTANCE = new OkHttpClient.Builder()//构建器
                .connectTimeout(15, TimeUnit.SECONDS)//连接超时
                .writeTimeout(20, TimeUnit.SECONDS)//写入超时
                .readTimeout(20, TimeUnit.SECONDS)//读取超时
                .build();
    }

    public static class FileEntry {
        private String name;
        private String fileName;
        private MediaType mediaType;
        private InputStream stream;

        public String getName() {
            return name;
        }

        public void setName(String name) {
            this.name = name;
        }

        public String getFileName() {
            return fileName;
        }

        public void setFileName(String fileName) {
            this.fileName = fileName;
        }

        public InputStream getStream() {
            return stream;
        }

        public void setStream(InputStream stream) {
            this.stream = stream;
        }

        public byte[] getStreamBytes() throws IOException {
            byte[] byt = new byte[stream.available()];
            stream.read(byt);
            return byt;
        }

        public MediaType getMediaType() {
            return mediaType;
        }

        public void setMediaType(MediaType mediaType) {
            this.mediaType = mediaType;
        }
    }
}
