package com.qhiot.core.util;

import org.apache.commons.lang.StringUtils;

import javax.servlet.ServletContext;
import javax.servlet.ServletRequest;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;

/**
 * The Class RequestUtil.
 *
 * @author 金建强(ptma @ 163.com)
 * @version 1.0.0
 * @since 2013-4-7 17:16:04
 */
public class RequestUtil {

    public static boolean isMultipart(HttpServletRequest request) {
        return ((request.getContentType() != null) && (request.getContentType().toLowerCase().startsWith("multipart")));
    }

    /**
     * 获取COOKIE
     *
     * @param name
     */
    public static Cookie getCookie(HttpServletRequest request, String name) {
        Cookie[] cookies = request.getCookies();
        if (cookies == null) {
            return null;
        }
        for (int i = 0; i < cookies.length; i++) {
            if (name.equals(cookies[i].getName())) {
                return cookies[i];
            }
        }
        return null;
    }

    /**
     * 设置COOKIE
     *
     * @param name
     * @param value
     * @param maxAge
     */
    public static void setCookie(HttpServletRequest request, HttpServletResponse response, String name, String value,
                                 int maxAge) {
        Cookie cookie = new Cookie(name, value);
        cookie.setMaxAge(maxAge);
        cookie.setPath("/");
        response.addCookie(cookie);
    }

    /**
     * 从URL地址中解析出URL前缀，例如 http://www.163.com:8081/index.jsp -> http://www.163.com:8081
     *
     * @param request
     * @return
     */
    public static String getUrlPrefix(HttpServletRequest request) {
        StringBuffer url = new StringBuffer(request.getScheme());
        url.append("://");
        url.append(request.getServerName());
        int port = request.getServerPort();
        if (port != 80) {
            url.append(":");
            url.append(port);
        }
        return url.toString();
    }

    /**
     * 获取访问的URL全路径
     *
     * @param request
     * @return
     */
    public static String getRequestURL(HttpServletRequest request, boolean withQueryString) {
        StringBuffer url = new StringBuffer(request.getRequestURI());
        String param = request.getQueryString();
        if (withQueryString && param != null) {
            url.append('?');
            url.append(param);
        }
        return url.substring(request.getContextPath().length());
    }

    /**
     * 获取请求的相对路径（相对于应用而言， 不含应用名）
     *
     * @param request
     * @return
     */
    public static String getRequestRelativeURL(HttpServletRequest request) {
        String requestUrl = StringUtil.removePrefix(request.getRequestURI(), RequestUtil.getContextPath(request) + "/");
        return StringUtil.removeSuffix(requestUrl, "/");
    }

    /**
     * 判断浏览器是否与Mozilla兼容
     *
     * @param request
     * @return
     */
    public static boolean isMozillaCompatible(HttpServletRequest request) {
        String userAgent = request.getHeader("user-agent");
        return userAgent == null || userAgent.indexOf("Mozilla") != -1;
    }

    /**
     * 是否为JSON请求
     *
     * @param request
     * @return
     */
    public static boolean isJsonRequest(HttpServletRequest request) {
        String acceptHeader = request.getHeader("Accept");
        return acceptHeader != null && acceptHeader.indexOf("application/json") != -1;
    }

    /**
     * 获取浏览器提交的整形参数
     *
     * @param param
     * @param defaultValue
     * @return
     */
    public static int getIntParam(HttpServletRequest request, String param, int defaultValue) {
        try {
            String value = request.getParameter(param);
            int idx = value.indexOf('#');
            if (idx != -1) {
                value = value.substring(0, idx);
            }
            return Integer.parseInt(value);
        } catch (Exception e) {
        }
        return defaultValue;
    }

    /**
     * 获取浏览器提交的整形参数
     *
     * @param param
     * @param defaultValue
     * @return
     */
    public static long getLongParam(HttpServletRequest request, String param, long defaultValue) {
        try {
            String value = request.getParameter(param);
            int idx = value.indexOf('#');
            if (idx != -1) {
                value = value.substring(0, idx);
            }
            return Long.parseLong(value);
        } catch (Exception e) {
        }
        return defaultValue;
    }

    /**
     * 获取浏览器提交的字符串参数
     *
     * @param param
     * @param defaultValue
     * @return
     */
    public static String getStringParam(HttpServletRequest request, String param, String defaultValue) {
        String value = request.getParameter(param);
        return (StringUtil.isEmpty(value)) ? defaultValue : value;
    }

    public static void attachment(HttpServletRequest request, HttpServletResponse response, String filename) throws UnsupportedEncodingException {
        String userAgent = request.getHeader("User-Agent");
        String attachmentFilename = filename;
        if (StringUtils.contains(userAgent, "MSIE") || StringUtils.contains(userAgent, "Edge")) {//IE浏览器，edge
            attachmentFilename = URLEncoder.encode(filename, "UTF8");
        } else if (StringUtils.contains(userAgent, "Mozilla")) {//google,火狐浏览器
            attachmentFilename = new String(filename.getBytes(), "ISO8859-1");
        } else {
            attachmentFilename = URLEncoder.encode(filename, "UTF8");//其他浏览器
        }
        response.setHeader("Content-Disposition", "attachment; filename=" + attachmentFilename);
        response.setCharacterEncoding("UTF-8");
        response.setContentType("application/x-download");
        // 重置 Content-Encoding, 避免被 GZIPFilter 压缩
        response.setHeader("Content-Encoding", "");
    }

    /**
     * 获取所有参数
     *
     * @param request
     * @return
     */
    public static Map<String, String> getParamMap(HttpServletRequest request) {
        Map<String, String> parameterMap = new HashMap<String, String>();
        Enumeration<String> parameterNames = request.getParameterNames();
        String parameterName;
        while (parameterNames.hasMoreElements()) {
            parameterName = parameterNames.nextElement();
            parameterMap.put(parameterName, request.getParameter(parameterName));
        }
        return parameterMap;
    }

    /**
     * 获取所有 RequestHeader
     *
     * @param request
     * @return
     */
    public static Map<String, String> getHeaderMap(HttpServletRequest request) {
        Map<String, String> headerMap = new HashMap<String, String>();
        Enumeration<String> headerNames = request.getHeaderNames();
        String headerName;
        while (headerNames.hasMoreElements()) {
            headerName = headerNames.nextElement();
            headerMap.put(headerName, request.getHeader(headerName));
        }
        return headerMap;
    }

    /**
     * 获取应用名
     *
     * @param request
     * @return
     */
    public static String getContextPath(ServletRequest request) {
        return getContextPath(request.getServletContext());
    }

    /**
     * 获取应用名
     *
     * @param context
     * @return
     */
    public static String getContextPath(ServletContext context) {
        String contextPath = context.getContextPath();
        if (contextPath == null || contextPath.length() == 0) {
            contextPath = "";
        }
        return contextPath;
    }
}
