package com.qhiot.core.util;

import org.apache.commons.codec.binary.Hex;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.InputStream;
import java.nio.charset.Charset;
import java.security.MessageDigest;
import java.security.SecureRandom;
import java.util.Arrays;

/**
 * <p>
 * User: 金建强(ptma@163.com)
 * </p>
 * <p>
 * Date: 2015-11-11 19:11.
 * </p>
 */
public final class SecureUtil {

    private static final Logger LOGGER = LoggerFactory.getLogger(SecureUtil.class);

    private static final String SHA1 = "SHA-1";
    private static final String SHA256 = "SHA-256";
    private static final String SHA384 = "SHA-384";
    private static final String SHA512 = "SHA-512";
    private static final String MD5 = "MD5";
    private static final int[] INV = new int[256];
    private static final char[] CHAR_ARRAY =
            "_-0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ".toCharArray();
    private static SecureRandom random = new SecureRandom();
    /**
     * base64码表
     */
    private static char[] base64EncodeTable =
            {'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q',
                    'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z', 'a', 'b', 'c', 'd', 'e', 'f', 'g',
                    'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w',
                    'x', 'y', 'z', '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '+', '/'};

    static {
        Arrays.fill(INV, -1);
        for (int i = 0, iS = base64EncodeTable.length; i < iS; i++) {
            INV[base64EncodeTable[i]] = i;
        }
        INV['='] = 0;
    }

    private SecureUtil() {
    }

    public static String md5(String input) {
        return digest(input, MD5, null, 1);
    }

    public static String sha1(String input) {
        return digest(input, SHA1, null, 1);
    }

    public static String sha256(String input) {
        return digest(input, SHA256, null, 1);
    }

    public static String sha384(String input) {
        return digest(input, SHA384, null, 1);
    }

    public static String sha512(String input) {
        return digest(input, SHA512, null, 1);
    }

    public static String md5(String input, byte[] salt, int iterations) {
        return digest(input, MD5, salt, iterations);
    }

    public static String sha1(String input, byte[] salt, int iterations) {
        return digest(input, SHA1, salt, iterations);
    }

    public static String sha256(String input, byte[] salt, int iterations) {
        return digest(input, SHA256, salt, iterations);
    }

    public static String sha384(String input, byte[] salt, int iterations) {
        return digest(input, SHA384, salt, iterations);
    }

    public static String sha512(String input, byte[] salt, int iterations) {
        return digest(input, SHA512, salt, iterations);
    }

    /**
     * 对文件进行md5散列.
     */
    public static String md5(InputStream input) {
        return digest(input, MD5);
    }

    /**
     * 对文件进行sha1散列.
     */
    public static String sha1(InputStream input) {
        return digest(input, SHA1);
    }

    /**
     * 生成随机 salt.
     *
     * @param saltLength 盐长度
     */
    public static String generateSalt(int saltLength) {
        StringBuilder salt = new StringBuilder();
        for (int i = 0; i < saltLength; i++) {
            salt.append(CHAR_ARRAY[random.nextInt(CHAR_ARRAY.length)]);
        }
        return salt.toString();
    }

    /**
     * 对字符串进行散列, 支持md5与sha算法.
     *
     * @param input      源
     * @param algorithm  Hash算法
     * @param salt       盐
     * @param iterations hash 次数
     * @return
     */
    private static String digest(String input, String algorithm, byte[] salt, int iterations) {
        try {
            MessageDigest digest = MessageDigest.getInstance(algorithm);
            if (salt != null) {
                digest.update(salt);
            }
            byte[] result = digest.digest(input.getBytes("UTF-8"));

            for (int i = 1; i < iterations; i++) {
                digest.reset();
                result = digest.digest(result);
            }
            return Hex.encodeHexString(result);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    private static String digest(InputStream input, String algorithm) {
        try {
            MessageDigest messageDigest = MessageDigest.getInstance(algorithm);
            int bufferLength = 8 * 1024;
            byte[] buffer = new byte[bufferLength];
            int read = input.read(buffer, 0, bufferLength);
            while (read > -1) {
                messageDigest.update(buffer, 0, read);
                read = input.read(buffer, 0, bufferLength);
            }
            return Hex.encodeHexString(messageDigest.digest());
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 编码为Base64
     *
     * @param arr     被编码的数组
     * @param lineSep 在76个char之后是CRLF还是EOF
     * @return 编码后的bytes
     */
    public static byte[] base64(byte[] arr, boolean lineSep) {
        int len = arr != null ? arr.length : 0;
        if (len == 0) {
            return new byte[0];
        }

        int evenlen = (len / 3) * 3;
        int cnt = ((len - 1) / 3 + 1) << 2;
        int destlen = cnt + (lineSep ? (cnt - 1) / 76 << 1 : 0);
        byte[] dest = new byte[destlen];

        for (int s = 0, d = 0, cc = 0; s < evenlen; ) {
            int i = (arr[s++] & 0xff) << 16 | (arr[s++] & 0xff) << 8 | (arr[s++] & 0xff);

            dest[d++] = (byte) base64EncodeTable[(i >>> 18) & 0x3f];
            dest[d++] = (byte) base64EncodeTable[(i >>> 12) & 0x3f];
            dest[d++] = (byte) base64EncodeTable[(i >>> 6) & 0x3f];
            dest[d++] = (byte) base64EncodeTable[i & 0x3f];

            if (lineSep && ++cc == 19 && d < destlen - 2) {
                dest[d++] = '\r';
                dest[d++] = '\n';
                cc = 0;
            }
        }

        int left = len - evenlen;
        if (left > 0) {
            int i = ((arr[evenlen] & 0xff) << 10) | (left == 2 ? ((arr[len - 1] & 0xff) << 2) : 0);

            dest[destlen - 4] = (byte) base64EncodeTable[i >> 12];
            dest[destlen - 3] = (byte) base64EncodeTable[(i >>> 6) & 0x3f];
            dest[destlen - 2] = left == 2 ? (byte) base64EncodeTable[i & 0x3f] : (byte) '=';
            dest[destlen - 1] = '=';
        }
        return dest;
    }

    /**
     * base64编码
     *
     * @param source  被编码的base64字符串
     * @param charset 字符集
     * @return 被加密后的字符串
     */
    public static String base64(String source, String charset) {
        return new String(base64(StringUtil.encode(source, charset), false),
                Charset.forName(charset));
    }

    /**
     * base64编码
     *
     * @param source  被编码的base64字符串
     * @param charset 字符集
     * @return 被加密后的字符串
     */
    public static String base64(byte[] source, String charset) {
        return new String(base64(source, false), Charset.forName(charset));
    }

    /**
     * base64解码
     *
     * @param source  被解码的base64字符串
     * @param charset 字符集
     * @return 被加密后的字符串
     */
    public static String decodeBase64(String source, String charset) {
        return new String(decodeBase64(StringUtil.encode(source, charset)),
                Charset.forName(charset));
    }

    /**
     * 解码Base64
     *
     * @param arr byte数组
     * @return 解码后的byte数组
     */
    public static byte[] decodeBase64(byte[] arr) {
        int length = arr.length;
        if (length == 0) {
            return new byte[0];
        }

        int sndx = 0, endx = length - 1;
        int pad = arr[endx] == '=' ? (arr[endx - 1] == '=' ? 2 : 1) : 0;
        int cnt = endx - sndx + 1;
        int sepCnt = length > 76 ? (arr[76] == '\r' ? cnt / 78 : 0) << 1 : 0;
        int len = ((cnt - sepCnt) * 6 >> 3) - pad;
        byte[] dest = new byte[len];

        int d = 0;
        for (int cc = 0, eLen = (len / 3) * 3; d < eLen; ) {
            int i = INV[arr[sndx++]] << 18 | INV[arr[sndx++]] << 12 | INV[arr[sndx++]] << 6
                    | INV[arr[sndx++]];

            dest[d++] = (byte) (i >> 16);
            dest[d++] = (byte) (i >> 8);
            dest[d++] = (byte) i;

            if (sepCnt > 0 && ++cc == 19) {
                sndx += 2;
                cc = 0;
            }
        }

        if (d < len) {
            int i = 0;
            for (int j = 0; sndx <= endx - pad; j++) {
                i |= INV[arr[sndx++]] << (18 - j * 6);
            }
            for (int r = 16; d < len; r -= 8) {
                dest[d++] = (byte) (i >> r);
            }
        }

        return dest;
    }
}
