package com.qhiot.core.util;

import com.google.common.util.concurrent.ThreadFactoryBuilder;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.*;

/**
 * 线程池工具
 *
 * @author 金建强(ptma @ 163.com)
 * @version 2016-01-20 23:38
 */
public final class ThreadUtil {

    /**
     * 收集并记录创建的线程池，禁止通过此map操作线程池
     */
    private static ConcurrentHashMap<String, ThreadPoolExecutor> poolCountMap = new ConcurrentHashMap<>();

    private static ThreadFactory threadFactory = new ThreadFactoryBuilder().setNameFormat("seem-commom-thread-%d").build();
    private static ExecutorService executor = newCachedThreadPool(8, 20000, threadFactory);

    public static Future<?> submit(Runnable runnable) {
        return executor.submit(runnable);
    }

    private static void rememberPool(String poolName, ThreadPoolExecutor pool) {
        try {
            _rememberPool(poolName + "_date&:" + DateUtil.nowString(), pool);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private static void _rememberPool(String poolName, ThreadPoolExecutor pool) {
        try {
            if (poolCountMap.containsKey(poolName)) {
                _rememberPool(poolName + "$", pool);
            } else {
                poolCountMap.put(poolName, pool);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public static ConcurrentHashMap<String, ThreadPoolExecutor> getPoolCountMap() {
        return poolCountMap;
    }

    /**
     * 在新的线程池执行可运行对象
     */
    public static void submitSingle(Runnable runnable) {
        ThreadPoolExecutor executor =
                new ThreadPoolExecutor(1, 1, 0L, TimeUnit.MILLISECONDS, new LinkedBlockingQueue<Runnable>(), threadFactory);
        executor.submit(runnable);
        rememberPool("oneCorePoolSize", executor);
    }

    /**
     * 新建一个线程池
     *
     * @param corePoolSize
     * @param maximumPoolSize
     * @return
     */
    public static ExecutorService newExecutor(int corePoolSize, int maximumPoolSize) {
        ThreadPoolExecutor executor = new ThreadPoolExecutor(corePoolSize, maximumPoolSize, 1, TimeUnit.MILLISECONDS,
                new LinkedBlockingQueue<Runnable>(), threadFactory);
        rememberPool("newThreadPool", executor);

        return executor;
    }

    /**
     * 获得一个新的线程池，只有单个线程
     *
     * @return ExecutorService
     */
    public static ScheduledExecutorService newSingleScheduledExecutor(String nameFormat) {
        ScheduledExecutorService executor = Executors.newSingleThreadScheduledExecutor(new ThreadFactoryBuilder().setNameFormat(
                nameFormat).build());
        return executor;
    }

    public static ExecutorService newSingleThreadExecutor() {
        ThreadPoolExecutor executor = new ThreadPoolExecutor(1, 1,
                0L, TimeUnit.MILLISECONDS,
                new LinkedBlockingQueue<Runnable>(), threadFactory);

        rememberPool("singleThreadPool", executor);
        return executor;
    }


    public static ThreadPoolExecutor newCachedThreadPool(int corePoolSize, int maximumPoolSize,
                                                         String nameFormat) {
        ThreadPoolExecutor executor = new ThreadPoolExecutor(corePoolSize, maximumPoolSize, 60L, TimeUnit.SECONDS,
                new LinkedBlockingQueue<Runnable>(), new ThreadFactoryBuilder().setNameFormat(
                nameFormat).build());

        rememberPool(nameFormat, executor);
        return executor;
    }

    public static ThreadPoolExecutor newCachedThreadPool(int corePoolSize, int maximumPoolSize,
                                                         String nameFormat, int capacity) {
        ThreadPoolExecutor executor = new ThreadPoolExecutor(corePoolSize, maximumPoolSize, 60L, TimeUnit.SECONDS,
                new LinkedBlockingQueue<Runnable>(capacity), new ThreadFactoryBuilder().setNameFormat(
                nameFormat).build());

        rememberPool(nameFormat, executor);
        return executor;
    }

    public static ThreadPoolExecutor newCachedThreadPool(int corePoolSize, int maximumPoolSize, long keepAliveTime,
                                                         String nameFormat) {
        ThreadPoolExecutor executor = new ThreadPoolExecutor(corePoolSize, maximumPoolSize, keepAliveTime, TimeUnit.SECONDS,
                new LinkedBlockingQueue<Runnable>(), new ThreadFactoryBuilder().setNameFormat(
                nameFormat).build());

        rememberPool(nameFormat, executor);
        return executor;
    }

    public static ThreadPoolExecutor newCachedThreadPool(int corePoolSize, int maximumPoolSize,
                                                         ThreadFactory threadFactory, int capacity) {
        ThreadPoolExecutor executor = new ThreadPoolExecutor(corePoolSize, maximumPoolSize, 60L, TimeUnit.SECONDS,
                new LinkedBlockingQueue<Runnable>(capacity), threadFactory);

        rememberPool("unNameCachedThreadPoolCapacity", executor);
        return executor;
    }

    public static ThreadPoolExecutor newCachedThreadPool(int corePoolSize, int maximumPoolSize,
                                                         ThreadFactory threadFactory) {
        ThreadPoolExecutor executor = newCachedThreadPool(corePoolSize, maximumPoolSize, threadFactory, Integer.MAX_VALUE);
//
//        rememberPool("unNameCachedThreadPool",executor);
        return executor;
    }

    public static ThreadPoolExecutor newCachedThreadPool(int corePoolSize, int maximumPoolSize, long keepAliveTime,
                                                         ThreadFactory threadFactory) {
        ThreadPoolExecutor executor = new ThreadPoolExecutor(corePoolSize, maximumPoolSize, keepAliveTime, TimeUnit.SECONDS,
                new LinkedBlockingQueue<Runnable>(), threadFactory);

        rememberPool("unNameCachedThreadPool2", executor);
        return executor;
    }

    public static ExecutorService newFixedThreadPool(int nThreads) {
        ThreadPoolExecutor executor = new ThreadPoolExecutor(nThreads, nThreads,
                0L, TimeUnit.MILLISECONDS,
                new LinkedBlockingQueue<Runnable>(),
                threadFactory);

        rememberPool("fixedThreadPool", executor);
        return executor;
    }

    /**
     * 重启公共线程池
     */
    public static void restart() {
        executor.shutdownNow();
        executor = newCachedThreadPool(8, 20000, threadFactory);
    }

    /**
     * @param callables 多个任务的集合LIST，请保持Callable的call方法的返回值类型和Callable的泛型一致
     * @param <T>
     * @return
     * @throws InterruptedException
     * @throws ExecutionException
     * @author luoj
     * 启动线程池，执行多个线程，并得到所有线程的结果，耗费时长取决于最慢的任务
     */
    public static <T> List<T> excuteAndResault(List<Callable<T>> callables) throws InterruptedException, ExecutionException {
        int size = callables.size();
        ExecutorService threadPool = newFixedThreadPool(size);
        return excuteAndResault(threadPool, callables);
    }

    /**
     * @param threadPool 传入线程池
     * @param callables  多个任务的集合LIST，请保持Callable的call方法的返回值类型和Callable的泛型一致
     * @param <T>
     * @return
     * @throws InterruptedException
     * @throws ExecutionException
     * @author luoj
     * 启动线程池，执行多个线程，并得到所有线程的结果，耗费时长取决于最慢的任务
     * 传入线程池不可重复使用，因为获取值的需要，线程池已被销毁
     */
    public static <T> List<T> excuteAndResault(ExecutorService threadPool, List<Callable<T>> callables) throws InterruptedException, ExecutionException {
        int size = callables.size();
        List<T> data = new ArrayList<T>();
        CompletionService<T> completionService = new ExecutorCompletionService<T>(threadPool);
        for (int j = 0; j < size; j++) {
            completionService.submit(callables.get(j));
        }
        threadPool.shutdown();
        StringBuffer info = new StringBuffer();
        for (int i = 0; i < size; i++) {
            data.add(completionService.take().get());
        }
        return data;
    }
}
