package com.qhiot.influxdb.connect;

import com.qhiot.core.util.CollectionUtil;
import com.qhiot.core.util.StringUtil;
import com.qhiot.influxdb.mapper.InfluxDBBaseMapper;
import com.qhiot.influxdb.model.RetentionPolicy;
import com.qhiot.influxdb.util.InfluxDBHandleUtil;
import okhttp3.OkHttpClient;
import org.influxdb.InfluxDB;
import org.influxdb.InfluxDBException;
import org.influxdb.InfluxDBFactory;
import org.influxdb.InfluxDBMapperException;
import org.influxdb.annotation.Column;
import org.influxdb.dto.*;
import org.influxdb.impl.InfluxDBMapper;

import java.lang.reflect.Field;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentMap;
import java.util.concurrent.TimeUnit;

import static org.influxdb.querybuilder.BuiltQuery.QueryBuilder.select;

/**
 * InfluxDB数据库连接操作类
 *
 * @author 罗军
 * res_URL:https://blog.csdn.net/x541211190/article/details/83216589
 */
public class InfluxDBConnection {

    // 用户名
    private String username;
    // 密码
    private String password;
    // 连接地址
    private String openurl;

    // 数据库
    private String database;
    // 保留策略
    private String retentionPolicy;

    private InfluxDB influxDB;

    private boolean isEnableBatch;

    private int maxPonits;

    private int maxTime;

    private long connectTimeOut;

    private long readTimeOut;

    private long writeTimeOut;

//    private InfluxDBMapper influxDBMapper;

    private InfluxDBBaseMapper influxDBMapper;

    public InfluxDBConnection(String username, String password, String openurl, String database,
                              String retentionPolicy) {
        this.username = username;
        this.password = password;
        this.openurl = openurl;
        this.database = database;
        this.retentionPolicy = retentionPolicy == null || retentionPolicy.equals("") ? "autogen" : retentionPolicy;
        influxDbBuild();
        createDBIfNotExists();
        influxDbMapperBuild();
    }

    public InfluxDBConnection(String username, String password, String openurl, String database,
                              String retentionPolicy, boolean isEnableBatch, int maxPonits, int maxTime,
                              long connectTimeOut, long readTimeOut, long writeTimeOut) {
        this.username = username;
        this.password = password;
        this.openurl = openurl;
        this.database = database;
        this.connectTimeOut = connectTimeOut;
        this.readTimeOut = readTimeOut;
        this.writeTimeOut = writeTimeOut;
        this.retentionPolicy = retentionPolicy == null || retentionPolicy.equals("") ? "autogen" : retentionPolicy;
        influxDbBuild();
        createDBIfNotExists();
        influxDbMapperBuild();
        setEnableBatch(isEnableBatch, maxPonits, maxTime);
    }

    public String getDatabase() {
        return database;
    }

    public String getRetentionPolicy() {
        return this.retentionPolicy;
    }

    public boolean isEnableBatch() {
        return this.isEnableBatch;
    }

    public int getMaxPonits() {
        return this.maxPonits;
    }

    public InfluxDB getInfluxDB() {
        return influxDB;
    }

    public InfluxDBBaseMapper getInfluxDBMapper() {
        return influxDBMapper;
    }

    /**
     * 当数据库不存在的时候，创建它
     */
    private void createDBIfNotExists() {
        if (!databaseExists(database)) {
            createDB(database);
        }
    }


    /**
     * 创建数据库
     *
     * @param dbName
     */
    @SuppressWarnings("deprecation")
    public void createDB(String dbName) {
        influxDB.createDatabase(dbName);
    }

    /**
     * 删除数据库
     *
     * @param dbName
     */
    @SuppressWarnings("deprecation")
    public void deleteDB(String dbName) {
        influxDB.deleteDatabase(dbName);
    }

    /**
     * 数据库是否存在*
     *
     * @param dbName
     */
    @SuppressWarnings("deprecation")
    public boolean databaseExists(String dbName) {
        return influxDB.databaseExists(dbName);
    }

    /**
     * 测试连接是否正常
     *
     * @return true 正常
     */
    public boolean ping() {
        boolean isConnected = false;
        Pong pong;
        try {
            pong = influxDB.ping();
            if (pong != null) {
                isConnected = true;
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return isConnected;
    }

    /**
     * 连接时序数据库
     *
     * @return
     */
    private InfluxDB influxDbBuild() {
        if (influxDB == null) {
//            influxDB = InfluxDBFactory.connect(openurl, username, password);
            OkHttpClient.Builder okHttpBuilder = new OkHttpClient.Builder()
                    .connectTimeout(connectTimeOut, TimeUnit.SECONDS)
                    .readTimeout(readTimeOut, TimeUnit.SECONDS)
                    .writeTimeout(writeTimeOut, TimeUnit.SECONDS);
            influxDB = InfluxDBFactory.connect(openurl, username, password, okHttpBuilder);
            influxDB.setLogLevel(InfluxDB.LogLevel.NONE);
            influxDB.setDatabase(database);
        }
        return influxDB;
    }

    /**
     * 设定启用
     *
     * @param isEnableBatch
     * @param maxPonits
     * @param maxTime
     */
    public void setEnableBatch(boolean isEnableBatch, int maxPonits, int maxTime) {
        this.isEnableBatch = isEnableBatch;
        this.maxPonits = maxPonits;
        this.maxTime = maxTime;
        setEnableBatch();
    }

    public void setEnableBatch() {
        if (influxDB.isBatchEnabled()) {
            influxDB.disableBatch();
        }
        if (isEnableBatch) {
            influxDB.enableBatch(maxPonits, maxTime, TimeUnit.MILLISECONDS);
        }
    }

    /**
     * 创建influxDBMapper
     *
     * @return
     */
    private InfluxDBMapper influxDbMapperBuild() {
        if (influxDBMapper == null) {
            influxDBMapper = new InfluxDBBaseMapper(influxDB);
        }
        return influxDBMapper;
    }

    /**
     * 创建自定义保留策略
     *
     * @param policyName  策略名
     * @param duration    保存天数
     * @param replication 保存副本数量
     * @param isDefault   是否设为默认保留策略
     */
    public void createRetentionPolicy(String policyName,String database, String duration, String shardDuration, int replication, Boolean isDefault) throws Exception {
        if(!retentionPolicyExists(database, policyName)){
            String sql = StringUtil.format("CREATE RETENTION POLICY {} ON {} DURATION {} REPLICATION {} ", policyName,
                    database, duration, replication);
            if(StringUtil.isNotEmpty(shardDuration)){
                sql = sql + " SHARD DURATION " + shardDuration;
            }
            if (isDefault) {
                sql = sql + " DEFAULT";
            }
            this.query(sql);
        }
    }

    /**
     * 创建或修改自定义保留策略
     *
     * @param policyName  策略名
     * @param duration    保存天数
     * @param replication 保存副本数量
     * @param isDefault   是否设为默认保留策略
     */
    public void createOrAlertRetentionPolicy(String policyName,String database, String duration, String shardDuration, int replication, Boolean isDefault) throws Exception {
        if(!retentionPolicyExists(database, policyName)){
            String sql = StringUtil.format("CREATE RETENTION POLICY {} ON {} DURATION {} REPLICATION {} ", policyName,
                    database, duration, replication);
            if(StringUtil.isNotEmpty(shardDuration)){
                sql = sql + " SHARD DURATION " + shardDuration;
            }
            if (isDefault) {
                sql = sql + " DEFAULT";
            }
            this.query(sql);
        }else {
            alertRetentionPolicy(policyName, database,  duration, shardDuration, isDefault);
        }
    }
    /**
     * 修改指定保留策略
     *
     * @param policyName  策略名
     * @param duration    保存天数
     * @param isDefault   是否设为默认保留策略
     */
    public void alertRetentionPolicy(String policyName,String database, String duration, String shardDuration, Boolean isDefault) throws Exception {
        if(retentionPolicyExists(database, policyName)){
            //ALTER RETENTION POLICY <retention_policy_name> ON <database_name> [DURATION <duration>] [REPLICATION <n>] [SHARD DURATION <duration>] [DEFAULT]
            String sql = StringUtil.format("ALTER RETENTION POLICY {} ON {} DURATION {} ", policyName,
                    database, duration);
            if(StringUtil.isNotEmpty(shardDuration)){
                sql = sql + " SHARD DURATION " + shardDuration;
            }
            if (isDefault) {
                sql = sql + " DEFAULT";
            }
            this.query(sql);
        }
    }
    /**
     * 修改指定保留策略
     *
     * @param policyName  策略名
     * @param duration    保存天数
     * @param replication 保存副本数量
     * @param isDefault   是否设为默认保留策略
     */
    public void alertRetentionPolicy(String policyName,String database, String duration, int replication, Boolean isDefault) throws Exception {
        if(retentionPolicyExists(database, policyName)){
            //ALTER RETENTION POLICY <retention_policy_name> ON <database_name> [DURATION <duration>] [REPLICATION <n>] [SHARD DURATION <duration>] [DEFAULT]
            String sql = StringUtil.format("ALTER RETENTION POLICY {} ON {} DURATION {} REPLICATION {} ", policyName,
                    database, duration, replication);
            if (isDefault) {
                sql = sql + " DEFAULT";
            }
            this.query(sql);
        }
    }

    /**
     * 删除指定保留策略
     *
     * @param policyName  策略名
     */
    public void deleteRetentionPolicy(String policyName,String database) throws Exception {
        if(retentionPolicyExists(database, policyName)){
            //DROP RETENTION POLICY <retention_policy_name> ON <database_name>
            String sql = StringUtil.format("DROP RETENTION POLICY {} ON {} ", policyName, database);
            this.query(sql);
        }
    }

    /**
     * 查询所有策略
     * @param database
     * @return
     * @throws Exception
     */
    public List<RetentionPolicy> selectAllRetentionPolicy(String database) throws Exception{
        try {
            String sql = StringUtil.format("SHOW RETENTION POLICIES ON {}",database);
            QueryResult queryResult = this.query(sql);
            return influxDBMapper.toPOJORetentionPolicies(queryResult);
        }catch (Exception e){
            throw new Exception("查询策略出错",e);
        }
    }

    /**
     * 查询指定名称策略
     * @param database
     * @param policyName
     * @return
     * @throws Exception
     */
    public RetentionPolicy selectRetentionPolicy(String database, String policyName) throws Exception{
        try {
            RetentionPolicy result = null;
            List<RetentionPolicy> retentionPolicies = selectAllRetentionPolicy(database);
            if(CollectionUtil.isEmpty(retentionPolicies)){
                return null;
            }
            for (RetentionPolicy retentionPolicy : retentionPolicies) {
                if(retentionPolicy.getName().equals(policyName)){
                    result = retentionPolicy;
                }
            }
            return result;
        }catch (Exception e){
            throw new Exception("查询策略出错",e);
        }
    }

    /**
     * 查询指定策略是否存在
     * @param database
     * @param policyName
     * @return
     * @throws Exception
     */
    public boolean retentionPolicyExists(String database, String policyName) throws Exception{
        try {
            return selectRetentionPolicy(database, policyName) == null ? false : true;
        }catch (Exception e){
            throw new Exception("查询策略出错",e);
        }
    }

    /**
     * 查询
     *
     * @param command 查询语句
     * @return
     */
    public QueryResult query(String command) throws InfluxDBException {
        return influxDB.query(new Query(command, database));
    }

    /**
     * 插入
     *
     * @param measurement 表
     * @param tags        标签
     * @param fields      字段
     */
    public void insert(String measurement, Map<String, String> tags, Map<String, Object> fields, long time,
                       TimeUnit timeUnit) throws InfluxDBException {
        Point.Builder builder = Point.measurement(measurement);
        builder.tag(tags);
        builder.fields(fields);
        if (0 != time) {
            builder.time(time, timeUnit);
        }
        influxDB.write(database, retentionPolicy, builder.build());
    }

    /**
     * 批量写入测点
     *
     * @param batchPoints
     */
    public void batchInsert(BatchPoints batchPoints) throws InfluxDBException {
        influxDB.write(batchPoints);
        // influxDB.enableGzip();
        // influxDB.enableBatch(2000,100,TimeUnit.MILLISECONDS);
        // influxDB.disableGzip();
        // influxDB.disableBatch();
    }

    /**
     * 批量写入数据
     *
     * @param database        数据库
     * @param retentionPolicy 保存策略
     * @param consistency     一致性
     * @param records         要保存的数据（调用BatchPoints.lineProtocol()可得到一条record）
     */
    public void batchInsert(final String database, final String retentionPolicy, final InfluxDB.ConsistencyLevel consistency,
                            final List<String> records) throws InfluxDBException {
        influxDB.write(database, retentionPolicy, consistency, records);
    }

    /**
     * 删除
     *
     * @param command 删除语句
     * @return 返回错误信息
     */
    public String deleteMeasurementData(String command) throws InfluxDBException {
        QueryResult result = influxDB.query(new Query(command, database));
        return result.getError();
    }

    /**
     * 关闭数据库
     */
    public void close() {
        influxDB.close();
    }

    /**
     * 构建Point
     *
     * @param measurement
     * @param time
     * @param fields
     * @return
     */
    public Point pointBuilder(String measurement, long time, Map<String, String> tags, Map<String, Object> fields) throws InfluxDBException {
        Point point = Point.measurement(measurement).time(time, TimeUnit.MILLISECONDS).tag(tags).fields(fields).build();
        return point;
    }

    /**
     * 执行指定的查询，并返回指定类型的结果集
     *
     * @param query
     * @param clazz
     * @param <T>
     * @return
     */
    public <T> List<T> query(Query query, Class<T> clazz) throws InfluxDBException {
        return influxDBMapper.query(query, clazz);
    }

    /**
     * 根据所给出的类（非对象），组装查询语句（全表查询），并返回指定类型的结果集
     *
     * @param clazz
     * @param <T>
     * @return
     */
    public <T> List<T> query(Class<T> clazz) throws InfluxDBException {
        return influxDBMapper.query(clazz);
    }

    /**
     * 存入一个记录
     *
     * @param <T> model
     * @return
     */
    public <T> void save(T model) throws InfluxDBException {
        influxDBMapper.save(model);
    }

    public <T> List<T> selectAll(Class<T> clazz) throws InfluxDBException {
        String table = InfluxDBHandleUtil.getMeasurementName(clazz);
        Query select = select().all().from(database, table);
        return influxDBMapper.query(select, clazz);
    }
}


