package com.qhiot.kafka.config;

import com.qhiot.kafka.serializer.ObjectDeserializer;
import com.qhiot.kafka.serializer.ObjectSerializer;
import org.apache.kafka.clients.admin.AdminClient;
import org.apache.kafka.clients.admin.KafkaAdminClient;
import org.apache.kafka.clients.consumer.ConsumerConfig;
import org.apache.kafka.clients.producer.ProducerConfig;
import org.apache.kafka.common.serialization.StringDeserializer;
import org.apache.kafka.common.serialization.StringSerializer;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.annotation.Order;
import org.springframework.kafka.annotation.EnableKafka;
import org.springframework.kafka.config.ConcurrentKafkaListenerContainerFactory;
import org.springframework.kafka.config.KafkaListenerContainerFactory;
import org.springframework.kafka.core.*;
import org.springframework.kafka.listener.AbstractMessageListenerContainer;
import org.springframework.kafka.listener.ConcurrentMessageListenerContainer;
import org.springframework.kafka.listener.ContainerProperties;

import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

/**
 * 此kafka配置类和qh-link模板因为使用的kafka版本不统一，存在不一样的代码
 *
 * @author 罗军(luoj@qh-iot.com)
 * @version 2021/6/17
 */

@Order(11)
@Configuration
@EnableKafka
public class KafkaConfig {
    private static final Logger LOGGER = LoggerFactory.getLogger("Init.KAFKA");

    @Value("${kafka.servers}")
    private String servers;

    /**
     * 发送失败重试次数
     * */
    @Value("${kafka.producer.retries:3}")
    private int retries;

    /**
     * 批处理内存大小
     * */
    @Value("${kafka.producer.batch.size:16384}")
    private int batchSize;

    /**
     * 批处理时间，毫秒
     * */
    @Value("${kafka.producer.linger:50}")
    private int linger;

    /**
     * 批处理缓冲区
     * */
    @Value("${kafka.producer.buffer.memory:33554432}")
    private int bufferMemory;

    /**
     * 如果为true，消费者的偏移量将在后台定期提交
     * */
    @Value("${kafka.consumer.enable.auto.commit:false}")
    private boolean enableAutoCommit;

    /**
     * 在使用Kafka的组管理时，用于检测消费者故障的超时
     * */
    @Value("${kafka.consumer.session.timeout:6000}")
    private String sessionTimeout;

    /**
     * 设置自动提交周期
     * */
    @Value("${kafka.consumer.auto.commit.interval:100}")
    private String autoCommitInterval;

    /**
     * 如果分区没有初始偏移量，或者当前偏移量服务器上不存在时，将使用的偏移量设置，earliest从头开始消费，latest从最近的开始消费，none抛出异常
     * */
    @Value("${kafka.consumer.auto.offset.reset}")
    private String autoOffsetReset;

    /**
     * 消费监听器容器并发数
     * */
    @Value("${kafka.consumer.concurrency}")
    private int concurrency;

    /**
     * 是否启用批量消费
     * */
    @Value("${kafka.consumer.batchListener:false}")
    private boolean batchListener;

    /**
     * 一次拉取请求的最大消息数，默认500条
     * */
    @Value("${kafka.consumer.maxPollRecords:1}")
    private int maxPollRecords;

    /**
     * 一次拉取消息最久处理时长，默认30s
     * */
    @Value("${kafka.consumer.maxPollIntervalMs:30000}")
    private int maxPollIntervalMs;

    public Map<String, Object> producerConfigs() {
        Map<String, Object> props = new HashMap<>();
        props.put(ProducerConfig.BOOTSTRAP_SERVERS_CONFIG, servers);
        props.put(ProducerConfig.RETRIES_CONFIG, retries);
        props.put(ProducerConfig.BATCH_SIZE_CONFIG, batchSize);
        props.put(ProducerConfig.LINGER_MS_CONFIG, linger);
        props.put(ProducerConfig.BUFFER_MEMORY_CONFIG, bufferMemory);
        props.put(ProducerConfig.KEY_SERIALIZER_CLASS_CONFIG, StringSerializer.class);
        props.put(ProducerConfig.VALUE_SERIALIZER_CLASS_CONFIG, ObjectSerializer.class);
        return props;
    }

    public ProducerFactory<String, String> producerFactory() {
        return new DefaultKafkaProducerFactory<>(producerConfigs());
    }

    /**
     * default byte[]
     * @return
     */
    @Bean
    public KafkaListenerContainerFactory<ConcurrentMessageListenerContainer<String, Object>> kafkaListenerContainerFactory() {
        LOGGER.info("kafka消费者启动");
        ConcurrentKafkaListenerContainerFactory<String, Object> factory = new ConcurrentKafkaListenerContainerFactory<>();
        factory.setConsumerFactory(consumerFactory());
        factory.setConcurrency(concurrency);
        factory.getContainerProperties().setPollTimeout(1500);
        factory.getContainerProperties().setAckMode(ContainerProperties.AckMode.MANUAL_IMMEDIATE);
        factory.setBatchListener(batchListener);
        return factory;
    }

    public ConsumerFactory<String, Object> consumerFactory() {
        return new DefaultKafkaConsumerFactory<>(consumerConfigs());
    }

    public Map<String, Object> consumerConfigs() {
        Map<String, Object> propsMap = new HashMap<>();
        propsMap.put(ConsumerConfig.BOOTSTRAP_SERVERS_CONFIG, servers);
        propsMap.put(ConsumerConfig.ENABLE_AUTO_COMMIT_CONFIG, enableAutoCommit);
        propsMap.put(ConsumerConfig.AUTO_COMMIT_INTERVAL_MS_CONFIG, autoCommitInterval);
        propsMap.put(ConsumerConfig.SESSION_TIMEOUT_MS_CONFIG, sessionTimeout);
        propsMap.put(ConsumerConfig.MAX_POLL_RECORDS_CONFIG, maxPollRecords);
        propsMap.put(ConsumerConfig.MAX_POLL_INTERVAL_MS_CONFIG, maxPollIntervalMs);
        propsMap.put(ConsumerConfig.KEY_DESERIALIZER_CLASS_CONFIG, StringDeserializer.class);
        propsMap.put(ConsumerConfig.VALUE_DESERIALIZER_CLASS_CONFIG, ObjectDeserializer.class);
        propsMap.put(ConsumerConfig.AUTO_OFFSET_RESET_CONFIG, autoOffsetReset);
        return propsMap;
    }

    @Bean
    public KafkaTemplate<String, String> kafkaTemplate() {
        LOGGER.info("kafka生产者启动");
        return new KafkaTemplate<String, String>(producerFactory());
    }
}
