package com.qhiot.log2es.appender;

import com.qhiot.core.util.StringUtil;
import com.qhiot.log2es.client.ElasticsearchClient;
import org.apache.commons.lang.exception.ExceptionUtils;
import org.apache.logging.log4j.core.Filter;
import org.apache.logging.log4j.core.Layout;
import org.apache.logging.log4j.core.LogEvent;
import org.apache.logging.log4j.core.appender.AbstractAppender;
import org.apache.logging.log4j.core.config.plugins.Plugin;
import org.apache.logging.log4j.core.config.plugins.PluginAttribute;
import org.apache.logging.log4j.core.config.plugins.PluginElement;
import org.apache.logging.log4j.core.config.plugins.PluginFactory;
import org.apache.logging.log4j.core.util.Booleans;
import org.elasticsearch.client.transport.TransportClient;
import org.elasticsearch.common.settings.Settings;
import org.elasticsearch.common.transport.InetSocketTransportAddress;
import org.elasticsearch.transport.client.PreBuiltTransportClient;

import java.io.Serializable;
import java.net.InetSocketAddress;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import static org.apache.commons.lang.StringUtils.substringAfterLast;
import static org.apache.commons.lang.StringUtils.substringBeforeLast;

/**
 * Log4j2 Appender 用于将 Log4j2 日志写入到 Elasticsearch 中
 *
 * @author 金建强(ptma@163.com)
 * @version 2016/6/7 22:43
 */
@Plugin(name = "Elasticsearch", category = "Core", elementType = "appender", printObject = true)
public class ElasticsearchAppender extends AbstractAppender {

    private ElasticsearchClient client;

    @PluginFactory
    public static ElasticsearchAppender createAppender(
            @PluginAttribute("name") final String name,
            @PluginElement("Layout") Layout<? extends Serializable> layout,
            @PluginElement("Filter") final Filter filter,
            @PluginAttribute(value = "cluster", defaultString = "elasticsearch") String cluster,
            @PluginAttribute(value = "clusterNodes", defaultString = "127.0.0.1:9300") String clusterNodes,
            @PluginAttribute(value = "index", defaultString = "log4j2") String index,
            @PluginAttribute(value = "type", defaultString = "log4j2") String type,
            @PluginAttribute(value = "maxActionsPerBulkRequest", defaultInt = 1000) Integer maxActionsPerBulkRequest,
            @PluginAttribute(value = "flushInterval", defaultInt = 5) Integer flushInterval,
            @PluginAttribute(value = "logNode", defaultString = "node-0") String logNode,
            @PluginAttribute(value = "ignoreExceptions", defaultBoolean = true) final String ignore) {

        if (name == null) {
            LOGGER.error("No name provided for Elasticsearch");
            return null;
        }

        Settings settings = Settings.builder()
            .put("cluster.name", cluster)
            .put("network.server", false)
            .put("client.transport.sniff", false)
            .put("client.transport.ping_timeout", "5s")
            .put("client.transport.ignore_cluster_name", false)
            .put("client.transport.nodes_sampler_interval", "30s")
            .put("http.type.default", "netty4")
            .put("transport.type.default", "netty4").build();

        TransportClient transportClient = new PreBuiltTransportClient(settings);
        for (String clusterNode : StringUtil.split(clusterNodes, StringUtil.COMMA)) {
            String hostName = substringBeforeLast(clusterNode, StringUtil.COLON);
            String port = substringAfterLast(clusterNode, StringUtil.COLON);
            InetSocketAddress address = new InetSocketAddress(hostName, Integer.parseInt(port));
            transportClient.addTransportAddress(new InetSocketTransportAddress(address));
        }

        ElasticsearchClient elasticsearchClient = new ElasticsearchClient(transportClient, index, type, flushInterval, maxActionsPerBulkRequest, logNode);

        final boolean ignoreExceptions = Booleans.parseBoolean(ignore, true);

        return new ElasticsearchAppender(name, layout, filter, ignoreExceptions, elasticsearchClient);
    }

    protected ElasticsearchAppender(final String name, final Layout<? extends Serializable> layout, final Filter filter,
                                    final boolean ignoreExceptions, ElasticsearchClient client) {
        super(name, filter, layout, ignoreExceptions);
        this.client = client;
    }

    /**
     * 产生新的日志时， 推送格式化后的日志文本到目标订阅地址
     *
     * @param event
     */
    @Override
    public void append(LogEvent event) {
        Map<String, Object> eventMap = new HashMap<String, Object>();

        eventMap.put("threadName", event.getThreadName());
        eventMap.put("millis", event.getTimeMillis());
        eventMap.put("date", new Date(event.getTimeMillis()));
        eventMap.put("loggerName", event.getLoggerName());
        eventMap.put("level", event.getLevel().name());
        eventMap.put("message", event.getMessage().getFormattedMessage());

        if (event.getSource() != null) {
            eventMap.put("lineNumber", event.getSource().getLineNumber());
            eventMap.put("className", event.getSource().getClassName());
            eventMap.put("methodName", event.getSource().getMethodName());
        }

        if (event.getThrown() != null) {
            eventMap.put("thrown", ExceptionUtils.getFullStackTrace(event.getThrown()));
        } else {
            eventMap.put("thrown", "");
        }
        client.index(eventMap);
    }

}
