package com.qhiot.log2es.client;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.core.JsonGenerator;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.JsonSerializer;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.SerializerProvider;
import com.qhiot.core.util.ThreadUtil;
import org.elasticsearch.action.bulk.BulkRequestBuilder;
import org.elasticsearch.action.bulk.BulkResponse;
import org.elasticsearch.action.index.IndexRequest;
import org.elasticsearch.client.transport.TransportClient;
import org.elasticsearch.common.xcontent.XContentType;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Map;
import java.util.Queue;
import java.util.TimeZone;
import java.util.concurrent.ConcurrentLinkedQueue;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.ReentrantLock;

/**
 * Elasticsearch 异步索引客户端
 *
 * @author 金建强(ptma@163.com)
 * @version 2017/4/27 0:20
 */
public class ElasticsearchClient {

    private static final Logger                  logger              =
                                                                             LoggerFactory
                                                                                     .getLogger(ElasticsearchClient.class);

    private static ThreadLocal<SimpleDateFormat> UTC_DATETIME_FORMAT =
                                                                             new ThreadLocal<SimpleDateFormat>() {
                                                                                 @Override
                                                                                 synchronized protected SimpleDateFormat initialValue() {
                                                                                     SimpleDateFormat sdf =
                                                                                             new SimpleDateFormat(
                                                                                                     "yyyy-MM-dd'T'HH:mm:ss.SSS'Z'");
                                                                                     sdf.setTimeZone(TimeZone
                                                                                             .getTimeZone("GMT"));
                                                                                     return sdf;
                                                                                 }
                                                                             };

    private final Queue<String>                  sources             =
                                                                             new ConcurrentLinkedQueue<String>();

    private final ReentrantLock                  lock                = new ReentrantLock(true);

    private final String                         index;
    private final String                         type;
    private final String                         logNode;

    private final ScheduledExecutorService       service;

    private final int                            maxActionsPerBulkRequest;

    private TransportClient                      client;
    private ObjectMapper                         objectMapper;

    public ElasticsearchClient(TransportClient client, String index, String type, int flushSecs,
                               int maxActionsPerBulkRequest, String logNode) {
        this.client = client;
        this.index = index;
        this.type = type;
        this.logNode = logNode;
        this.maxActionsPerBulkRequest = maxActionsPerBulkRequest;

        objectMapper = new ObjectMapper();
        objectMapper.setSerializationInclusion(JsonInclude.Include.ALWAYS);
        objectMapper.configure(JsonGenerator.Feature.QUOTE_FIELD_NAMES, true);
        objectMapper.configure(JsonGenerator.Feature.WRITE_NUMBERS_AS_STRINGS, true);
        objectMapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);
        // 所有日期格式都统一为以下样式
        objectMapper.setDateFormat(UTC_DATETIME_FORMAT.get());

        objectMapper.getSerializerProvider().setNullValueSerializer(new JsonSerializer<Object>() {
            @Override
            public void serialize(Object value, JsonGenerator jg, SerializerProvider sp)
                    throws IOException {
                jg.writeString("");
            }
        });

        service = ThreadUtil.newSingleScheduledExecutor("seem-log2es-flusher-%d");
        service.scheduleAtFixedRate(new Runnable() {
            @Override
            public void run() {
                flush();
            }
        }, flushSecs, flushSecs, TimeUnit.SECONDS);
    }

    /**
     * 添加索引到队列（不会立即写入）
     *
     * @param source
     * @return
     */
    public ElasticsearchClient index(Map<String, Object> source) {
        try {
            source.put("logNode", logNode);
            sources.add(objectMapper.writeValueAsString(source));
        } catch (Exception e) {
            logger.error(e.getMessage(), e);
        }
        return this;
    }

    /**
     * 将队列中的索引写入到 ES
     */
    public void flush() {
        lock.lock();
        try {
            while (!sources.isEmpty()) {
                String source;
                int i = maxActionsPerBulkRequest;
                BulkRequestBuilder builder = client.prepareBulk();
                // 从队列取索引, 批量提交, 但每次最多只能提交 maxActionsPerBulkRequest 条
                while ((source = sources.poll()) != null && (i-- >= 0)) {
                    IndexRequest request =
                            client.prepareIndex(index, type).setSource(source, XContentType.JSON)
                                    .request();

                    builder.add(request);
                }
                if (!builder.request().requests().isEmpty()) {
                    BulkResponse response = builder.execute().actionGet();
                    if (response.hasFailures()) {
                        logger.warn("Elasticsearch 索引失败: " + response.buildFailureMessage());
                    }
                }
            }
        } catch (Throwable t) {
            logger.warn(t.getMessage(), t);
        } finally {
            lock.unlock();
        }
    }

    public void close() {
        service.shutdownNow();
        if (!sources.isEmpty()) {
            flush();
        }
    }

}
