/*!
 * CodeGenerator(https://git.oschina.net/ptma/CodeGenerator)
 *
 * Date: 2021-05-31 03:06:03
 */
package com.qhiot.transport.db.service;

import com.qhiot.cache.util.CacheUtil;
import com.qhiot.core.exception.ServiceException;
import com.qhiot.core.service.BaseService;
import com.qhiot.core.util.CollectionUtil;
import com.qhiot.transport.db.mapper.KafkaSwitchConfigMapper;
import com.qhiot.transport.db.model.KafkaSwitchConfig;
import net.oschina.j2cache.CacheObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.cache.annotation.EnableCaching;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import tk.mybatis.mapper.entity.Example;

import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;

/**
 * 业务层 <br/>
 * Table: kafka_switch_config - 消息推送配置 <br/>
 *
 * @author 罗军(luoj@qh-iot.com)
 * @version 2021-05-31 03:06:03
 */
@Service
@EnableCaching
@Transactional(rollbackFor = Exception.class)
public class KafkaSwitchConfigService extends BaseService<KafkaSwitchConfig> {

    private static final String KAFKA_SWITCH_CACHE_KEY = "seem:kafka:switch:default";

    @Value("${kafka.destinationPrefix:seem.}")
    private String kafkaDestinationPrefix;

    @Autowired
    private KafkaSwitchConfigMapper kafkaSwitchConfigMapper;

    private static final Map<String,KafkaSwitchConfig> KAFKA_SWITCH_DEFAULT = new HashMap<>();

    static {
        getAllKafkaTopic();
    }


    //获取所有定义好了的kafka发送通道
    private static void getAllKafkaTopic(){
        try {
            Class clazz = Class.forName("com.qhiot.kafka.KafkaTopicDestinations");
            Field[] fields = clazz.getDeclaredFields();
            if (CollectionUtil.isNotEmpty(fields)) {

                Field descriptionsField = clazz.getDeclaredField("DESCRIPTIONS");
                descriptionsField.setAccessible(true);
                Map<String,String> descriptions = (Map<String, String>) descriptionsField.get(clazz);

                for (Field field : fields) {
                    if (field.getType().toString().endsWith("java.lang.String") && Modifier.isFinal(field.getModifiers()) && Modifier.isStatic(field.getModifiers())) {
                        KafkaSwitchConfig config = new KafkaSwitchConfig();
                        config.setTopicDestinations(String.valueOf(field.get(clazz)));
                        config.setPass(1);
                        if(descriptions.containsKey(config.getTopicDestinations())){
                            config.setRemark(descriptions.get(config.getTopicDestinations()));
                        }
                        KAFKA_SWITCH_DEFAULT.put(config.getTopicDestinations(), config);
                    }
                }
            }
            //暂时废弃log推送，没完了这个
//            KAFKA_SWITCH_DEFAULT.put("application.logs",false);
        }catch (Exception e){
            //出错什么都不做
        }
    }

    public int saveOrUpdate(String key, String value) throws ServiceException {
        try {
            KafkaSwitchConfig kafkaSwitchConfig = new KafkaSwitchConfig();
            if(Integer.parseInt(value) == 1){
                kafkaSwitchConfig.setPass(0);
            }else {
                kafkaSwitchConfig.setPass(1);
            }

            kafkaSwitchConfig.setTopicDestinations(key);
            CacheUtil.set(KAFKA_SWITCH_CACHE_KEY,key,kafkaSwitchConfig);
            if (null == kafkaSwitchConfigMapper.selectOne(kafkaSwitchConfig) ) {
                return kafkaSwitchConfigMapper.insert(kafkaSwitchConfig);
            } else {
                Example example = new Example(KafkaSwitchConfig.class);
                example.createCriteria().andEqualTo("topicDestinations", key);
                return kafkaSwitchConfigMapper.updateByExample(kafkaSwitchConfig, example);
            }
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public Map<String, KafkaSwitchConfig> all() throws ServiceException {
        ConcurrentHashMap<String,KafkaSwitchConfig> allKVMap = new ConcurrentHashMap();
        allKVMap.putAll(KAFKA_SWITCH_DEFAULT);
        List<KafkaSwitchConfig> list = selectAll();
        if(CollectionUtil.isNotEmpty(list)){
            for (KafkaSwitchConfig kafkaSwitchConfig : list) {
                allKVMap.put(kafkaSwitchConfig.getTopicDestinations(),kafkaSwitchConfig);
            }
        }
        return allKVMap;
    }

    public KafkaSwitchConfig select(String key) throws ServiceException {
        try {
            //缓存
            KafkaSwitchConfig kafkaSwitchConfig = (KafkaSwitchConfig) CacheUtil.get(KAFKA_SWITCH_CACHE_KEY, key);;
            if(kafkaSwitchConfig != null){
                return kafkaSwitchConfig;
            }
            //数据库
            kafkaSwitchConfig = new KafkaSwitchConfig();
            kafkaSwitchConfig.setTopicDestinations(key);
            kafkaSwitchConfig = kafkaSwitchConfigMapper.selectOne(kafkaSwitchConfig);
            if(kafkaSwitchConfig != null){
                CacheUtil.set(KAFKA_SWITCH_CACHE_KEY,key,kafkaSwitchConfig);
                return kafkaSwitchConfig;
            }else {
                CacheUtil.set(KAFKA_SWITCH_CACHE_KEY,key, KAFKA_SWITCH_DEFAULT.get(key));
                return KAFKA_SWITCH_DEFAULT.get(key);
            }
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public List<KafkaSwitchConfig> selectAll() throws ServiceException {
        //缓存 数据库 本地
        //本地
        ConcurrentHashMap<String,KafkaSwitchConfig> allKVMap = new ConcurrentHashMap();
        allKVMap.putAll(KAFKA_SWITCH_DEFAULT);

        //数据库 顺带更新数据库
        List<KafkaSwitchConfig> dbDatas = super.selectAll();
        if(CollectionUtil.isNotEmpty(dbDatas)){
            for (KafkaSwitchConfig data : dbDatas) {
                CacheUtil.set(KAFKA_SWITCH_CACHE_KEY,data.getTopicDestinations(),data);
                allKVMap.put(data.getTopicDestinations(),data);
            }
        }
        //缓存
        Collection<String> keys = CacheUtil.keys(KAFKA_SWITCH_CACHE_KEY);
        Map<String, CacheObject> dataCache = CacheUtil.get(KAFKA_SWITCH_CACHE_KEY, keys);
        if(CollectionUtil.isNotEmpty(dataCache)){
            for (CacheObject value : dataCache.values()) {
                KafkaSwitchConfig k = (KafkaSwitchConfig) value.getValue();
                allKVMap.put(k.getTopicDestinations(),k);
            }
        }
        List<KafkaSwitchConfig> all = new ArrayList<>(allKVMap.values());
        return all;
    }

    public boolean checkCanPass(String key) throws ServiceException {
        key = key.replace(kafkaDestinationPrefix,"");
        KafkaSwitchConfig config = select(key);
        if(config==null){
            return false;
        }else {
            return config.canPass();
        }
    }
}
