/*!
 * CodeGenerator(https://git.oschina.net/ptma/CodeGenerator)
 *
 * Date: 2017-04-20 10:18:47
 */
package com.qhiot.transport.db.service;

import com.qhiot.core.exception.DuplicateDataException;
import com.qhiot.core.exception.ServiceException;
import com.qhiot.core.service.BaseService;
import com.qhiot.core.util.DateUtil;
import com.qhiot.transport.db.dto.MonitoringChartData;
import com.qhiot.transport.db.dto.MonitoringDataDayDTO;
import com.qhiot.transport.db.dto.MonitoringDataHistoryDTO;
import com.qhiot.transport.db.dto.MonitoringGisDataHistoryDTO;
import com.qhiot.transport.db.mapper.MonitoringDataHistoryMapper;
import com.qhiot.transport.db.model.MonitoringDataHistory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import tk.mybatis.mapper.entity.Example;

import java.util.Date;
import java.util.List;

/**
 * 业务层 <br/>
 * Table: monitoring_data_history - 测点历史数据 <br/>
 *
 * @author 金建强(ptma @ 163.com)
 * @version 2017-04-20 10:18:47
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class MonitoringDataHistoryService extends BaseService<MonitoringDataHistory> {

    @Autowired
    private MonitoringDataHistoryMapper monitoringDataHistoryMapper;

    /**
     * 插入多个表，仅 monitoring_data_day, monitoring_data_month, monitoring_data_history
     *
     * @param entity 实体类
     * @return 影响的行数
     * @throws ServiceException 业务出错
     */
    public int insertToMultipleTableSelective(MonitoringDataHistory entity) throws ServiceException, DuplicateDataException {
        try {
            // 是否是今日的数据
            boolean today = DateUtil.isToday(entity.getDataTime());

            // 是否是最近一个月内的数据
            int diffDays = DateUtil.diffDays(entity.getDataTime(), DateUtil.now());
            boolean recentlyMonth = diffDays <= 31 && diffDays >= 0;
            if (today || recentlyMonth) {
                return monitoringDataHistoryMapper.insertToMultipleTableSelective(entity, today, recentlyMonth);
            }
            return 0;
        } catch (Exception e) {
            if (e instanceof DuplicateKeyException) {
                throw new DuplicateDataException("监测数据重复");
            } else {
                throw new ServiceException(e.getMessage(), e);
            }
        }
    }

    public List<MonitoringChartData> selectChartDataByExampleFromDay(Example example) throws ServiceException {
        try {
            return monitoringDataHistoryMapper.selectChartDataByExampleFromDay(example);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }


    public List<MonitoringChartData> selectChartDataByExampleFromMonth(Example example) throws ServiceException {
        try {
            return monitoringDataHistoryMapper.selectChartDataByExampleFromMonth(example);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public List<MonitoringDataHistoryDTO> selectDTOByExampleFromMonth(Example example) throws ServiceException {
        try {
            return monitoringDataHistoryMapper.selectDTOByExampleFromMonth(example);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public List<MonitoringGisDataHistoryDTO> selectMonitoringChartDataFromMonth(Example example) throws ServiceException {
        try {
            return monitoringDataHistoryMapper.selectMonitoringChartDataFromMonth(example);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    /**
     * 删除日表中指定时间之前的数据, 时间转为最小ID
     *
     * @param dataId
     * @return
     * @throws ServiceException
     */
    public int deleteMonitoringDataDayLessThanId(long dataId) throws ServiceException {
        try {
            return monitoringDataHistoryMapper.deleteMonitoringDataDayLessThanId(dataId);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    /**
     * 删除月表中指定时间之前的数据, 时间转为最小ID
     *
     * @param dataId
     * @return
     * @throws ServiceException
     */
    public int deleteMonitoringDataMonthLessThanId(long dataId) throws ServiceException {
        try {
            return monitoringDataHistoryMapper.deleteMonitoringDataMonthLessThanId(dataId);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    /**
     * 根据时间范围查询测点数据
     *
     * @param deviceId     终端编号
     * @param indexId      监测指标编号
     * @param monitoringId 测点编号
     * @param smId         站点测点编号
     * @param startTime    开始时间
     * @param endTime      结束时间
     * @return
     * @throws ServiceException
     */
    public MonitoringDataHistory selectMonitoringDataHistoryByTime(String deviceId,
                                                                   Long indexId,
                                                                   Long monitoringId,
                                                                   Long smId,
                                                                   Date startTime,
                                                                   Date endTime) throws ServiceException {
        try {
            return monitoringDataHistoryMapper.selectMonitoringDataHistoryByTime(deviceId, indexId, monitoringId, smId, startTime, endTime);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public MonitoringDataHistory selectMonitoringDataHistoryAllByTime(String deviceId,
                                                                      Long indexId,
                                                                      Long monitoringId,
                                                                      Long smId,
                                                                      Date startTime,
                                                                      Date endTime) throws ServiceException {
        try {
            return monitoringDataHistoryMapper.selectMonitoringDataHistoryAllByTime(deviceId, indexId, monitoringId, smId, startTime, endTime);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public MonitoringDataHistory selectBySmId(Long smId) throws ServiceException {
        try {
            return monitoringDataHistoryMapper.selectBySmId(smId);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public List<MonitoringDataHistory> selectMonitoringDataHistoryAllByTimeSM(Long smId,
                                                                              Date startTime,
                                                                              Date endTime) throws ServiceException {
        try {
            return monitoringDataHistoryMapper.selectMonitoringDataHistoryAllByTimeSM(smId, startTime, endTime);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public List<MonitoringDataHistory> selectMonitoringDataHistoryByTimeSM(Long smId,
                                                                           Date startTime,
                                                                           Date endTime) throws ServiceException {
        try {
            return monitoringDataHistoryMapper.selectMonitoringDataHistoryByTimeSM(smId, startTime, endTime);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public List<MonitoringDataDayDTO> selectAVGHistoryByTimeSM(Long smId,
                                                               Date startTime,
                                                               Date endTime) throws ServiceException {
        try {
            return monitoringDataHistoryMapper.selectAVGHistoryByTimeSM(smId, startTime, endTime);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public List<MonitoringDataDayDTO> selectAVGHistoryAllByTimeSM(Long smId,
                                                                  Date startTime,
                                                                  Date endTime) throws ServiceException {
        try {
            return monitoringDataHistoryMapper.selectAVGHistoryAllByTimeSM(smId, startTime, endTime);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }
}
