/*!
 * CodeGenerator(https://git.oschina.net/ptma/CodeGenerator)
 *
 * Date: 2017-04-13 10:12:30
 */
package com.qhiot.transport.db.service;

import com.qhiot.cache.util.CacheUtil;
import com.qhiot.core.exception.ServiceException;
import com.qhiot.transport.db.dto.MonitoringDTO;
import com.qhiot.transport.db.dto.TransportMonitoringDataDTO;
import com.qhiot.transport.db.dto.TransportResourceMonitoringIndexDTO;
import com.qhiot.transport.db.mapper.TransportResourceMonitoringMapper;
import com.qhiot.transport.db.model.TransportResourceMonitoring;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import tk.mybatis.mapper.entity.Example;

import java.util.List;

/**
 * 业务层 <br/>
 * Table: transport_resource_monitoring - 终端资源测点对应配置 <br/>
 * 注意：如果需要新增insert,save,uodate,delete相关的方法，请检查ConfigServiceAspect的AOP拦截
 *
 * @author 金建强(ptma @ 163.com)
 * @version 2017-04-13 10:12:30
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class TransportResourceMonitoringService {

    @Autowired
    private TransportResourceMonitoringMapper transportResourceMonitoringMapper;

    public TransportResourceMonitoring selectByPrimaryKey(Long monitoringId) throws ServiceException {
        try {
            return transportResourceMonitoringMapper.selectByPrimaryKey(monitoringId);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public void update2Null(String deviceId) throws ServiceException {
        try {
            transportResourceMonitoringMapper.update2Null(deviceId);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    /**
     * 保存或更新， 判断的依据是 deviceId, configIndex, dataIndex
     *
     * @param entity 实体类
     * @return
     * @throws ServiceException
     */
    @CacheEvict(value = "seem:transport:resource:monitoring", key = "#entity.deviceId+'_'+#entity.configIndex")
    public TransportResourceMonitoring saveOrUpdateSelective(TransportResourceMonitoring entity) throws ServiceException {
        try {
            CacheUtil.deleteByPrefix("seem:transport:resource:monitoring", entity.getDeviceId());

            Example example = new Example(TransportResourceMonitoring.class);
            example.createCriteria().andEqualTo("deviceId", entity.getDeviceId())
                    .andEqualTo("configIndex", entity.getConfigIndex())
                    .andEqualTo("dataIndex", entity.getDataIndex());
            List<TransportResourceMonitoring> list = transportResourceMonitoringMapper.selectByExample(example);
            if (list == null || list.isEmpty()) {
                transportResourceMonitoringMapper.insertSelective(entity);
            } else {
                entity.setMonitoringId(list.get(0).getMonitoringId());
                transportResourceMonitoringMapper.updateByPrimaryKeySelective(entity);
            }
            return entity;
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    /**
     * 根据终端编号和配置序号找出对应的所有测点配置
     *
     * @param deviceId
     * @param configIndex
     * @return
     * @throws ServiceException
     */
    @Cacheable(value = "seem:transport:resource:monitoring", key = "#deviceId+'_'+#configIndex")
    public List<TransportResourceMonitoringIndexDTO> selectByDeviceAndConfigIndex(String deviceId, int configIndex) throws ServiceException {
        try {
            Example example = new Example(TransportResourceMonitoring.class);
            example.createCriteria().andEqualTo("deviceId", deviceId)
                    .andEqualTo("configIndex", configIndex);
            example.orderBy("dataIndex");
            return transportResourceMonitoringMapper.selectIndexDTOByExample(example);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public List<TransportResourceMonitoringIndexDTO> selectByDeviceAndConfigIndexAll(String deviceId) throws ServiceException {
        try {
            return transportResourceMonitoringMapper.selectIndexDTOAll(deviceId);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }


    public List<TransportResourceMonitoring> selectByExample(Example example) throws ServiceException {
        try {
            return transportResourceMonitoringMapper.selectByExample(example);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }


    /**
     * 将测点的有效标志全部变为无效
     *
     * @param deviceId
     * @param configIndex
     * @return
     * @throws ServiceException
     */
    public int invalidByDeviceAndConfigIndex(String deviceId, int configIndex) throws ServiceException {
        try {
            CacheUtil.deleteByPrefix("seem:transport:resource:monitoring", deviceId);

            Example example = new Example(TransportResourceMonitoring.class);
            example.createCriteria().andEqualTo("deviceId", deviceId)
                    .andEqualTo("configIndex", configIndex);
            TransportResourceMonitoring entity = new TransportResourceMonitoring();
            entity.setValid((short) 0);
            return transportResourceMonitoringMapper.updateByExampleSelective(entity, example);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public int deleteByDeviceAndConfigIndex(String deviceId, int configIndex) throws ServiceException {
        try {
            CacheUtil.deleteByPrefix("seem:transport:resource:monitoring", deviceId);

            Example example = new Example(TransportResourceMonitoring.class);
            example.createCriteria().andEqualTo("deviceId", deviceId)
                    .andEqualTo("configIndex", configIndex);
            return transportResourceMonitoringMapper.deleteByExample(example);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public int deleteByDeviceAndConfigIndexAndDataIndex(String deviceId, int configIndex, int dataIndex) throws ServiceException {
        try {
            CacheUtil.deleteByPrefix("seem:transport:resource:monitoring", deviceId);

            Example example = new Example(TransportResourceMonitoring.class);
            example.createCriteria().andEqualTo("deviceId", deviceId)
                    .andEqualTo("configIndex", configIndex).andEqualTo("dataIndex", dataIndex);
            return transportResourceMonitoringMapper.deleteByExample(example);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public int deleteByPage(String deviceId, int page) throws ServiceException {
        try {
            CacheUtil.deleteByPrefix("seem:transport:resource:monitoring", deviceId);

            Example example = new Example(TransportResourceMonitoring.class);
            example.createCriteria().andEqualTo("deviceId", deviceId)
                    .andBetween("configIndex", page * 10, page * 10 + 9);

            return transportResourceMonitoringMapper.deleteByExample(example);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public TransportResourceMonitoringIndexDTO selectIndexDTOByMonitoringId(Long monitoringId) throws ServiceException {
        try {
            return transportResourceMonitoringMapper.selectIndexDTOByMonitoringId(monitoringId);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public List<TransportResourceMonitoring> selectByDevice(String deviceId) throws ServiceException {
        try {
            Example example = new Example(TransportResourceMonitoring.class);
            example.createCriteria().andEqualTo("deviceId", deviceId).andNotEqualTo("configIndex", -1L);
            return transportResourceMonitoringMapper.selectByExample(example);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public int deleteByDevice(String deviceId) throws ServiceException {
        try {
            CacheUtil.deleteByPrefix("seem:transport:resource:monitoring", deviceId);

            Example example = new Example(TransportResourceMonitoring.class);
            example.createCriteria().andEqualTo("deviceId", deviceId);
            return transportResourceMonitoringMapper.deleteByExample(example);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }


    public List<TransportMonitoringDataDTO> selectDTOExample(Example example) throws ServiceException {
        try {
            return transportResourceMonitoringMapper.selectValidMonitoringDataDTOByExample(example);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }


    /**
     * 按条件查询
     *
     * @param example 查询条件
     * @return 实体集合
     * @throws ServiceException 业务出错
     */
    public List<TransportMonitoringDataDTO> selectValidMonitoringDataDTO(Example example) throws ServiceException {
        try {
            return transportResourceMonitoringMapper.selectValidMonitoringDataDTOByExample(example);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    /**
     * 根据Example的条件进行更新实体中的非null字段
     *
     * @param transportResourceMonitoring 实体
     * @param example                     条件
     * @return 一个分页的实体集合
     * @throws ServiceException 业务出错
     */
    public int updateByExampleSelective(TransportResourceMonitoring transportResourceMonitoring, Example example) throws ServiceException {
        try {
            return transportResourceMonitoringMapper.updateByExampleSelective(transportResourceMonitoring, example);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public List<MonitoringDTO> selectMonitoringDataMonthDTOByExample(Example example) throws ServiceException {
        try {
            return transportResourceMonitoringMapper.selectMonitoringDataMonthDTOByExample(example);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public int updateByPrimaryKey(TransportResourceMonitoring entity) throws ServiceException {
        try {
            CacheUtil.deleteByPrefix("seem:transport:resource:config", entity.getDeviceId());
            return transportResourceMonitoringMapper.updateByPrimaryKey(entity);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

}
