package com.qhiot.transport.influxdb.service;

import com.qhiot.core.exception.ServiceException;
import com.qhiot.core.util.DateUtil;
import com.qhiot.influxdb.service.InfluxDBBaseService;
import com.qhiot.transport.influxdb.bean.PacketV2Log;
import com.qhiot.transport.influxdb.model.DecodeLogInflux;
import org.influxdb.dto.Query;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

/**
 * 解码解析失败才会存入influxdb，此表按照日期划分，存入${influx.database}公共库中
 * 解析失败存储时间为 ${decode.error.logtime},默认20d(20天)
 * @author 罗军(luoj@qh-iot.com)
 * @version 2021-05-24 15:00:46
 */
@Service
public class DecodeLogInfluxService extends InfluxDBBaseService<DecodeLogInflux> {

    @Value("${decode.error.log.retention.policy:decode_log_policy}")
    private String decodeLogPolicy;

    /**
     * measurement_yyyy-MM-dd
     * @return
     */
    @Override
    public String getMeasurementName(){
        return super.getMeasurementName() + "_" + DateUtil.today();
    }

    public String getMeasurementName(Date date){
        return super.getMeasurementName() + "_" + DateUtil.formatDate(date);
    }

    /**
     * 存入日志
     * @param decodeLogInflux
     */
    public void insert(DecodeLogInflux decodeLogInflux) throws ServiceException {
        try {
            appointedInsert(decodeLogInflux,null,getMeasurementName(),decodeLogPolicy);
        }catch (Exception e){
            throw new ServiceException(e.getMessage(), e);
        }
    }

    /**
     * 存入日志到指定日期的表
     * @param decodeLogInflux
     */
    public void insert(DecodeLogInflux decodeLogInflux, Date date) throws ServiceException{
        try {
            appointedInsert(decodeLogInflux,null,getMeasurementName(date),decodeLogPolicy);
        }catch (Exception e){
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public List<DecodeLogInflux> selectDecodeLogToday() throws ServiceException {
        try {
            StringBuilder sqlBuilder = new StringBuilder();
            sqlBuilder.append("SELECT * FROM ").append("\"").append(decodeLogPolicy).append("\".").append(getMeasurementName());
            Query query = new Query(sqlBuilder.toString());
            return query(query,DecodeLogInflux.class);
        }catch (Exception e){
            throw new ServiceException(e.getMessage(), e);
        }
    }

    /**
     * 查询指定日期的数据
     * @return
     * @throws ServiceException
     */
    public List<DecodeLogInflux> selectDecodeLogByTime(Date start, Date end) throws ServiceException {
        try {
            StringBuilder sqlBuilder = new StringBuilder();
            sqlBuilder.append("SELECT * FROM ");

            int a = DateUtil.getDaysOfYear(end) - DateUtil.getDaysOfYear(start) + 1;
            if(a > 0){
                for (int i = 1; i <= a; i++) {
                    sqlBuilder.append("\"").append(decodeLogPolicy).append("\".").append(getMeasurementName(DateUtil.offsetDay(start,i)));
                    if(i != a){
                        sqlBuilder.append(",");
                    }
                }
            }
            Query query = new Query(sqlBuilder.toString());
            return query(query,DecodeLogInflux.class);
        }catch (Exception e){
            throw new ServiceException(e.getMessage(), e);
        }
    }

    /**
     * 查询指定日期的数据
     * @return
     * @throws ServiceException
     */
    public List<DecodeLogInflux> selectDecodeLogByDay(Date date) throws ServiceException {
        try {
            StringBuilder sqlBuilder = new StringBuilder();
            sqlBuilder.append("SELECT * FROM ").append("\"").append(decodeLogPolicy).append("\".").append(getMeasurementName(date));
            Query query = new Query(sqlBuilder.toString());
            return query(query,DecodeLogInflux.class);
        }catch (Exception e){
            throw new ServiceException(e.getMessage(), e);
        }
    }

}
