package com.qhiot.transport.message.bean;

import org.hibernate.validator.constraints.Range;

import java.io.Serializable;

/**
 * 资源配置
 *
 * @author 金建强(ptma@163.com)
 * @version 2017-03-21 10:09
 */
public class ResourceConfig implements Serializable {

    private static final long serialVersionUID = 7582122431174072648L;

    public static final int DATA_LENGTH = 56;
    public static final int DATA_COUNT  = 30;

    /**
     * 配置有效位(1 byte), 0 或 1
     */
    @Range(min=0, max=1, message = "message.bean.ResourceConfig.valid.Range")
    protected byte          valid;

    /**
     * 协议类型(1 byte)<br/>
     * 0: 终端内部资源, <br/>
     * 1: Modbus外设, <br/>
     * 2: 电表协议1997, <br/>
     * 3: 电表协议2007, <br/>
     * 4: Hostlink协议, <br/>
     * 5: 其它协议
     */
    @Range(min=0, max=5, message = "message.bean.ResourceConfig.protocol.Range")
    protected byte          protocol;

    /**
     * 资源类型(1 byte)<br/>
     * 0-15: 4-20ma, <br/>
     * 16-23: 0-5V, <br/>
     * 24: 温度, <br/>
     * 25: 湿度, <br/>
     * 26: 掉电检测电压(V), <br/>
     * 30-45: 电量数据(安培), <br/>
     * 50-69: 开关量输入, <br/>
     * 70-89: 开关量输出, <br/>
     * 100: 外设
     */
    @Range(min=0, max=100, message = "message.bean.ResourceConfig.resource.Range")
    protected byte          resource;

    // -------------------------- 串口通讯参数 ------------------------------
    /**
     * 串口号(1 byte)<br/>
     * 1: 串口1, <br/>
     * 2: 串口2, <br/>
     * 3: 串口3, <br/>
     * 4: 串口4, <br/>
     * 5: 串口5, <br/>
     * 6: 其它
     */
    @Range(min=0, max=6, message = "message.bean.ResourceConfig.commPort.Range")
    protected byte          commPort;

    /**
     * 波特率(1 byte)<br/>
     * 0: 1200, <br/>
     * 1: 2400, <br/>
     * 2: 4800, <br/>
     * 3: 9600, <br/>
     * 4: 19200, <br/>
     * 5: 38400, <br/>
     * 6: 57600, <br/>
     * 7: 115200
     */
    @Range(min=0, max=7, message = "message.bean.ResourceConfig.baudRate.Range")
    protected byte          baudRate;

    /**
     * 数据位(1byte)<br/>
     * 0: 5位数据, <br/>
     * 1: 6位数据, <br/>
     * 2: 7位数据, <br/>
     * 3: 8位数据
     */
    @Range(min=0, max=3, message = "message.bean.ResourceConfig.dataBit.Range")
    protected byte          dataBit;

    /**
     * 停止位(1 byte)<br/>
     * 0: 1位, <br/>
     * 1: 1.5位, <br/>
     * 2: 2位
     */
    @Range(min=0, max=2, message = "message.bean.ResourceConfig.stopBit.Range")
    protected byte          stopBit;

    /**
     * 奇偶校验位(1 byte)<br/>
     * 0: 无校验, <br/>
     * 1: 奇校验, <br/>
     * 2: 偶校验
     */
    @Range(min=0, max=2, message = "message.bean.ResourceConfig.parityBit.Range")
    protected byte          parityBit;

    /**
     * 采集周期(2 byte) (0-65535)秒
     */
    @Range(min=1, max=65535, message = "message.bean.ResourceConfig.period.Range")
    protected int           period = 300;

    // ------------------------ 指令参数 ---------------------------

    /**
     * 设备地址(1 byte)<br/>
     */
    protected short          address;

    /**
     * 功能码(1 byte)<br/>
     * 01 读线圈
     * 02 读离散输入
     * 03 读保持寄存器
     * 04 读输入寄存器
     */
    @Range(min=0, max=4, message = "message.bean.ResourceConfig.functionCode.Range")
    protected byte          functionCode;

    /**
     * 起始地址(2 byte)<br/>
     */
    protected int         startingAddr;

    /**
     * 数据解码(1 byte) <br/><br/>
     *
     * 0 :Bit <br/>
     * 1 :INT8S <br/>
     * 2 :INT8U <br/>
     * 3 :INT16S_AB <br/>
     * 4 :INT16S_BA <br/>
     * 5 :INT16U_AB <br/>
     * 6 :INT16U_BA <br/>
     * 7 :Long ABCD <br/>
     * 8 :Long CDAB <br/>
     * 9 :Long BADC <br/>
     * 10:Long DCBA <br/>
     * 11:float ABCD <br/>
     * 12:float CDAB <br/>
     * 13:float BADC <br/>
     * 14:float DCBA <br/>
     */
    @Range(min=0, max=14, message = "message.bean.ResourceConfig.method.Range")
    protected byte          method;

    /**
     * 测点数量(2 byte)<br/>
     */
    protected int         dataQuantity;

    /**
     * 指令长度(1 byte) 0-32 <br/>
     */
    protected byte         commandLength = 0;

    /**
     * 采集指令(32 byte) 不足 32byte 的补 0x00
     */
    protected byte[]        command     = new byte[32];

    /**
     * 寄存器类型(1 byte)<br/>
     * 0: bit/只读, <br/>
     * 1: bit/可读可写, <br/>
     * 2: 2字节/只读, <br/>
     * 3: 2字节/可读可写
     */
    @Range(min=0, max=4, message = "message.bean.ResourceConfig.registerType.Range")
    protected byte          registerType;

    /**
     * 存放地址(2 byte) 0-65535
     */
    @Range(min=0, max=200, message = "message.bean.ResourceConfig.storeAddress.Range")
    protected int           storeAddress;

    /**
     * 上限(4 byte) Float DCBA
     */
    protected float upperBound;

    /**
     * 下限(4 byte) Float DCBA
     */
    protected float lowerBound;

    /**
     * 上限报警启用标志(1 byte)
     */
    protected byte          upperBoundEnabled;

    /**
     * 下限报警启用标志(1 byte)
     */
    protected byte          lowerBoundEnabled;

    /**
     * 预留2(2 byte)
     */
    protected byte          reserved;

    public byte getValid() {
        return valid;
    }

    public void setValid(byte valid) {
        this.valid = valid;
    }

    public byte getProtocol() {
        return protocol;
    }

    public void setProtocol(byte protocol) {
        this.protocol = protocol;
    }

    public byte getResource() {
        return resource;
    }

    public void setResource(byte resource) {
        this.resource = resource;
    }

    public byte getCommPort() {
        return commPort;
    }

    public void setCommPort(byte commPort) {
        this.commPort = commPort;
    }

    public byte getBaudRate() {
        return baudRate;
    }

    public void setBaudRate(byte baudRate) {
        this.baudRate = baudRate;
    }

    public byte getDataBit() {
        return dataBit;
    }

    public void setDataBit(byte dataBit) {
        this.dataBit = dataBit;
    }

    public byte getStopBit() {
        return stopBit;
    }

    public void setStopBit(byte stopBit) {
        this.stopBit = stopBit;
    }

    public byte getParityBit() {
        return parityBit;
    }

    public void setParityBit(byte parityBit) {
        this.parityBit = parityBit;
    }

    public int getPeriod() {
        return period;
    }

    public void setPeriod(int period) {
        this.period = period;
    }

    public short getAddress() {
        return address;
    }

    public void setAddress(short address) {
        this.address = address;
    }

    public byte getFunctionCode() {
        return functionCode;
    }

    public void setFunctionCode(byte functionCode) {
        this.functionCode = functionCode;
    }

    public int getStartingAddr() {
        return startingAddr;
    }

    public void setStartingAddr(int startingAddr) {
        this.startingAddr = startingAddr;
    }

    public int getDataQuantity() {
        return dataQuantity;
    }

    public void setDataQuantity(int dataQuantity) {
        this.dataQuantity = dataQuantity;
    }

    public byte getMethod() {
        return method;
    }

    public void setMethod(byte method) {
        this.method = method;
    }

    public byte getCommandLength() {
        return commandLength;
    }

    public void setCommandLength(byte commandLength) {
        this.commandLength = commandLength;
    }

    public byte[] getCommand() {
        return command;
    }

    public void setCommand(byte[] command) {
        this.command = command;
    }

    public byte getRegisterType() {
        return registerType;
    }

    public void setRegisterType(byte registerType) {
        this.registerType = registerType;
    }

    public int getStoreAddress() {
        return storeAddress;
    }

    public void setStoreAddress(int storeAddress) {
        this.storeAddress = storeAddress;
    }

    public float getUpperBound() {
        return upperBound;
    }

    public void setUpperBound(float upperBound) {
        this.upperBound = upperBound;
    }

    public float getLowerBound() {
        return lowerBound;
    }

    public void setLowerBound(float lowerBound) {
        this.lowerBound = lowerBound;
    }

    public byte getReserved() {
        return reserved;
    }

    public void setReserved(byte reserved) {
        this.reserved = reserved;
    }

    public byte getUpperBoundEnabled() {
        return upperBoundEnabled;
    }

    public void setUpperBoundEnabled(byte upperBoundEnabled) {
        this.upperBoundEnabled = upperBoundEnabled;
    }

    public byte getLowerBoundEnabled() {
        return lowerBoundEnabled;
    }

    public void setLowerBoundEnabled(byte lowerBoundEnabled) {
        this.lowerBoundEnabled = lowerBoundEnabled;
    }
}
