package com.qhiot.transport.codec;

import com.qhiot.transport.codec.decoder.Decoder;
import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;
import org.springframework.util.Assert;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

/**
 * 协议解析器链
 *
 * @author 金建强(ptma@163.com)
 */
public class DecoderChain {

    private List<Decoder>     decoders;
    private Iterator<Decoder> iterator;

    public DecoderChain(){
        this.decoders = new LinkedList<Decoder>();
    }

    public DecoderChain(List<Decoder> decoders){
        Assert.notNull(decoders, "decoders cannot be null");
        this.decoders = decoders;
    }

    /**
     * 执行链路中的下一个解码器
     *
     * @param channel 通道
     * @param buf 缓冲器
     * @return 回应
     */
    public byte[] doDecode(Channel channel, ByteBuf buf, ChainContext context) {
        if (this.iterator == null) {
            this.iterator = this.decoders.iterator();
        }
        if (this.iterator.hasNext()) {
            Decoder nextDecoder = this.iterator.next();
            /**
             * 此处必须捕获 RuntimeException <br>
             * 如果解码器链路中的某一个解码器因为没有处理好异常，抛出了 RuntimeException<br>
             * 捕获异常后， 直接调用链路中的下一个解码器 <br>
             * 否则将会导致后续解码器失效，漏掉数据
             */
            try {
                return nextDecoder.decode(channel, buf, context, this);
            } catch (Exception e) {
                return doDecode(channel, buf, context);
            }
        }
        return null;
    }

    /**
     * 开始执行一个解码器链路
     */
    public byte[] execute(Channel channel, ByteBuf buf, ChainContext context) {
        // 开始一个链路时， 为了保证线程安全， 需要新建一个 DecoderChain 的拷贝
        // 链路中的解码器不用 Clone，可复用
        DecoderChain chain = new DecoderChain();
        chain.addDecoders(decoders);
        chain.iterator = chain.decoders.iterator();
        return chain.doDecode(channel, buf, context);
    }

    public List<Decoder> getDecoders() {
        return decoders;
    }

    /**
     * 添加解码器到链路中
     *
     * @param decoder 解码器
     * @return DecoderChain 返回解码器链本身
     */
    public DecoderChain addDecoder(Decoder decoder) {
        decoders.add(decoder);
        return this;
    }

    /**
     * 添加解码器到链路中
     *
     * @param decoders 解码器集合
     * @return DecoderChain 返回解码器链本身
     */
    public DecoderChain addDecoders(List<Decoder> decoders) {
        this.decoders.addAll(decoders);
        return this;
    }

    /**
     * 清空链路中的解码器
     *
     * @return DecoderChain 返回解码器链本身
     */
    public DecoderChain clearDecoders() {
        decoders.clear();
        return this;
    }

    /**
     * 清空链路中的解码器
     *
     * @return DecoderChain 返回解码器链本身
     */
    public boolean hasDecoder() {
        return decoders!=null && !decoders.isEmpty();
    }
}
