package com.qhiot.transport.codec;

import com.qhiot.core.util.ByteUtil;
import com.qhiot.core.util.StringUtil;
import com.qhiot.transport.codec.bean.AttrKey;
import com.qhiot.transport.codec.bean.PacketV1;
import com.qhiot.transport.codec.bean.PacketV2;
import com.qhiot.transport.codec.decoder.BaseDecoder;
import com.qhiot.transport.codec.util.ByteBufUtil;
import com.qhiot.transport.db.model.TransportDecoder;
import com.qhiot.transport.db.service.TransportDecoderService;
import com.qhiot.transport.jms.kafka.CodecLogKafkaSender;
import com.qhiot.transport.message.CodecLogMessage;
import com.qhiot.transport.message.define.Device;
import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;
import io.netty.util.AttributeKey;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.List;

//import com.qhiot.transport.codec.util.GroovyDecoderUtil;

/**
 * 包协议解析器， 内部调用协议解析器链完成多协议的解析
 *
 * @author 金建强(ptma @ 163.com)
 * @version 2015-12-15 12:08.
 */
public class FrameDecoder extends BaseDecoder {

    public static final String DECODER_CATEGORY = "root";

    private static final Logger LOGGER = LoggerFactory.getLogger("transport.codec.FrameDecoder");

    private static final AttributeKey<byte[]> CUMULATIVE_BUF_KEY = AttributeKey.valueOf("CUMULATIVE_BUF");

    @Autowired(required = false)
    private CodecLogKafkaSender codecLogSender;

    @Autowired
    private TransportDecoderService transportDecoderService;

    @Override
    public String version() {
        return "root";
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        // 从数据库读取根解码器, 并追加到链路中
        List<TransportDecoder> decoders = transportDecoderService.selectByCategory(DECODER_CATEGORY);
        for (TransportDecoder transportDecoder : decoders) {
//            Decoder decoder = GroovyDecoderUtil.parseDecoder(transportDecoder.getContent());
//            if (null != decoder) {
//                addGroovySubDecoder(decoder);
//            }
        }
    }

    @Override
    public byte[] decode(Channel channel, ByteBuf originalBuf, ChainContext context, DecoderChain chain) {
        byte[] rawBytes = ByteBufUtil.getBytesRemaining(originalBuf);
        // 没有断包缓存时，去掉头部的 0x00
        byte[] cumulativeBytes = channel.attr(CUMULATIVE_BUF_KEY).get();

        ByteBuf buf = ByteBufUtil.directBuffer(rawBytes.length);
        buf.writeBytes(rawBytes);
        try {
            int bytesLengthBefore;
            int bytesLengthAfter;
            while (true) {
                bytesLengthBefore = buf.readableBytes();
                byte[] responsePacket = executeSubDecoderChain(channel, buf, context);
                if (responsePacket != null && responsePacket.length > 0) {
                    ByteBuf frames = ByteBufUtil.directBuffer(responsePacket.length);
                    frames.writeBytes(responsePacket);
                    channel.writeAndFlush(frames);
                    //ByteBufUtil.safeRelease(frames);
                }
                bytesLengthAfter = buf.readableBytes();

                // 解码器链路执行结束后, 比对前后剩余的可读字节
                if (bytesLengthAfter == 0 || bytesLengthBefore == bytesLengthAfter) {
                    /**
                     * bytesLengthBefore == bytesLengthAfter 前后相等, 表示没有数据可被链路处理, 也就是不符合任何子协议 有两种可能：
                     * 1、断包了
                     * 处理思路，剩余未能处理的数据缓存起来， 与下一个数据包拼接起来
                     * 2、协议不符
                     * 如果遇到这种情况，需要在根解析链的最后添加一个特殊解码器，判断所有协议的帧头帧尾，不符的直接吞噬掉
                     * 否则会导致断包处理的死循环（不符合任何协议的数据一直在数据包的前部）
                     */
                    if (bytesLengthAfter != 0) {
                        byte[] remainingBytes = ByteBufUtil.getBytesRemaining(buf);
                        if (cumulativeBytes == null) {
                            // 查找帧头，不符合帧头的去掉
                            while (remainingBytes.length > 0 && !ByteUtil.startsWith(remainingBytes, PacketV2.PREFIX_IOT)
                                    && !ByteUtil.startsWith(remainingBytes, PacketV2.PREFIX_STA)
                                    && !ByteUtil.startsWith(remainingBytes, PacketV1.PREFIX)) {
                                remainingBytes = ByteUtil.subBytes(remainingBytes, 1);
                            }
                            if (remainingBytes.length > 0) {
                                channel.attr(CUMULATIVE_BUF_KEY).set(remainingBytes);
                            }
                        } else {
                            byte[] combineBytes = ByteUtil.combine(cumulativeBytes, remainingBytes);
                            ByteBuf combineBuf = null;
                            try {
                                // 需要将之前的断包缓存清理掉， 因为后续的递归过程中如果处理不了合并后的报文，会把合并后的报文再次增加到缓存里
                                channel.attr(CUMULATIVE_BUF_KEY).set(null);
                                combineBuf = ByteBufUtil.directBuffer(combineBytes.length);
                                combineBuf.writeBytes(combineBytes);
                                decode(channel, combineBuf, context, chain);
                            } finally {
                                ByteBufUtil.safeRelease(combineBuf);
                            }
                        }
                        break;
                    }
                    /**
                     * bytesLengthAfter==0 处理完了
                     */
                    break;
                }
                /**
                 * bytesLengthBefore != bytesLengthAfter() && bytesLengthAfter > 0 处理了一部分， 还有数据剩余， 说明粘包了， 继续循环处理
                 */
            }

            if (rawBytes.length == buf.readableBytes()) {
                Device device = channel.attr(AttrKey.MASTER_DEVICE).get();
                String deviceId;
                if (device != null) {
                    deviceId = device.getDeviceId();
                } else {
                    deviceId = "";
                }

                CodecLogMessage logMessage = CodecLogMessage.analyzeMessage(channel.id().asShortText(), deviceId, context.getId(),
                        StringUtil.format("未知[{}] = {}", rawBytes.length, ByteUtil.toHexString(rawBytes, true)));

                LOGGER.warn(logMessage.getMessage());
                if (codecLogSender != null && device != null) {
                    codecLogSender.sendMessage(logMessage);
                }
            }
        } finally {
            ByteBufUtil.safeRelease(originalBuf);
            ByteBufUtil.safeRelease(buf);
        }
        return null;
    }
}
