package com.qhiot.transport.codec.bean;

import com.qhiot.core.util.ByteUtil;
import com.qhiot.core.util.CRCUtil;
import com.qhiot.transport.codec.ChainContext;
import com.qhiot.transport.codec.exception.DecodeException;
import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;

import java.nio.ByteOrder;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

/**
 * 杭州亿脑基本协议(v20141113)
 *
 * @author 金建强(ptma@163.com)
 * @version 2015-12-17 16:19.
 */
public class PacketV1 extends Packet {

    private static final long      serialVersionUID = -3003633434419612788L;

    public static final List<Byte> PREFIX           =
            Collections.unmodifiableList(Arrays.asList((byte) 0xF0, (byte) 0xF0));
    public static final int        MIN_LENGTH       = 26;

    private Channel channel;
    private String              contextId;

    private byte                   packageType;
    private byte                   reserve;
    private int                    dataFieldLength;
    private int                    srcDevAddr;
    private int                    dstDevAddr;
    private short                  sequenceNumber;
    private short                  acknowledgeNumber;
    private int                    transmissionPkgTimestamp;
    private int                    crc;

    private byte                   dataType;
    private short                  dataLength;
    private byte[]                 data;

    public static PacketV1 read(Channel channel, ChainContext context, ByteBuf buf) throws DecodeException {
        buf.markReaderIndex();
        try {
            byte[] rawBytes = new byte[buf.readableBytes()];
            buf.getBytes(0, rawBytes);

            PacketV1 pkg = new PacketV1(channel);
            pkg.contextId = context.getId();

            byte[] prefix = new byte[2];
            buf.readBytes(prefix);
            pkg.packageType = buf.readByte();
            pkg.reserve = buf.readByte();
            pkg.dataFieldLength = buf.readIntLE();
            pkg.srcDevAddr = buf.readIntLE();
            pkg.dstDevAddr = buf.readIntLE();
            pkg.sequenceNumber = buf.readShortLE();
            pkg.acknowledgeNumber = buf.readShortLE();
            pkg.transmissionPkgTimestamp = buf.readIntLE();
            int len = buf.readableBytes();
            if (len < (pkg.dataFieldLength + 2)) {
                buf.resetReaderIndex();
                return null;
            }
            pkg.dataType = buf.readByte();
            pkg.dataLength = buf.readShortLE();
            pkg.data = new byte[pkg.dataFieldLength - 3];
            buf.readBytes(pkg.data);

            // CRC 为大端模式
            byte[] crcBytes = new byte[2];
            buf.readBytes(crcBytes);
            pkg.crc = ByteUtil.toUnsignedShort(crcBytes, ByteOrder.LITTLE_ENDIAN);
            int crc2 = ByteUtil.toUnsignedShort(crcBytes, ByteOrder.BIG_ENDIAN);

            // CRC校验从包类型开始到数据域结束

            byte[] needCheckBytes = ByteUtil.subBytes(rawBytes, 2, pkg.dataFieldLength + 22 + 2);
            int crcChecker = CRCUtil.crc16(needCheckBytes);
            if (pkg.crc != crcChecker && crc2 != crcChecker) {// CRCUtil 校验失败
                buf.resetReaderIndex();
                return null;
            } else {
                return pkg;
            }
        } catch (RuntimeException e) {
            buf.resetReaderIndex();
            throw new DecodeException(e);
        }
    }

    private PacketV1(Channel channel){
        this.channel = channel;
    }

    @Override
    public String getChannelId(){
        return channel.id().asShortText();
    }

    public byte getPackageType() {
        return packageType;
    }

    public void setPackageType(byte packageType) {
        this.packageType = packageType;
    }

    public byte getReserve() {
        return reserve;
    }

    public void setReserve(byte reserve) {
        this.reserve = reserve;
    }

    public int getDataFieldLength() {
        return dataFieldLength;
    }

    public void setDataFieldLength(int dataFieldLength) {
        this.dataFieldLength = dataFieldLength;
    }

    public int getSrcDevAddr() {
        return srcDevAddr;
    }

    @Override
    public String getDeviceId() {
        return String.valueOf(srcDevAddr);
    }

    public void setSrcDevAddr(int srcDevAddr) {
        this.srcDevAddr = srcDevAddr;
    }

    public int getDstDevAddr() {
        return dstDevAddr;
    }

    public void setDstDevAddr(int destDevAddr) {
        dstDevAddr = destDevAddr;
    }

    public short getSequenceNumber() {
        return sequenceNumber;
    }

    public void setSequenceNumber(short sequenceNumber) {
        this.sequenceNumber = sequenceNumber;
    }

    public short getAcknowledgeNumber() {
        return acknowledgeNumber;
    }

    public void setAcknowledgeNumber(short acknowledgeNumber) {
        this.acknowledgeNumber = acknowledgeNumber;
    }

    public int getTransmissionPkgTimestamp() {
        return transmissionPkgTimestamp;
    }

    public void setTransmissionPkgTimestamp(int transmissionPkgTimestamp) {
        this.transmissionPkgTimestamp = transmissionPkgTimestamp;
    }

    public byte[] getData() {
        return data == null ? null : data.clone();
    }

    public void setData(byte[] data) {
        this.data = data == null ? null : data.clone();
    }

    public int getCrc() {
        return crc;
    }

    public void setCrc(int crc) {
        this.crc = crc;
    }

    public byte getDataType() {
        return dataType;
    }

    public PacketV1 setDataType(byte dataType) {
        this.dataType = dataType;
        return this;
    }

    public short getDataLength() {
        return dataLength;
    }

    public PacketV1 setDataLength(short dataLength) {
        this.dataLength = dataLength;
        return this;
    }

    @Override
    public String getContextId() {
        return contextId;
    }

}
