package com.qhiot.transport.codec.bean;

import com.qhiot.core.util.ByteUtil;
import com.qhiot.core.util.CRCUtil;
import com.qhiot.transport.codec.ChainContext;
import com.qhiot.transport.codec.exception.DecodeException;
import com.qhiot.transport.codec.util.ByteBufUtil;
import com.qhiot.transport.message.define.DeviceType;
import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;

/**
 * 心跳实体
 *
 * @author 金建强(ptma@163.com)
 * @version 2015-12-17 16:19
 */
public class PacketV2 extends Packet {

    private static final long      serialVersionUID = 4040591981205234330L;

    /** 帧头：IOT 3个字节 */
    public static final byte[] PREFIX_IOT       = new byte[] {(byte) 0x49, (byte) 0x4f, (byte) 0x54};

    /** 帧头：STA 3个字节 */
    public static final byte[] PREFIX_STA       = new byte[] {(byte) 0x53, (byte) 0x54, (byte) 0x41};

    /** 帧尾: END 3个字节 */
    public static final byte[] SUFFIX_END       = new byte[] {(byte) 0x45, (byte) 0x4e, (byte) 0x44};

    public static final int    MIN_LENGTH       = 21;

    private Channel channel;

    private String              contextId;

    /** 帧头 */
    private byte[]               prefix;
    /** 目标终端类型 */
    private byte                 dstDevType;
    /** 目标终端编号 */
    private int                  dstDevAddr;
    /** 源终端类型 */
    private byte                 deviceType;
    /** 源终端编号 */
    private String               deviceId;
    /** 命令 */
    private byte                 command;
    /** 长度(数据段的长度) */
    private short                commandDataLength;
    /** 数据 */
    private byte[]               commandData;
    /** CRC16 校验 */
    private int                  crc;

    public static PacketV2 read(Channel channel, ChainContext context, ByteBuf buf) throws DecodeException {
        buf.markReaderIndex();
        try {
            PacketV2 pkg = new PacketV2(channel);
            pkg.contextId = context.getId();

            /*
             * buf.readBytes(3); 注意此用法
             *
             * @see http://netty.io/wiki/reference-counted-objects.html ByteBuf.copy() 以及 ByteBuf.readBytes(int) 返回的
             * ByteBuf 是非衍生的， 需要单独 release.
             */
            pkg.prefix = new byte[3];
            buf.readBytes(pkg.prefix);
            if (!ByteUtil.equals(pkg.prefix, PacketV2.PREFIX_IOT) && !ByteUtil.equals(pkg.prefix, PacketV2.PREFIX_STA)) {
                buf.resetReaderIndex();
                return null;
            }

            pkg.dstDevType = buf.readByte();
            pkg.dstDevAddr = buf.readIntLE();
            pkg.deviceType = buf.readByte();
            pkg.deviceId = String.valueOf(buf.readUnsignedIntLE());
            pkg.command = buf.readByte();
            pkg.commandDataLength = buf.readShortLE();
            int len = buf.readableBytes();
            if (len < (pkg.commandDataLength + 5)) {
                buf.resetReaderIndex();
                return null;
            }
            if (pkg.commandDataLength > 0) {
                pkg.commandData = new byte[pkg.commandDataLength];
                buf.readBytes(pkg.commandData);
            } else {
                pkg.commandData = new byte[0];
            }
            pkg.crc = buf.readUnsignedShortLE();

            byte[] suffix = new byte[3];
            buf.readBytes(suffix);

            // CRC校验数据域
            int crcChecker;
            if (ByteUtil.equals(pkg.prefix, PacketV2.PREFIX_STA)) {
                crcChecker = CRCUtil.crc16CcittExt(pkg.commandData);
            } else {
                crcChecker = CRCUtil.crc16(pkg.commandData);
            }
            if (pkg.crc != crcChecker) {// CRC 校验失败
                buf.resetReaderIndex();
                return null;
            } else {
                return pkg;
            }
        } catch (Exception e) {
            buf.resetReaderIndex();
            throw new DecodeException(e);
        }
    }

    public ByteBuf toByteBuf() {
        ByteBuf responseBuf = ByteBufUtil.directBuffer(PacketV2.MIN_LENGTH + getCommandDataLength());

        responseBuf.writeBytes(getPrefix());
        responseBuf.writeByte(getDstDevType());
        responseBuf.writeIntLE(getDstDevAddr());
        responseBuf.writeByte(getDeviceType());
        responseBuf.writeIntLE(Integer.parseInt(getDeviceId()));
        responseBuf.writeByte(getCommand());
        responseBuf.writeShortLE(getCommandDataLength());
        if (getCommandDataLength() > 0) {
            responseBuf.writeBytes(getCommandData());
        }
        if (ByteUtil.equals(getPrefix(), PacketV2.PREFIX_STA)) {
            crc = CRCUtil.crc16CcittExt(getCommandData());
        } else {
            crc = CRCUtil.crc16(getCommandData());
        }
        responseBuf.writeShortLE(getCrc());
        responseBuf.writeBytes(PacketV2.SUFFIX_END);

        return responseBuf;
    }

    private PacketV2(Channel channel){
        this.channel = channel;
    }

    public byte[] getPrefix() {
        return prefix;
    }

    public void setPrefix(byte[] prefix) {
        this.prefix = prefix;
    }

    @Override
    public String getChannelId(){
        return channel.id().asShortText();
    }

    public byte getDstDevType() {
        return dstDevType;
    }

    public void setDstDevType(byte dstDevType) {
        this.dstDevType = dstDevType;
    }

    public int getDstDevAddr() {
        return dstDevAddr;
    }

    public void setDstDevAddr(int dstDevAddr) {
        this.dstDevAddr = dstDevAddr;
    }

    public byte getDeviceType() {
        return deviceType;
    }

    public String getDeviceName() {
        return DeviceType.nameOf(deviceType);
    }

    public void setDeviceType(byte deviceType) {
        this.deviceType = deviceType;
    }

    @Override
    public String getDeviceId() {
        return deviceId;
    }

    public void setDeviceId(String deviceId) {
        this.deviceId = deviceId;
    }

    public byte getCommand() {
        return command;
    }

    public void setCommand(byte command) {
        this.command = command;
    }

    public short getCommandDataLength() {
        return commandDataLength;
    }

    public void setCommandDataLength(short commandDataLength) {
        this.commandDataLength = commandDataLength;
    }

    public byte[] getCommandData() {
        return commandData == null ? new byte[0] : commandData.clone();
    }

    public void setCommandData(byte[] commandData) {
        this.commandData = commandData == null ? null : commandData.clone();
    }

    public int getCrc() {
        return crc;
    }

    public void setCrc(int crc) {
        this.crc = crc;
    }

    @Override
    public String getContextId() {
        return contextId;
    }

}
