package com.qhiot.transport.codec.bean.v2;

import com.qhiot.core.util.CRCUtil;
import com.qhiot.transport.codec.util.ByteBufUtil;
import com.qhiot.transport.message.define.DeviceType;
import io.netty.buffer.ByteBuf;

import java.io.Serializable;
import java.util.LinkedList;
import java.util.List;

/**
 * 代码更新请求的封装
 *
 * @author 金建强(ptma@163.com)
 */
public class FirmwareRequest implements Serializable {

    private static final long serialVersionUID = 3675418331382812074L;

    public static final int       SEGMENT_LENGTH_512 = 512;
    public static final int       SEGMENT_LENGTH_128 = 128;

    private boolean               ignoreErrors;

    /** 目标终端类型 */
    private DeviceType            deviceType;
    /** 目标终端编号 */
    private String                deviceId;
    /** 总片数 */
    private int                   totalSegment;
    /** 文件CRC */
    private int                   fileCrc;

    private byte[]                firmware;

    private List<FirmwareSegment> segments           = new LinkedList<FirmwareSegment>();
    private int                   segmentIndex;

    public FirmwareRequest(boolean ignoreErrors, DeviceType deviceType, String deviceId, byte[] firmware, int segmentLength) {
        ByteBuf buf = ByteBufUtil.directBuffer();
        try {
            this.ignoreErrors = ignoreErrors;
            segmentIndex = 0;
            this.deviceType = deviceType;
            this.deviceId = deviceId;
            fileCrc = CRCUtil.crc16(firmware);
            this.firmware = firmware;

            totalSegment = (int) Math.ceil(((double) firmware.length / segmentLength));

            buf.writeBytes(firmware);

            for (int i = 0; i < totalSegment; i++) {
                int dataLength = Math.min(segmentLength, buf.readableBytes());
                byte[] segmentData = new byte[dataLength];
                buf.readBytes(segmentData);
                segments.add(new FirmwareSegment(deviceType, deviceId, i + 1, segmentData));
            }
        } finally {
            ByteBufUtil.safeRelease(buf);
        }
    }

    public byte[] toBytes() {
        ByteBuf buf = ByteBufUtil.directBuffer(9);
        try {
            buf.writeByte(deviceType.getValue());
            buf.writeIntLE(Integer.parseInt(deviceId));
            buf.writeShortLE(totalSegment);
            buf.writeShortLE(fileCrc);

            return ByteBufUtil.getBytesRemaining(buf);
        } finally {
            ByteBufUtil.safeRelease(buf);
        }
    }

    /**
     * 是否有剩余分片
     */
    public boolean hasNextSegment() {
        return segments.size() > 0 && segmentIndex < segments.size();
    }


    /**
     * 切换到下一个分片
     */
    public void moveToNextSegment() {
        segmentIndex++;
    }

    /**
     * 切换到指定的分片
     */
    public void moveToSegment(int segmentIndex) {
        this.segmentIndex = segmentIndex;
    }

    /**
     * 获取下一个分片
     */
    public FirmwareSegment getCurrentSegment() {
        return segments.get(segmentIndex);
    }

    public DeviceType getDeviceType() {
        return deviceType;
    }

    public String getDeviceId() {
        return deviceId;
    }

    public int getTotalSegment() {
        return totalSegment;
    }

    public byte[] getFirmware() {
        return firmware;
    }

    public boolean isIgnoreErrors() {
        return ignoreErrors;
    }

    public void setSegmentIndex(int segmentIndex) {
        this.segmentIndex = segmentIndex;
    }

    public int getSegmentIndex() {
        return segmentIndex;
    }
}
