package com.qhiot.transport.codec.decoder;

import com.qhiot.core.util.StringUtil;
import com.qhiot.transport.codec.ChainContext;
import com.qhiot.transport.codec.DecoderChain;
import com.qhiot.transport.codec.bean.Packet;
import com.qhiot.transport.db.service.TransportDecoderService;
import com.qhiot.transport.jms.kafka.*;
import com.qhiot.transport.message.CodecLogMessage;
import com.qhiot.transport.message.data.*;
import com.qhiot.transport.message.response.ResponseMessage;
import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;

/**
 * 加码器基类， Spring Bean，提供一些常用的工具或服务类
 *
 * @author 金建强(ptma@163.com)
 * @version 2016/3/10 22:04
 */
@Component
public abstract class BaseDecoder implements InitializingBean, Decoder {

    protected final Logger            logger = LoggerFactory.getLogger("transport.codec." + version() + "." + getClass().getSimpleName());

//    @Autowired
//    private CodecLogSender            codecLogSender;
//    @Autowired
//    private TransportResponseSender   transportResponseSender;
//    @Autowired
//    private TransportDataQueueSender  transportDataQueueSender;
//    @Autowired
//    private TransportDataTopicSender  transportDataTopicSender;

    /**---kafka---*/
    @Autowired
    private CodecLogKafkaSender codecLogKafkaSender;
    @Autowired
    private OnlineDataKafkaSender onlineDataKafkaSender;
    @Autowired
    private CoulometerDataKafkaSender coulometerDataKafkaSender;
    @Autowired
    private ElectricCabinetDataKafkaSender electricCabinetDataKafkaSender;
    @Autowired
    private IccidDataKafkaSender iccidDataKafkaSender;
    @Autowired
    private InputStateDataKafkaSender inputStateDataKafkaSender;
    @Autowired
    private OutputStateDataKafkaSender outputStateDataKafkaSender;
    @Autowired
    private MonitoringHistoryDataKafkaSender monitoringHistoryDataKafkaSender;
    @Autowired
    private OnboardTagDataKafkaSender onboardTagDataKafkaSender;
    @Autowired
    private PowerDataKafkaSender powerDataKafkaSender;
    @Autowired
    private Rs485DataKafkaSender rs485DataKafkaSender;
    @Autowired
    private Tag24gDataKafkaSender tag24gDataKafkaSender;
    @Autowired
    private Tag780mDataKafkaSender tag780mDataKafkaSender;
    @Autowired
    private TransformerStateKafkaSender transformerStateKafkaSender;
    @Autowired
    private TransportResponseKafkaSender transportResponseKafkaSender;
    /**------*/
    @Autowired
    protected TransportDecoderService transportDecoderService;

    private DecoderChain              subDecoderChain;

    protected List<Decoder>           duplicateSubDecoders;

    abstract public String version();

    /**
     * 执行子解码器链
     *
     * @param channel
     * @param buf
     * @return 子解码器返回回应包的数据域部分, 返回 null 表示无需回应
     */
    public byte[] executeSubDecoderChain(final Channel channel, ByteBuf buf,
                                         final ChainContext context) {
        if (subDecoderChain != null && subDecoderChain.hasDecoder()) {
            return subDecoderChain.execute(channel, buf, context);
        } else {
            return null;
        }
    }

    /**
     * 设置子解码器列表， 会清除原有的列表
     *
     * @param subDecoders
     */
    public void setSubDecoders(List<Decoder> subDecoders) {
        if (subDecoderChain == null) {
            subDecoderChain = new DecoderChain();
        }
        subDecoderChain.clearDecoders();
        subDecoderChain.addDecoders(subDecoders);
        // 保存一份由配置文件设置的解码器列表副本
        if (duplicateSubDecoders == null) {
            duplicateSubDecoders = new ArrayList<Decoder>();
        }
        duplicateSubDecoders.clear();
        duplicateSubDecoders.addAll(subDecoders);
    }

    /**
     * 添加脚本解码器到子解码器链
     *
     * @param decoder
     */
    public void addGroovySubDecoder(Decoder decoder) {
        if (subDecoderChain == null) {
            subDecoderChain = new DecoderChain();
        }
        subDecoderChain.addDecoder(decoder);
    }

    /**
     * 是否存在子解码器
     *
     * @return
     */
    public boolean hasSubDecoder() {
        return subDecoderChain != null && subDecoderChain.hasDecoder();
    }

    /**
     * 重置解码器链
     */
    public void resetSubDecoders() {
        try {
            subDecoderChain.clearDecoders();
            if (duplicateSubDecoders != null) {
                subDecoderChain.addDecoders(duplicateSubDecoders);
            }
            afterPropertiesSet();
        } catch (Exception e) {
            logger.info(e.getMessage(), e);
        }
    }

    /**
     * 输出并传输消息日志到传输监控消息队列， 消息监听者会负责将消息通过 websocket 发送到浏览器前端
     *
     * @param packet 解码包
     * @param format 消息模板
     * @param arguments 消息参数
     */
    public void printAnalyzeMessageToLogTopic(Packet packet, String format, Object... arguments) {
        CodecLogMessage logMessage = CodecLogMessage.analyzeMessage(packet.getChannelId(), packet.getDeviceId(), packet.getContextId(), StringUtil.format(format, arguments));
        logger.debug(logMessage.getMessage());
        if (codecLogKafkaSender != null) {
            codecLogKafkaSender.sendMessage(logMessage);
        }
    }

    /**
     * 输出并传输下行日志到传输监控消息队列， 消息监听者会负责将消息通过 websocket 发送到浏览器前端
     *
     * @param packet 解码包
     * @param format 消息模板
     * @param arguments 消息参数
     */
    public void printDownstreamMessageToLogTopic(Packet packet, String format, Object... arguments) {
        CodecLogMessage logMessage =
                CodecLogMessage.downstreamMessage(packet.getChannelId(), packet.getDeviceId(), packet.getContextId(),
                    StringUtil.format(format, arguments));
//        logger.debug(logMessage.getMessage());
        if (codecLogKafkaSender != null) {
            codecLogKafkaSender.sendMessage(logMessage);
        }
    }

    /**
     * 输出普通日志到传输监控消息队列， 消息监听者会负责将消息通过 websocket 发送到浏览器前端
     *
     * @param packet
     * @param format
     * @param arguments
     */
    public void printNormalMessageToLogTopic(Packet packet, String format, Object... arguments) {
        CodecLogMessage logMessage =
            CodecLogMessage.normalMessage(packet.getChannelId(), packet.getDeviceId(), packet.getContextId(),
                StringUtil.format(format, arguments));
        logger.debug(logMessage.getMessage());
        if (codecLogKafkaSender != null) {
            codecLogKafkaSender.sendMessage(logMessage);
        }
    }


    /**
     * 发送响应消息对象到 MQ topic://transport.response
     */
    public void sendTransportResponseToTopic(final ResponseMessage message) {
//        if (transportResponseSender != null) {
//            transportResponseSender.sendMessage(message);
//        }
        transportResponseKafkaSender.sendMessage(message);
    }

    /**
     * 发送数据到 MQ queue://transport.data.queue
     */
//    public void sendTransportDataToQueue(final Data message) {
//        if (transportDataQueueSender != null) {
//            transportDataQueueSender.sendMessage(message);
//        }
//    }

    /**
     * 发送数据到 MQ topic://transport.data.topic
     */
//    public void sendTransportDataToTopic(final Data message) {
//        if (transportDataTopicSender != null) {
//            transportDataTopicSender.sendMessage(message);
//        }
//    }

    public void sendOnlieDataToKafka(final OnlineData message) {
        if (onlineDataKafkaSender != null) {
            onlineDataKafkaSender.sendMessage(message);
        }
    }

    /**
     * 发送数据到 MQ queue://transport.data.queue, topic://transport.data.topic
     */
//    public void sendTransportDataToQueueAndTopic(final Data message) {
//        if (transportDataQueueSender != null) {
//            transportDataQueueSender.sendMessage(message);
//        }
//        if (transportDataTopicSender != null) {
//            transportDataTopicSender.sendMessage(message);
//        }
//    }



    public void sendCoulometerData(final CoulometerData message) {
        coulometerDataKafkaSender.sendMessage(message);
    }

    public void sendElectricCabinetData(final ElectricCabinetData message) {
        electricCabinetDataKafkaSender.sendMessage(message);
    }

    public void sendIccidData(final IccidData message) {
        iccidDataKafkaSender.sendMessage(message);
    }

    public void sendInputStateData(final InputStateData message) {
        inputStateDataKafkaSender.sendMessage(message);
    }

    public void sendOutputStateData(final OutputStateData message) {
        outputStateDataKafkaSender.sendMessage(message);
    }

    public void sendMonitoringDataHistory(final MonitoringHistoryData message) {
        monitoringHistoryDataKafkaSender.sendMessage(message);
    }

    public void sendOnboardTagData(final OnboardTagData message) {
        onboardTagDataKafkaSender.sendMessage(message);
    }

    public void sendPowerData(final PowerData message) {
        powerDataKafkaSender.sendMessage(message);
    }

    public void sendRs485Data(final Rs485Data message) {
        rs485DataKafkaSender.sendMessage(message);
    }
    public void sendTag24gData(final Tag24gData message) {
        tag24gDataKafkaSender.sendMessage(message);
    }
    public void sendTag780mData(final Tag780mData message) {
        tag780mDataKafkaSender.sendMessage(message);
    }
    public void sendTransformerState(final TransformerState message) {
        transformerStateKafkaSender.sendMessage(message);
    }
}
