package com.qhiot.transport.codec.decoder;

import com.qhiot.core.util.ByteUtil;
import com.qhiot.core.util.CRCUtil;
import com.qhiot.core.util.DateUtil;
import com.qhiot.transport.codec.ChainContext;
import com.qhiot.transport.codec.DecoderChain;
import com.qhiot.transport.codec.bean.PacketV1;
import com.qhiot.transport.codec.exception.DecodeException;
import com.qhiot.transport.codec.util.ByteBufUtil;
//import com.qhiot.transport.codec.util.GroovyDecoderUtil;
import com.qhiot.transport.db.model.TransportDecoder;
import com.qhiot.transport.message.data.OnlineData;
import com.qhiot.transport.message.define.DeviceType;
import com.qhiot.transport.netty.handler.ChannelManager;
import com.qhiot.transport.netty.handler.ChannelUtil;
import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.List;

/**
 * 杭州亿脑基本协议(v20141113) 解码器
 *
 * @author 金建强(ptma@163.com)
 * @version 2015-12-17 16:19.
 */
public class PacketV1Decoder extends BaseV1Decoder {

    public static final String DECODER_CATEGORY = "packetV1";

    @Autowired
    private ChannelUtil channelUtil;

    @Override
    public byte[] decode(Channel channel, ByteBuf buf, ChainContext context, DecoderChain chain) {
        try {
            byte[] rawBytes = ByteBufUtil.getBytesRemaining(buf);

            if (!ByteUtil.startsWith(rawBytes, PacketV1.PREFIX) || rawBytes.length < PacketV1.MIN_LENGTH) {
                return chain.doDecode(channel, buf, context);
            }

            buf.markReaderIndex();

            final PacketV1 packetV1 = PacketV1.read(channel, context, buf);
            if (packetV1 == null) {
                buf.resetReaderIndex();
                return chain.doDecode(channel, buf, context);
            }
            ChannelManager.getInstance().add(DeviceType.GATEWAY.getValue(), packetV1.getDeviceId(), channel);

            OnlineData onlineData = new OnlineData(DeviceType.GATEWAY.getValue(), packetV1.getDeviceId());
            onlineData.setConnected(true);
            channelUtil.updateOnline(channel, onlineData);
//            super.sendTransportDataToTopic(onlineData);
            super.sendOnlieDataToKafka(onlineData);

            return decodeData(channel, packetV1, context);
        } catch (DecodeException e) {
            logger.info(e.getMessage(), e);
            return chain.doDecode(channel, buf, context);
        }
    }

    private byte[] decodeData(Channel channel, PacketV1 packetV1, ChainContext context) {
        context.put(ChainContext.PACKET_V1, packetV1);
        ByteBuf dataBuf = ByteBufUtil.directBuffer();
        try {
            dataBuf.writeBytes(packetV1.getData());

//            super.sendTransportDataToTopic(new OnlineData(packetV1.getDeviceId()));
            super.sendOnlieDataToKafka(new OnlineData(packetV1.getDeviceId()));


            byte[] responseDataFiled = executeSubDecoderChain(channel, dataBuf, context);

            if (responseDataFiled != null) {

                byte[] responsePacket = encoderResponsePacklet(packetV1, responseDataFiled);
                if (responsePacket != null && responsePacket.length > 0) {
                    super.printDownstreamMessageToLogTopic(packetV1, "平台回应[{}] = {}", responsePacket.length, ByteUtil.toHexString(responsePacket, true));
                }
                return responsePacket;
            } else {
                return null;
            }
        } finally {
            ByteBufUtil.safeRelease(dataBuf);
        }
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        // 从数据库读取根解码器, 并追加到链路中
//        List<TransportDecoder> decoders = transportDecoderService.selectByCategory(DECODER_CATEGORY);
//        for (TransportDecoder transportDecoder : decoders) {
//            Decoder decoder = GroovyDecoderUtil.parseDecoder(transportDecoder.getContent());
//            if (null != decoder) {
//                addGroovySubDecoder(decoder);
//            }
//        }
    }

    private byte[] encoderResponsePacklet(PacketV1 pkg, byte[] responseDataFiled) {
        if (responseDataFiled == null) {
            return new byte[0];
        }
        ByteBuf responseBuf = ByteBufUtil.directBuffer(PacketV1.MIN_LENGTH + responseDataFiled.length);
        try {
            responseBuf.writeBytes(new byte[] { (byte) 0xF0, (byte) 0xF0 });
            responseBuf.writeByte(pkg.getPackageType());
            responseBuf.writeByte(pkg.getReserve());
            responseBuf.writeIntLE(responseDataFiled.length);
            responseBuf.writeIntLE(pkg.getDstDevAddr());
            responseBuf.writeIntLE(pkg.getSrcDevAddr());
            responseBuf.writeShortLE(pkg.getAcknowledgeNumber());
            responseBuf.writeShortLE(pkg.getSequenceNumber());
            responseBuf.writeIntLE(DateUtil.getUTCSeconds());
            if (responseDataFiled.length > 0) {
                responseBuf.writeBytes(responseDataFiled);
            }
            // CRC 校验区域从包类型开始到数据域结束
            byte[] crcBytes = ByteUtil.subBytes(ByteBufUtil.getBytesRemaining(responseBuf), 2);
            int crc = CRCUtil.crc16(crcBytes);
            // CRC 校验为大端模式
            responseBuf.writeShort(crc);

            return ByteBufUtil.getBytesRemaining(responseBuf);
        } finally {
            ByteBufUtil.safeRelease(responseBuf);
        }
    }
}
