package com.qhiot.transport.codec.decoder;

import com.qhiot.core.exception.ServiceException;
import com.qhiot.core.util.ByteUtil;
import com.qhiot.core.util.ThreadUtil;
import com.qhiot.transport.codec.ChainContext;
import com.qhiot.transport.codec.DecoderChain;
import com.qhiot.transport.codec.bean.AttrKey;
import com.qhiot.transport.codec.bean.PacketV2;
import com.qhiot.transport.codec.exception.DecodeException;
import com.qhiot.transport.codec.util.*;
import com.qhiot.transport.codec.util.InnerQueueManager.InnerQueue;
import com.qhiot.transport.db.model.TransportDecoder;
import com.qhiot.transport.db.model.TransportResourceConfig;
import com.qhiot.transport.db.model.TransportTransfer;
import com.qhiot.transport.db.service.TransportResourceConfigService;
import com.qhiot.transport.db.service.TransportTransferService;
import com.qhiot.transport.db.util.BeanTransformUtil;
import com.qhiot.transport.message.bean.ResourceConfig;
import com.qhiot.transport.message.data.OnlineData;
import com.qhiot.transport.message.define.Device;
import com.qhiot.transport.message.define.DeviceType;
import com.qhiot.transport.netty.handler.ChannelManager;
import com.qhiot.transport.netty.handler.ChannelUtil;
import com.qhiot.transport.netty.transfer.Transferrer;
import io.netty.bootstrap.Bootstrap;
import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;
import org.springframework.beans.factory.annotation.Autowired;

import javax.annotation.Resource;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.TimeUnit;

/**
 * 新版协议外层解码器
 *
 * @author 金建强(ptma@163.com)
 * @version 2015-12-17 16:19
 */
public class PacketV2Decoder extends BaseV2Decoder {

    public static final String             DECODER_CATEGORY = "packetV2";

    @Resource(name = "transferBootstrap")
    private Bootstrap bootstrap;

    @Autowired
    private TransportTransferService       transportTransferService;
    @Autowired
    private TransportResourceConfigService transportResourceConfigService;

    @Autowired
    private ChannelUtil channelUtil;

    private static ExecutorService         executor         = ThreadUtil.newCachedThreadPool(10, 10000, 600L, "seem-resource-config-update-%d");

    @Override
    public byte[] decode(Channel channel, ByteBuf buf, ChainContext context, DecoderChain chain) {
        buf.markReaderIndex();
        try {
            byte[] rawBytes = ByteBufUtil.getBytesRemaining(buf);



            // 以下代码需配合 DelimiterBasedFrameDecoder, delimiter="END" 使用, 用于分帧
            // 粘包时可能前一帧未正常结束，就会导致有多个帧头， 这时只有最后一个帧头是有效的
            int offset = -1;
            PacketV2 packetV2 = PacketV2.read(channel, context, buf);
            if (packetV2 == null) {
                offset = findPrefixNotAtFront(rawBytes);
            }
            while (packetV2 == null && offset != -1) {
                rawBytes = ByteUtil.subBytes(rawBytes, offset);
                buf.readBytes(offset);

                if (rawBytes.length < PacketV2.MIN_LENGTH) {
                    return chain.doDecode(channel, buf, context);
                }

                packetV2 = PacketV2.read(channel, context, buf);
                if (packetV2 == null) {
                    offset = findPrefixNotAtFront(rawBytes);
                }
            }

            if (packetV2 == null) {
                buf.resetReaderIndex();
                return chain.doDecode(channel, buf, context);
            }

            if (DeviceType.valueOf(packetV2.getDeviceType()).equals(DeviceType.UNKNOW)) {
                super.printAnalyzeMessageToLogTopic(packetV2, "未知终端类型 0x{}, 终端编号={}", ByteUtil.toHexString(packetV2.getDeviceType()), packetV2.getDeviceId());
                return null;
            }

            ChannelManager.getInstance().add(packetV2.getDeviceType(), packetV2.getDeviceId(), channel);

            if (ByteUtil.equals(packetV2.getPrefix(), PacketV2.PREFIX_STA)) {
                channel.attr(AttrKey.PACKET_STA).set(Boolean.TRUE);
            }

            return decodePacketV2(channel, packetV2, context);
        } catch (DecodeException e) {
            buf.resetReaderIndex();
            Device device = channel.attr(AttrKey.MASTER_DEVICE).get();
            String deviceId;
            if (device != null) {
                deviceId = device.getDeviceId();
            } else {
                deviceId = "";
            }
            logger.info("[{}, {}] {}", channel.id().asShortText(), deviceId, e.getMessage(), e);
            return chain.doDecode(channel, buf, context);
        }
    }

    /**
     * 解码 PacketV2 报文
     *
     * @param channel Socket 通道
     * @param packetV2 已经解码出来的packetV2
     */
    private byte[] decodePacketV2(Channel channel, final PacketV2 packetV2, ChainContext context) {
        context.put(ChainContext.PACKET_V2, packetV2);

        // 任何命令都认为是连接数据
        OnlineData onlineData = new OnlineData(packetV2.getDeviceType(), packetV2.getDeviceId());
        onlineData.setConnected(true);
        onlineData.setChannelId(channel.id().asShortText());
//        super.sendTransportDataToTopic(onlineData);
        super.sendOnlieDataToKafka(onlineData);
        channelUtil.updateOnline(channel, onlineData);

        // 定时抓拍会导致高并发， 忽略图片分片的上传
        if (packetV2.getCommand() != Command.CAM_SNAP_UPLOADING && packetV2.getCommand() != Command.HISTORY_DATA_UPLOAD) {
            // 检查需要下发的资源配置, 同一会话只发一次
            Boolean resourceConfigSynchronous = channel.attr(AttrKey.RESOURCE_CONFIG_SYNCHRONOUS).get();
            if (resourceConfigSynchronous == null && packetV2.getCommand() != Command.RESOURCE_CONFIG_WRITE_ONE && packetV2.getCommand() != Command.RESOURCE_CONFIG_WRITE_TEN) {
                channel.attr(AttrKey.RESOURCE_CONFIG_SYNCHRONOUS).set(Boolean.TRUE);
                updateResourceConfigInThreadPool(channel, packetV2);
            }
        }

        // 检查数据转发
        transferBufferInThreadPool(channel, packetV2);

        ByteBuf commandDataBuf = ByteBufUtil.directBuffer();
        try {
            commandDataBuf.writeBytes(packetV2.getCommandData());
            byte[] responseDataFiled = executeSubDecoderChain(channel, commandDataBuf, context);

            if (responseDataFiled != null) {
                byte[] responsePacket = encoderResponsePacklet(packetV2, responseDataFiled);
                if (responsePacket != null && responsePacket.length > 0) {
                    if (Boolean.TRUE.equals(channel.attr(AttrKey.UDP).get())) {
                        try {
                            Thread.sleep(500L);
                        } catch (InterruptedException e) {

                        }
                    }

                    super.printDownstreamMessageToLogTopic(packetV2, "平台回应[{}] = {}", responsePacket.length, ByteUtil.toHexString(responsePacket, true));
                }
                return responsePacket;
            } else {
                return new byte[0];
            }
        } finally {
            ByteBufUtil.safeRelease(commandDataBuf);
        }
    }

    /**
     * 检查是否有需要更新的资源配置， 如果有就逐条下发
     *
     * @param channel
     * @param packetV2
     */
    private void updateResourceConfigInThreadPool(final Channel channel, final PacketV2 packetV2) {
        executor.submit(new Runnable() {
            @Override
            public void run() {
                List<TransportResourceConfig> configs = null;
                try {
                    configs = transportResourceConfigService.selectNeedsUpdate(packetV2.getDeviceId());
                } catch (ServiceException e) {
                    logger.error(e.getMessage(), e);
                }

                if (configs != null && !configs.isEmpty()) {
                    // 逐条下发配置
                    for (TransportResourceConfig config : configs) {
                        if (0 == config.getValid()) {
                            // 配置未生效时， 跳过
                            continue;
                        }

                        try {
                            Thread.sleep(1000L);
                        } catch (InterruptedException e) {

                        }
                        InnerQueue queue = InnerQueueManager.getAndClearQueue("SENSOR_CONFIG_WRITE_{}", config.getDeviceId());
                        ResourceConfig resourceConfig = BeanTransformUtil.transform(config);
                        Object response = sendRequestAndWait(queue, channel, packetV2.getDeviceId(), config.getConfigIndex(), resourceConfig);
                        if (response == null) {
                            continue;
                        }
                        config.setNeedsUpdate((short) 0);
                        try {
                            transportResourceConfigService.updateSelective(config);
                        } catch (ServiceException e) {
                            logger.error(e.getMessage(), e);
                        }
                    }
                }
            }
        });
    }

    /**
     * 将数据转发到其它服务器
     *
     * @param channel
     * @param packetV2
     */
    private void transferBufferInThreadPool(final Channel channel, final PacketV2 packetV2) {
        ThreadUtil.submit(new Runnable() {
            @Override
            public void run() {
                try {
                    Transferrer transferrer = channel.attr(AttrKey.TRANSFERRER).get();
                    if (transferrer == null) {
                        transferrer = new Transferrer(channel, bootstrap);
                        channel.attr(AttrKey.TRANSFERRER).set(transferrer);
                    }

                    List<TransportTransfer> transferList = transportTransferService.selectByDeviceId(packetV2.getDeviceId());

                    StringBuffer urls = new StringBuffer();

                    if (transferList != null && !transferList.isEmpty()) {
                        for (TransportTransfer t : transferList) {
                            urls.append(t.getDestHost()).append(":").append(t.getDestPort());
                            transferrer.addTransfer(t.getDestHost(), t.getDestPort());
                        }
                    }
                    ByteBuf transBuf = packetV2.toByteBuf();
                    if(urls.length() != 0){
                        logger.debug("[{},{}] == 进行数据转发，目标地址组={}", channel.id().asShortText(),packetV2.getDeviceId(),urls.toString());
                    }
                    transferrer.writeAndFlush(transBuf);
                    ByteBufUtil.safeRelease(transBuf);
                } catch (Exception e) {
                    logger.error(e.getMessage(), e);
                }
            }
        });

    }

    /**
     * 尝试发送配置， 重试3次
     *
     * @param queue
     * @param channel
     * @param gatewayId
     * @param index
     * @param config
     * @return
     */
    protected Object sendRequestAndWait(InnerQueueManager.InnerQueue queue, Channel channel, String gatewayId, int index, ResourceConfig config) {
        Object response = null;
        try {
            int tryCount = 0;
            while (tryCount < 3) {
                CommandUtil.resourceConfigWrite(channel, gatewayId, index, config);
                response = queue.poll(4, TimeUnit.SECONDS);
                if (response != null) {
                    return response;
                }
                tryCount++;
            }
        } catch (Exception e) {

        }
        return response;
    }

    /**
     * 查找不在头部的帧头（IOT|STA）出现的位置， 没有时返回 -1
     *
     * @param source
     * @return
     */
    private int findPrefixNotAtFront(byte[] source) {
        for (int i = 1, n = source.length; i < n; i++) {
            if (ByteUtil.startsWith(source, i, PacketV2.PREFIX_IOT) || ByteUtil.startsWith(source, i, PacketV2.PREFIX_STA)) {
                return i;
            }
        }
        return -1;
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        // 从数据库读取根解码器, 并追加到链路中
//        List<TransportDecoder> decoders = transportDecoderService.selectByCategory(DECODER_CATEGORY);
//        for (TransportDecoder transportDecoder : decoders) {
//            Decoder decoder = GroovyDecoderUtil.parseDecoder(transportDecoder.getContent());
//            if (null != decoder) {
//                addGroovySubDecoder(decoder);
//            }
//        }
    }
}
