package com.qhiot.transport.codec.decoder.v1;

import com.qhiot.transport.codec.ChainContext;
import com.qhiot.transport.codec.DecoderChain;
import com.qhiot.transport.codec.bean.PacketV1;
import com.qhiot.transport.codec.decoder.BaseV1Decoder;
import com.qhiot.transport.codec.decoder.Decoder;
import com.qhiot.transport.codec.exception.DecodeException;
import com.qhiot.transport.codec.util.ByteBufUtil;
//import com.qhiot.transport.codec.util.GroovyDecoderUtil;
import com.qhiot.transport.db.model.TransportDecoder;
import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;

import java.util.List;

/**
 * 传感器数据包解码器
 *
 * <p>
 * 上一级解码器 {@link com.qhiot.transport.codec.decoder.PacketV1Decoder}
 * </p>
 *
 * @author 金建强(ptma@163.com)
 * @version 2015-12-17 16:19
 */
public class SensorDataDecoder extends BaseV1Decoder {

    public static final String DECODER_CATEGORY  = "packetV1_sensor";
    public static final int    SENSOR_BASE_BYTES = 16;

    @Override
    public byte[] decode(Channel channel, ByteBuf dataBuf, ChainContext context,
                         DecoderChain chain) {
        dataBuf.markReaderIndex();
        try {
            final PacketV1 packetV1 = (PacketV1) context.get(ChainContext.PACKET_V1);
            if (packetV1 == null || packetV1.getDataType() != 0x01) {
                return chain.doDecode(channel, dataBuf, context);
            }

            int dataCount = packetV1.getDataLength() / SENSOR_BASE_BYTES;
            if (packetV1.getDataLength() >= SENSOR_BASE_BYTES) {
                ByteBuf sensorDataBuf = ByteBufUtil.directBuffer();
                sensorDataBuf.writeBytes(packetV1.getData());
                decodeSensor(channel, sensorDataBuf, context);
            }

            return encoderResponsePacklet(packetV1, dataCount);
        } catch (DecodeException e) {
            dataBuf.resetReaderIndex();
            return chain.doDecode(channel, dataBuf, context);
        }
    }

    /**
     * 递归逐条解析传感器数据， 子解码器需要注意2点： 1. 解析前必须标记读索引 dataBuf.markReaderIndex()， 无法解析的必须重置读索引
     * dataBuf.resetReaderIndex(); 2. 不需要把自身的报文数据读完， 不能多读或少读， 否则会影响后续数据的解析
     *
     * @param channel
     * @param dataBuf
     * @param context
     */
    private void decodeSensor(Channel channel, ByteBuf dataBuf, ChainContext context) {
        try {
            int beforeRemaining = dataBuf.readableBytes();
            executeSubDecoderChain(channel, dataBuf, context);
            int afterRemaining = dataBuf.readableBytes();
            if (beforeRemaining > afterRemaining && afterRemaining >= SENSOR_BASE_BYTES) {
                ByteBuf bufRemaining = ByteBufUtil.directBuffer();
                bufRemaining.writeBytes(ByteBufUtil.getBytesRemaining(dataBuf));
                decodeSensor(channel, bufRemaining, context);
            }
        } finally {
            ByteBufUtil.safeRelease(dataBuf);
        }
    }

    /**
     * 只需要回复数据域部分
     */
    private byte[] encoderResponsePacklet(PacketV1 packetV1, int dataCount) {
        ByteBuf repDataBuf = ByteBufUtil.directBuffer(19);
        try {
            repDataBuf.writeByte((byte) 0x81);
            repDataBuf.writeShortLE(3);
            repDataBuf.writeByte(dataCount);
            repDataBuf.writeByte(0x00);
            repDataBuf.writeByte(0x00);

            return ByteBufUtil.getBytesRemaining(repDataBuf);
        } finally {
            ByteBufUtil.safeRelease(repDataBuf);
        }
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        // 从数据库读取根解码器, 并追加到链路中
//        List<TransportDecoder> decoders = transportDecoderService.selectByCategory(DECODER_CATEGORY);
//        for (TransportDecoder transportDecoder : decoders) {
//            Decoder decoder = GroovyDecoderUtil.parseDecoder(transportDecoder.getContent());
//            if (null != decoder) {
//                addGroovySubDecoder(decoder);
//            }
//        }
    }

}
