package com.qhiot.transport.codec.decoder.v1.sensor;

import com.qhiot.core.util.ByteUtil;
import com.qhiot.transport.codec.ChainContext;
import com.qhiot.transport.codec.DecoderChain;
import com.qhiot.transport.codec.bean.PacketV1;
import com.qhiot.transport.codec.decoder.v1.SensorDataDecoder;
import com.qhiot.transport.codec.processor.ModbusDataProcessor;
import com.qhiot.transport.codec.util.ByteBufUtil;
import com.qhiot.transport.message.bean.Modbus;
import com.qhiot.transport.message.data.OnlineData;
import com.qhiot.transport.message.define.DeviceType;
import com.qhiot.transport.netty.handler.ChannelUtil;
import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.ArrayList;
import java.util.List;

/**
 * 64. 透传模块-modbus
 *
 * <p>
 * 上一级解码器 {@link SensorDataDecoder}
 * </p>
 *
 * 解码器需要注意2点： 1. 解析前必须标记读索引 dataBuf.markReaderIndex()， 无法解析的必须重置读索引 dataBuf.resetReaderIndex(); 2.
 * 不需要把自身的报文数据读完， 不能多读或少读， 否则会影响后续数据的解析
 *
 * @author 金建强(ptma@163.com)
 * @version 2016-07-28 12:14
 */
public class ModbusDecoder extends BaseSensorDecoder {

    @Autowired
    private ModbusDataProcessor modbusDataProcessor;

    @Autowired
    private ChannelUtil channelUtil;

    @Override
    public byte[] decode(Channel channel, ByteBuf dataBuf, ChainContext context,
                         DecoderChain chain) {
        final PacketV1 packetV1 = (PacketV1) context.get(ChainContext.PACKET_V1);
        if (packetV1 == null) {
            return chain.doDecode(channel, dataBuf, context);
        }
        dataBuf.markReaderIndex();

        byte d1 = dataBuf.readByte();
        byte type = (byte) ByteUtil.getHigh4(d1);

        /**
         * 00 16*1（字节），N=1 01 16*2（字节），N=2 10 16*4（字节），N=4 11 16*6（字节），N=6
         */
        byte pkgLen = (byte) ((ByteUtil.getLow4(d1) & 0x0C) >> 2);
        byte jump = (byte) (ByteUtil.getLow4(d1) & 0x03);
        byte dataType = dataBuf.readByte();

        if (type != 0x0b || dataType != 0x70) {
            dataBuf.resetReaderIndex();
            return chain.doDecode(channel, dataBuf, context);
        }

        double battery = calcBattery(dataBuf.readUnsignedByte());
        byte rssi = dataBuf.readByte();
        int nodeId = dataBuf.readIntLE();

        OnlineData onlineData = new OnlineData(DeviceType.GATEWAY.getValue(), String.valueOf(nodeId));
        onlineData.setConnected(true);
        channelUtil.updateOnline(channel, onlineData);
//        super.sendTransportDataToTopic(onlineData);
        super.sendOnlieDataToKafka(onlineData);


        byte[] time = new byte[6];
        dataBuf.readBytes(time);
        int timestamp = (int) (((time[0] & 0xffL) | ((time[1] & 0xffL) << 8)
                | ((time[2] & 0xffL) << 16) | ((time[3] & 0xffL) << 24) | ((time[4] & 0xffL) << 32)
                | ((time[5] & 0xffL) << 40)) / 1000);

        //设备类型
        int deviceTypes = dataBuf.readUnsignedShortLE();

        //Modbus 数据字段数
        byte dataN = dataBuf.readByte();

        ByteBuf modbusBuf = ByteBufUtil.directBuffer();

        int pkgLength;
        switch (pkgLen){
            case 0:
                pkgLength = 16;
                break;
            case 1:
                pkgLength = 16*2;
                break;
            case 2:
                pkgLength = 16*4;
                break;
            case 3:
                pkgLength = 16*6;
                break;
            default:
                pkgLength = 16;
        }
        if(pkgLength == 16){
            dataBuf.resetReaderIndex();
            return chain.doDecode(channel, dataBuf, context);
        }
        byte[] b16 = new byte[pkgLength-17];
        dataBuf.readBytes(b16);
        modbusBuf.writeBytes(b16);

        try {
            //在不知道每一段modbus数据长度，且每一段modbus数据长度都不固定的情况下，先getByte读取第5个byte位置上的数字，
            //计算当前寄存器内容的长度，截取当前寄存器内容，进行处理，余下部分重复当前逻辑
            List<Modbus> modbuses = new ArrayList<>();
            int count = 0;
            for (int i = 0;i < dataN;i++){
                try {
                    short address = modbusBuf.readUnsignedByte();
                    short functionCode = modbusBuf.readUnsignedByte();
                    int storeAddress = modbusBuf.readUnsignedShortLE();
                    short length = modbusBuf.readUnsignedByte();
                    if(length == 0){
                        break;
                    }
                    byte[] data = new byte[2*length];
                    modbusBuf.readBytes(data);
                    ByteUtil.toHexString(data,true);
                    Modbus modbus = new Modbus();
                    modbus.setDeviceId(nodeId);
                    modbus.setTimestamp(timestamp);
                    modbus.setAddress(address);
                    modbus.setFunctionCode(functionCode);
                    modbus.setStoreAddress(storeAddress);
                    modbus.setLength(length);
                    modbus.setData(data);
                    modbuses.add(modbus);
                    logger.debug("modbus"+modbus);
                    logger.debug("length:"+length+"|nodeId:"+nodeId+"|storeAddress:"+storeAddress);
                    count = count + 5 + length;
                }catch (Exception e){
                    logger.error(e.getMessage(),e);
                }
            }
            modbusDataProcessor.processorModbusData(packetV1,modbuses);
            byte[] unAnalys = new byte[modbusBuf.readableBytes()];
            modbusBuf.readBytes(unAnalys);
            ByteUtil.toHexString(unAnalys,true);
        } finally {
            ByteBufUtil.safeRelease(modbusBuf);
        }

        return null;
    }

    @Override
    public void afterPropertiesSet() throws Exception {

    }
}
