package com.qhiot.transport.codec.decoder.v2;

import com.qhiot.core.util.StringUtil;
import com.qhiot.transport.codec.ChainContext;
import com.qhiot.transport.codec.DecoderChain;
import com.qhiot.transport.codec.bean.PacketV2;
import com.qhiot.transport.codec.decoder.BaseV2Decoder;
import com.qhiot.transport.codec.exception.DecodeException;
import com.qhiot.transport.codec.util.Command;
import com.qhiot.transport.db.model.TransportAsciiParam;
import com.qhiot.transport.db.service.TransportAsciiParamService;
import com.qhiot.transport.message.bean.AsciiParam;
import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * ASCII 参数请求， 服务器发送 ASCII? 询问服务器是否有需要读写的参数, 读参数使用 key? , 写参数使用 key:value , 多个参数指令之间使用 0x00 进行分隔
 *
 * @author 金建强(ptma@163.com)
 * @version 2015-12-17 16:19.
 */
public class AsciiParamAskDecoder extends BaseV2Decoder {

    @Autowired
    private TransportAsciiParamService transportAsciiParamService;

    @Override
    public byte[] decode(Channel channel, ByteBuf commandDataBuf, ChainContext context, DecoderChain chain) {
        commandDataBuf.markReaderIndex();
        try {
            final PacketV2 packetV2 = (PacketV2) context.get(ChainContext.PACKET_V2);
            if (packetV2 == null || packetV2.getCommand() != Command.ASCII_PARAM_ASK) {
                return chain.doDecode(channel, commandDataBuf, context);
            }
            printAnalyzeMessageToLogTopic(packetV2, "ASCII?");

            // 读取数据库， 是否有需要读写的 ASCII 参数
            AsciiParam asciiParam = selectParamsByDeviceId(packetV2.getDeviceId());

            return asciiParam.getData();
        } catch (DecodeException e) {
            logger.info(e.getMessage(), e);
            commandDataBuf.resetReaderIndex();
            return chain.doDecode(channel, commandDataBuf, context);
        }
    }

    /**
     * 从数据库中读取需要读写的 ASCII 参数
     * @param deviceId
     * @return
     */
    private AsciiParam selectParamsByDeviceId(String deviceId) {
        AsciiParam asciiParam = new AsciiParam();
        try {
            Map<String, String> paramMap = new HashMap<String, String>();
            List<TransportAsciiParam> params = transportAsciiParamService.selectNotSynchronousAndReadableByDeviceId(deviceId);
            for (TransportAsciiParam param: params) {
                if (StringUtil.isNotBlank(param.getParamValue())) {
                    if (param.getSynchronousTime()==null || param.getSynchronousTime().before(param.getSetupTime())) {
                        paramMap.put(param.getParamKey(), StringUtil.trim(param.getParamValue()));
                    }
                }
                if (1==param.getReadable() && !paramMap.containsKey(param.getParamKey())){
                    paramMap.put(param.getParamKey(), "?");
                }
            }
            asciiParam.setParamMap(paramMap);
        } catch (Exception e) {
            logger.error(e.getMessage(), e);
        }
        return asciiParam;
    }


    @Override
    public void afterPropertiesSet() throws Exception {
        // Do nothing
    }

}
