package com.qhiot.transport.codec.decoder.v2;

import com.qhiot.core.util.DateUtil;
import com.qhiot.core.util.ThreadUtil;
import com.qhiot.transport.codec.ChainContext;
import com.qhiot.transport.codec.DecoderChain;
import com.qhiot.transport.codec.bean.PacketV2;
import com.qhiot.transport.codec.decoder.BaseV2Decoder;
import com.qhiot.transport.codec.exception.DecodeException;
import com.qhiot.transport.codec.util.Command;
import com.qhiot.transport.db.model.TransportAsciiParam;
import com.qhiot.transport.db.model.TransportAsciiParamHistory;
import com.qhiot.transport.db.service.TransportAsciiParamHistoryService;
import com.qhiot.transport.db.service.TransportAsciiParamService;
import com.qhiot.transport.message.bean.AsciiParam;
import com.qhiot.transport.message.bean.AsciiParamBuilder;
import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.Date;
import java.util.Iterator;
import java.util.Map;

/**
 *  ASCII 参数上报， 该报文在({@link AsciiParamAskDecoder})读写参数回应后才会上传<br/>
 *  此时表示读写参数的指令已被终端正确接收， 就可以将写参数的参数之灵清除掉了
 *
 * @author 金建强(ptma@163.com)
 * @version 2017/4/19 15:58
 */
public class AsciiParamDecoder extends BaseV2Decoder {

    @Override
    public byte[] decode(Channel channel, ByteBuf commandDataBuf, ChainContext context, DecoderChain chain) {
        commandDataBuf.markReaderIndex();
        try {
            final PacketV2 packetV2 = (PacketV2) context.get(ChainContext.PACKET_V2);
            if (packetV2 == null
                || packetV2.getCommand() != Command.ASCII_PARAM_REPORT ) {
                return chain.doDecode(channel, commandDataBuf, context);
            }

            final AsciiParam asciiParam = AsciiParamBuilder.read(commandDataBuf, packetV2.getCommandDataLength());

            printAnalyzeMessageToLogTopic(packetV2, "ASCII参数回应: {}", asciiParam.getDataString());

            ThreadUtil.submit(new Runnable() {
                @Override
                public void run() {
                    saveToHistoryAndClearWriteParam(packetV2, asciiParam);
                }
            });

            return new byte[0];
        } catch (DecodeException e) {
            logger.info(e.getMessage(), e);
            commandDataBuf.resetReaderIndex();
            return chain.doDecode(channel, commandDataBuf, context);
        }
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        // Do nothing
    }

    @Autowired
    private TransportAsciiParamService transportAsciiParamService;
    @Autowired
    private TransportAsciiParamHistoryService transportAsciiParamHistoryService;

    /**
     * 保存上报的参数到历史记录， 并且删除写参数指令
     *
     * @param packetV2
     * @param asciiParam
     */
    private void saveToHistoryAndClearWriteParam(PacketV2 packetV2, AsciiParam asciiParam) {
        try {
            Map<String,String> paramMap = asciiParam.getParamMap();
            // 上报的参数是空的(只有时间)， 代表的是写参数成功
            if (paramMap == null || paramMap.isEmpty() || (paramMap.size()==1 && paramMap.containsKey("时间"))) {
                transportAsciiParamService.updateSynchronousByDeviceId(packetV2.getDeviceId());
            } else {
                // 遍历上报的参数， 保存参数历史记录
                Date paramTime = null;
                Iterator<Map.Entry<String, String>> itr = paramMap.entrySet().iterator();
                while (itr.hasNext()) {
                    Map.Entry<String, String> entry = itr.next();
                    if("时间".equals(entry.getKey())) {
                        paramTime = DateUtil.convertUTC2Date(Integer.parseInt(entry.getValue()));
                        break;
                    }
                }

                if (paramTime == null) {
                    paramTime = DateUtil.now();
                }

                itr = paramMap.entrySet().iterator();
                while (itr.hasNext()) {
                    Map.Entry<String, String> entry = itr.next();

                    if(entry.getValue() == null || "时间".equals(entry.getKey())) {
                        continue;
                    }

                    // 更新同步值和同步时间
                    TransportAsciiParam transportAsciiParam = new TransportAsciiParam();
                    transportAsciiParam.setDeviceId(packetV2.getDeviceId());
                    transportAsciiParam.setParamKey(entry.getKey());
                    transportAsciiParam.setSynchronousValue(entry.getValue());
                    transportAsciiParam.setSynchronousTime(paramTime);
                    transportAsciiParamService.updateByDeviceIdAndParamKey(transportAsciiParam);

                    // 插入历史
                    TransportAsciiParamHistory paramHis = new TransportAsciiParamHistory();
                    paramHis.setDeviceId(packetV2.getDeviceId());
                    paramHis.setUploadTime(DateUtil.now());
                    paramHis.setParamKey(entry.getKey());
                    paramHis.setParamValue(entry.getValue());
                    paramHis.setParamTime(paramTime);
                    transportAsciiParamHistoryService.insertSelective(paramHis);
                }
            }
        } catch (Exception e) {
            logger.error(e.getMessage(), e);
        }
    }
}
