package com.qhiot.transport.codec.decoder.v2;

import com.qhiot.transport.codec.ChainContext;
import com.qhiot.transport.codec.DecoderChain;
import com.qhiot.transport.codec.bean.PacketV2;
import com.qhiot.transport.codec.decoder.BaseV2Decoder;
import com.qhiot.transport.codec.exception.DecodeException;
import com.qhiot.transport.codec.util.Command;
import com.qhiot.transport.message.bean.CameraConfig;
import com.qhiot.transport.message.bean.CameraConfigBuilder;
import com.qhiot.transport.message.response.camrea.*;
import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;

import java.util.ArrayList;
import java.util.List;

/**
 * 摄像头配置
 *
 * @author 金建强(ptma@163.com)
 * @version 2015-12-17 16:19
 */
public class CameraConfigDecoder extends BaseV2Decoder {

    @Override
    public byte[] decode(Channel channel, ByteBuf commandDataBuf, ChainContext context, DecoderChain chain) {
        commandDataBuf.markReaderIndex();
        try {
            final PacketV2 packetV2 = (PacketV2) context.get(ChainContext.PACKET_V2);
            if (packetV2 == null
                || (packetV2.getCommand() != Command.CAM_CONFIG_WRITE
                    && packetV2.getCommand() != Command.CAM_CONFIG_READ
                    && packetV2.getCommand() != Command.CAM_CONFIG_DEL
                    && packetV2.getCommand() != Command.CAM_CONFIG_DEL_ALL && packetV2.getCommand() != Command.CAM_CONFIG_READ_ALL)) {
                return chain.doDecode(channel, commandDataBuf, context);
            }

            switch (packetV2.getCommand()) {
                case Command.CAM_CONFIG_WRITE: // 写摄像头配置
                    CameraConfigWriteResponse response = new CameraConfigWriteResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                    super.sendTransportResponseToTopic(response);
                    super.printAnalyzeMessageToLogTopic(packetV2, "写摄像头配置回应");
                    break;
                case Command.CAM_CONFIG_READ: // 读摄像头配置
                    CameraConfig cameraConfig = CameraConfigBuilder.read(commandDataBuf);
                    CameraConfigReadResponse readResponse = new CameraConfigReadResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                    readResponse.setConfig(cameraConfig);
                    super.sendTransportResponseToTopic(readResponse);
                    super.printAnalyzeMessageToLogTopic(packetV2, "读摄像头配置回应");
                    break;
                case Command.CAM_CONFIG_READ_ALL: // 读全部摄像头配置
                    List<CameraConfig> cameraConfigItems = new ArrayList<CameraConfig>();
                    CameraConfig cameraConfigItem;
                    for (int i = 0; i < CameraConfig.DATA_COUNT; i++) {
                        cameraConfigItem = CameraConfigBuilder.read(commandDataBuf);
                        cameraConfigItems.add(cameraConfigItem);
                    }
                    CameraConfigReadAllResponse readAllResponse = new CameraConfigReadAllResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                    readAllResponse.setConfigs(cameraConfigItems);
                    super.sendTransportResponseToTopic(readAllResponse);
                    super.printAnalyzeMessageToLogTopic(packetV2, "读摄像头全部配置回应");
                    break;
                case Command.CAM_CONFIG_DEL: // 删除摄像头配置
                    CameraConfigDeleteResponse delResponse = new CameraConfigDeleteResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                    super.sendTransportResponseToTopic(delResponse);
                    super.printAnalyzeMessageToLogTopic(packetV2, "删除摄像头配置回应");
                    break;
                case Command.CAM_CONFIG_DEL_ALL: // 删除摄像头全部配置
                    CameraConfigDeleteAllResponse delAllResponse = new CameraConfigDeleteAllResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                    super.sendTransportResponseToTopic(delAllResponse);
                    super.printAnalyzeMessageToLogTopic(packetV2, "删除摄像头全部配置回应");
                    break;
                default:
                    return chain.doDecode(channel, commandDataBuf, context);
            }

            return null;
        } catch (DecodeException e) {
            logger.info(e.getMessage(), e);
            commandDataBuf.resetReaderIndex();
            return chain.doDecode(channel, commandDataBuf, context);
        }
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        // Do nothing
    }

}
