package com.qhiot.transport.codec.decoder.v2;

import com.qhiot.cache.util.CacheUtil;
import com.qhiot.core.exception.ServiceException;
import com.qhiot.core.sql.generator.SnowflakeID;
import com.qhiot.core.util.*;
import com.qhiot.transport.codec.ChainContext;
import com.qhiot.transport.codec.DecoderChain;
import com.qhiot.transport.codec.bean.PacketV2;
import com.qhiot.transport.codec.cache.SnapCache;
import com.qhiot.transport.codec.decoder.BaseV2Decoder;
import com.qhiot.transport.codec.exception.DecodeException;
import com.qhiot.transport.codec.util.ByteBufUtil;
import com.qhiot.transport.codec.util.Command;
import com.qhiot.transport.jms.kafka.*;
import com.qhiot.transport.message.bean.PictureSegmentBuilder;
import com.qhiot.transport.message.bean.SnapSegment;
import com.qhiot.transport.message.bean.TransportSnapsegment;
import com.qhiot.transport.message.data.SnapDataBurst;
import com.qhiot.transport.message.response.camrea.SnapUploadingResponse;
import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;
import org.springframework.beans.factory.annotation.Autowired;

import java.io.IOException;
import java.util.Date;
import java.util.concurrent.ExecutorService;

/**
 * 摄像头图片上传解析
 *
 * @author 金建强(ptma@163.com)
 * @version 2015-12-17 16:19
 */
public class CameraSegmentDecoder extends BaseV2Decoder {

    private static final String               CACHE_REGION       = "seem:transport:snap_segment";
    private static final String               CACHE_KEY_TEMPLATE = "{}_{}_{}";

    private static final int                  BURST_LENGTH = 10*1024; //byte;

    @Autowired
    private SnowflakeID snowflakeID;

    private static ExecutorService            executor           = ThreadUtil.newCachedThreadPool(10, 10000, 600L, "seem-camera-segment-%d");

    @Override
    public byte[] decode(Channel channel, ByteBuf commandDataBuf, ChainContext context, DecoderChain chain) {
        commandDataBuf.markReaderIndex();
        try {
            final PacketV2 packetV2 = (PacketV2) context.get(ChainContext.PACKET_V2);
            if (packetV2 == null || packetV2.getCommand() != Command.CAM_SNAP_UPLOADING) {
                return chain.doDecode(channel, commandDataBuf, context);
            }

            SnapSegment snapSegment = PictureSegmentBuilder.read(commandDataBuf);

            // 1、为了保证响应速度， 先回应
            byte[] responseData = encoderResponsePacklet(snapSegment);
            writeResponseV2(channel, packetV2, responseData);

            // 2、然后再处理数据
            printAnalyzeMessageToLogTopic(packetV2, "上传照片区块: {}/{}", snapSegment.getSegmentIndex(), snapSegment.getTotalSegments());

            saveSnapSegment(packetV2, snapSegment);

            return null;
        } catch (DecodeException e) {
            logger.info(e.getMessage(), e);
            commandDataBuf.resetReaderIndex();
            return chain.doDecode(channel, commandDataBuf, context);
        }
    }

    /**
     * 只需要回复数据域部分
     */
    private byte[] encoderResponsePacklet(SnapSegment snapSegment) {
        ByteBuf repDataBuf = ByteBufUtil.directBuffer(3);
        try {
            repDataBuf.writeByte(snapSegment.getCameraAddr());
            repDataBuf.writeShortLE(snapSegment.getSegmentIndex());

            return ByteBufUtil.getBytesRemaining(repDataBuf);
        } finally {
            ByteBufUtil.safeRelease(repDataBuf);
        }
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        // Do nothing
    }

    /**
     * 处理图片分片数据
     */
    public void saveSnapSegment(final PacketV2 packetV2, final SnapSegment snapSegment) {
        executor.submit(new Runnable() {

            @Override
            public void run() {
                try {
                    final String cacheKey = StringUtil.format(CACHE_KEY_TEMPLATE, packetV2.getDeviceId(), snapSegment.getCameraAddr(), snapSegment.getTimestamp());
                    SnapCache snapCache = (SnapCache) CacheUtil.get(CACHE_REGION, cacheKey);
                    if (snapCache == null) {
                        snapCache = new SnapCache(packetV2.getDeviceId(), snapSegment.getCameraAddr(), snapSegment.getTotalSegments(), snapSegment.getTimestamp());
                    }

                    if (snapSegment.getSegmentIndex() <= snapSegment.getTotalSegments()) {
                        snapCache.setBlockData(snapSegment.getSegmentIndex(), snapSegment.getSegmentData());

                        SnapUploadingResponse response = new SnapUploadingResponse(packetV2.getDeviceType(), packetV2.getDeviceId());

                        int progressVal = snapCache.completedBlock() * 100 / snapSegment.getTotalSegments();
                        response.setProgress(progressVal);
                        response.setCompleted(false);
                        response.setSnapTime(DateUtil.convertUTC2Date(snapSegment.getTimestamp()));

                        sendTransportResponseToTopic(response);
                    }
                    if (snapCache.isCompleted()) {

                        TransportSnapsegment transportSnapsegmentDetail = new TransportSnapsegment();
                        transportSnapsegmentDetail.setDeviceId(packetV2.getDeviceId());
                        transportSnapsegmentDetail.setCameraIndex(Integer.valueOf(snapSegment.getCameraAddr()));
                        transportSnapsegmentDetail.setCompletedblock(snapCache.completedBlock());
                        transportSnapsegmentDetail.setCreateTime(DateUtil.now());
                        transportSnapsegmentDetail.setTotalsegments(Integer.valueOf(snapSegment.getTotalSegments()));


//                        processSnapCache(packetV2, snapCache);
                        sendSnapBurst2Kafka(snapCache);
                        printNormalMessageToLogTopic(packetV2, "上传完毕");
                        // 保存图片后， 清除缓存
                        CacheUtil.del(CACHE_REGION, cacheKey);
                    } else {
                        CacheUtil.set(CACHE_REGION, cacheKey, snapCache);
                    }
                } catch (Exception e) {
                    logger.error(e.getMessage(), e);
                }
            }

            @Autowired
            private SnapDataBurstKafkaSender snapDataBurstKafkaSender;
            /**
             * 以分片形式传输数据
             * 1.将文件拆分为多个片段，顺序发送到kafka，消费端接收消息时，识别是否是头数据，如果是，暂存，直到收到尾部数据
             *
             * @param data
             * @throws IOException
             * @throws ServiceException
             */
            public void sendSnapBurst2Kafka(final SnapCache data) throws IOException, ServiceException {
                //序列化对象，分片对象，发送
                Date now = DateUtil.now();
                long id = snowflakeID.next();
                byte[] fileBytes = data.getBytes();
                int burstCount = (int)Math.ceil(fileBytes.length*1.0/BURST_LENGTH);
                SnapDataBurst snapDataBurst;
                for (int i = 0; i < burstCount; i++) {
                    //读取 i*BURST_LENGTH ~ (i+1)*BURST_LENGTH/length
                    byte[] burst;
                    if((i+1)*BURST_LENGTH > fileBytes.length){
                        burst = ArrayUtil.subarray(fileBytes,i*BURST_LENGTH, fileBytes.length - i*BURST_LENGTH);
                    }else {
                        burst = ArrayUtil.subarray(fileBytes,i*BURST_LENGTH, BURST_LENGTH);
                    }
                    snapDataBurst = new SnapDataBurst(data.getDeviceId(),id,burstCount,i,data.getCameraAddr(),data.getSnapTime(),burst);
                    if(i == 0){
                        snapDataBurst.setHeader(true);
                    }
                    //发送到
                    snapDataBurstKafkaSender.sendMessage(snapDataBurst);
                }

            }
        });
    }
}
