package com.qhiot.transport.codec.decoder.v2;

import com.qhiot.transport.codec.ChainContext;
import com.qhiot.transport.codec.DecoderChain;
import com.qhiot.transport.codec.bean.PacketV2;
import com.qhiot.transport.codec.decoder.BaseV2Decoder;
import com.qhiot.transport.codec.exception.DecodeException;
import com.qhiot.transport.codec.util.Command;
import com.qhiot.transport.message.bean.CameraTimePointConfig;
import com.qhiot.transport.message.bean.CameraTimePointConfigBuilder;
import com.qhiot.transport.message.response.camrea.*;
import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;

import java.util.ArrayList;
import java.util.List;

/**
 * 摄像头拍照时间点指令解析
 * <p>
 * User: 金建强(ptma@163.com)
 * </p>
 * <p>
 * Date: 2015-12-17 16:19.
 * </p>
 */
public class CameraTimePointDecoder extends BaseV2Decoder {

    @Override
    public byte[] decode(Channel channel, ByteBuf commandDataBuf, ChainContext context, DecoderChain chain) {
        commandDataBuf.markReaderIndex();
        try {
            final PacketV2 packetV2 = (PacketV2) context.get(ChainContext.PACKET_V2);
            if (packetV2 == null
                || (packetV2.getCommand() != Command.CAM_CONFIG_TIME_POINT_WRITE
                    && packetV2.getCommand() != Command.CAM_CONFIG_TIME_POINT_READ
                    && packetV2.getCommand() != Command.CAM_CONFIG_TIME_POINT_DEL
                    && packetV2.getCommand() != Command.CAM_CONFIG_TIME_POINT_DEL_ALL && packetV2.getCommand() != Command.CAM_CONFIG_TIME_POINT_READ_ALL)) {
                return chain.doDecode(channel, commandDataBuf, context);
            }

            switch (packetV2.getCommand()) {
                case Command.CAM_CONFIG_TIME_POINT_WRITE: // 写摄像头时间点配置
                    CameraTimePointWriteResponse response = new CameraTimePointWriteResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                    super.sendTransportResponseToTopic(response);

                    printAnalyzeMessageToLogTopic(packetV2, "写摄像头时间点配置回应");
                    break;
                case Command.CAM_CONFIG_TIME_POINT_READ: // 读摄像头时间点配置
                    CameraTimePointConfig config = CameraTimePointConfigBuilder.read(commandDataBuf);
                    CameraTimePointReadResponse readResponse = new CameraTimePointReadResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                    readResponse.setConfig(config);
                    super.sendTransportResponseToTopic(readResponse);

                    printAnalyzeMessageToLogTopic(packetV2, "读摄像头时间点配置回应");
                    break;
                case Command.CAM_CONFIG_TIME_POINT_READ_ALL: // 读全部摄像头时间点配置
                    List<CameraTimePointConfig> configItems = new ArrayList<CameraTimePointConfig>();
                    CameraTimePointConfig configItem;
                    for (int i = 0; i < CameraTimePointConfig.DATA_COUNT; i++) {
                        configItem = CameraTimePointConfigBuilder.read(commandDataBuf);
                        configItems.add(configItem);
                    }
                    CameraTimePointReadAllResponse readAllResponse = new CameraTimePointReadAllResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                    readAllResponse.setConfigs(configItems);
                    super.sendTransportResponseToTopic(readAllResponse);

                    printAnalyzeMessageToLogTopic(packetV2, "读摄像头时间点全部配置回应");
                    break;
                case Command.CAM_CONFIG_TIME_POINT_DEL: // 删除摄像头时间点配置
                    CameraTimePointDeleteResponse delResponse = new CameraTimePointDeleteResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                    super.sendTransportResponseToTopic(delResponse);

                    printAnalyzeMessageToLogTopic(packetV2, "删除摄像头时间点配置回应");
                    break;
                case Command.CAM_CONFIG_TIME_POINT_DEL_ALL: // 删除摄像头时间点全部配置
                    CameraTimePointDeleteAllResponse delAllResponse = new CameraTimePointDeleteAllResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                    super.sendTransportResponseToTopic(delAllResponse);

                    printAnalyzeMessageToLogTopic(packetV2, "删除摄像头时间点全部配置回应");
                    break;
                default:
                    return chain.doDecode(channel, commandDataBuf, context);
            }

            return null;
        } catch (DecodeException e) {
            logger.info(e.getMessage(), e);
            commandDataBuf.resetReaderIndex();
            return chain.doDecode(channel, commandDataBuf, context);
        }
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        // Do nothing
    }

}
