package com.qhiot.transport.codec.decoder.v2;

import com.qhiot.core.util.ByteUtil;
import com.qhiot.core.util.DateUtil;
import com.qhiot.core.util.StringUtil;
import com.qhiot.core.util.ThreadUtil;
import com.qhiot.transport.codec.ChainContext;
import com.qhiot.transport.codec.DecoderChain;
import com.qhiot.transport.codec.bean.PacketV2;
import com.qhiot.transport.codec.decoder.BaseV2Decoder;
import com.qhiot.transport.codec.exception.DecodeException;
import com.qhiot.transport.codec.util.ByteBufUtil;
import com.qhiot.transport.codec.util.Command;
import com.qhiot.transport.codec.util.UgradeThreadManager;
import com.qhiot.transport.db.model.TransportUpgrade;
import com.qhiot.transport.db.model.TransportWarning;
import com.qhiot.transport.db.service.TransportUpgradeService;
import com.qhiot.transport.jms.kafka.TransportWarningKafkaSender;
import com.qhiot.transport.message.define.DeviceType;
import com.qhiot.transport.message.request.UpgradeRequest;
import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;
import org.springframework.beans.factory.annotation.Autowired;

import java.nio.ByteOrder;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

/**
 * 心跳 0x01
 *
 * @author 金建强(ptma@163.com)
 * @version 2015-12-17 16:19
 */
public class HeartbeatDecoder extends BaseV2Decoder {

    @Autowired
    private TransportUpgradeService transportUpgradeService;

//    @Autowired
//    private InsightWarningQueueSender insightWarningQueueSender;

    /**---kafka---*/
    @Autowired
    private TransportWarningKafkaSender transportWarningKafkaSender;
    /**------*/

    @Override
    public byte[] decode(Channel channel, ByteBuf commandDataBuf, ChainContext context, DecoderChain chain) {
        commandDataBuf.markReaderIndex();
        try {
            final PacketV2 packetV2 = (PacketV2) context.get(ChainContext.PACKET_V2);
            if (packetV2 == null
                || (!DeviceType.GATEWAY.equals(packetV2.getDeviceType())
                && !DeviceType.EXCITER.equals(packetV2.getDeviceType())
                && !DeviceType.REPEATER.equals(packetV2.getDeviceType())
                && !DeviceType.TILTED_SENSOR.equals(packetV2.getDeviceType())
                && !DeviceType.HUMITURE_SENSOR.equals(packetV2.getDeviceType())
                && !DeviceType.TILTED_SENSOR.equals(packetV2.getDeviceType())
                && !DeviceType.GPRS_GPS.equals(packetV2.getDeviceType()))
                || packetV2.getCommand() != Command.HEARTBEAT) {
                commandDataBuf.resetReaderIndex();
                return chain.doDecode(channel, commandDataBuf, context);
            }

            /**
             * 检测是否有该终端的未启动的固件升级任务， 如果有， 立即启动相应的升级任务；
             * 为了不影响对心跳的及时回应，需要另起线程执行
             */
            ThreadUtil.submit(new Runnable() {
                @Override
                public void run() {
                    try {
                        // 是否有待升级的任务
                        TransportUpgrade transportUpgrade = transportUpgradeService.selectByDevice(packetV2.getDeviceId(), packetV2.getDeviceType());
                        if (transportUpgrade != null) {
                            UpgradeRequest request = new UpgradeRequest(packetV2.getDeviceType(), packetV2.getDeviceId());
                            request.setIgnoreErrors(1==transportUpgrade.getIgnoreErrors());
                            request.setFirmwireData(transportUpgrade.getFirmware());
                            request.setStartupTime(transportUpgrade.getStartupTime());
                            request.setSegmentIndex(transportUpgrade.getSegmentIndex());
                            request.setTimeout(transportUpgrade.getTimeout());
                            request.setRetryCount(transportUpgrade.getRetryCount());
                            request.setSegmentLength(transportUpgrade.getSegmentLength());
                            request.setFirmwareId(transportUpgrade.getFirmwareId());
                            UgradeThreadManager.getInstance().upgrade(request);
                        }
                    } catch (Exception e) {
                        logger.error("查询升级任务出错: {}", e.getMessage(), e);
                    }
                }
            });

            try {
                byte[] commandData = packetV2.getCommandData();
                if (packetV2.getCommandDataLength()>2) {
                    commandDataBuf.readBytes(commandData);

                    Map<String, byte[]> asciiMap = new TreeMap<String, byte[]>();
                    List<byte[]> items = ByteUtil.split(commandData, (byte) 0x3B);
                    for (byte[] item : items) {
                        List<byte[]> kv = ByteUtil.splitKeyValue(item, (byte) 0x3A);
                        if (kv.size() == 2) {
                            String key = new String(kv.get(0));
                            byte[] value = kv.get(1);
                            if (StringUtil.isNotBlank(key)) {
                                asciiMap.put(key.replaceAll("@", "").trim(), value);
                            }
                        }
                    }

                    StringBuffer messageBuf = new StringBuffer("心跳: ");
                    messageBuf.append(StringUtil.format("{}={}", DeviceType.nameOf(packetV2.getDeviceType()), packetV2.getDeviceId()));
                    List<Object> values = new ArrayList<Object>();
                    boolean first = true;
                    for (Map.Entry<String, byte[]> entry : asciiMap.entrySet()) {
                        String key = entry.getKey();
                        byte[] value = entry.getValue();
                        messageBuf.append(", ");
                        switch (key.charAt(0)) {
                            case 'A':
                                messageBuf.append("GPRS信号值={}");
                                values.add(ByteUtil.toInt(value[0]));
                                break;
                            case 'B':
                                messageBuf.append("终端温度={}℃");
                                values.add(ByteUtil.toShort(value, ByteOrder.LITTLE_ENDIAN) / 100d);
                                break;
                            case 'C':
                                messageBuf.append("终端湿度={}%");
                                values.add(ByteUtil.toShort(value, ByteOrder.LITTLE_ENDIAN) / 100d);
                                break;
                            case 'D':
                                messageBuf.append("电池电压={}V");
                                values.add(ByteUtil.toShort(value, ByteOrder.LITTLE_ENDIAN) / 1000d);
                                break;
                            case 'G':
                                messageBuf.append("GPS坐标=({}, {})");
                                values.add(ByteUtil.toFloat(value, ByteOrder.LITTLE_ENDIAN));
                                values.add(ByteUtil.toFloat(value, 4, ByteOrder.LITTLE_ENDIAN));
                                break;
                            default: {
                                messageBuf.append(entry.getKey()).append("={}");
                                values.add(ByteUtil.toHexString(value));
                            }
                        }
                    }
                    printAnalyzeMessageToLogTopic(packetV2, messageBuf.toString(), values.toArray());
                } else {
                    printAnalyzeMessageToLogTopic(packetV2, "心跳: {}={}", DeviceType.nameOf(packetV2.getDeviceType()), packetV2.getDeviceId());
                }

                ThreadUtil.submit(new Runnable() {
                    @Override
                    public void run() {
                        cancelWarning(packetV2.getDeviceId());
                    }
                });

            } catch (Exception e) {
                logger.info(e.getMessage(), e);
            }

            return encoderResponsePacklet();
        } catch (DecodeException e) {
            logger.info(e.getMessage(), e);
            commandDataBuf.resetReaderIndex();
            return chain.doDecode(channel, commandDataBuf, context);
        }
    }

    /**
     * 只需要回复数据域部分
     */
    private byte[] encoderResponsePacklet() {
        ByteBuf repDataBuf = ByteBufUtil.directBuffer(4);
        try {
            repDataBuf.writeIntLE(DateUtil.getUTCSeconds());
            return ByteBufUtil.getBytesRemaining(repDataBuf);
        } finally {
            ByteBufUtil.safeRelease(repDataBuf);
        }
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        // Do nothing
    }

    private void cancelWarning(final String deviceId){
        try {
            TransportWarning warning = new TransportWarning();
            warning.setDeviceId(deviceId);
            warning.setWarningCode(TransportWarning.WarningCode.POWER);
            warning.setFirstTime(DateUtil.now());
            warning.setLatestTime(DateUtil.now());
            warning.setWarningCount(1);
            //上电数据推送
            warning.setWarning("终端上电");
            transportWarningKafkaSender.sendMessage(warning);

        }catch (Exception e) {
            logger.warn("上电告警恢复出错: ", e.getMessage(), e);
        }
    }
}
