package com.qhiot.transport.codec.decoder.v2;

import com.qhiot.transport.codec.ChainContext;
import com.qhiot.transport.codec.DecoderChain;
import com.qhiot.transport.codec.bean.PacketV2;
import com.qhiot.transport.codec.decoder.BaseV2Decoder;
import com.qhiot.transport.codec.exception.DecodeException;
import com.qhiot.transport.codec.processor.HistoryDataProcessor;
import com.qhiot.transport.codec.util.Command;
import com.qhiot.transport.message.bean.History;
import com.qhiot.transport.message.bean.HistoryBuilder;
import com.qhiot.transport.message.bean.HistoryCount;
import com.qhiot.transport.message.bean.HistoryCountBuilder;
import com.qhiot.transport.message.response.HistoryCountResponse;
import com.qhiot.transport.message.response.HistoryDeleteResponse;
import com.qhiot.transport.message.response.HistoryReadResponse;
import com.qhiot.transport.message.response.ResourceConfigReadFailResponse;
import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;
import io.netty.util.Attribute;
import io.netty.util.AttributeKey;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

/**
 * 历史数据
 *
 * @author 金建强(ptma@163.com)
 * @version 2015-12-17 16:19
 */
public class HistoryDataDecoder extends BaseV2Decoder {

    @Autowired
    private HistoryDataProcessor historyDataProcessor;

    public static final AttributeKey<Long> HISTORY_TIMESTAMP                  = AttributeKey.valueOf("HISTORY_TIMESTAMP");

    public static final AttributeKey<Integer> HISTORY_UPLOAD_COUNT                  = AttributeKey.valueOf("HISTORY_UPLOAD_COUNT");

    @Value("${transport.codec.block.upload.count:5}")
    private int uploadCountBlock;

    @Value("${transport.codec.block.time.quantum:2000}")
    private long timeQuantumBlock;

    @Override
    public byte[] decode(Channel channel, ByteBuf commandDataBuf, ChainContext context, DecoderChain chain) {
        commandDataBuf.markReaderIndex();
        try {
            final PacketV2 packetV2 = (PacketV2) context.get(ChainContext.PACKET_V2);
            if (packetV2 == null
                || (packetV2.getCommand() != Command.HISTORY_DATA_READ
                    && packetV2.getCommand() != Command.HISTORY_DATA_DELETE
                    && packetV2.getCommand() != Command.HISTORY_DATA_COUNT
                    && packetV2.getCommand() != Command.HISTORY_DATA_UPLOAD
                    && packetV2.getCommand() != Command.DATA_COLLECT_FAIL)) {
                return chain.doDecode(channel, commandDataBuf, context);
            }

            switch (packetV2.getCommand()) {
                case Command.DATA_COLLECT_FAIL:
                    ResourceConfigReadFailResponse readFailResponse = new ResourceConfigReadFailResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                    readFailResponse.setResult(commandDataBuf.readByte());
                    super.sendTransportResponseToTopic(readFailResponse);

                    super.printAnalyzeMessageToLogTopic(packetV2, "采集数据失败");
                    break;
                case Command.HISTORY_DATA_READ:
                    List<History> historyList = new LinkedList<History>();
                    History his;
                    int dataCount = 0;
                    while (commandDataBuf.readableBytes() >= History.MIN_DATA_LENGTH) {
                        try {
                            his = HistoryBuilder.read(commandDataBuf);
                            historyList.add(his);
                            dataCount++;
                        } catch (Exception e){
                            break;
                        }
                    }

                    HistoryReadResponse readResponse = new HistoryReadResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                    readResponse.setHistoryList(historyList);
                    super.sendTransportResponseToTopic(readResponse);

                    super.printAnalyzeMessageToLogTopic(packetV2, "读取历史数据回应: {}条", historyList.size());

                    historyDataProcessor.processorUploadData(packetV2, historyList);
                    break;
                case Command.HISTORY_DATA_DELETE:
                    HistoryDeleteResponse delResponse = new HistoryDeleteResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                    super.sendTransportResponseToTopic(delResponse);

                    super.printAnalyzeMessageToLogTopic(packetV2, "删除历史数据回应");
                    break;
                case Command.HISTORY_DATA_COUNT:
                    HistoryCount historyCount = HistoryCountBuilder.read(commandDataBuf);
                    HistoryCountResponse cntResponse = new HistoryCountResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                    cntResponse.setHistoryCount(historyCount);
                    super.sendTransportResponseToTopic(cntResponse);

                    super.printAnalyzeMessageToLogTopic(packetV2, "读取历史数据总数回应");
                    break;
                case Command.HISTORY_DATA_UPLOAD:
                    Attribute<Long> timeAttr = channel.attr(HISTORY_TIMESTAMP);
                    Attribute<Integer> uploadCountAttr = channel.attr(HISTORY_UPLOAD_COUNT);

                    Integer uploadCount = uploadCountAttr.get();
                    if (uploadCount == null) {
                        uploadCount = 0;
                    }
                    uploadCount++;

                    Long lastTime = timeAttr.get();
                    Long currentTime = System.currentTimeMillis();
                    // 连续传输 5 次以内的不控制频率
                    if (uploadCount > uploadCountBlock && lastTime != null) {
                        if (currentTime - lastTime < timeQuantumBlock) {
                            return null;
                        }
                    }
                    uploadCountAttr.set(uploadCount);
                    timeAttr.set(currentTime);

                    // 1、为了保证响应速度， 先回应
                    byte[] responseData = new byte[0];
                    writeResponseV2(channel, packetV2, responseData);

                    // 2、然后再处理数据
                    History history;
                    int uploadDataCount = 0;
                    List<History> histories = new ArrayList<History>();
                    while (commandDataBuf.readableBytes() >= (History.MIN_DATA_LENGTH+1)) {
                        try {
                            short singleDataLength = commandDataBuf.readUnsignedByte();
                            history = HistoryBuilder.read(commandDataBuf);
                            histories.add(history);
                            uploadDataCount++;
                        } catch (Exception e){
                            break;
                        }
                    }

                    super.printAnalyzeMessageToLogTopic(packetV2, "上传历史数据: {}条", uploadDataCount);
                    historyDataProcessor.processorUploadData(packetV2, histories);

                    // 3、由于 1 中已经回应， 所以不必重复回应
                    return null;
                default:
                    return chain.doDecode(channel, commandDataBuf, context);
            }

            return null;
        } catch (DecodeException e) {
            logger.info(e.getMessage(), e);
            commandDataBuf.resetReaderIndex();
            return chain.doDecode(channel, commandDataBuf, context);
        }
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        // Do nothing
    }

}
