package com.qhiot.transport.codec.decoder.v2;

import com.qhiot.transport.codec.ChainContext;
import com.qhiot.transport.codec.DecoderChain;
import com.qhiot.transport.codec.bean.PacketV2;
import com.qhiot.transport.codec.decoder.BaseV2Decoder;
import com.qhiot.transport.codec.exception.DecodeException;
import com.qhiot.transport.codec.util.Command;
import com.qhiot.transport.message.bean.InputConfig;
import com.qhiot.transport.message.bean.InputConfigBuilder;
import com.qhiot.transport.message.response.io.*;
import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;

import java.util.ArrayList;
import java.util.List;

/**
 * 输入开关量控制配置指令解析
 *
 * @author 金建强(ptma@163.com)
 * @version 2016 -07-01 02:37:09
 */
public class InputConfigDecoder extends BaseV2Decoder {

    @Override
    public byte[] decode(Channel channel, ByteBuf commandDataBuf, ChainContext context, DecoderChain chain) {
        commandDataBuf.markReaderIndex();
        try {
            final PacketV2 packetV2 = (PacketV2) context.get(ChainContext.PACKET_V2);
            if (packetV2 == null
                || (packetV2.getCommand() != Command.SWITCH_INPUT_CONFIG_WRITE
                    && packetV2.getCommand() != Command.SWITCH_INPUT_CONFIG_READ
                    && packetV2.getCommand() != Command.SWITCH_INPUT_CONFIG_DEL
                    && packetV2.getCommand() != Command.SWITCH_INPUT_CONFIG_DEL_ALL && packetV2.getCommand() != Command.SWITCH_INPUT_CONFIG_READ_ALL)) {
                return chain.doDecode(channel, commandDataBuf, context);
            }

            switch (packetV2.getCommand()) {
                case Command.SWITCH_INPUT_CONFIG_WRITE: // 写输入开关量配置
                    InputConfigWriteResponse response = new InputConfigWriteResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                    sendTransportResponseToTopic(response);

                    printAnalyzeMessageToLogTopic(packetV2, "写输入开关量配置回应");
                    break;
                case Command.SWITCH_INPUT_CONFIG_READ: // 读输入开关量配置
                    InputConfig config = InputConfigBuilder.read(commandDataBuf);

                    InputConfigReadResponse readResponse = new InputConfigReadResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                    readResponse.setConfig(config);
                    sendTransportResponseToTopic(readResponse);

                    printAnalyzeMessageToLogTopic(packetV2, "读输入开关量配置回应");
                    break;
                case Command.SWITCH_INPUT_CONFIG_READ_ALL: // 读全部输入开关量配置
                    List<InputConfig> configItems = new ArrayList<InputConfig>();
                    InputConfig configItem;
                    int configItemCount = packetV2.getCommandDataLength()/InputConfig.DATA_LENGTH;
                    for (int i = 0; i < configItemCount; i++) {
                        configItem = InputConfigBuilder.read(commandDataBuf);
                        configItems.add(configItem);
                    }

                    InputConfigReadAllResponse readAllResponse = new InputConfigReadAllResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                    readAllResponse.setConfigs(configItems);
                    sendTransportResponseToTopic(readAllResponse);

                    printAnalyzeMessageToLogTopic(packetV2, "读输入开关量全部配置回应");
                    break;
                case Command.SWITCH_INPUT_CONFIG_DEL: // 删除输入开关量配置
                    InputConfigDeleteResponse delResponse = new InputConfigDeleteResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                    sendTransportResponseToTopic(delResponse);

                    printAnalyzeMessageToLogTopic(packetV2, "删除输入开关量配置回应");
                    break;
                case Command.SWITCH_INPUT_CONFIG_DEL_ALL: // 删除输入开关量全部配置
                    InputConfigDeleteAllResponse delAllResponse = new InputConfigDeleteAllResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                    sendTransportResponseToTopic(delAllResponse);

                    printAnalyzeMessageToLogTopic(packetV2, "删除输入开关量全部配置回应");
                    break;
                default:
                    return chain.doDecode(channel, commandDataBuf, context);
            }

            return null;
        } catch (DecodeException e) {
            logger.info(e.getMessage(), e);
            commandDataBuf.resetReaderIndex();
            return chain.doDecode(channel, commandDataBuf, context);
        }
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        // Do nothing
    }

}
