package com.qhiot.transport.codec.decoder.v2;

import com.qhiot.transport.codec.ChainContext;
import com.qhiot.transport.codec.DecoderChain;
import com.qhiot.transport.codec.bean.PacketV2;
import com.qhiot.transport.codec.decoder.BaseV2Decoder;
import com.qhiot.transport.codec.exception.DecodeException;
import com.qhiot.transport.codec.util.Command;
import com.qhiot.transport.message.bean.OutputConfig;
import com.qhiot.transport.message.bean.OutputConfigBuilder;
import com.qhiot.transport.message.response.io.*;
import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;

import java.util.ArrayList;
import java.util.List;

/**
 * 输出开关量控制配置指令解析
 */
public class OutputConfigDecoder extends BaseV2Decoder {

    @Override
    public byte[] decode(Channel channel, ByteBuf commandDataBuf, ChainContext context, DecoderChain chain) {
        commandDataBuf.markReaderIndex();
        try {
            final PacketV2 packetV2 = (PacketV2) context.get(ChainContext.PACKET_V2);
            if (packetV2 == null
                || (packetV2.getCommand() != Command.SWITCH_OUTPUT_CONFIG_WRITE
                    && packetV2.getCommand() != Command.SWITCH_OUTPUT_CONFIG_READ
                    && packetV2.getCommand() != Command.SWITCH_OUTPUT_CONFIG_DEL
                    && packetV2.getCommand() != Command.SWITCH_OUTPUT_CONFIG_DEL_ALL
                    && packetV2.getCommand() != Command.SWITCH_OUTPUT_CONFIG_READ_ALL

                    && packetV2.getCommand() != Command.SWITCH_OUTPUT_OPEN && packetV2.getCommand() != Command.SWITCH_OUTPUT_CLOSE)) {
                return chain.doDecode(channel, commandDataBuf, context);
            }

            switch (packetV2.getCommand()) {
                case Command.SWITCH_OUTPUT_CONFIG_WRITE: // 写输出开关量配置
                    OutputConfigWriteResponse response = new OutputConfigWriteResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                    sendTransportResponseToTopic(response);

                    printAnalyzeMessageToLogTopic(packetV2, "写输出开关量配置回应");
                    break;
                case Command.SWITCH_OUTPUT_CONFIG_READ: // 读输出开关量配置
                    OutputConfig config = OutputConfigBuilder.read(commandDataBuf);

                    OutputConfigReadResponse readResponse = new OutputConfigReadResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                    readResponse.setConfig(config);
                    sendTransportResponseToTopic(readResponse);

                    printAnalyzeMessageToLogTopic(packetV2, "读输出开关量配置回应");
                    break;
                case Command.SWITCH_OUTPUT_CONFIG_READ_ALL: // 读全部输出开关量配置
                    List<OutputConfig> configItems = new ArrayList<OutputConfig>();
                    OutputConfig configItem;
                    for (int i = 0; i < OutputConfig.DATA_COUNT; i++) {
                        configItem = OutputConfigBuilder.read(commandDataBuf);
                        configItems.add(configItem);
                    }

                    OutputConfigReadAllResponse readAllResponse = new OutputConfigReadAllResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                    readAllResponse.setConfigs(configItems);
                    sendTransportResponseToTopic(readAllResponse);

                    printAnalyzeMessageToLogTopic(packetV2, "读输出开关量全部配置回应");
                    break;
                case Command.SWITCH_OUTPUT_CONFIG_DEL: // 删除输出开关量配置
                    OutputConfigDeleteResponse delResponse = new OutputConfigDeleteResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                    sendTransportResponseToTopic(delResponse);

                    printAnalyzeMessageToLogTopic(packetV2, "删除输出开关量配置回应");
                    break;
                case Command.SWITCH_OUTPUT_CONFIG_DEL_ALL: // 删除输出开关量全部配置
                    OutputConfigDeleteAllResponse delAllResponse = new OutputConfigDeleteAllResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                    sendTransportResponseToTopic(delAllResponse);

                    printAnalyzeMessageToLogTopic(packetV2, "删除输出开关量全部配置回应");
                    break;

                case Command.SWITCH_OUTPUT_OPEN: // 输出开关量手动打开
                    OutputOpenResponse openResponse = new OutputOpenResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                    sendTransportResponseToTopic(openResponse);

                    printAnalyzeMessageToLogTopic(packetV2, "输出开关量手动打开回应");
                    break;
                case Command.SWITCH_OUTPUT_CLOSE: // 输出开关量手动关闭
                    OutputCloseResponse closeResponse = new OutputCloseResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                    sendTransportResponseToTopic(closeResponse);

                    printAnalyzeMessageToLogTopic(packetV2, "输出开关量手动关闭回应");
                    break;

                default:
                    return chain.doDecode(channel, commandDataBuf, context);
            }

            return null;
        } catch (DecodeException e) {
            logger.info(e.getMessage(), e);
            commandDataBuf.resetReaderIndex();
            return chain.doDecode(channel, commandDataBuf, context);
        }
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        // Do nothing
    }
}
