package com.qhiot.transport.codec.decoder.v2;

import com.google.common.util.concurrent.ThreadFactoryBuilder;
import com.qhiot.core.exception.ServiceException;
import com.qhiot.transport.codec.ChainContext;
import com.qhiot.transport.codec.DecoderChain;
import com.qhiot.transport.codec.bean.PacketV2;
import com.qhiot.transport.codec.decoder.BaseV2Decoder;
import com.qhiot.transport.codec.exception.DecodeException;
import com.qhiot.transport.codec.util.Command;
import com.qhiot.transport.codec.util.CommandUtil;
import com.qhiot.transport.codec.util.InnerQueueManager;
import com.qhiot.transport.db.model.TransportResourceConfig;
import com.qhiot.transport.db.service.TransportResourceConfigService;
import com.qhiot.transport.db.util.BeanTransformUtil;
import com.qhiot.transport.message.bean.ResourceConfig;
import com.qhiot.transport.message.define.DeviceType;
import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.SynchronousQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

/**
 * 低功耗终端询问是否有设置信息
 *
 * @author 金建强(ptma@163.com)
 * @version 2017-03-21 13:59.
 */
public class ResourceConfigAskDecoder extends BaseV2Decoder {

    @Autowired
    private TransportResourceConfigService transportResourceConfigService;

    private static ExecutorService         executor =
                                                            new ThreadPoolExecutor(
                                                                    0,
                                                                    100,
                                                                    60L,
                                                                    TimeUnit.SECONDS,
                                                                    new SynchronousQueue<Runnable>(),
                                                                    new ThreadFactoryBuilder()
                                                                            .setNameFormat(
                                                                                    "seem-resource-config-update-%d")
                                                                            .build());

    @Override
    public byte[] decode(final Channel channel, ByteBuf commandDataBuf, ChainContext context,
                         DecoderChain chain) {
        commandDataBuf.markReaderIndex();
        try {
            final PacketV2 packetV2 = (PacketV2) context.get(ChainContext.PACKET_V2);
            if (packetV2 == null
                    || (!DeviceType.GATEWAY.equals(packetV2.getDeviceType()) && !DeviceType.IO_BOARD
                            .equals(packetV2.getDeviceType()))
                    || packetV2.getCommand() != Command.RESOURCE_CONFIG_ASK) {
                return chain.doDecode(channel, commandDataBuf, context);
            }

            printAnalyzeMessageToLogTopic(packetV2, "是否有设置信息");

            executor.submit(new Runnable() {
                @Override
                public void run() {
                    List<TransportResourceConfig> configs = null;
                    try {
                        configs = transportResourceConfigService.selectNeedsUpdate(packetV2
                                        .getDeviceId());
                    } catch (ServiceException e) {
                        logger.error(e.getMessage(), e);
                    }
                    // 回应需要更新的配置条数
                    int count = (configs == null) ? 0 : configs.size();
                    CommandUtil.sendCommand(channel, DeviceType.GATEWAY, packetV2.getDeviceId(),
                            Command.RESOURCE_CONFIG_ASK, new byte[] {(byte) count});

                    if (configs != null && !configs.isEmpty()) {
                        // 逐条下发配置
                        for (TransportResourceConfig config : configs) {
                            InnerQueueManager.InnerQueue queue =
                                    InnerQueueManager.getAndClearQueue("SENSOR_CONFIG_WRITE_{}",
                                            config.getDeviceId());
                            ResourceConfig resourceConfig = BeanTransformUtil.transform(config);
                            Object response =
                                    sendRequestAndWait(queue, channel, packetV2.getDeviceId(),
                                            config.getConfigIndex(), resourceConfig);
                            if (response == null) {
                                continue;
                            }
                            config.setNeedsUpdate((short) 0);
                            try {
                                transportResourceConfigService.updateSelective(config);
                            } catch (ServiceException e) {
                                logger.error(e.getMessage(), e);
                            }
                        }
                    }
                }
            });

            return null;
        } catch (DecodeException e) {
            logger.info(e.getMessage(), e);
            commandDataBuf.resetReaderIndex();
            return chain.doDecode(channel, commandDataBuf, context);
        }
    }

    /**
     * 尝试发送配置， 重试3次
     *
     * @param queue
     * @param channel
     * @param gatewayId
     * @param index
     * @param config
     * @return
     */
    protected Object sendRequestAndWait(InnerQueueManager.InnerQueue queue, Channel channel,
                                        String gatewayId, int index, ResourceConfig config) {
        Object response = null;
        try {
            int tryCount = 0;
            while (tryCount < 3) {
                CommandUtil.resourceConfigWrite(channel, gatewayId, index, config);
                response = queue.poll(4, TimeUnit.SECONDS);
                if (response != null) {
                    return response;
                }
                tryCount++;
            }
        } catch (Exception e) {

        }
        return response;
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        // Do nothing
    }
}
