package com.qhiot.transport.codec.decoder.v2;

import com.qhiot.core.util.JsonUtil;
import com.qhiot.transport.codec.ChainContext;
import com.qhiot.transport.codec.DecoderChain;
import com.qhiot.transport.codec.bean.PacketV2;
import com.qhiot.transport.codec.decoder.BaseV2Decoder;
import com.qhiot.transport.codec.exception.DecodeException;
import com.qhiot.transport.codec.util.ByteBufUtil;
import com.qhiot.transport.codec.util.Command;
import com.qhiot.transport.codec.util.InnerQueueManager;
import com.qhiot.transport.message.bean.ResourceConfig;
import com.qhiot.transport.message.bean.ResourceConfigBuilder;
import com.qhiot.transport.message.bean.ResourceTest;
import com.qhiot.transport.message.define.DeviceType;
import com.qhiot.transport.message.response.*;
import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;

import java.util.ArrayList;
import java.util.List;

/**
 * 资源配置解析
 *
 * @author 金建强(ptma@163.com)
 * @version 2017-03-21 13:59.
 */
public class ResourceConfigDecoder extends BaseV2Decoder {

    @Override
    public byte[] decode(Channel channel, ByteBuf commandDataBuf, ChainContext context, DecoderChain chain) {
        commandDataBuf.markReaderIndex();
        try {
            final PacketV2 packetV2 = (PacketV2) context.get(ChainContext.PACKET_V2);
            if (packetV2 == null
                || !DeviceType.GATEWAY.equals(packetV2.getDeviceType())
                || (packetV2.getCommand() != Command.RESOURCE_CONFIG_READ_TEN
                    && packetV2.getCommand() != Command.RESOURCE_CONFIG_DELETE_TEN
                    && packetV2.getCommand() != Command.RESOURCE_CONFIG_WRITE_TEN
                    && packetV2.getCommand() != Command.RESOURCE_CONFIG_READ_ONE
                    && packetV2.getCommand() != Command.RESOURCE_CONFIG_DELETE_ONE
                    && packetV2.getCommand() != Command.RESOURCE_CONFIG_WRITE_ONE
                    && packetV2.getCommand() != Command.RESOURCE_CONFIG_DELETE_ALL
                    && packetV2.getCommand() != Command.RESOURCE_CONFIG_TEST_DATA)) {
                return chain.doDecode(channel, commandDataBuf, context);
            }

            switch (packetV2.getCommand()) {
                case Command.RESOURCE_CONFIG_READ_TEN: // 读10条资源配置
                    List<ResourceConfig> configItems = new ArrayList<ResourceConfig>();
                    ResourceConfig configItem;
                    for (int i = 0; i < 10; i++) {
                        configItem = ResourceConfigBuilder.read(commandDataBuf);
                        configItems.add(configItem);
                    }

                    ResourceConfigReadTenResponse readTenResponse = new ResourceConfigReadTenResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                    readTenResponse.setConfigs(configItems);
                    sendTransportResponseToTopic(readTenResponse);

                    printAnalyzeMessageToLogTopic(packetV2, "读取10条资源配置回应");
                    break;

                case Command.RESOURCE_CONFIG_DELETE_TEN: // 删除10条资源配置
                    ResourceConfigDeleteTenResponse delTenResponse = new ResourceConfigDeleteTenResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                    sendTransportResponseToTopic(delTenResponse);

                    printAnalyzeMessageToLogTopic(packetV2, "删除10条资源配置回应");
                    break;

                case Command.RESOURCE_CONFIG_WRITE_TEN: // 写10条资源配置
                    ResourceConfigWriteTenResponse writeTenResponse = new ResourceConfigWriteTenResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                    sendTransportResponseToTopic(writeTenResponse);

                    printAnalyzeMessageToLogTopic(packetV2, "写入10条资源配置回应");
                    break;
                case Command.RESOURCE_CONFIG_READ_ONE: // 读1条资源配置
                    ResourceConfig config = ResourceConfigBuilder.read(commandDataBuf);
                    ResourceConfigReadResponse readResponse = new ResourceConfigReadResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                    readResponse.setConfig(config);
                    sendTransportResponseToTopic(readResponse);

                    printAnalyzeMessageToLogTopic(packetV2, "读取1条资源配置回应");
                    break;

                case Command.RESOURCE_CONFIG_DELETE_ONE: // 删除1条资源配置
                    ResourceConfigDeleteResponse delResponse = new ResourceConfigDeleteResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                    sendTransportResponseToTopic(delResponse);

                    printAnalyzeMessageToLogTopic(packetV2, "删除1条资源配置回应");
                    break;

                case Command.RESOURCE_CONFIG_WRITE_ONE: // 写1条资源配置
                    ResourceConfigWriteResponse writeResponse = new ResourceConfigWriteResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                    writeResponse.setResult(commandDataBuf.readByte());

                    InnerQueueManager.getQueue("SENSOR_CONFIG_WRITE_{}", writeResponse.getDeviceId()).offer(Byte.valueOf(writeResponse.getResult()));


                    sendTransportResponseToTopic(writeResponse);

                    printAnalyzeMessageToLogTopic(packetV2, "写入1条资源配置回应");
                    break;

                case Command.RESOURCE_CONFIG_TEST_DATA: // 根据资源配置透传数据
                    ResourceTest resourceTest = new ResourceTest();
                    resourceTest.setData(ByteBufUtil.getBytesRemaining(commandDataBuf));

                    ResourceConfigTestDataResponse testDataResponse = new ResourceConfigTestDataResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                    testDataResponse.setData(resourceTest);
                    sendTransportResponseToTopic(testDataResponse);

                    printAnalyzeMessageToLogTopic(packetV2, "资源配置测试回应: {}", JsonUtil.toJson(resourceTest));
                    break;

                default:
                    return chain.doDecode(channel, commandDataBuf, context);
            }

            return null;
        } catch (DecodeException e) {
            logger.info(e.getMessage(), e);
            commandDataBuf.resetReaderIndex();
            return chain.doDecode(channel, commandDataBuf, context);
        }
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        // Do nothing
    }

}
