package com.qhiot.transport.codec.decoder.v2;

import com.qhiot.transport.codec.ChainContext;
import com.qhiot.transport.codec.DecoderChain;
import com.qhiot.transport.codec.bean.PacketV2;
import com.qhiot.transport.codec.decoder.BaseV2Decoder;
import com.qhiot.transport.codec.exception.DecodeException;
import com.qhiot.transport.codec.util.Command;
import com.qhiot.transport.message.bean.Rs485Config;
import com.qhiot.transport.message.bean.Rs485ConfigBuilder;
import com.qhiot.transport.message.define.DeviceType;
import com.qhiot.transport.message.response.*;
import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;

import java.util.ArrayList;
import java.util.List;

/**
 * 485指令配置解析
 * <p>
 * User: 金建强(ptma@163.com)
 * </p>
 * <p>
 * Date: 2015-12-17 16:19.
 * </p>
 */
public class Rs485ConfigDecoder extends BaseV2Decoder {

    @Override
    public byte[] decode(Channel channel, ByteBuf commandDataBuf, ChainContext context, DecoderChain chain) {
        commandDataBuf.markReaderIndex();
        try {
            final PacketV2 packetV2 = (PacketV2) context.get(ChainContext.PACKET_V2);
            if (packetV2 == null
                || !DeviceType.GATEWAY.equals(packetV2.getDeviceType())
                || (packetV2.getCommand() != Command.RS485_CONFIG_WRITE
                    && packetV2.getCommand() != Command.RS485_CONFIG_READ
                    && packetV2.getCommand() != Command.RS485_CONFIG_DEL
                    && packetV2.getCommand() != Command.RS485_CONFIG_DEL_ALL
                    && packetV2.getCommand() != Command.RS485_CONFIG_READ_ALL )) {
                return chain.doDecode(channel, commandDataBuf, context);
            }

            switch (packetV2.getCommand()) {
                case Command.RS485_CONFIG_WRITE: // 写485配置
                    Rs485ConfigWriteResponse response = new Rs485ConfigWriteResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                    sendTransportResponseToTopic(response);

                    printAnalyzeMessageToLogTopic(packetV2, "写入485配置回应");
                    break;
                case Command.RS485_CONFIG_READ: // 读485配置
                    Rs485Config config = Rs485ConfigBuilder.read(commandDataBuf);

                    Rs485ConfigReadResponse readResponse = new Rs485ConfigReadResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                    readResponse.setConfig(config);
                    sendTransportResponseToTopic(readResponse);

                    printAnalyzeMessageToLogTopic(packetV2, "读取485配置回应");
                    break;
                case Command.RS485_CONFIG_READ_ALL: // 读全部485配置
                    List<Rs485Config> configItems = new ArrayList<Rs485Config>();
                    Rs485Config configItem;
                    for (int i = 0; i < Rs485Config.DATA_COUNT; i++) {
                        configItem = Rs485ConfigBuilder.read(commandDataBuf);
                        configItems.add(configItem);
                    }

                    Rs485ConfigReadAllResponse readAllResponse = new Rs485ConfigReadAllResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                    readAllResponse.setConfigs(configItems);
                    sendTransportResponseToTopic(readAllResponse);

                    printAnalyzeMessageToLogTopic(packetV2, "读取485全部配置回应");
                    break;
                case Command.RS485_CONFIG_DEL: // 删除485配置
                    Rs485ConfigDeleteResponse delResponse = new Rs485ConfigDeleteResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                    sendTransportResponseToTopic(delResponse);

                    printAnalyzeMessageToLogTopic(packetV2, "删除485配置回应");
                    break;
                case Command.RS485_CONFIG_DEL_ALL: // 删除485全部配置
                    Rs485ConfigDeleteAllResponse delAllResponse = new Rs485ConfigDeleteAllResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                    sendTransportResponseToTopic(delAllResponse);

                    printAnalyzeMessageToLogTopic(packetV2, "删除485全部配置回应");
                    break;
                default:
                    return chain.doDecode(channel, commandDataBuf, context);
            }

            return null;
        } catch (DecodeException e) {
            logger.info(e.getMessage(), e);
            commandDataBuf.resetReaderIndex();
            return chain.doDecode(channel, commandDataBuf, context);
        }
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        // Do nothing
    }

}
