package com.qhiot.transport.codec.decoder.v2;

import com.qhiot.transport.codec.ChainContext;
import com.qhiot.transport.codec.DecoderChain;
import com.qhiot.transport.codec.bean.PacketV2;
import com.qhiot.transport.codec.decoder.BaseV2Decoder;
import com.qhiot.transport.codec.exception.DecodeException;
import com.qhiot.transport.codec.util.ByteBufUtil;
import com.qhiot.transport.codec.util.Command;
import com.qhiot.transport.codec.util.InnerQueueManager;
import com.qhiot.transport.message.bean.WarningConfig;
import com.qhiot.transport.message.bean.WarningConfigBuilder;
import com.qhiot.transport.message.define.DeviceType;
import com.qhiot.transport.message.response.*;
import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;

import java.util.ArrayList;
import java.util.List;

/**
 * 新数据采集配置 - 报警配置
 *
 * @author 金建强(ptma@163.com)
 * @version 2017-03-21 13:59.
 */
public class WarningConfigDecoder extends BaseV2Decoder {

    public static final byte WRITE_ONE  = (byte) 0x00;
    public static final byte READ_ONE   = (byte) 0x01;
    public static final byte DELETE_ONE = (byte) 0x02;
    public static final byte DELETE_ALL = (byte) 0x03;
    public static final byte READ_TEN   = (byte) 0x04;

    @Override
    public byte[] decode(Channel channel, ByteBuf commandDataBuf, ChainContext context, DecoderChain chain) {
        commandDataBuf.markReaderIndex();
        try {
            final PacketV2 packetV2 = (PacketV2) context.get(ChainContext.PACKET_V2);
            if (packetV2 == null
                || !DeviceType.GATEWAY.equals(packetV2.getDeviceType())
                || (packetV2.getCommand() != Command.WARNING_CONFIG)) {
                return chain.doDecode(channel, commandDataBuf, context);
            }
            byte subCommand = commandDataBuf.readByte();
            short configIndex = commandDataBuf.readShortLE();

            ByteBuf dataBuf = ByteBufUtil.directBuffer();
            try {
                dataBuf.writeBytes(ByteBufUtil.getBytesRemaining(commandDataBuf));
                switch (subCommand) {
                    case READ_TEN: // 读10条配置
                        List<WarningConfig> configItems = new ArrayList<WarningConfig>();
                        WarningConfig configItem;
                        for (int i = 0; i < 10; i++) {
                            configItem = WarningConfigBuilder.read(dataBuf);
                            if (!configItem.isAllFieldZero()) {
                                configItems.add(configItem);
                            }
                        }

                        WarningConfigReadTenResponse readTenResponse = new WarningConfigReadTenResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                        readTenResponse.setConfigs(configItems);
                        sendTransportResponseToTopic(readTenResponse);

                        printAnalyzeMessageToLogTopic(packetV2, "读取10条报警配置回应");
                        break;

                    case READ_ONE: // 读1条配置
                        WarningConfig config = WarningConfigBuilder.read(dataBuf);
                        WarningConfigReadResponse readResponse = new WarningConfigReadResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                        readResponse.setConfig(config);
                        sendTransportResponseToTopic(readResponse);

                        printAnalyzeMessageToLogTopic(packetV2, "读取1条报警配置回应");
                        break;

                    case WRITE_ONE: // 写1条配置
                        WarningConfigWriteResponse writeResponse = new WarningConfigWriteResponse(packetV2.getDeviceType(), packetV2.getDeviceId());

                        InnerQueueManager.getQueue("WARNING_CONFIG_WRITE_{}", writeResponse.getDeviceId()).offer(Boolean.TRUE);

                        sendTransportResponseToTopic(writeResponse);

                        printAnalyzeMessageToLogTopic(packetV2, "写入1条报警配置回应");
                        break;

                    case DELETE_ONE: // 删除1条配置
                        WarningConfigDeleteResponse delResponse = new WarningConfigDeleteResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                        sendTransportResponseToTopic(delResponse);

                        printAnalyzeMessageToLogTopic(packetV2, "删除1条报警配置回应");
                        break;

                    case DELETE_ALL: // 删除全部配置
                        WarningConfigDeleteAllResponse delAllResponse = new WarningConfigDeleteAllResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                        sendTransportResponseToTopic(delAllResponse);

                        printAnalyzeMessageToLogTopic(packetV2, "删除全部报警配置回应");
                        break;

                    default:
                        return chain.doDecode(channel, commandDataBuf, context);
                }
            } finally {
                ByteBufUtil.safeRelease(dataBuf);
            }

            return null;
        } catch (DecodeException e) {
            logger.warn(e.getMessage(), e);
            commandDataBuf.resetReaderIndex();
            return chain.doDecode(channel, commandDataBuf, context);
        }
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        // Do nothing
    }

}
