package com.qhiot.transport.codec.processor;

import com.qhiot.core.exception.ServiceException;
import com.qhiot.core.util.*;
import com.qhiot.transport.codec.bean.PacketV2;
import com.qhiot.transport.db.dto.TransportResourceMonitoringIndexDTO;
import com.qhiot.transport.db.model.TransportEquipmentMonitoring;
import com.qhiot.transport.db.model.TransportResourceConfig;
import com.qhiot.transport.db.model.TransportWarning;
import com.qhiot.transport.db.model.TransportWarning.WarningCode;
import com.qhiot.transport.db.model.TransportWarningConfig;
import com.qhiot.transport.db.service.*;
import com.qhiot.transport.influxdb.bean.PacketV2Log;
import com.qhiot.transport.influxdb.model.DecodeLogInflux;
import com.qhiot.transport.influxdb.model.TransportWarningInflux;
import com.qhiot.transport.influxdb.service.DecodeLogInfluxService;
import com.qhiot.transport.influxdb.service.TransportWarningInfluxService;
import com.qhiot.transport.jms.*;
import com.qhiot.transport.jms.kafka.CodecLogKafkaSender;
import com.qhiot.transport.jms.kafka.MonitoringHistoryDataKafkaSender;
import com.qhiot.transport.jms.kafka.TransportWarningKafkaSender;
import com.qhiot.transport.jms.kafka.WarningDataKafkaSender;
import com.qhiot.transport.message.CodecLogMessage;
import com.qhiot.transport.message.bean.History;
import com.qhiot.transport.message.data.MonitoringHistoryData;
import com.qhiot.transport.message.data.WarningData;
import groovy.lang.Binding;
import groovy.lang.GroovyShell;
import groovy.lang.Script;
import org.codehaus.groovy.control.CompilationFailedException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.kafka.core.KafkaTemplate;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.nio.ByteOrder;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ExecutorService;

/**
 * 历史数据上传处理器, 分为两个子任务<br/>
 * 1. 保存历史数据原始记录到历史记录表<br/>
 * 2. 按照测点配置解析各测点数据
 *
 *  20200605改造日志 - by 罗军
 *  消息队列中，只要推送对象和推送方式一致，则归并到一个收发队列
 *
 */
@Component
public class HistoryDataProcessor {

    protected final Logger                      LOGGER                         = LoggerFactory.getLogger("transport.codec." + getClass().getSimpleName());

    private static final String                 DATA_UPLOAD_TEMPLATE           = "监测数据: {}={}{}, 时间={}, HEX=[{}]";
    private static final String                 DATA_UPLOAD_EXCEPTION_TEMPLATE = "监测数据解析出错: 寄存器类型={}, 寄存器地址={}, 解码方式={}, 数据=[{}], 错误={}";
    private static final String                 TIMESTAMP_WARNING_TEMPLATE     = "监测数据时间戳异常: 时间={}, 系统时间={}";
    private static final String                 NO_CONFIG_WARNING_TEMPLATE     = "未找到测点配置: deviceId={}, registerType={}, storeAddress={}";
    private static final String                 DATA_BOUND_WARNING_TEMPLATE    = "测点\"{}\"监测值超出{}限, 当前值为 {}";

//    @Autowired
//    private CodecLogSender                      codecLogSender;

//    @Autowired
//    private TransportDataQueueSender transportDataQueueSender;
//    @Autowired
//    private TransportDataTopicSender transportDataTopicSender;
//    @Autowired
//    private InsightWarningQueueSender insightWarningQueueSender;
//    @Autowired
//    private InsightDataQueueSender insightDataQueueSender;
//    @Autowired
//    private ApiPushDataQueueSender apiPushDataQueueSender;

    /**---kafka---*/
    @Autowired
    private CodecLogKafkaSender codecLogSender;
    @Autowired
    private MonitoringHistoryDataKafkaSender monitoringHistoryDataKafkaSender;
    @Autowired
    private WarningDataKafkaSender warningDataKafkaSender;
    @Autowired
    private TransportWarningKafkaSender transportWarningKafkaSender;
    /**------*/


    @Autowired
    private TransportResourceConfigService      transportResourceConfigService;
    @Autowired
    private TransportEquipmentConfigService     transportEquipmentConfigService;
    @Autowired
    private TransportEquipmentMonitoringService transportEquipmentMonitoringService;
    @Autowired
    private TransportResourceMonitoringService  transportResourceMonitoringService;
    @Autowired
    private TransportWarningConfigService       transportWarningConfigService;

    @Autowired
    private TransportWarningService             transportWarningService;

    @Autowired
    private TransportWarningInfluxService transportWarningInfluxService;

    @Autowired
    private DecodeLogInfluxService decodeLogInfluxService;

    @Autowired
    private HistoryDataSaveProcessor historyDataSaveProcessor;

    @Value("${app.node:0}")
    private int node;
    private ExecutorService                     executor                       = ThreadUtil.newCachedThreadPool(10, 1000, "seem-history-data-processor-%d");

    /** 公式脚本缓存 */
    private Map<String, Script>                 formulaScriptMap               = new ConcurrentHashMap<String, Script>();

    public void processorUploadData(final PacketV2 packetV2, final List<History> histories) {
        executor.submit(new Runnable() {
            @Override
            public void run() {
                for (History history : histories) {
                    process(packetV2, history);

                }
            }

            private void process(final PacketV2 packetV2, final History history) {
                Date samplingTime = DateUtil.convertUTC2Date(history.getTimestamp());

                // 检测时间超过当前系统时间 10 分钟
                if (DateUtil.diffMinutes(DateUtil.now(), samplingTime) > 10) {
                    String samplingTimeStr = DateUtil.formatDateTime(samplingTime);
                    logWarn(packetV2, StringUtil.format(TIMESTAMP_WARNING_TEMPLATE, samplingTimeStr, DateUtil.nowString()));
                    return;
                }

                boolean processed;
                // 新配置的寄存器类型肯定为 0
                if (history.getRegisterType() == 0) {
                    processed = processWithNewConfig(packetV2, history);
                } else {
                    processed = processWithOldConfig(packetV2, history);
                }
                if (!processed) {
                    logWarn(packetV2, StringUtil.format(NO_CONFIG_WARNING_TEMPLATE, packetV2.getDeviceId(), history.getRegisterType(), history.getStoreAddress()));
                }
            }

            /**
             * 按照新的数据采集配置解析
             *
             * @param history
             */
            private boolean processWithNewConfig(final PacketV2 packetV2, final History history) {
                try {
                    Date samplingTime = DateUtil.convertUTC2Date(history.getTimestamp());
                    String samplingTimeStr = DateUtil.formatDateTime(samplingTime);
                    byte registerType = history.getRegisterType();
                    int storeAddress = history.getStoreAddress();

                    TransportEquipmentMonitoring config = transportEquipmentMonitoringService.selectByRegister(packetV2.getDeviceId(), registerType, storeAddress);
                    if (config == null) {
                        TransportWarningConfig warningConfig = transportWarningConfigService.selectByAddress(packetV2.getDeviceId(), storeAddress);
                        if (warningConfig != null) {
                            if (history.getData()[0] == 0x01) {
                                String warningMessage = StringUtil.format("规则报警,配置序号为 {}, 报警输出寄存器为 {}", warningConfig.getConfigIndex(), warningConfig.getWarningRegisterAddress());
                                WarningData warningData = new WarningData(packetV2.getDeviceId());
                                warningData.setFirstTime(samplingTime);
                                warningData.setLatestTime(samplingTime);
                                warningData.setWarningCount(1);
                                warningData.setWarningMessage(warningMessage);
//                                transportDataTopicSender.sendMessage(warningData);
//                                apiPushDataQueueSender.sendMessage(warningData);
                                warningDataKafkaSender.sendMessage(warningData);

                                TransportWarning warning = new TransportWarning();
                                warning.setDeviceId(packetV2.getDeviceId());
                                warning.setWarningCode(WarningCode.RULE_MATCH);
                                warning.setWarning(warningMessage);
                                warning.setWarningCount(1);
                                warning.setFirstTime(samplingTime);
                                warning.setLatestTime(samplingTime);
//                                transportWarningService.insertSelective(warning);
                                TransportWarningInflux warningInflux = new TransportWarningInflux(warning);
                                transportWarningInfluxService.insert(warningInflux);

//                                insightWarningQueueSender.sendMessage(warning);
                                transportWarningKafkaSender.sendMessage(warning);
                            }
                            return true;
                        }
                        return false;
                    }

                    byte[] data = history.getData();

                    List<TransportResourceMonitoringIndexDTO> monitoringList = transportResourceMonitoringService.selectByDeviceAndConfigIndex(packetV2.getDeviceId(), config.getConfigIndex());
                    if (monitoringList == null || monitoringList.isEmpty() || config.getDataQuantity() == null) {
                        return false;
                    }
                    // 数据数量 和 测点配置个数中取小的那一个，防止测点配置的数量大于自己数据的个数
                    int dataCount = Math.min(config.getDataQuantity(), monitoringList.size());
                    for (int j = 0; j < dataCount; j++) {
                        TransportResourceMonitoringIndexDTO monitoring = monitoringList.get(j);
                        if (monitoring.getValid().byteValue() != 1) {
                            continue;
                        }
                        /*
                         * 计算方法
                         */
                        int dataIndex = monitoring.getDataIndex();
                        BigDecimal bigDecimal = null;
                        String parsed = "";
                        switch (history.getMethod()) {
                            case 0: // Bit
                                int byteIndex = (int) Math.floor((double) dataIndex / 8);
                                if (data.length > byteIndex) {
                                    byte b0 = data[byteIndex];
                                    parsed = ByteUtil.toBinaryString(b0);
                                    int v0 = ByteUtil.getBit(b0, dataIndex % 8);
                                    bigDecimal = new BigDecimal(v0);
                                }
                                break;
                            case 1: // INT8S 8位有符号
                                if (data.length > dataIndex) {
                                    byte b1 = data[dataIndex];
                                    parsed = ByteUtil.toHexString(b1);
                                    bigDecimal = new BigDecimal(b1);
                                }
                                break;
                            case 2: // INT8U 8位无符号
                                if (data.length > dataIndex) {
                                    byte b2 = data[dataIndex];
                                    parsed = ByteUtil.toHexString(b2);
                                    bigDecimal = new BigDecimal(ByteUtil.getUnsigned(b2));
                                }
                                break;
                            case 3: // INT16S_AB
                                if (data.length >= dataIndex * 2 + 2) {
                                    byte[] b3 = ByteUtil.subBytes(data, dataIndex * 2, dataIndex * 2 + 2);
                                    parsed = ByteUtil.toHexString(b3);
                                    bigDecimal = new BigDecimal(ByteUtil.toShort(b3));
                                }
                                break;
                            case 4: // INT16S_BA
                                if (data.length >= dataIndex * 2 + 2) {
                                    byte[] b4 = ByteUtil.subBytes(data, dataIndex * 2, dataIndex * 2 + 2);
                                    parsed = ByteUtil.toHexString(b4);
                                    short v4 = ByteUtil.toShort(b4, ByteOrder.LITTLE_ENDIAN);
                                    bigDecimal = new BigDecimal(v4);
                                }
                                break;
                            case 5: // INT16U_AB
                                if (data.length >= dataIndex * 2 + 2) {
                                    byte[] b5 = ByteUtil.subBytes(data, dataIndex * 2, dataIndex * 2 + 2);
                                    parsed = ByteUtil.toHexString(b5);
                                    int v5 = ByteUtil.toUnsignedShort(b5);
                                    bigDecimal = new BigDecimal(v5);
                                }
                                break;
                            case 6: // INT16U_BA
                                if (data.length >= dataIndex * 2 + 2) {
                                    byte[] b6 = ByteUtil.subBytes(data, dataIndex * 2, dataIndex * 2 + 2);
                                    parsed = ByteUtil.toHexString(b6);
                                    int v6 = ByteUtil.toUnsignedShort(b6, ByteOrder.LITTLE_ENDIAN);
                                    bigDecimal = new BigDecimal(v6);
                                }
                                break;
                            case 7: // Long ABCD
                                if (data.length >= dataIndex * 4 + 4) {
                                    byte[] b7 = ByteUtil.subBytes(data, dataIndex * 4, dataIndex * 4 + 4);
                                    parsed = ByteUtil.toHexString(b7);
                                    int v7 = ByteUtil.toInt(b7);
                                    bigDecimal = new BigDecimal(v7);
                                }
                                break;
                            case 8: // Long CDAB
                                if (data.length >= dataIndex * 4 + 4) {
                                    byte[] b8 = ByteUtil.subBytes(data, dataIndex * 4, dataIndex * 4 + 4);
                                    parsed = ByteUtil.toHexString(b8);
                                    int v8 = ByteUtil.toInt(ByteUtil.convert(b8, "CDAB"));
                                    bigDecimal = new BigDecimal(v8);
                                }
                                break;
                            case 9: // Long BADC
                                if (data.length >= dataIndex * 4 + 4) {
                                    byte[] b9 = ByteUtil.subBytes(data, dataIndex * 4, dataIndex * 4 + 4);
                                    parsed = ByteUtil.toHexString(b9);
                                    int v9 = ByteUtil.toInt(ByteUtil.convert(b9, "BADC"));
                                    bigDecimal = new BigDecimal(v9);
                                }
                                break;
                            case 10: // Long DCBA
                                if (data.length >= dataIndex * 4 + 4) {
                                    byte[] b10 = ByteUtil.subBytes(data, dataIndex * 4, dataIndex * 4 + 4);
                                    parsed = ByteUtil.toHexString(b10);
                                    int v10 = ByteUtil.toInt(b10, ByteOrder.LITTLE_ENDIAN);
                                    bigDecimal = new BigDecimal(v10);
                                }
                                break;
                            case 11: // float ABCD
                                if (data.length >= dataIndex * 4 + 4) {
                                    byte[] b11 = ByteUtil.subBytes(data, dataIndex * 4, dataIndex * 4 + 4);
                                    parsed = ByteUtil.toHexString(b11);
                                    float v11 = ByteUtil.toFloat(b11);
                                    bigDecimal = new BigDecimal(v11);
                                }
                                break;
                            case 12: // float CDAB
                                if (data.length >= dataIndex * 4 + 4) {
                                    byte[] b12 = ByteUtil.subBytes(data, dataIndex * 4, dataIndex * 4 + 4);
                                    parsed = ByteUtil.toHexString(b12);
                                    float v12 = ByteUtil.toFloat(ByteUtil.convert(b12, "CDAB"));
                                    bigDecimal = new BigDecimal(v12);
                                }
                                break;
                            case 13: // float BADC
                                if (data.length >= dataIndex * 4 + 4) {
                                    byte[] b13 = ByteUtil.subBytes(data, dataIndex * 4, dataIndex * 4 + 4);
                                    parsed = ByteUtil.toHexString(b13);
                                    float v13 = ByteUtil.toFloat(ByteUtil.convert(b13, "BADC"));
                                    bigDecimal = new BigDecimal(v13);
                                }
                                break;
                            case 14: // float DCBA
                                if (data.length >= dataIndex * 4 + 4) {
                                    byte[] b14 = ByteUtil.subBytes(data, dataIndex * 4, dataIndex * 4 + 4);
                                    parsed = ByteUtil.toHexString(b14);
                                    float v14 = ByteUtil.toFloat(b14, ByteOrder.LITTLE_ENDIAN);
                                    bigDecimal = new BigDecimal(v14);
                                }
                                break;
                            default:
                                parsed = "";
                        }
                        if (bigDecimal != null) {
                            bigDecimal = calculateWithFormula(bigDecimal, monitoring.getFormula());
                            if (monitoring.getValuePrecision() != null && monitoring.getValuePrecision() >= 0) {
                                bigDecimal = bigDecimal.setScale(monitoring.getValuePrecision().intValue(), BigDecimal.ROUND_HALF_UP);
                            } else {
                                bigDecimal = bigDecimal.setScale(2, BigDecimal.ROUND_HALF_UP);
                            }

//                            String debugStr = StringUtil.format(DATA_UPLOAD_TEMPLATE, monitoring.getMonitoringName(), bigDecimal.toString(), monitoring.getUnit(), samplingTimeStr, parsed);
//
//                            logDebug(packetV2, debugStr);

                            MonitoringHistoryData monitoringHistoryData = new MonitoringHistoryData(packetV2.getDeviceType(), packetV2.getDeviceId());
                            monitoringHistoryData.setContextId(packetV2.getContextId());

                            monitoringHistoryData.setMonitoringId(monitoring.getMonitoringId());
                            monitoringHistoryData.setMonitoringName(monitoring.getMonitoringName());
                            monitoringHistoryData.setDataType(monitoring.getDataType());
                            monitoringHistoryData.setDataMap(monitoring.getDataMap());
                            monitoringHistoryData.setValuePrecision(monitoring.getValuePrecision());
                            monitoringHistoryData.setUnit(monitoring.getUnit());
                            monitoringHistoryData.setIcon(monitoring.getIcon());
                            monitoringHistoryData.setValue(bigDecimal.doubleValue());
                            monitoringHistoryData.setDataTime(samplingTime);
                            monitoringHistoryData.setMonitoringIndexId(monitoring.getMonitoringIndexId());

                            if (monitoringHistoryData.getMonitoringIndexId() == 6338941382618513408L && monitoringHistoryData.getValue() < 0) {
                                continue;
                            }

//                            if (transportDataTopicSender != null) {
//                                transportDataTopicSender.sendMessage(monitoringHistoryData);
//                                transportDataQueueSender.sendMessage(monitoringHistoryData);
//                            }
//                            if (insightDataQueueSender != null) {
//                                insightDataQueueSender.sendMessage(monitoringHistoryData);
//                            }
//                            if (apiPushDataQueueSender != null) {
//                                apiPushDataQueueSender.sendMessage(monitoringHistoryData);
//                            }
                            monitoringHistoryDataKafkaSender.sendMessage(monitoringHistoryData);
                            historyDataSaveProcessor.process(monitoringHistoryData);
                        }
                    }
                } catch (Exception e) {
                    String debugStr =
                            StringUtil.format(DATA_UPLOAD_EXCEPTION_TEMPLATE, history.getRegisterType(), history.getStoreAddress(), history.getMethod(), history.getDataHex(), e.getMessage());

                    logError(packetV2, debugStr, e);
                }
                return true;
            }

            /**
             * 按照旧的资源配置解析
             *
             * @param history
             */
            private boolean processWithOldConfig(final PacketV2 packetV2, final History history) {
                // 根据配置解析数据
                try {
                    Date samplingTime = DateUtil.convertUTC2Date(history.getTimestamp());
                    Date inTime = DateUtil.now();
                    byte registerType = history.getRegisterType();
                    int storeAddress = history.getStoreAddress();

                    TransportResourceConfig config = transportResourceConfigService.selectByRegister(packetV2.getDeviceId(), registerType, storeAddress);
                    if (config == null) {
                        return false;
                    }

                    byte[] data = history.getData();

                    List<TransportResourceMonitoringIndexDTO> monitoringList = transportResourceMonitoringService.selectByDeviceAndConfigIndex(packetV2.getDeviceId(), config.getConfigIndex());
                    if (monitoringList == null || monitoringList.isEmpty() || config.getDataQuantity() == null) {
                        return false;
                    }
                    // 数据数量 和 测点配置个数中取小的那一个，防止测点配置的数量大于自己数据的个数
                    int dataCount = Math.min(config.getDataQuantity(), monitoringList.size());
                    for (int j = 0; j < dataCount; j++) {
                        TransportResourceMonitoringIndexDTO monitoring = monitoringList.get(j);
                        if (monitoring.getValid().byteValue() != 1) {
                            continue;
                        }
                        /*
                         * 计算方法
                         */
                        int dataIndex = monitoring.getDataIndex();
                        BigDecimal bigDecimal = null;
                        String parsed = "";
                        switch (history.getMethod()) {
                            case 0: // Bit
                                int byteIndex = (int) Math.floor((double) dataIndex / 8);
                                if (data.length > byteIndex) {
                                    byte b0 = data[byteIndex];
                                    parsed = ByteUtil.toBinaryString(b0);
                                    int v0 = ByteUtil.getBit(b0, dataIndex % 8);
                                    bigDecimal = new BigDecimal(v0);
                                }
                                break;
                            case 1: // INT8S 8位有符号
                                if (data.length > dataIndex) {
                                    byte b1 = data[dataIndex];
                                    parsed = ByteUtil.toHexString(b1);
                                    bigDecimal = new BigDecimal(b1);
                                }
                                break;
                            case 2: // INT8U 8位无符号
                                if (data.length > dataIndex) {
                                    byte b2 = data[dataIndex];
                                    parsed = ByteUtil.toHexString(b2);
                                    bigDecimal = new BigDecimal(ByteUtil.getUnsigned(b2));
                                }
                                break;
                            case 3: // INT16S_AB
                                if (data.length >= dataIndex * 2 + 2) {
                                    byte[] b3 = ByteUtil.subBytes(data, dataIndex * 2, dataIndex * 2 + 2);
                                    parsed = ByteUtil.toHexString(b3);
                                    bigDecimal = new BigDecimal(ByteUtil.toShort(b3));
                                }
                                break;
                            case 4: // INT16S_BA
                                if (data.length >= dataIndex * 2 + 2) {
                                    byte[] b4 = ByteUtil.subBytes(data, dataIndex * 2, dataIndex * 2 + 2);
                                    parsed = ByteUtil.toHexString(b4);
                                    short v4 = ByteUtil.toShort(b4, ByteOrder.LITTLE_ENDIAN);
                                    bigDecimal = new BigDecimal(v4);
                                }
                                break;
                            case 5: // INT16U_AB
                                if (data.length >= dataIndex * 2 + 2) {
                                    byte[] b5 = ByteUtil.subBytes(data, dataIndex * 2, dataIndex * 2 + 2);
                                    parsed = ByteUtil.toHexString(b5);
                                    int v5 = ByteUtil.toUnsignedShort(b5);
                                    bigDecimal = new BigDecimal(v5);
                                }
                                break;
                            case 6: // INT16U_BA
                                if (data.length >= dataIndex * 2 + 2) {
                                    byte[] b6 = ByteUtil.subBytes(data, dataIndex * 2, dataIndex * 2 + 2);
                                    parsed = ByteUtil.toHexString(b6);
                                    int v6 = ByteUtil.toUnsignedShort(b6, ByteOrder.LITTLE_ENDIAN);
                                    bigDecimal = new BigDecimal(v6);
                                }
                                break;
                            case 7: // Long ABCD
                                if (data.length >= dataIndex * 4 + 4) {
                                    byte[] b7 = ByteUtil.subBytes(data, dataIndex * 4, dataIndex * 4 + 4);
                                    parsed = ByteUtil.toHexString(b7);
                                    int v7 = ByteUtil.toInt(b7);
                                    bigDecimal = new BigDecimal(v7);
                                }
                                break;
                            case 8: // Long CDAB
                                if (data.length >= dataIndex * 4 + 4) {
                                    byte[] b8 = ByteUtil.subBytes(data, dataIndex * 4, dataIndex * 4 + 4);
                                    parsed = ByteUtil.toHexString(b8);
                                    int v8 = ByteUtil.toInt(ByteUtil.convert(b8, "CDAB"));
                                    bigDecimal = new BigDecimal(v8);
                                }
                                break;
                            case 9: // Long BADC
                                if (data.length >= dataIndex * 4 + 4) {
                                    byte[] b9 = ByteUtil.subBytes(data, dataIndex * 4, dataIndex * 4 + 4);
                                    parsed = ByteUtil.toHexString(b9);
                                    int v9 = ByteUtil.toInt(ByteUtil.convert(b9, "BADC"));
                                    bigDecimal = new BigDecimal(v9);
                                }
                                break;
                            case 10: // Long DCBA
                                if (data.length >= dataIndex * 4 + 4) {
                                    byte[] b10 = ByteUtil.subBytes(data, dataIndex * 4, dataIndex * 4 + 4);
                                    parsed = ByteUtil.toHexString(b10);
                                    int v10 = ByteUtil.toInt(b10, ByteOrder.LITTLE_ENDIAN);
                                    bigDecimal = new BigDecimal(v10);
                                }
                                break;
                            case 11: // float ABCD
                                if (data.length >= dataIndex * 4 + 4) {
                                    byte[] b11 = ByteUtil.subBytes(data, dataIndex * 4, dataIndex * 4 + 4);
                                    parsed = ByteUtil.toHexString(b11);
                                    float v11 = ByteUtil.toFloat(b11);
                                    bigDecimal = new BigDecimal(v11);
                                }
                                break;
                            case 12: // float CDAB
                                if (data.length >= dataIndex * 4 + 4) {
                                    byte[] b12 = ByteUtil.subBytes(data, dataIndex * 4, dataIndex * 4 + 4);
                                    parsed = ByteUtil.toHexString(b12);
                                    float v12 = ByteUtil.toFloat(ByteUtil.convert(b12, "CDAB"));
                                    bigDecimal = new BigDecimal(v12);
                                }
                                break;
                            case 13: // float BADC
                                if (data.length >= dataIndex * 4 + 4) {
                                    byte[] b13 = ByteUtil.subBytes(data, dataIndex * 4, dataIndex * 4 + 4);
                                    parsed = ByteUtil.toHexString(b13);
                                    float v13 = ByteUtil.toFloat(ByteUtil.convert(b13, "BADC"));
                                    bigDecimal = new BigDecimal(v13);
                                }
                                break;
                            case 14: // float DCBA
                                if (data.length >= dataIndex * 4 + 4) {
                                    byte[] b14 = ByteUtil.subBytes(data, dataIndex * 4, dataIndex * 4 + 4);
                                    parsed = ByteUtil.toHexString(b14);
                                    float v14 = ByteUtil.toFloat(b14, ByteOrder.LITTLE_ENDIAN);
                                    bigDecimal = new BigDecimal(v14);
                                }
                                break;
                            default:
                                parsed = "";
                        }
                        if (bigDecimal != null) {
                            bigDecimal = calculateWithFormula(bigDecimal, monitoring.getFormula());
                            if (monitoring.getValuePrecision() != null && monitoring.getValuePrecision() >= 0) {
                                bigDecimal = bigDecimal.setScale(monitoring.getValuePrecision().intValue(), BigDecimal.ROUND_HALF_UP);
                            } else {
                                bigDecimal = bigDecimal.setScale(2, BigDecimal.ROUND_HALF_UP);
                            }

                            String debugStr = StringUtil.format(DATA_UPLOAD_TEMPLATE, monitoring.getMonitoringName(), bigDecimal.toString(), monitoring.getUnit(), "123", parsed);

                            logDebug(packetV2, debugStr);

                            MonitoringHistoryData monitoringHistoryData = new MonitoringHistoryData(packetV2.getDeviceType(), packetV2.getDeviceId());
                            monitoringHistoryData.setContextId(packetV2.getContextId());

                            monitoringHistoryData.setMonitoringId(monitoring.getMonitoringId());
                            monitoringHistoryData.setMonitoringName(monitoring.getMonitoringName());
                            monitoringHistoryData.setDataType(monitoring.getDataType());
                            monitoringHistoryData.setDataMap(monitoring.getDataMap());
                            monitoringHistoryData.setValuePrecision(monitoring.getValuePrecision());
                            monitoringHistoryData.setUnit(monitoring.getUnit());
                            monitoringHistoryData.setIcon(monitoring.getIcon());
                            monitoringHistoryData.setValue(bigDecimal.doubleValue());
                            monitoringHistoryData.setDataTime(samplingTime);
                            monitoringHistoryData.setMonitoringIndexId(monitoring.getMonitoringIndexId());
                            monitoringHistoryData.setProcessTime(DateUtil.now());
                            if (monitoringHistoryData.getMonitoringIndexId() == 6338941382618513408L && monitoringHistoryData.getValue() < 0) {
                                continue;
                            }

//                            if (transportDataTopicSender != null) {
//                                transportDataTopicSender.sendMessage(monitoringHistoryData);
//                                transportDataQueueSender.sendMessage(monitoringHistoryData);
//                            }
//                            if (insightDataQueueSender != null) {
//                                insightDataQueueSender.sendMessage(monitoringHistoryData);
//                            }
//                            if (apiPushDataQueueSender != null) {
//                                apiPushDataQueueSender.sendMessage(monitoringHistoryData);
//                            }
//                            if (virtualTransportDataQueueSender != null) {
//                                virtualTransportDataQueueSender.sendMessage(monitoringHistoryData);
//                            }

                            monitoringHistoryDataKafkaSender.sendMessage(monitoringHistoryData);
                            historyDataSaveProcessor.process(monitoringHistoryData);
                            /* 告警状态: ‘0’:低于下限值, ‘1’:正常, ‘2’:高于上限值 */
                            if (history.getAlert() == '0' || history.getAlert() == '2') {
                                processWarn(monitoringHistoryData, history.getAlert());
                            }

//                            kafkaTemplate.send(KafkaTopicDestinations.TOPIC_TEST_DATA,monitoringHistoryData);
                        }
                    }
                } catch (Exception e) {
                    String debugStr =
                            StringUtil.format(DATA_UPLOAD_EXCEPTION_TEMPLATE, history.getRegisterType(), history.getStoreAddress(), history.getMethod(), history.getDataHex(), e.getMessage());

                    logError(packetV2, debugStr, e);

                    saveErrorDecodeLog(packetV2);
                }
                return true;
            }

            private void processWarn(MonitoringHistoryData data, byte alertCode) {
                try {
                    String message = StringUtil.format(DATA_BOUND_WARNING_TEMPLATE, data.getMonitoringName(), alertCode == 0x30 ? "下" : "上", data.getValue());

                    WarningData warningData = new WarningData(packetV2.getDeviceId());
                    warningData.setFirstTime(data.getDataTime());
                    warningData.setLatestTime(data.getDataTime());
                    warningData.setWarningCount(1);
                    warningData.setWarningMessage(message);
//                    transportDataTopicSender.sendMessage(warningData);
//                    apiPushDataQueueSender.sendMessage(warningData);
                    warningDataKafkaSender.sendMessage(warningData);

                    TransportWarning warning = new TransportWarning();
                    warning.setDeviceId(packetV2.getDeviceId());
                    warning.setWarningCode(TransportWarning.WarningCode.TRANSFINITE);
                    warning.setWarning(message);
                    warning.setWarningCount(1);
                    warning.setFirstTime(data.getDataTime());
                    warning.setLatestTime(data.getDataTime());
//                    transportWarningService.insertSelective(warning);
                    TransportWarningInflux warningInflux = new TransportWarningInflux(warning);
                    transportWarningInfluxService.insert(warningInflux);

//                    insightWarningQueueSender.sendMessage(warning);
                    transportWarningKafkaSender.sendMessage(warning);

                } catch (Exception e) {
                    LOGGER.error("处理告警数据出错: {}", e.getMessage(), e);
                }
            }

            /**
             * 使用公式对原始值进行计算
             *
             * @param originalValue
             * @param formula
             * @return
             */
            private BigDecimal calculateWithFormula(BigDecimal originalValue, String formula) {
                if (StringUtil.isBlank(formula)) {
                    return originalValue;
                }
                try {
                    Script script;
                    if (formulaScriptMap.containsKey(formula)) {
                        script = formulaScriptMap.get(formula);
                    } else {
                        script = new GroovyShell().parse(formula);
                        formulaScriptMap.put(formula, script);
                    }
                    Binding binding = new Binding();
                    binding.setVariable("x", originalValue.doubleValue());
                    script.setBinding(binding);

                    Double value = Double.valueOf(script.run().toString());
                    return new BigDecimal(value);
                } catch (CompilationFailedException e) {
                    LOGGER.error("非法的换算公式: formula={}, {}", formula, e.getMessage());
                }
                return originalValue;
            }

            private void logDebug(final PacketV2 packetV2, String debugStr) {
                CodecLogMessage logMessage = CodecLogMessage.analyzeMessage(packetV2.getChannelId(), packetV2.getDeviceId(), packetV2.getContextId(), debugStr);
                LOGGER.debug(logMessage.getMessage());
                if (codecLogSender != null) {
                    codecLogSender.sendMessage(logMessage);
                }
            }

            private void logWarn(final PacketV2 packetV2, String debugStr) {
                CodecLogMessage logMessage = CodecLogMessage.analyzeMessage(packetV2.getChannelId(), packetV2.getDeviceId(), packetV2.getContextId(), debugStr);
                LOGGER.warn(logMessage.getMessage());
                if (codecLogSender != null) {
                    codecLogSender.sendMessage(logMessage);
                }
            }

            private void logError(final PacketV2 packetV2, String debugStr, Throwable e) {
                CodecLogMessage logMessage = CodecLogMessage.analyzeMessage(packetV2.getChannelId(), packetV2.getDeviceId(), packetV2.getContextId(), debugStr);
                LOGGER.error(logMessage.getMessage(), e);
                if (codecLogSender != null) {
                    codecLogSender.sendMessage(logMessage);
                }
            }

            private void saveErrorDecodeLog(PacketV2 packetV2){
                DecodeLogInflux decodeLogInflux = new DecodeLogInflux();
                PacketV2Log packetV2Log = new PacketV2Log();
                BeanUtils.copyProperties(packetV2,packetV2Log);
                decodeLogInflux.setDate(DateUtil.now());
                decodeLogInflux.setAppNode(String.valueOf(node));
                decodeLogInflux.setDecoderNum(1);
                decodeLogInflux.setPacketV2LogStr(JsonUtil.toJson(packetV2Log));
                try {
                    decodeLogInfluxService.insert(decodeLogInflux);
                } catch (ServiceException e) {
                    LOGGER.error(e.getMessage(),e);
                }
            }
        });
    }


}
