package com.qhiot.transport.codec.processor;

import com.qhiot.cache.util.CacheUtil;
import com.qhiot.core.util.DateUtil;
import com.qhiot.core.util.ThreadUtil;
import com.qhiot.transport.codec.cache.PowerCache;
import com.qhiot.transport.db.model.DevicePowerState;
import com.qhiot.transport.db.model.TransportWarning;
import com.qhiot.transport.db.service.DevicePowerStateService;
import com.qhiot.transport.db.service.TransportWarningService;
import com.qhiot.transport.influxdb.model.TransportWarningInflux;
import com.qhiot.transport.influxdb.service.TransportWarningInfluxService;
import com.qhiot.transport.jms.kafka.TransportWarningKafkaSender;
import com.qhiot.transport.jms.kafka.WarningDataKafkaSender;
import com.qhiot.transport.message.data.PowerData;
import com.qhiot.transport.message.data.WarningData;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.concurrent.ExecutorService;

/**
 * 掉电数据处理器
 *
 * @author 金建强(ptma 163.com)
 * @version 2018 -02-06 13:37:44
 */
@Component
public class PowerDataProcessor {

    private static final Logger     LOGGER   = LoggerFactory.getLogger(PowerDataProcessor.class);

    private static ExecutorService  executor = ThreadUtil.newCachedThreadPool(0, 1000, "seem-power-data-processor-%d");

    @Autowired
    private TransportWarningService transportWarningService;
    @Autowired
    private TransportWarningInfluxService transportWarningInfluxService;
    @Autowired
    private DevicePowerStateService devicePowerStateService;

//    @Autowired
//    private InsightWarningQueueSender insightWarningQueueSender;
//
//    @Autowired
//    private TransportDataTopicSender transportDataTopicSender;

    /**---kafka---*/
    @Autowired
    private WarningDataKafkaSender warningDataKafkaSender;
    @Autowired
    private TransportWarningKafkaSender transportWarningKafkaSender;
    /**------*/

    public void process(final PowerData data) {
        if (data.getPowerState() == null) {
            return;
        }
        executor.submit(new Runnable() {
            @Override
            public void run() {
                processData(data);
            }
        });
    }

    public void processData(final PowerData data) {
        try {
            final String cacheKey = data.getDeviceId();
            // 如果是上电信号，则删除掉电缓存
            if (data.getPowerState() == 1) {
                CacheUtil.del(PowerCache.CACHE_REGION, cacheKey);
                /**
                 * 记录上电信息
                 */
                savePowerState(data, null);
            } else {
                Date now = DateUtil.now();
                PowerCache cacheData =
                        (PowerCache) CacheUtil.get(PowerCache.CACHE_REGION, cacheKey);
                if (cacheData == null) {
                    cacheData = new PowerCache(data.getDeviceId());
                    cacheData.setFirstTime(now);
                }
                cacheData.setLatestTime(now);
                cacheData.increaseCount();

                saveWarning(data, cacheData);
                savePowerState(data, cacheData);

                CacheUtil.set(PowerCache.CACHE_REGION, cacheKey, cacheData);
            }
        } catch (Exception e) {
            LOGGER.warn("处理掉电数据出错: ", e.getMessage(), e);
        }
    }

    private void saveWarning(final PowerData data, PowerCache cacheData) {
        try {
            TransportWarning warning = new TransportWarning();
            warning.setDeviceId(cacheData.getDeviceId());
            warning.setWarningCode(TransportWarning.WarningCode.POWER);
            warning.setWarning("终端掉电");
            warning.setFirstTime(cacheData.getFirstTime());
            warning.setLatestTime(cacheData.getLatestTime());
            warning.setWarningCount(cacheData.getCount());

            // 掉电告警ID 未被缓存了（新的掉电数据）
            if (cacheData.getWarningId() == null) {
//                transportWarningService.insertSelective(warning);
                TransportWarningInflux warningInflux = new TransportWarningInflux(warning);
                transportWarningInfluxService.insert(warningInflux);
                cacheData.setWarningId(warning.getId());

                // 掉电数据推送
                WarningData warningData = new WarningData(warning.getDeviceId());
                warningData.setFirstTime(warning.getFirstTime());
                warningData.setLatestTime(warning.getLatestTime());
                warningData.setWarningCount(warning.getWarningCount());
                warningData.setWarningMessage(warning.getWarning());
//                transportDataTopicSender.sendMessage(warningData);
                warningDataKafkaSender.sendMessage(warningData);

            } else {
                warning.setId(cacheData.getWarningId());
//                transportWarningService.updateSelective(warning);
                TransportWarningInflux warningInflux = new TransportWarningInflux(warning);
                transportWarningInfluxService.updateById(warningInflux);
            }

        } catch (Exception e) {
            LOGGER.warn("保存掉电告警出错: ", e.getMessage(), e);
        }
    }

    private void savePowerState(final PowerData data, PowerCache cacheData) {
        try {
            DevicePowerState state = new DevicePowerState();
            state.setDeviceId(data.getDeviceId());
            state.setPowerState(data.getPowerState());
            state.setStateTime(data.getTimestamp());

            // 掉电记录ID 未被缓存了（新的掉电数据）
            if (cacheData == null || cacheData.getStateId() == null) {
                devicePowerStateService.insertSelective(state);
                if (cacheData != null) {
                    cacheData.setStateId(state.getStateId());
                }
            } else {
                state.setStateId(cacheData.getStateId());
                devicePowerStateService.updateSelective(state);
            }

            if (cacheData != null) {
                TransportWarning warning = new TransportWarning();
                warning.setDeviceId(cacheData.getDeviceId());
                warning.setWarningCode(TransportWarning.WarningCode.POWER);
                warning.setFirstTime(cacheData.getFirstTime());
                warning.setLatestTime(cacheData.getLatestTime());
                warning.setWarningCount(cacheData.getCount());
                if (data.getPowerState() == 1) {
                    //上电数据推送
                    warning.setWarning("终端上电");
                } else {
                    //掉电数据推送
                    warning.setWarning("终端掉电");
                }
//                insightWarningQueueSender.sendMessage(warning);
                transportWarningKafkaSender.sendMessage(warning);
            }
        } catch (Exception e) {
            LOGGER.warn("保存掉电记录出错: ", e.getMessage(), e);
        }
    }

}
