package com.qhiot.transport.codec.util;

import com.qhiot.core.util.ByteUtil;
import com.qhiot.core.util.CRCUtil;
import com.qhiot.core.util.SpringUtil;
import com.qhiot.core.util.StringUtil;
import com.qhiot.transport.codec.bean.AttrKey;
import com.qhiot.transport.codec.bean.PacketV2;
import com.qhiot.transport.codec.bean.v2.FirmwareRequest;
import com.qhiot.transport.codec.bean.v2.FirmwareSegment;
import com.qhiot.transport.codec.decoder.v2.EquipmentConfigDecoder;
import com.qhiot.transport.codec.decoder.v2.EquipmentMonitoringDecoder;
import com.qhiot.transport.codec.decoder.v2.WarningConfigDecoder;
import com.qhiot.transport.jms.kafka.CodecLogKafkaSender;
import com.qhiot.transport.message.CodecLogMessage;
import com.qhiot.transport.message.bean.*;
import com.qhiot.transport.message.define.Device;
import com.qhiot.transport.message.define.DeviceType;
import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;
import io.netty.channel.ChannelFutureListener;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.nio.ByteOrder;
import java.util.List;

/**
 *  向硬件发送命令
 *
 * @author 金建强(ptma@163.com)
 * @version 2017/4/13 20:06
 */
public class CommandUtil {

    private static final Logger logger = LoggerFactory.getLogger("transport.codec.CommandUtil");

    /**
     * 发送读取终端\激励器\中继信息指令
     *
     * @param channel
     * @param deviceId
     */
    public static void deviceParamRead(Channel channel, DeviceType deviceType, String deviceId) {
        sendCommand(channel, deviceType, deviceId, Command.READ_DEV_PARAM, Command.EMPTY_DATA);
    }

    /**
     * 写入终端参数
     *
     * @param channel
     * @param gatewayId
     * @param gatewayParam
     */
    public static void gatewayParamWrite(Channel channel, String gatewayId, GatewayParam gatewayParam) {
        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.WRITE_DEV_PARAM,
                GatewayParamBuilder.toBytes(gatewayParam));
    }

    /**
     * 低级写入终端参数
     *
     * @param channel
     * @param gatewayId
     * @param gatewayParam
     */
    public static void gatewayParamLowLevelWrite(Channel channel, String gatewayId,
                                                 GatewayParam gatewayParam) {
        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.LOW_LEVEL_WRITE,
                GatewayParamBuilder.toBytes(gatewayParam));
    }

    /**
     * 写入 激励器\中继 参数
     *
     * @param channel
     * @param deviceId
     * @param deviceParam
     */
    public static void deviceParamWrite(Channel channel, DeviceType deviceType, String deviceId,
                                        DeviceParam deviceParam) {
        sendCommand(channel, deviceType, deviceId, Command.WRITE_DEV_PARAM,
                DeviceParamBuilder.toBytes(deviceParam));
    }

    /**
     * 低级写入 激励器\中继 参数
     *
     * @param channel
     * @param deviceId
     * @param deviceParam
     */
    public static void deviceParamLowLevelWrite(Channel channel, DeviceType deviceType,
                                                String deviceId, DeviceParam deviceParam) {
        sendCommand(channel, deviceType, deviceId, Command.LOW_LEVEL_WRITE,
                DeviceParamBuilder.toBytes(deviceParam));
    }

    /**
     * 重启终端
     *
     * @param channel
     * @param deviceType
     * @param deviceId
     */
    public static void deviceReboot(Channel channel, DeviceType deviceType, String deviceId) {
        sendCommand(channel, deviceType, deviceId, Command.REBOOT, Command.EMPTY_DATA);
    }

    /**
     * 终端恢复出厂设置
     *
     * @param channel
     * @param deviceId
     */
    public static void deviceRestore(Channel channel, DeviceType deviceType, String deviceId) {
        sendCommand(channel, deviceType, deviceId, Command.RESTORE, Command.EMPTY_DATA);
    }

    /**
     * 终端低级格式化
     *
     * @param channel
     * @param deviceId
     */
    public static void deviceLowLevelFormat(Channel channel, DeviceType deviceType, String deviceId) {
        sendCommand(channel, deviceType, deviceId, Command.LOW_LEVEL_FORMAT, Command.EMPTY_DATA);
    }

    /**
     * 发送固件更新请求
     *
     * @param channel
     * @param firmwareRequest
     */
    public static void writeFirmwareUpgradeRequest(Channel channel, FirmwareRequest firmwareRequest) {
        sendCommand(channel, firmwareRequest.getDeviceType(), firmwareRequest.getDeviceId(),
                Command.FIRMWARE_UPGRADE, firmwareRequest.toBytes());
    }

    /**
     * 发送固件更新数据包
     *
     * @param channel
     * @param firmwareSegment
     */
    public static void writeFirmwareUpgradeSegment(Channel channel, FirmwareSegment firmwareSegment) {
        sendCommand(channel, firmwareSegment.getDeviceType(), firmwareSegment.getDeviceId(),
                Command.FIRMWARE_DATA, firmwareSegment.toBytes());
    }

    /**
     * 透传指令
     *
     * @param channel
     * @param deviceId 终端编号
     * @param baudrate 模特率序号 [0-2400, 1-4800, 2-9600, 3-19200, 4-38400, 5-57600, 6-115200, 7-1200]
     * @param checkMode 校验[0-无校验, 1-奇校验, 2-偶校验]
     * @param dc 数据通道[0:PLC, 1:摄像头, 2:流量计, 3:其它]
     * @param commandHex 十六进制指令数据
     */
    public static void sendTransmitTransparently(Channel channel, DeviceType deviceType,
                                                 String deviceId, int baudrate, int checkMode, int dc, String commandHex) {
        byte[] commandData =
                ByteUtil.concat(new byte[] {(byte) baudrate, (byte) checkMode, (byte) dc},
                        ByteUtil.hexToBytes(commandHex));
        sendCommand(channel, deviceType, deviceId, Command.TRANSMIT_TRANSPARENTLY, commandData);
    }

    /**
     * 写入摄像头配置指令
     *
     * @param channel
     * @param deviceId
     * @param cameraIndex 序号(0-7)
     * @param config 配置命令
     */
    public static void cameraConfigWrite(Channel channel, String deviceId, int cameraIndex,
                                         CameraConfig config) {
        byte[] cmd =
                ByteUtil.combine(new byte[] {(byte) cameraIndex},
                        CameraConfigBuilder.toBytes(config));
        sendCommand(channel, DeviceType.GATEWAY, deviceId, Command.CAM_CONFIG_WRITE, cmd);
    }

    /**
     * 读取一条摄像头配置指令
     *
     * @param channel
     * @param deviceId
     * @param cameraIndex 序号(0-7)
     */
    public static void cameraConfigRead(Channel channel, String deviceId, int cameraIndex) {
        byte[] cmd = {(byte) cameraIndex};
        sendCommand(channel, DeviceType.GATEWAY, deviceId, Command.CAM_CONFIG_READ, cmd);
    }

    /**
     * 删除一条摄像头配置指令
     *
     * @param channel
     * @param deviceId
     * @param cameraIndex 序号(0-7)
     */
    public static void cameraConfigDelete(Channel channel, String deviceId, int cameraIndex) {
        byte[] cmd = {(byte) cameraIndex};
        sendCommand(channel, DeviceType.GATEWAY, deviceId, Command.CAM_CONFIG_DEL, cmd);
    }

    /**
     * 删除全部摄像头配置指令
     *
     * @param channel
     * @param deviceId
     */
    public static void cameraConfigDeleteAll(Channel channel, String deviceId) {
        sendCommand(channel, DeviceType.GATEWAY, deviceId, Command.CAM_CONFIG_DEL_ALL,
                Command.EMPTY_DATA);
    }

    /**
     * 读取全部摄像头配置指令
     *
     * @param channel
     * @param deviceId
     */
    public static void cameraConfigReadAll(Channel channel, String deviceId) {
        sendCommand(channel, DeviceType.GATEWAY, deviceId, Command.CAM_CONFIG_READ_ALL,
                Command.EMPTY_DATA);
    }

    /**
     * 拍照指令
     *
     * @param channel
     * @param deviceId 终端编号
     * @param cameraIndex 序号(0-7)
     */
    public static void takePicture(Channel channel, String deviceId, int cameraIndex) {
        byte[] cmd = {(byte) cameraIndex};
        sendCommand(channel, DeviceType.GATEWAY, deviceId, Command.CAM_SNAP, cmd);
    }

    /**
     * 停止照片上传
     *
     * @param channel
     * @param deviceId
     * @param cameraIndex 序号(0-7)
     */
    public static void stopPictureUploading(Channel channel, String deviceId, int cameraIndex) {
        byte[] cmd = {(byte) cameraIndex};
        sendCommand(channel, DeviceType.GATEWAY, deviceId, Command.CAM_SNAP_STOP_UPLOADING, cmd);
    }

    /**
     * 写入摄像头拍照时间点配置指令
     *
     * @param channel
     * @param gatewayId
     * @param cameraIndex 序号(0-19)
     * @param config 配置命令
     */
    public static void cameraTimePointConfigWrite(Channel channel, String gatewayId, int cameraIndex,
                                                  CameraTimePointConfig config) {
        byte[] cmd =
                ByteUtil.combine(new byte[] {(byte) cameraIndex},
                        CameraTimePointConfigBuilder.toBytes(config));
        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.CAM_CONFIG_TIME_POINT_WRITE,
                cmd);
    }

    /**
     * 读取一条摄像头拍照时间点配置指令
     *
     * @param channel
     * @param gatewayId
     * @param cameraIndex 序号(0-19)
     */
    public static void cameraTimePointConfigRead(Channel channel, String gatewayId, int cameraIndex) {
        byte[] cmd = {(byte) cameraIndex};
        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.CAM_CONFIG_TIME_POINT_READ, cmd);
    }

    /**
     * 删除一条摄像头拍照时间点配置指令
     *
     * @param channel
     * @param gatewayId
     * @param cameraIndex 序号(0-19)
     */
    public static void cameraTimePointConfigDelete(Channel channel, String gatewayId, int cameraIndex) {
        byte[] cmd = {(byte) cameraIndex};
        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.CAM_CONFIG_TIME_POINT_DEL, cmd);
    }

    /**
     * 删除全部摄像头拍照时间点配置指令
     *
     * @param channel
     * @param gatewayId
     */
    public static void cameraTimePointConfigDeleteAll(Channel channel, String gatewayId) {
        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.CAM_CONFIG_TIME_POINT_DEL_ALL,
                Command.EMPTY_DATA);
    }

    /**
     * 读取全部摄像头拍照时间点配置指令
     *
     * @param channel
     * @param gatewayId
     */
    public static void cameraTimePointConfigReadAll(Channel channel, String gatewayId) {
        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.CAM_CONFIG_TIME_POINT_READ_ALL,
                Command.EMPTY_DATA);
    }

    // ---------------------------
    /**
     * 写入485外设采集相关配置指令
     *
     * @param channel
     * @param gatewayId
     * @param index 序号(0-39)
     * @param config 配置命令
     */
    public static void rs485ConfigWrite(Channel channel, String gatewayId, int index,
                                        Rs485Config config) {
        byte[] cmd =
                ByteUtil.combine(new byte[] {(byte) index}, Rs485ConfigBuilder.toBytes(config));
        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.RS485_CONFIG_WRITE, cmd);
    }

    /**
     * 读取一天485外设采集相关配置指令
     *
     * @param channel
     * @param gatewayId
     * @param index 序号(0-39)
     */
    public static void rs485ConfigRead(Channel channel, String gatewayId, int index) {
        byte[] cmd = {(byte) index};
        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.RS485_CONFIG_READ, cmd);
    }

    /**
     * 删除一条485外设采集相关配置指令
     *
     * @param channel
     * @param gatewayId
     * @param index 序号(0-39)
     */
    public static void rs485ConfigDelete(Channel channel, String gatewayId, int index) {
        byte[] cmd = {(byte) index};
        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.RS485_CONFIG_DEL, cmd);
    }

    /**
     * 删除全部485外设采集相关配置指令
     *
     * @param channel
     * @param gatewayId
     */
    public static void rs485ConfigDeleteAll(Channel channel, String gatewayId) {
        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.RS485_CONFIG_DEL_ALL,
                Command.EMPTY_DATA);
    }

    /**
     * 读取全部485外设采集相关配置指令
     *
     * @param channel
     * @param gatewayId
     */
    public static void rs485ConfigReadAll(Channel channel, String gatewayId) {
        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.RS485_CONFIG_READ_ALL,
                Command.EMPTY_DATA);
    }

    /**
     * 采集/下发 配置的485外设采集指令
     *
     * @param channel
     * @param gatewayId
     * @param index
     */
    public static void rs485Collect(Channel channel, String gatewayId, int index) {
        byte[] cmd = {(byte) index};
        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.RS485_COLLECT, cmd);
    }

    // --------------------------------
    /**
     * 写入输入开关量控制配置指令
     *
     * @param channel
     * @param gatewayId
     * @param index 序号(0-39)
     * @param config 配置命令
     */
    public static void switchInputConfigWrite(Channel channel, String gatewayId, int index,
                                              InputConfig config) {
        byte[] cmd =
                ByteUtil.combine(new byte[] {(byte) index}, InputConfigBuilder.toBytes(config));
        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.SWITCH_INPUT_CONFIG_WRITE, cmd);
    }

    /**
     * 读取一天输入开关量控制配置指令
     *
     * @param channel
     * @param gatewayId
     * @param index 序号(0-39)
     */
    public static void switchInputConfigRead(Channel channel, String gatewayId, int index) {
        byte[] cmd = {(byte) index};
        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.SWITCH_INPUT_CONFIG_READ, cmd);
    }

    /**
     * 删除一条输入开关量控制配置指令
     *
     * @param channel
     * @param gatewayId
     * @param index 序号(0-39)
     */
    public static void switchInputConfigDelete(Channel channel, String gatewayId, int index) {
        byte[] cmd = {(byte) index};
        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.SWITCH_INPUT_CONFIG_DEL, cmd);
    }

    /**
     * 删除全部输入开关量控制配置指令
     *
     * @param channel
     * @param gatewayId
     */
    public static void switchInputConfigDeleteAll(Channel channel, String gatewayId) {
        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.SWITCH_INPUT_CONFIG_DEL_ALL,
                Command.EMPTY_DATA);
    }

    /**
     * 读取全部输入开关量控制配置指令
     *
     * @param channel
     * @param gatewayId
     */
    public static void switchInputConfigReadAll(Channel channel, String gatewayId) {
        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.SWITCH_INPUT_CONFIG_READ_ALL,
                Command.EMPTY_DATA);
    }

    /**
     * 读取输入开关量状态
     *
     * @param channel
     * @param gatewayId
     */
    public static void switchInputStateRead(Channel channel, String gatewayId) {
        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.SWITCH_INPUT_STATE_READ,
                Command.EMPTY_DATA);
    }

    // --------------------------------

    /**
     * 写入输出开关量控制配置指令
     *
     * @param channel
     * @param gatewayId
     * @param index 序号(0-39)
     * @param config 配置命令
     */
    public static void switchOutputConfigWrite(Channel channel, String gatewayId, int index,
                                               OutputConfig config) {
        byte[] cmd =
                ByteUtil.combine(new byte[] {(byte) index}, OutputConfigBuilder.toBytes(config));
        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.SWITCH_OUTPUT_CONFIG_WRITE, cmd);
    }

    /**
     * 读取一天输出开关量控制配置指令
     *
     * @param channel
     * @param gatewayId
     * @param index 序号(0-39)
     */
    public static void switchOutputConfigRead(Channel channel, String gatewayId, int index) {
        byte[] cmd = {(byte) index};
        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.SWITCH_OUTPUT_CONFIG_READ, cmd);
    }

    /**
     * 删除一条输出开关量控制配置指令
     *
     * @param channel
     * @param gatewayId
     * @param index 序号(0-39)
     */
    public static void switchOutputConfigDelete(Channel channel, String gatewayId, int index) {
        byte[] cmd = {(byte) index};
        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.SWITCH_OUTPUT_CONFIG_DEL, cmd);
    }

    /**
     * 删除全部输出开关量控制配置指令
     *
     * @param channel
     * @param gatewayId
     */
    public static void switchOutputConfigDeleteAll(Channel channel, String gatewayId) {
        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.SWITCH_OUTPUT_CONFIG_DEL_ALL,
                Command.EMPTY_DATA);
    }

    /**
     * 读取全部输出开关量控制配置指令
     *
     * @param channel
     * @param gatewayId
     */
    public static void switchOutputConfigReadAll(Channel channel, String gatewayId) {
        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.SWITCH_OUTPUT_CONFIG_READ_ALL,
                Command.EMPTY_DATA);
    }

    /**
     * 手动控制, 打开某一路开关量
     *
     * @param channel
     * @param gatewayId
     * @param index
     */
    public static void switchOutputOpen(Channel channel, String gatewayId, int index) {
        byte[] cmd = {(byte) index};
        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.SWITCH_OUTPUT_OPEN, cmd);
    }

    /**
     * 手动控制, 关闭某一路开关量
     *
     * @param channel
     * @param gatewayId
     * @param index
     */
    public static void switchOutputClose(Channel channel, String gatewayId, int index) {
        byte[] cmd = {(byte) index};
        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.SWITCH_OUTPUT_CLOSE, cmd);
    }

    /**
     * 写入输出开关量周期指令配置指令
     *
     * @param channel
     * @param gatewayId
     * @param index 序号(0-39)
     * @param config 配置命令
     */
    public static void switchOutputPeriodConfigWrite(Channel channel, String gatewayId, int index,
                                                     OutputPeriodConfig config) {
        byte[] cmd =
                ByteUtil.combine(new byte[] {(byte) index},
                        OutputPeriodConfigBuilder.toBytes(config));
        sendCommand(channel, DeviceType.GATEWAY, gatewayId,
                Command.SWITCH_OUTPUT_PERIOD_CONFIG_WRITE, cmd);
    }

    /**
     * 读取一条输出开关量周期指令配置指令
     *
     * @param channel
     * @param gatewayId
     * @param index 序号(0-39)
     */
    public static void switchOutputPeriodConfigRead(Channel channel, String gatewayId, int index) {
        byte[] cmd = {(byte) index};
        sendCommand(channel, DeviceType.GATEWAY, gatewayId,
                Command.SWITCH_OUTPUT_PERIOD_CONFIG_READ, cmd);
    }

    /**
     * 删除一条输出开关量周期指令配置指令
     *
     * @param channel
     * @param gatewayId
     * @param index 序号(0-39)
     */
    public static void switchOutputPeriodConfigDelete(Channel channel, String gatewayId, int index) {
        byte[] cmd = {(byte) index};
        sendCommand(channel, DeviceType.GATEWAY, gatewayId,
                Command.SWITCH_OUTPUT_PERIOD_CONFIG_DEL, cmd);
    }

    /**
     * 删除全部输出开关量周期指令配置指令
     *
     * @param channel
     * @param gatewayId
     */
    public static void switchOutputPeriodConfigDeleteAll(Channel channel, String gatewayId) {
        sendCommand(channel, DeviceType.GATEWAY, gatewayId,
                Command.SWITCH_OUTPUT_PERIOD_CONFIG_DEL_ALL, Command.EMPTY_DATA);
    }

    /**
     * 读取全部输出开关量周期指令配置指令
     *
     * @param channel
     * @param gatewayId
     */
    public static void switchOutputPeriodConfigReadAll(Channel channel, String gatewayId) {
        sendCommand(channel, DeviceType.GATEWAY, gatewayId,
                Command.SWITCH_OUTPUT_PERIOD_CONFIG_READ_ALL, Command.EMPTY_DATA);
    }

    /**
     * 读取输出开关量状态
     *
     * @param channel
     * @param gatewayId
     */
    public static void switchOutputStateRead(Channel channel, String gatewayId) {
        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.SWITCH_OUTPUT_STATE_READ,
                Command.EMPTY_DATA);
    }

    // -----------------------------------

    /**
     * 设置输入输出运行模式
     *
     * @param channel
     * @param gatewayId
     * @param model
     */
    public static void ioWorkModelWrite(Channel channel, String gatewayId, int model) {
        byte[] cmd = {(byte) model};
        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.IO_WORK_MODEL_WRITE, cmd);
    }

    /**
     * 读取输入输出运行模式
     *
     * @param channel
     * @param gatewayId
     */
    public static void ioWorkModelRead(Channel channel, String gatewayId) {
        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.IO_WORK_MODEL_READ,
            Command.EMPTY_DATA);
    }

    /**
     * modbus 指令控制 IO 输出
     *
     * @param channel
     * @param gatewayId
     * @param registerAddress 寄存器地址，2字节(大端模式)
     * @param open 开关
     */
    public static void modbusIOControl(Channel channel, String gatewayId, int registerAddress, boolean open) {
        byte[] commandData = new byte[8];
        commandData[0] = 0x01;
        commandData[1] = 0x05;
        ByteUtil.writeUnsignedShort(commandData, 2, registerAddress, ByteOrder.BIG_ENDIAN);
        commandData[4] = open ? (byte) 0xFF : 0x00;
        commandData[5] = 0x00;

        int crc = CRCUtil.crcModbus(ByteUtil.subBytes(commandData, 0, 6));
        ByteUtil.writeUnsignedShort(commandData, 6, (short) crc, ByteOrder.LITTLE_ENDIAN);

        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.MODBUS_IO_CONTROL, commandData);
    }
    // --------------------------------

    /**
     * modbus 参数写入
     *
     * @param channel
     * @param gatewayId
     * @param registerAddress 寄存器地址，2字节(大端模式)
     * @param method 解码方法
     * @param data 数据
     */
    public static void modbusDataWrite(Channel channel, String gatewayId, int registerAddress, byte registerType, byte method, byte[] data) {
        byte[] commandData = new byte[4 + data.length];
        ByteUtil.writeUnsignedShort(commandData, 0, registerAddress, ByteOrder.BIG_ENDIAN);
        commandData[2] = registerType;
        commandData[3] = method;
        ByteUtil.writeBytes(commandData, 4, data);
        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.MODBUS_DATA_WRITE, commandData);
    }

    /**
     * 读互感器状态
     *
     * @param channel
     * @param gatewayId
     */
    public static void transformerStateRead(Channel channel, String gatewayId) {
        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.TRANSFORMER_STATE_READ,
                Command.EMPTY_DATA);
    }

    /**
     * 读电量计数据
     *
     * @param channel
     * @param gatewayId
     */
    public static void coulometerRead(Channel channel, String gatewayId) {
        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.COULOMETER_DATA_READ,
                Command.EMPTY_DATA);
    }

    // --------------------------------

    /**
     * 读取历史数据
     *
     * @param channel
     * @param gatewayId
     * @param offset 从第几条开始读
     * @param limit 总共读取几条
     * @throws Exception
     */
    public static void historyDataRead(Channel channel, String gatewayId, int offset, short limit) {
        byte[] commandData = new byte[5];
        ByteUtil.writeInt(commandData, 0, offset, ByteOrder.LITTLE_ENDIAN);
        commandData[4] = (byte) limit;
        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.HISTORY_DATA_READ, commandData);
    }

    /**
     * 删除历史数据
     *
     * @param channel
     * @param gatewayId
     * @throws Exception
     */
    public static void historyDataDelete(Channel channel, String gatewayId) {
        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.HISTORY_DATA_DELETE,
                Command.EMPTY_DATA);
    }

    public static void historyDataCount(Channel channel, String gatewayId) {
        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.HISTORY_DATA_COUNT,
                Command.EMPTY_DATA);
    }

    // --------------------------------

    /**
     * 发送短信目标号码
     *
     * @param channel
     * @param gatewayId
     * @param mobile 目标号码 Unicode格式
     */
    public static void smsSendMobile(Channel channel, String gatewayId, byte[] mobile) {
        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.SMS_SEND_MOBILE, mobile);
    }

    /**
     * 发送短信内容
     *
     * @param channel
     * @param gatewayId
     * @param content Unicode格式
     */
    public static void smsSendContent(Channel channel, String gatewayId, byte[] content) {
        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.SMS_SEND_CONTENT, content);
    }

    /**
     * 读短信
     *
     * @param channel
     * @param gatewayId
     */
    public static void smsRead(Channel channel, String gatewayId) {
        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.SMS_READ, Command.EMPTY_DATA);
    }








    /**
     * 读10条资源配置信息
     */
    public static void resourceConfigReadTen(Channel channel, String gatewayId, int page) {
        byte[] cmd = {(byte) page};
        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.RESOURCE_CONFIG_READ_TEN, cmd);
    }
    /**
     * 删除10条资源配置信息
     */
    public static void resourceConfigDeleteTen(Channel channel, String gatewayId, int page) {
        byte[] cmd = {(byte) page};
        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.RESOURCE_CONFIG_DELETE_TEN, cmd);
    }
    /**
     * 写入10条资源配置信息
     */
    public static void resourceConfigWriteTen(Channel channel, String gatewayId, int page, List<ResourceConfig> configs) {
        byte[] cmd = {(byte) page};
        for (int i=0;i<configs.size();i++) {
            cmd = ByteUtil.combine(cmd, ResourceConfigBuilder.toBytes(configs.get(i)));
        }
        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.RESOURCE_CONFIG_WRITE_TEN, cmd);
    }
    /**
     * 读1条资源配置信息
     */
    public static void resourceConfigRead(Channel channel, String gatewayId, int index) {
        byte[] cmd = {(byte) index};
        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.RESOURCE_CONFIG_READ_ONE, cmd);
    }
    /**
     * 删除1条资源配置信息
     */
    public static void resourceConfigDelete(Channel channel, String gatewayId, int index) {
        byte[] cmd = {(byte) index};
        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.RESOURCE_CONFIG_DELETE_ONE, cmd);
    }
    /**
     * 写入1条资源配置信息
     */
    public static void resourceConfigWrite(Channel channel, String gatewayId, int index, ResourceConfig config) {
        byte[] cmd = ByteUtil.combine(new byte[]{(byte) index}, ResourceConfigBuilder.toBytes(config));
        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.RESOURCE_CONFIG_WRITE_ONE, cmd);
    }
    /**
     * 读数据测试
     */
    public static void resourceConfigTest(Channel channel, String gatewayId, int index) {
        byte[] cmd = {(byte) index};
        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.RESOURCE_CONFIG_TEST_DATA, cmd);
    }



    // ================================


    /**
     * 写入一条动力设备监测电流配置
     *
     * @param channel
     * @param gatewayId
     */
    public static void overloadCurrentWriteOne(Channel channel, String gatewayId, int index,
                                               OverloadCurrent config) {
        byte[] cmd =
            ByteUtil.combine(new byte[] {(byte) index}, OverloadCurrentBuilder.toBytes(config));
        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.OVERLOAD_CURRENT_WRITE_ONE, cmd);
    }

    /**
     * 读取一条动力设备监测电流配置
     *
     * @param channel
     * @param gatewayId
     */
    public static void overloadCurrentReadOne(Channel channel, String gatewayId, int index) {
        byte[] cmd = {(byte) index};
        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.OVERLOAD_CURRENT_READ_ONE, cmd);
    }

    /**
     * 删除一条动力设备监测电流配置
     *
     * @param channel
     * @param gatewayId
     */
    public static void overloadCurrentDeleteOne(Channel channel, String gatewayId, int index) {
        byte[] cmd = {(byte) index};
        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.OVERLOAD_CURRENT_DELETE_ONE, cmd);
    }

    /**
     * 删除全部动力设备监测电流配置
     *
     * @param channel
     * @param gatewayId
     */
    public static void overloadCurrentDeleteAll(Channel channel, String gatewayId) {
        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.OVERLOAD_CURRENT_DELETE_ALL, Command.EMPTY_DATA);
    }

    /**
     * 读全部输动力设备监测电流配置
     *
     * @param channel
     * @param gatewayId
     */
    public static void overloadCurrentReadAll(Channel channel, String gatewayId) {
        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.OVERLOAD_CURRENT_READ_ALL, Command.EMPTY_DATA);
    }



    // ---------------------------------------------------------------------------------------

    /**
     * 读取一条动力设备监测电流状态
     *
     * @param channel
     * @param gatewayId
     */
    public static void overloadWarningReadOne(Channel channel, String gatewayId, int index, int type) {
        byte[] cmd = {(byte) index, (byte) type};
        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.OVERLOAD_WARNING_READ_ONE, cmd);
    }

    /**
     * 删除一条动力设备监测电流状态
     *
     * @param channel
     * @param gatewayId
     */
    public static void overloadWarningDeleteOne(Channel channel, String gatewayId, int index, int type) {
        byte[] cmd = {(byte) index, (byte) type};
        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.OVERLOAD_WARNING_CLEAR, cmd);
    }

    /**
     * 删除全部动力设备监测电流状态
     *
     * @param channel
     * @param gatewayId
     */
    public static void overloadWarningDeleteAll(Channel channel, String gatewayId) {
        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.OVERLOAD_WARNING_CLEAR_ALL, Command.EMPTY_DATA);
    }

    /**
     * 读全部动力设备监测电流状态
     *
     * @param channel
     * @param gatewayId
     */
    public static void overloadWarningReadAll(Channel channel, String gatewayId) {
        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.OVERLOAD_WARNING_READ_ALL, Command.EMPTY_DATA);
    }


    // =======================================

    /**
     * 打开无线开关
     * @param channel
     * @param gatewayId
     * @param index
     */
    public static void wirelessSwitchOpen(Channel channel, String gatewayId, int index) {
        byte[] cmd = {(byte) index};
        sendCommand(channel, DeviceType.WIRELESS_SWITCH, gatewayId, Command.WIRELESS_SWITCH_OPEN, cmd);
    }

    /**
     * 关闭无线开关
     * @param channel
     * @param gatewayId
     * @param index
     */
    public static void wirelessSwitchClose(Channel channel, String gatewayId, int index) {
        byte[] cmd = {(byte) index};
        sendCommand(channel, DeviceType.WIRELESS_SWITCH, gatewayId, Command.WIRELESS_SWITCH_CLOSE, cmd);
    }

    /**
     * 打开所有无线开关
     * @param channel
     * @param gatewayId
     */
    public static void wirelessSwitchOpenAll(Channel channel, String gatewayId) {
        sendCommand(channel, DeviceType.WIRELESS_SWITCH, gatewayId, Command.WIRELESS_SWITCH_OPEN_ALL, Command.EMPTY_DATA);
    }

    /**
     * 关闭所有无线开关
     * @param channel
     * @param gatewayId
     */
    public static void wirelessSwitchCloseAll(Channel channel, String gatewayId) {
        sendCommand(channel, DeviceType.WIRELESS_SWITCH, gatewayId, Command.WIRELESS_SWITCH_CLOSE_ALL, Command.EMPTY_DATA);
    }

    // ================================

    /**
     * AT 指令
     *
     * @param channel
     * @param deviceId 终端编号
     * @param command 指令字符串
     */
    public static void sendAtCommand(Channel channel, DeviceType deviceType,
                                     String deviceId, String command) {
        byte[] commandData = command.getBytes();
        sendCommand(channel, deviceType, deviceId, Command.AT_COMMAND_SEND, commandData);
    }

    // ==================================

    /**
     * 读配置信息
     */
    public static void equipmentConfigRead(Channel channel, String gatewayId, int index) {
        ByteBuf buf = ByteBufUtil.directBuffer();
        buf.writeByte(EquipmentConfigDecoder.READ_ONE);
        buf.writeShortLE(index);
        byte[] cmd = ByteBufUtil.getBytesRemaining(buf);
        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.EQUIPMENT_CONFIG, cmd);
    }

    /**
     * 读10配置信息
     */
    public static void equipmentConfigReadTen(Channel channel, String gatewayId, int page) {
        ByteBuf buf = ByteBufUtil.directBuffer();
        buf.writeByte(EquipmentConfigDecoder.READ_TEN);
        buf.writeShortLE(page);
        byte[] cmd = ByteBufUtil.getBytesRemaining(buf);

        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.EQUIPMENT_CONFIG, cmd);
    }

    /**
     * 写入1条配置信息
     */
    public static void equipmentConfigWrite(Channel channel, String gatewayId, int index, EquipmentConfig config) {
        ByteBuf buf = ByteBufUtil.directBuffer();
        buf.writeByte(EquipmentConfigDecoder.WRITE_ONE);
        buf.writeShortLE(index);
        buf.writeBytes(EquipmentConfigBuilder.toBytes(config));
        byte[] cmd = ByteBufUtil.getBytesRemaining(buf);

        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.EQUIPMENT_CONFIG, cmd);
    }

    /**
     * 删除1条配置信息
     */
    public static void equipmentConfigDelete(Channel channel, String gatewayId, int index) {
        ByteBuf buf = ByteBufUtil.directBuffer();
        buf.writeByte(EquipmentConfigDecoder.DELETE_ONE);
        buf.writeShortLE(index);
        byte[] cmd = ByteBufUtil.getBytesRemaining(buf);

        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.EQUIPMENT_CONFIG, cmd);
    }

    public static void equipmentConfigDeleteAll(Channel channel, String gatewayId) {
        ByteBuf buf = ByteBufUtil.directBuffer();
        buf.writeByte(EquipmentConfigDecoder.DELETE_ALL);
        buf.writeShortLE(0);
        byte[] cmd = ByteBufUtil.getBytesRemaining(buf);
        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.EQUIPMENT_CONFIG, cmd);
    }

    /**
     * 读测点配置信息
     */
    public static void equipmentMonitoringRead(Channel channel, String gatewayId, int index) {
        ByteBuf buf = ByteBufUtil.directBuffer();
        buf.writeByte(EquipmentMonitoringDecoder.READ_ONE);
        buf.writeShortLE(index);
        byte[] cmd = ByteBufUtil.getBytesRemaining(buf);
        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.EQUIPMENT_MONITORING_CONFIG, cmd);
    }

    /**
     * 读10测点配置信息
     */
    public static void equipmentMonitoringReadTen(Channel channel, String gatewayId, int page) {
        ByteBuf buf = ByteBufUtil.directBuffer();
        buf.writeByte(EquipmentMonitoringDecoder.READ_TEN);
        buf.writeShortLE(page);
        byte[] cmd = ByteBufUtil.getBytesRemaining(buf);

        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.EQUIPMENT_MONITORING_CONFIG, cmd);
    }

    /**
     * 写入1条测点配置信息
     */
    public static void equipmentMonitoringWrite(Channel channel, String gatewayId, int index, EquipmentMonitoring config) {
        ByteBuf buf = ByteBufUtil.directBuffer();
        buf.writeByte(EquipmentMonitoringDecoder.WRITE_ONE);
        buf.writeShortLE(index);
        buf.writeBytes(EquipmentMonitoringBuilder.toBytes(config));
        byte[] cmd = ByteBufUtil.getBytesRemaining(buf);

        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.EQUIPMENT_MONITORING_CONFIG, cmd);
    }

    /**
     * 删除1条测点配置信息
     */
    public static void equipmentMonitoringDelete(Channel channel, String gatewayId, int index) {
        ByteBuf buf = ByteBufUtil.directBuffer();
        buf.writeByte(EquipmentMonitoringDecoder.DELETE_ONE);
        buf.writeShortLE(index);
        byte[] cmd = ByteBufUtil.getBytesRemaining(buf);

        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.EQUIPMENT_MONITORING_CONFIG, cmd);
    }

    public static void equipmentMonitoringDeleteAll(Channel channel, String gatewayId) {
        ByteBuf buf = ByteBufUtil.directBuffer();
        buf.writeByte(EquipmentMonitoringDecoder.DELETE_ALL);
        buf.writeShortLE(0);
        byte[] cmd = ByteBufUtil.getBytesRemaining(buf);

        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.EQUIPMENT_MONITORING_CONFIG, cmd);
    }

    /**
     * 测点参数写入
     *
     * @param channel
     * @param gatewayId
     * @param registerAddress 寄存器地址，2字节(大端模式)
     * @param data 数据
     */
    public static void equipmentMonitoringParamWrite(Channel channel, String gatewayId, short configIndex,
                                                     short dataIndex, int registerAddress, byte[] data) {
//        ByteBuf buf = ByteBufUtil.directBuffer();
//        buf.writeByte(EquipmentMonitoringDecoder.WRITE_DATA);
//        buf.writeShort(configIndex);
//        buf.writeShort(dataIndex);
//        buf.writeShort(registerAddress);
//        buf.writeBytes(new byte[]{0x00, 0x00});
//        buf.writeBytes(data);
//        byte[] cmd = ByteBufUtil.getBytesRemaining(buf);


        byte[] commandData = new byte[9 + data.length];
        commandData[0] = EquipmentMonitoringDecoder.WRITE_DATA;
        ByteUtil.writeUnsignedShort(commandData, 1, configIndex, ByteOrder.LITTLE_ENDIAN);
        ByteUtil.writeUnsignedShort(commandData, 3, dataIndex, ByteOrder.LITTLE_ENDIAN);
        ByteUtil.writeUnsignedShort(commandData, 5, registerAddress, ByteOrder.LITTLE_ENDIAN);
        commandData[7] = 0x00;
        commandData[8] = 0x00;
        ByteUtil.writeBytes(commandData, 9, data);

        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.EQUIPMENT_MONITORING_CONFIG, commandData);
    }

    //-------- 报警配置 -------
    /**
     * 读配置信息
     */
    public static void warningConfigRead(Channel channel, String gatewayId, int index) {
        ByteBuf buf = ByteBufUtil.directBuffer();
        buf.writeByte(WarningConfigDecoder.READ_ONE);
        buf.writeShortLE(index);
        byte[] cmd = ByteBufUtil.getBytesRemaining(buf);
        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.WARNING_CONFIG, cmd);
    }

    /**
     * 读10配置信息
     */
    public static void warningConfigReadTen(Channel channel, String gatewayId, int page) {
        ByteBuf buf = ByteBufUtil.directBuffer();
        buf.writeByte(WarningConfigDecoder.READ_TEN);
        buf.writeShortLE(page);
        byte[] cmd = ByteBufUtil.getBytesRemaining(buf);

        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.WARNING_CONFIG, cmd);
    }

    /**
     * 写入1条配置信息
     */
    public static void warningConfigWrite(Channel channel, String gatewayId, int index, WarningConfig config) {
        ByteBuf buf = ByteBufUtil.directBuffer();
        buf.writeByte(WarningConfigDecoder.WRITE_ONE);
        buf.writeShortLE(index);
        buf.writeBytes(WarningConfigBuilder.toBytes(config));
        byte[] cmd = ByteBufUtil.getBytesRemaining(buf);

        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.WARNING_CONFIG, cmd);
    }

    /**
     * 删除1条配置信息
     */
    public static void warningConfigDelete(Channel channel, String gatewayId, int index) {
        ByteBuf buf = ByteBufUtil.directBuffer();
        buf.writeByte(WarningConfigDecoder.DELETE_ONE);
        buf.writeShortLE(index);
        byte[] cmd = ByteBufUtil.getBytesRemaining(buf);

        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.WARNING_CONFIG, cmd);
    }

    public static void warningConfigDeleteAll(Channel channel, String gatewayId) {
        ByteBuf buf = ByteBufUtil.directBuffer();
        buf.writeByte(WarningConfigDecoder.DELETE_ALL);
        buf.writeShortLE(0);
        byte[] cmd = ByteBufUtil.getBytesRemaining(buf);
        sendCommand(channel, DeviceType.GATEWAY, gatewayId, Command.WARNING_CONFIG, cmd);
    }

    // ============================================================================================

    /**
     * 发送指令
     *
     * @param channel 会话
     * @param deviceType 目标终端类型
     * @param deviceId 目标终端编号
     * @param command 命令
     * @param commandData 数据
     */
    public static void sendCommand(Channel channel, DeviceType deviceType, String deviceId,
                                   byte command, byte[] commandData) {
        CodecLogKafkaSender codecLogSender = SpringUtil.getBean(CodecLogKafkaSender.class);

        if (channel == null) {
            CodecLogMessage logMessage = CodecLogMessage.downstreamMessage("null", deviceId, "",
                "指令发送失败, 连接已丢失");
            logger.debug(logMessage.getMessage());
            if (codecLogSender != null) {
                codecLogSender.sendMessage(logMessage);
            }
            return;
        }

        Device device = channel.attr(AttrKey.MASTER_DEVICE).get();

        if (!channel.isWritable()) {
            CodecLogMessage logMessage = CodecLogMessage.downstreamMessage(channel.id().asShortText(), deviceId, "",
                "指令发送失败, 连接不可写入");
            logger.debug(logMessage.getMessage());
            if (codecLogSender != null && device != null) {
                codecLogSender.sendMessage(logMessage);
            }
            return;
        }

        if (commandData==null) {
            commandData = Command.EMPTY_DATA;
        }

        ByteBuf commandBuf = ByteBufUtil.directBuffer(PacketV2.MIN_LENGTH + commandData.length);
        if (Boolean.TRUE.equals(channel.attr(AttrKey.PACKET_STA).get())) {
            commandBuf.writeBytes(PacketV2.PREFIX_STA);
        } else {
            commandBuf.writeBytes(PacketV2.PREFIX_IOT);
        }
        commandBuf.writeByte(deviceType.getValue());
        commandBuf.writeIntLE(Integer.parseInt(deviceId));
        commandBuf.writeByte(DeviceType.SERVER.getValue());
        commandBuf.writeIntLE(-1);
        commandBuf.writeByte(command);
        commandBuf.writeShortLE((short) commandData.length);
        if (commandData.length > 0) {
            commandBuf.writeBytes(commandData);
        }

        // CRC校验数据域
        int crc = CRCUtil.crc16(commandData);
        commandBuf.writeShortLE(crc);
        commandBuf.writeBytes(PacketV2.SUFFIX_END);

        byte[] commandBytes = ByteBufUtil.getBytesRemaining(commandBuf);

        CodecLogMessage logMessage = CodecLogMessage.downstreamMessage(channel.id().asShortText(), deviceId, "",
            StringUtil.format("平台指令[{}] = {}", commandBytes.length, ByteUtil.toHexString(commandBytes, true)));
        logger.debug(logMessage.getMessage());

        if (codecLogSender != null && device != null) {
            codecLogSender.sendMessage(logMessage);
        }

        channel.writeAndFlush(commandBuf).addListener(ChannelFutureListener.CLOSE_ON_FAILURE);
    }

}
