package com.qhiot.transport.codec.util;

import com.google.common.cache.CacheBuilder;
import com.google.common.cache.CacheLoader;
import com.google.common.cache.LoadingCache;
import com.qhiot.core.util.StringUtil;

import java.util.Collection;
import java.util.Iterator;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.TimeUnit;

/**
 * 传输层内部阻塞队列管理器
 *
 * @author 金建强(ptma@163.com)
 */
public class InnerQueueManager {

    private final LoadingCache<String, InnerQueue> queuesCache;
    private final static int                       EXPIRATION = 600;

    private static class InnerQueueManagerHolder {
        final static InnerQueueManager INSTANCE = new InnerQueueManager();
    }

    private InnerQueueManager() {
        // 指定一个如果数据不存在获取数据的方法
        CacheLoader<String, InnerQueue> cacheLoader = new CacheLoader<String, InnerQueue>() {
            @Override
            public InnerQueue load(String key) throws Exception {
                return new InnerQueue();
            }
        };

        queuesCache =
                CacheBuilder.newBuilder().expireAfterAccess(EXPIRATION, TimeUnit.SECONDS)
                        .build(cacheLoader);
    }

    public static InnerQueueManager getInstance() {
        return InnerQueueManagerHolder.INSTANCE;
    }

    public static InnerQueue getQueue(String queueName) {
        return InnerQueueManager.getInstance().get(queueName);
    }

    public static InnerQueue getAndClearQueue(String queueName) {
        return InnerQueueManager.getInstance().getAndClear(queueName);
    }

    public static InnerQueue getQueue(String template, Object... values) {
        return getQueue(StringUtil.format(template, values));
    }

    public static InnerQueue getAndClearQueue(String template, Object... values) {
        return getAndClearQueue(StringUtil.format(template, values));
    }

    public InnerQueue get(String queueName) {
        try {
            return queuesCache.get(queueName);
        } catch (Exception e) {

        }
        InnerQueue queue = new InnerQueue();
        queuesCache.put(queueName, queue);
        return queue;
    }

    public InnerQueue getAndClear(String queueName) {
        InnerQueue queue = get(queueName);
        queue.clear();
        return queue;
    }

    public static class InnerQueue extends LinkedBlockingQueue<Object> {

        private long            timeMillis;

        public InnerQueue() {
            super(Integer.MAX_VALUE);
            timeMillis = System.currentTimeMillis();
        }

        public InnerQueue(int capacity) {
            super(capacity);
            timeMillis = System.currentTimeMillis();
        }

        public InnerQueue(Collection<Object> c) {
            super(c);
            timeMillis = System.currentTimeMillis();
        }

        public long getTimeMillis() {
            return timeMillis;
        }

        @Override
        public void put(Object e) throws InterruptedException {
            timeMillis = System.currentTimeMillis();
            super.put(e);
        }

        @Override
        public boolean offer(Object e, long timeout, TimeUnit unit) throws InterruptedException {
            timeMillis = System.currentTimeMillis();
            return super.offer(e, timeout, unit);
        }

        @Override
        public Object take() throws InterruptedException {
            timeMillis = System.currentTimeMillis();
            return super.take();
        }

        @Override
        public boolean add(Object o) {
            timeMillis = System.currentTimeMillis();
            return super.add(o);
        }

        @Override
        public boolean offer(Object e) {
            timeMillis = System.currentTimeMillis();
            return super.offer(e);
        }

        public boolean offerTrue() {
            timeMillis = System.currentTimeMillis();
            return super.offer(Boolean.TRUE);
        }

        @Override
        public Object remove() {
            timeMillis = System.currentTimeMillis();
            return super.remove();
        }

        @Override
        public Object poll(long timeout, TimeUnit unit) throws InterruptedException {
            timeMillis = System.currentTimeMillis();
            return super.poll(timeout, unit);
        }

        @Override
        public int remainingCapacity() {
            timeMillis = System.currentTimeMillis();
            return super.remainingCapacity();
        }

        @Override
        public boolean remove(Object o) {
            timeMillis = System.currentTimeMillis();
            return super.remove(o);
        }

        @Override
        public boolean containsAll(Collection<?> c) {
            timeMillis = System.currentTimeMillis();
            return super.containsAll(c);
        }

        @Override
        public boolean addAll(Collection<?> c) {
            timeMillis = System.currentTimeMillis();
            return super.addAll(c);
        }

        @Override
        public boolean removeAll(Collection<?> c) {
            timeMillis = System.currentTimeMillis();
            return super.removeAll(c);
        }

        @Override
        public boolean retainAll(Collection<?> c) {
            timeMillis = System.currentTimeMillis();
            return super.retainAll(c);
        }

        @Override
        public void clear() {
            timeMillis = System.currentTimeMillis();
            super.clear();
        }

        @Override
        public int size() {
            timeMillis = System.currentTimeMillis();
            return super.size();
        }

        @Override
        public boolean isEmpty() {
            timeMillis = System.currentTimeMillis();
            return super.isEmpty();
        }

        @Override
        public boolean contains(Object o) {
            timeMillis = System.currentTimeMillis();
            return super.contains(o);
        }

        @Override
        public Iterator<Object> iterator() {
            timeMillis = System.currentTimeMillis();
            return super.iterator();
        }

        @Override
        public Object[] toArray() {
            timeMillis = System.currentTimeMillis();
            return super.toArray();
        }

        @Override
        public <T> T[] toArray(T[] a) {
            timeMillis = System.currentTimeMillis();
            return super.toArray(a);
        }

        @Override
        public int drainTo(Collection<? super Object> c) {
            timeMillis = System.currentTimeMillis();
            return super.drainTo(c);
        }

        @Override
        public int drainTo(Collection<? super Object> c, int maxElements) {
            timeMillis = System.currentTimeMillis();
            return super.drainTo(c, maxElements);
        }

        @Override
        public Object poll() {
            timeMillis = System.currentTimeMillis();
            return super.poll();
        }

        @Override
        public Object element() {
            timeMillis = System.currentTimeMillis();
            return super.element();
        }

        @Override
        public Object peek() {
            timeMillis = System.currentTimeMillis();
            return super.peek();
        }
    }
}
