package com.qhiot.transport.codec.util;

import com.qhiot.core.util.DateUtil;
import com.qhiot.core.util.SpringUtil;
import com.qhiot.core.util.StringUtil;
import com.qhiot.core.util.ThreadUtil;
import com.qhiot.transport.codec.util.UpgradeThread.UpgradeResult;
import com.qhiot.transport.db.model.TransportUpgradeLog;
import com.qhiot.transport.db.service.TransportUpgradeLogService;
import com.qhiot.transport.db.service.TransportUpgradeService;
import com.qhiot.transport.message.request.UpgradeRequest;
import com.qhiot.transport.netty.handler.ChannelManager;
import io.netty.channel.Channel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Iterator;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

/**
 * 固件升级线程管理器
 *
 * @author 金建强(ptma@163.com)
 */
public class UgradeThreadManager {

    private static final Logger LOGGER                       =
        LoggerFactory.getLogger("transport.codec.UpgradeThread");

    private static class UgradeThreadManagerHolder {
        final static UgradeThreadManager INSTANCE = new UgradeThreadManager();
    }

    private Map<String, UpgradeThread> threads = new ConcurrentHashMap<String, UpgradeThread>();

    private Map<String, UpgradeRequest> requestWatchList = new ConcurrentHashMap<String, UpgradeRequest>();

    private ScheduledExecutorService service = null;
    private ExecutorService threadPool;

    private UgradeThreadManager(){
        threadPool = ThreadUtil.newCachedThreadPool(10, 500, "seem-firmwire-upgrade-%d");

        service = ThreadUtil.newSingleScheduledExecutor("seem-firmwire-upgrade-timeout-%d");
        service.scheduleAtFixedRate(new Scanner(requestWatchList), 5, 5, TimeUnit.MINUTES);
    }

    public static UgradeThreadManager getInstance() {
        return UgradeThreadManagerHolder.INSTANCE;
    }

    public UpgradeThread getThread(byte deviceType, String deviceId) {
        String deviceKey = StringUtil.format("{}-{}", deviceType, deviceId);
        return threads.get(deviceKey);
    }

    public void removeUgradeThread(byte deviceType, String deviceId) {
        String deviceKey = StringUtil.format("{}-{}", deviceType, deviceId);
        threads.remove(deviceKey);
    }

    public void upgrade(UpgradeRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());

        if (channel == null) {
            return;
        }

        String deviceKey = StringUtil.format("{}-{}", request.getDeviceType(), request.getDeviceId());
        UpgradeThread upgradeThread = threads.get(deviceKey);
        if (upgradeThread == null) {
            requestWatchList.put(deviceKey, request);
            upgradeThread = new UpgradeThread(channel, request);
            upgradeThread.setRetryCount(request.getRetryCount());
            threads.put(deviceKey, upgradeThread);
            threadPool.submit(upgradeThread);
        }
    }


    static class Scanner implements Runnable {

        private Map<String, UpgradeRequest> requestWatchList;

        public Scanner(Map<String, UpgradeRequest> requestWatchList) {
            this.requestWatchList = requestWatchList;
        }

        @Override
        public void run() {
            try {
                Iterator<Map.Entry<String, UpgradeRequest>> itr = requestWatchList.entrySet().iterator();
                while (itr.hasNext()) {
                    Map.Entry<String, UpgradeRequest> entry = itr.next();
                    UpgradeRequest request = entry.getValue();
                    if (request.getStartupTime() == null) {
                        itr.remove();
                    } else {
                        //超时时间为 0 表示永不超时
                        if (request.getTimeout()>0 && DateUtil.diffMinutes(request.getStartupTime(), DateUtil.now()) > request.getTimeout()) {
                            itr.remove();
                            saveUpgradeLog(request);
                        }
                    }
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }

        private void saveUpgradeLog(UpgradeRequest request) {
            try {
                String deviceId = String.valueOf(request.getDeviceId());
                byte deviceType = request.getDeviceType();

                UgradeThreadManager.getInstance().removeUgradeThread(deviceType, deviceId);

                TransportUpgradeService upgradeService = SpringUtil.getBean(TransportUpgradeService.class);
                if (upgradeService != null) {
                    upgradeService.deleteByDevice(deviceId, deviceType);
                }


                TransportUpgradeLog transportUpgradeLog = new TransportUpgradeLog();
                transportUpgradeLog.setDeviceId(deviceId);
                transportUpgradeLog.setDeviceType(deviceType);
                transportUpgradeLog.setStartTime(request.getStartupTime());
                transportUpgradeLog.setCompleteTime(DateUtil.now());
                transportUpgradeLog.setUpgradeResult(UpgradeResult.FAIL.ordinal());
                transportUpgradeLog.setResultMessage("任务超时");
                transportUpgradeLog.setFirmware(request.getFirmwireData());

                transportUpgradeLog.setIgnoreErrors(request.isIgnoreErrors()?0:1);
                transportUpgradeLog.setFirmwareId(request.getFirmwareId());
                transportUpgradeLog.setSegmentLength(request.getSegmentLength());

                TransportUpgradeLogService upgradeLogService = SpringUtil.getBean(TransportUpgradeLogService.class);
                if (upgradeLogService != null) {
                    upgradeLogService.insert(transportUpgradeLog);
                }

            } catch (Exception e) {
                LOGGER.error(e.getMessage(), e);
            }
        }
    }
}
