package com.qhiot.transport.codec.util;

import com.qhiot.core.util.DateUtil;
import com.qhiot.core.util.SpringUtil;
import com.qhiot.core.util.StringUtil;
import com.qhiot.core.util.ThreadUtil;
import com.qhiot.transport.codec.bean.v2.FirmwareRequest;
import com.qhiot.transport.codec.bean.v2.FirmwareSegment;
import com.qhiot.transport.db.model.TransportUpgradeLog;
import com.qhiot.transport.db.service.TransportUpgradeLogService;
import com.qhiot.transport.db.service.TransportUpgradeService;
import com.qhiot.transport.jms.kafka.CodecLogKafkaSender;
import com.qhiot.transport.jms.kafka.TransportResponseKafkaSender;
import com.qhiot.transport.message.CodecLogMessage;
import com.qhiot.transport.message.define.DeviceType;
import com.qhiot.transport.message.define.UpgradeCode;
import com.qhiot.transport.message.request.UpgradeRequest;
import com.qhiot.transport.message.response.UpgradeProgress;
import com.qhiot.transport.message.response.UpgradeResponse;
import io.netty.channel.Channel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.concurrent.TimeUnit;

/**
 * 固件更新线程， 支持断点续传
 *
 * @author 金建强(ptma @ 163.com)
 */
public class UpgradeThread implements Runnable {

    private static final Logger LOGGER =
            LoggerFactory.getLogger("transport.codec.UpgradeThread");

    private static final int UPGRADE_REQUEST_WAIT_SECONDS = 5;

    private final FirmwareRequest firmware;

    private final Long firmwareId;
    private final Integer segmentLength;

    private final Channel channel;
    private final UpgradeRequest request;
    //    private TransportResponseSender transportResponseSender;
    private final TransportResponseKafkaSender transportResponseKafkaSender;
    private boolean abort = false;
    private int retryCount = 30;

    public UpgradeThread(Channel channel, UpgradeRequest request) {
        this.channel = channel;
        this.request = request;
        this.firmwareId = request.getFirmwareId();
        this.segmentLength = request.getSegmentLength();
        firmware = new FirmwareRequest(request.isIgnoreErrors(), DeviceType.valueOf(request.getDeviceType()),
                request.getDeviceId(), request.getFirmwireData(), request.getSegmentLength());
        firmware.setSegmentIndex(request.getSegmentIndex());
        transportResponseKafkaSender = SpringUtil.getBean(TransportResponseKafkaSender.class);
    }

    @Override
    public void run() {
        debug("固件升级: 开始升级");
        if (request.getStartupTime() == null) {
            request.setStartupTime(DateUtil.now());
            ThreadUtil.submit(new Runnable() {
                @Override
                public void run() {
                    try {
                        saveStartupTime();
                    } catch (Exception e) {
                        LOGGER.error("记录升级开始时间出错: {}", e.getMessage(), e);
                    }
                }
            });
        }
        try {
            if (firmware.getSegmentIndex() == 0) {
                sendRequest();
            } else {
                sendAllSegment();
            }
            removeFromThreadManager();
        } catch (Exception e) {
            // 手动 cancelUpgrade 时会产生 InterruptedException, 所以应该忽略该异常
            if (!(e instanceof InterruptedException)) {
                LOGGER.error("升级失败: " + e.getMessage(), e);
                debug("固件升级: 升级失败:{}", e.getMessage());
                finishUpgrade(UpgradeResult.FAIL, "升级失败: " + e.getMessage());
            }
        }
    }

    /**
     * 发送更新请求
     */
    private void sendRequest() throws Exception {
        int count = 0;
        while (!abort && count < retryCount) {
            if (channel == null || !channel.isActive() || !channel.isOpen()) {
                debug("固件升级: 中断升级, 连接丢失或无效!");
                return;
            }
            UpgradeQueue queue = UpgradeQueueManager.getAndClearQueue(UpgradeQueue.FIRMWARE_REQUEST_QUEUE, firmware.getDeviceId());
            debug("固件升级: 升级请求, 第 {} 次发送", count + 1);
            CommandUtil.writeFirmwareUpgradeRequest(channel, firmware);
            UpgradeResponse upgradeResponse = (UpgradeResponse) queue.poll(UPGRADE_REQUEST_WAIT_SECONDS, TimeUnit.SECONDS);

            if (upgradeResponse == null) {
                // 请求超时无响应
                if (count >= retryCount - 1) {
                    // 达到最多重试次数
                    debug("固件升级: 中断升级, 达到最多重试次数!");
                    return;
                } else {
                    count++;
                    continue;
                }
            } else {
                // 收到响应
                UpgradeCode upgradeCode = UpgradeCode.valueOf(upgradeResponse.getUpgradeCode());
                if (upgradeCode.equals(UpgradeCode.ENABLED)) {
                    // 允许更新， 表示终端已接受升级请求
                    // 开始发送分片数据包
                    sendAllSegment();
                    // 发送完分片后, 需要退出方法, 避免再次进入循环
                    return;
                } else {
                    // 其它响应代码直接抛出异常，升级失败
                    throw new Exception(upgradeCode.toString());
                }
            }
        }
    }

    private void sendAllSegment() throws Exception {
        while (!abort && firmware.hasNextSegment()) {
            sendSegment(firmware.getCurrentSegment());
        }
        if (abort) {
            debug("固件升级: 中断升级!");
            abortUpgradeAndSaveSegmentIndex(firmware.getSegmentIndex());
        }
    }

    /**
     * 发送分片数据包
     */
    private void sendSegment(FirmwareSegment segment) throws Exception {
        int count = 0;
        while (!abort && count < retryCount) {
            if (channel == null || !channel.isActive() || !channel.isOpen()) {
                debug("固件升级: 连接丢失或无效!");
                abortUpgradeAndSaveSegmentIndex(firmware.getSegmentIndex());
                return;
            }
            UpgradeQueue queue = UpgradeQueueManager.getAndClearQueue(UpgradeQueue.FIRMWARE_SEGMENT_QUEUE, firmware.getDeviceId());
            debug("固件升级: 分片 {}/{}, 第 {} 次发送", segment.getSegmentIndex(), firmware.getTotalSegment(), count + 1);
            CommandUtil.writeFirmwareUpgradeSegment(channel, segment);

            UpgradeResponse segmentResponse = (UpgradeResponse) queue.poll(UPGRADE_REQUEST_WAIT_SECONDS, TimeUnit.SECONDS);

            if (segmentResponse == null) {
                // 请求超时无响应
                if (count >= retryCount - 1) {
                    // 达到最多重试次数
                    debug("固件升级: 达到最多重试次数!");
                    abortUpgradeAndSaveSegmentIndex(firmware.getSegmentIndex());
                    return;
                } else {
                    count++;
                    continue;
                }
            }

            // 收到响应
            UpgradeCode upgradeCode = UpgradeCode.valueOf(segmentResponse.getUpgradeCode());
            // 允许更新， 或者 （终端类型错误， 并设置为忽略的）
            boolean gatewayTypeErrorAndIgnore = upgradeCode.equals(UpgradeCode.GATEWAY_TYPE_ERROR) && firmware.isIgnoreErrors();
            if (upgradeCode.equals(UpgradeCode.ENABLED) || gatewayTypeErrorAndIgnore) {
                // 允许更新(表示终端已收到当前分片)
                if (transportResponseKafkaSender != null) {
                    // 报告进度
                    int progressVal = segment.getSegmentIndex() * 100 / firmware.getTotalSegment();
                    UpgradeProgress progress = new UpgradeProgress(request.getDeviceType(), request.getDeviceId());
                    progress.setProgress(progressVal);
                    progress.setCompleted(segment.getSegmentIndex() == firmware.getTotalSegment());
                    transportResponseKafkaSender.sendMessage(progress);
                }

                // 最后一个分片包， 升级成功
                if (segment.getSegmentIndex() == firmware.getTotalSegment()) {
                    debug("固件升级: 升级结束!");
                    finishUpgrade(UpgradeResult.SUCCESS, "升级成功");
                }

                // 新的协议终端会返回确认收到的分片序号, 那么
                if (segmentResponse.getSegmentIndex() != null) {
                    // 不能 +1 ， 因为协议里的序号是从 1 开始的
                    firmware.moveToSegment(segmentResponse.getSegmentIndex());
                } else {
                    firmware.moveToNextSegment();
                }
                // 结束当前分片的发送(已发送)
                return;
            } else {
                // 其它响应代码直接抛出异常，升级失败
                throw new Exception(upgradeCode.toString());
            }
        }
    }

    /**
     * 手动取消升级
     */
    public void cancelUpgrade() {
        try {
            abort = true;
            debug("固件升级: 手动取消!");
            finishUpgrade(UpgradeResult.CANCEL, "手动取消");
        } catch (Exception ie) {

        }
    }

    /**
     * 中断升级
     */
    public void abortUpgrade() {
        abort = true;
    }

    /**
     * 记录升级开始时间
     */
    private void saveStartupTime() {
        try {
            TransportUpgradeService upgradeService = SpringUtil.getBean(TransportUpgradeService.class);
            if (upgradeService != null) {
                upgradeService.updateStartupTime(String.valueOf(firmware.getDeviceId()),
                        firmware.getDeviceType().getValue());
            }
        } catch (Exception e) {
            LOGGER.error(e.getMessage(), e);
        }
    }

    /**
     * 中断升级过程, 并记录分片序号
     */
    private void abortUpgradeAndSaveSegmentIndex(int index) {
        try {
            abort = true;

            String deviceId = String.valueOf(firmware.getDeviceId());
            byte deviceType = firmware.getDeviceType().getValue();

            UgradeThreadManager.getInstance().removeUgradeThread(deviceType, deviceId);

            if (index < 0) {
                index = 0;
            }
            TransportUpgradeService upgradeService = SpringUtil.getBean(TransportUpgradeService.class);
            if (upgradeService != null) {
                upgradeService.updateSegmentIndex(String.valueOf(firmware.getDeviceId()),
                        firmware.getDeviceType().getValue(), index);
            }
        } catch (Exception e) {
            LOGGER.error(e.getMessage(), e);
        }
    }

    /**
     * 将升级线程移出线程管理器
     */
    private void removeFromThreadManager() {
        String deviceId = String.valueOf(firmware.getDeviceId());
        byte deviceType = firmware.getDeviceType().getValue();

        UgradeThreadManager.getInstance().removeUgradeThread(deviceType, deviceId);
    }

    /**
     * 结束升级, 保存升级日志， 并删除升级任务
     */
    private void finishUpgrade(UpgradeResult upgradeResult, String upgradeMessage) {
        try {
            String deviceId = String.valueOf(firmware.getDeviceId());
            byte deviceType = firmware.getDeviceType().getValue();

            removeFromThreadManager();

            TransportUpgradeService upgradeService = SpringUtil.getBean(TransportUpgradeService.class);
            if (upgradeService != null) {
                upgradeService.deleteByDevice(deviceId, deviceType);
            }


            TransportUpgradeLog transportUpgradeLog = new TransportUpgradeLog();
            transportUpgradeLog.setDeviceId(deviceId);
            transportUpgradeLog.setDeviceType(deviceType);
            transportUpgradeLog.setStartTime(request.getStartupTime());
            transportUpgradeLog.setCompleteTime(DateUtil.now());
            transportUpgradeLog.setUpgradeResult(upgradeResult.ordinal());
            transportUpgradeLog.setResultMessage(upgradeMessage);
            transportUpgradeLog.setFirmware(firmware.getFirmware());

            transportUpgradeLog.setFirmwareId(firmwareId);
            transportUpgradeLog.setSegmentLength(segmentLength);
            transportUpgradeLog.setIgnoreErrors(request.isIgnoreErrors() ? 0 : 1);

            TransportUpgradeLogService upgradeLogService = SpringUtil.getBean(TransportUpgradeLogService.class);
            if (upgradeLogService != null) {
                upgradeLogService.insert(transportUpgradeLog);
            }

        } catch (Exception e) {
            LOGGER.error(e.getMessage(), e);
        }
    }

    private void debug(String format, Object... arguments) {
        CodecLogMessage logMessage = CodecLogMessage.analyzeMessage(channel.id().asShortText(),
                String.valueOf(firmware.getDeviceId()), "", StringUtil.format(format, arguments));

        LOGGER.debug(logMessage.getMessage());
        CodecLogKafkaSender codecLogSender = SpringUtil.getBean(CodecLogKafkaSender.class);
        if (codecLogSender != null) {
            codecLogSender.sendMessage(logMessage);
        }
    }

    public void setRetryCount(int retryCount) {
        this.retryCount = retryCount > 0 ? retryCount : 30;
    }

    enum UpgradeResult {
        /**
         * 失败
         */
        FAIL,
        /**
         * 成功
         */
        SUCCESS,
        /**
         * 取消
         */
        CANCEL
    }
}
