package com.qhiot.transport.config;

import com.qhiot.transport.jms.kafka.*;
import com.qhiot.transport.netty.channel.NioUdpServerChannel;
import com.qhiot.transport.netty.handler.ChannelHandler;
import com.qhiot.transport.netty.handler.*;
import com.qhiot.transport.netty.server.SocketServer;
import com.qhiot.transport.netty.server.UdpServer;
import com.qhiot.transport.netty.transfer.TransferHandler;
import io.netty.bootstrap.Bootstrap;
import io.netty.bootstrap.ServerBootstrap;
import io.netty.buffer.ByteBufAllocator;
import io.netty.buffer.PooledByteBufAllocator;
import io.netty.channel.*;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.SocketChannel;
import io.netty.channel.socket.nio.NioServerSocketChannel;
import io.netty.channel.socket.nio.NioSocketChannel;
import io.netty.util.concurrent.DefaultThreadFactory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.DependsOn;
import org.springframework.context.annotation.ImportResource;
import org.springframework.core.annotation.Order;
import org.springframework.kafka.core.KafkaTemplate;

/**
 * 传输层服务器配置
 *
 * @author 金建强(ptma 163.com)
 * @version 2017 -04-13 19:53
 */
@Order(9999999)
@Configuration
//@EnableJms
@ImportResource(value = "classpath:spring/codec.xml")
public class TransportConfig {

    private static final Logger LOGGER = LoggerFactory.getLogger("Init.Transport.Server");

    @Value("${transport.bossCount:2}")
    private int bossCount;

    @Value("${transport.workerCount:4}")
    private int workerCount;

    @Value("${transport.port:61000}")
    private int transportPort;

    @Value("${transport.udp.port:62000}")
    private int udpPort;

    @Value("${transport.udp.bossCount:2}")
    private int udpBossCount;

    @Value("${transport.udp.workerCount:4}")
    private int udpWorkerCount;

    @Value("${transport.backlog:1024}")
    private int backlog;

    @Value("${transport.sndbuf:8192}")
    private int sndbuf;
    @Value("${transport.rcvbuf:8192}")
    private int rcvbuf;

    @Value("${transport.online.flush:5}")
    private int onlineFlush;

    // 缓冲区配置
    @Value("${transport.receive.buffer.minimum:64}")
    private int recvBufferMinimum;
    @Value("${transport.receive.buffer.initial:256}")
    private int recvBufferInitial;
    @Value("${transport.receive.buffer.maximum:1024}")
    private int recvBufferMaximum;

    @Value("${kafka.destinationPrefix:seem.}")
    private String destinationPrefix;

    @Bean(destroyMethod = "stop")
    public SocketServer server() throws Exception {
        LOGGER.info("tcpServer");
        SocketServer server = new SocketServer();
        server.setServerBootstrap(bootstrap());
        server.setPort(transportPort);
        return server;
    }

    @Bean(initMethod = "init", destroyMethod = "close")
    public OnlinePersistence onlinePersistence() {
        OnlinePersistence bean = new OnlinePersistence();
        bean.setFlushSecs(onlineFlush);
        return bean;
    }

    @Bean
    public ChannelHandler channelHandler() {
        ChannelHandler channelHandler = new ChannelHandler();
        return channelHandler;
    }

    @Bean
    public DefaultChannelInitializer channelInitalizer() {
        DefaultChannelInitializer protocolInitalizer = new DefaultChannelInitializer();
        protocolInitalizer.setChannelHandler(channelHandler());
        return protocolInitalizer;
    }

    @Bean(name = "serverBootstrap")
    public ServerBootstrap bootstrap() {
        ServerBootstrap bootstrap = new ServerBootstrap();
        bootstrap.group(bossGroup(), workerGroup());
        bootstrap.channel(NioServerSocketChannel.class);
        bootstrap.childHandler(channelInitalizer());

        bootstrap.option(ChannelOption.ALLOCATOR, new PooledByteBufAllocator(true, 16, 16, 8192, 11));
        bootstrap.childOption(ChannelOption.ALLOCATOR, new PooledByteBufAllocator(true, 16, 16, 8192, 11));
        RecvByteBufAllocator recvByteBufAllocator = new AdaptiveRecvByteBufAllocator(recvBufferMinimum,
                recvBufferInitial,
                recvBufferMaximum);
        bootstrap.childOption(ChannelOption.RCVBUF_ALLOCATOR, recvByteBufAllocator);

        bootstrap.childOption(ChannelOption.SO_SNDBUF, sndbuf);
        bootstrap.childOption(ChannelOption.SO_RCVBUF, rcvbuf);

        bootstrap.option(ChannelOption.SO_BACKLOG, backlog);
        // SO_LINT=0 后，在调用了close()方法后，不会再进入TIME_WAIT状态
        bootstrap.childOption(ChannelOption.SO_LINGER, 0);
        bootstrap.childOption(ChannelOption.TCP_NODELAY, true);
        bootstrap.childOption(ChannelOption.SO_KEEPALIVE, true);

        return bootstrap;
    }

    @Bean(name = "bossGroup", destroyMethod = "shutdownGracefully")
    public NioEventLoopGroup bossGroup() {
        return new NioEventLoopGroup(bossCount, new DefaultThreadFactory("TCP SERVER 1", true));
    }

    @Bean(name = "workerGroup", destroyMethod = "shutdownGracefully")
    public NioEventLoopGroup workerGroup() {
        return new NioEventLoopGroup(workerCount, new DefaultThreadFactory("TCP SERVER 2", true));
    }


    // UDP ==========================================
    @Bean(destroyMethod = "stop")
    public UdpServer udpServer() throws Exception {
        LOGGER.info("udpServer");
        UdpServer server = new UdpServer();
        server.setBootstrap(udpBootstrap());
        server.setPort(udpPort);
        return server;
    }

    @Bean(name = "udpBossGroup", destroyMethod = "shutdownGracefully")
    public NioEventLoopGroup udpBossGroup() {
        return new NioEventLoopGroup(udpBossCount, new DefaultThreadFactory("UDP SERVER 1", true));
    }

    @Bean(name = "udpWorkerGroup", destroyMethod = "shutdownGracefully")
    public EventLoopGroup udpWorkerGroup() {
        return new DefaultEventLoopGroup(udpWorkerCount, new DefaultThreadFactory("UDP SERVER 2", true));
    }

    @Bean(name = "udpBootstrap")
    public ServerBootstrap udpBootstrap() {
        ServerBootstrap bootstrap = new ServerBootstrap()
                .group(udpBossGroup(), udpWorkerGroup())
                .channel(NioUdpServerChannel.class);

        bootstrap.option(ChannelOption.ALLOCATOR, ByteBufAllocator.DEFAULT);
        bootstrap.childOption(ChannelOption.ALLOCATOR, ByteBufAllocator.DEFAULT);
        RecvByteBufAllocator recvByteBufAllocator = new AdaptiveRecvByteBufAllocator(recvBufferMinimum,
                recvBufferInitial,
                recvBufferMaximum);
        bootstrap.childOption(ChannelOption.RCVBUF_ALLOCATOR, recvByteBufAllocator);

        bootstrap.childOption(ChannelOption.SO_SNDBUF, sndbuf);
        bootstrap.childOption(ChannelOption.SO_RCVBUF, rcvbuf);

        bootstrap.option(ChannelOption.SO_BACKLOG, backlog);
        // SO_LINT=0 后，在调用了close()方法后，不会再进入TIME_WAIT状态
        bootstrap.childOption(ChannelOption.SO_LINGER, 0);
        bootstrap.childOption(ChannelOption.TCP_NODELAY, true);
        bootstrap.childOption(ChannelOption.SO_KEEPALIVE, true);

        bootstrap.childHandler(udpChannelInitalizer());

        return bootstrap;
    }

    @Bean("udpChannelInitalizer")
    public UdpChannelInitializer udpChannelInitalizer() {
        UdpChannelInitializer udpChannelInitalizer = new UdpChannelInitializer();
        udpChannelInitalizer.setUdpChannelHandler(udpChannelHandler());
        return udpChannelInitalizer;
    }

    @Bean
    public UdpChannelHandler udpChannelHandler() {
        return new UdpChannelHandler();
    }


    // 数据转发客户端， 共用一个 Bootstrap 和 NioEventLoopGroup;
    @Bean(name = {"bootstrap", "transferBootstrap"})
    public Bootstrap transferBootstrap() {
        Bootstrap bootstrap = new Bootstrap();
        bootstrap.group(new NioEventLoopGroup());
        bootstrap.option(ChannelOption.SO_KEEPALIVE, true);
        bootstrap.option(ChannelOption.TCP_NODELAY, true);
        bootstrap.option(ChannelOption.CONNECT_TIMEOUT_MILLIS, 3000);

        bootstrap.channel(NioSocketChannel.class);

        bootstrap.handler(new ChannelInitializer<SocketChannel>() {
            @Override
            public void initChannel(SocketChannel ch) throws Exception {
                ch.pipeline().addLast(new TransferHandler());
            }
        });

        return bootstrap;
    }

    @Autowired
    private KafkaTemplate kafkaTemplate;

    @Bean
    @DependsOn("kafkaTemplate")
    public CodecLogKafkaSender codecLogKafkaSender() {
        return new CodecLogKafkaSender(kafkaTemplate, destinationPrefix);
    }

    @Bean
    @DependsOn("kafkaTemplate")
    public ConnectionDataKafkaSender connectionDataKafkaSender() {
        return new ConnectionDataKafkaSender(kafkaTemplate,destinationPrefix);
    }

    @Bean
    @DependsOn("kafkaTemplate")
    public CoulometerDataKafkaSender coulometerDataKafkaSender() {
        return new CoulometerDataKafkaSender(kafkaTemplate,destinationPrefix);
    }

    @Bean
    @DependsOn("kafkaTemplate")
    public ElectricCabinetDataKafkaSender electricCabinetDataKafkaSender() {
        return new ElectricCabinetDataKafkaSender(kafkaTemplate,destinationPrefix);
    }

    @Bean
    @DependsOn("kafkaTemplate")
    public IccidDataKafkaSender iccidDataKafkaSender() {
        return new IccidDataKafkaSender(kafkaTemplate,destinationPrefix);
    }

    @Bean
    @DependsOn("kafkaTemplate")
    public InputStateDataKafkaSender inputStateDataKafkaSender() {
        return new InputStateDataKafkaSender(kafkaTemplate,destinationPrefix);
    }

    @Bean
    @DependsOn("kafkaTemplate")
    public MonitoringHistoryDataKafkaSender monitoringHistoryDataKafkaSender() {
        return new MonitoringHistoryDataKafkaSender(kafkaTemplate,destinationPrefix);
    }

    @Bean
    @DependsOn("kafkaTemplate")
    public OnboardTagDataKafkaSender onboardTagDataKafkaSender() {
        return new OnboardTagDataKafkaSender(kafkaTemplate,destinationPrefix);
    }

    @Bean
    @DependsOn("kafkaTemplate")
    public OnlineDataKafkaSender onlineDataKafkaSender() {
        return new OnlineDataKafkaSender(kafkaTemplate,destinationPrefix);
    }

    @Bean
    @DependsOn("kafkaTemplate")
    public OutputStateDataKafkaSender outputStateDataKafkaSender() {
        return new OutputStateDataKafkaSender(kafkaTemplate,destinationPrefix);
    }

    @Bean
    @DependsOn("kafkaTemplate")
    public PowerDataKafkaSender powerDataKafkaSender() {
        return new PowerDataKafkaSender(kafkaTemplate,destinationPrefix);
    }

    @Bean
    @DependsOn("kafkaTemplate")
    public Rs485DataKafkaSender rs485DataKafkaSender() {
        return new Rs485DataKafkaSender(kafkaTemplate,destinationPrefix);
    }

    @Bean
    @DependsOn("kafkaTemplate")
    public SnapDataBurstKafkaSender snapDataBurstKafkaSender() {
        return new SnapDataBurstKafkaSender(kafkaTemplate,destinationPrefix);
    }

    @Bean
    @DependsOn("kafkaTemplate")
    public Tag24gDataKafkaSender tag24gDataKafkaSender() {
        return new Tag24gDataKafkaSender(kafkaTemplate,destinationPrefix);
    }

    @Bean
    @DependsOn("kafkaTemplate")
    public Tag780mDataKafkaSender tag780mDataKafkaSender() {
        return new Tag780mDataKafkaSender(kafkaTemplate,destinationPrefix);
    }

    @Bean
    @DependsOn("kafkaTemplate")
    public TrafficDataKafkaSender trafficDataKafkaSender() {
        return new TrafficDataKafkaSender(kafkaTemplate,destinationPrefix);
    }


    @Bean
    @DependsOn("kafkaTemplate")
    public TransformerStateKafkaSender transformerStateKafkaSender() {
        return new TransformerStateKafkaSender(kafkaTemplate,destinationPrefix);
    }


    @Bean
    @DependsOn("kafkaTemplate")
    public TransportResponseKafkaSender transportResponseKafkaSender() {
        return new TransportResponseKafkaSender(kafkaTemplate,destinationPrefix);
    }


    @Bean
    @DependsOn("kafkaTemplate")
    public TransportWarningKafkaSender transportWarningKafkaSender() {
        return new TransportWarningKafkaSender(kafkaTemplate,destinationPrefix);
    }

    @Bean
    @DependsOn("kafkaTemplate")
    public WarningDataKafkaSender warningDataKafkaSender() {
        return new WarningDataKafkaSender(kafkaTemplate,destinationPrefix);
    }
}
