package com.qhiot.transport.jms.kafka.listener.handle;

import com.qhiot.core.util.SpringUtil;
import com.qhiot.transport.codec.FrameDecoder;
import com.qhiot.transport.codec.bean.AttrKey;
import com.qhiot.transport.codec.decoder.BaseDecoder;
import com.qhiot.transport.codec.decoder.PacketV1Decoder;
import com.qhiot.transport.codec.decoder.PacketV2Decoder;
import com.qhiot.transport.codec.decoder.v1.SensorDataDecoder;
import com.qhiot.transport.codec.decoder.v2.Rs485CollectionDecoder;
import com.qhiot.transport.codec.util.CommandUtil;
import com.qhiot.transport.codec.util.UgradeThreadManager;
import com.qhiot.transport.codec.util.UpgradeThread;
import com.qhiot.transport.jms.kafka.TransportResponseKafkaSender;
import com.qhiot.transport.message.define.DeviceType;
import com.qhiot.transport.message.request.*;
import com.qhiot.transport.message.request.camera.*;
import com.qhiot.transport.message.request.io.*;
import com.qhiot.transport.message.response.UpgradeCancelResponse;
import com.qhiot.transport.netty.handler.ChannelManager;
import com.qhiot.transport.netty.transfer.Transferrer;
import io.netty.channel.Channel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * 1 * @Author: luoj
 * 2 * @Date: 2021/5/27 16:00
 * 3
 */
@Component
public class HandleRequest {

    private static final Logger LOGGER = LoggerFactory.getLogger(HandleRequest.class);

    @Autowired
    private TransportResponseKafkaSender transportResponseKafkaSender;

    public void handleRequest(RequestMessage request) {
        LOGGER.error("未定义实际的请求相应方法 handleRequest({})", request.getClass().getName());
    }

    /**
     * 解码器变更请求， 来自解码器脚本被修改后
     *
     * @param request
     */
    public void handleRequest(DecoderChangedRequest request) {
        BaseDecoder decoder = null;
        if (PacketV2Decoder.DECODER_CATEGORY.equals(request.getCategory())) {
            decoder = SpringUtil.getBean(PacketV2Decoder.class);
        } else if (PacketV1Decoder.DECODER_CATEGORY.equals(request.getCategory())) {
            decoder = SpringUtil.getBean(PacketV1Decoder.class);
        } else if (Rs485CollectionDecoder.DECODER_CATEGORY.equals(request.getCategory())) {
            decoder = SpringUtil.getBean(Rs485CollectionDecoder.class);
        } else if (SensorDataDecoder.DECODER_CATEGORY.equals(request.getCategory())) {
            decoder = SpringUtil.getBean(SensorDataDecoder.class);
        } else if (FrameDecoder.DECODER_CATEGORY.equals(request.getCategory())) {
            decoder = SpringUtil.getBean(FrameDecoder.class);
        }

        if (decoder != null) {
            decoder.resetSubDecoders();
        }
    }


    /**
     * 终端参数读取请求
     *
     * @param request
     */
    public void handleRequest(DeviceParamReadRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.deviceParamRead(channel, DeviceType.valueOf(request.getDeviceType()), request.getDeviceId());
        }
    }

    /**
     * 终端参数写入请求
     *
     * @param request
     */
    public void handleRequest(GatewayParamWriteRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.gatewayParamWrite(channel, request.getDeviceId(), request.getParam());
        }
    }

    /**
     * 终端参数低级写入请求
     *
     * @param request
     */
    public void handleRequest(GatewayParamLowlevelWriteRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.gatewayParamLowLevelWrite(channel, request.getDeviceId(), request.getParam());
        }
    }

    /**
     * 终端参数写入请求
     *
     * @param request
     */
    public void handleRequest(DeviceParamWriteRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.deviceParamWrite(channel, DeviceType.valueOf(request.getDeviceType()), request.getDeviceId(), request.getParam());
        }
    }

    /**
     * 终端参数低级写入请求
     *
     * @param request
     */
    public void handleRequest(DeviceParamLowlevelWriteRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.deviceParamLowLevelWrite(channel, DeviceType.valueOf(request.getDeviceType()), request.getDeviceId(), request.getParam());
        }
    }

    /**
     * 终端重启请求
     *
     * @param request
     */
    public void handleRequest(DeviceRebootRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.deviceReboot(channel, DeviceType.valueOf(request.getDeviceType()), request.getDeviceId());
        }
    }

    /**
     * 终端重置请求
     *
     * @param request
     */
    public void handleRequest(DeviceRestoreRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.deviceRestore(channel, DeviceType.valueOf(request.getDeviceType()), request.getDeviceId());
        }
    }

    /**
     * 终端格式化请求
     *
     * @param request
     */
    public void handleRequest(DeviceFormatRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.deviceLowLevelFormat(channel, DeviceType.valueOf(request.getDeviceType()), request.getDeviceId());
        }
    }

    /**
     * 读短信请求
     *
     * @param request
     */
    public void handleRequest(SmsReadRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.smsRead(channel, request.getDeviceId());
        }
    }

    /**
     * 写入发送短信的手机号请求
     *
     * @param request
     */
    public void handleRequest(SmsMobileRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.smsSendMobile(channel, request.getDeviceId(), request.getMobile());
        }
    }

    /**
     * 写入发送短信的内容请求
     *
     * @param request
     */
    public void handleRequest(SmsContentRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.smsSendContent(channel, request.getDeviceId(), request.getContent());
        }
    }

    /**
     * 摄像头配置读取请求
     *
     * @param request
     */
    public void handleRequest(CameraConfigReadRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.cameraConfigRead(channel, request.getDeviceId(), request.getCameraIndex());
        }
    }

    /**
     * 摄像头配置读取全部请求
     *
     * @param request
     */
    public void handleRequest(CameraConfigReadAllRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.cameraConfigReadAll(channel, request.getDeviceId());
        }
    }

    /**
     * 摄像头配置写入请求
     *
     * @param request
     */
    public void handleRequest(CameraConfigWriteRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.cameraConfigWrite(channel, request.getDeviceId(), request.getCameraIndex(), request.getCameraConfig());
        }
    }

    /**
     * 摄像头配置删除请求
     *
     * @param request
     */
    public void handleRequest(CameraConfigDeleteRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.cameraConfigDelete(channel, request.getDeviceId(), request.getCameraIndex());
        }
    }

    /**
     * 摄像头配置删除全部请求
     *
     * @param request
     */
    public void handleRequest(CameraConfigDeleteAllRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.cameraConfigDeleteAll(channel, request.getDeviceId());
        }
    }

    /**
     * 拍照请求
     */
    public void handleRequest(TakePictureRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.takePicture(channel, request.getDeviceId(), request.getCameraIndex());
        }
    }

    /**
     * 停止照片上传请求
     *
     * @param request
     */
    public void handleRequest(StopUploadRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.stopPictureUploading(channel, request.getDeviceId(), request.getCameraIndex());
        }
    }

    /**
     * 摄像头周期配置读取请求
     *
     * @param request
     */
    public void handleRequest(CameraTimePointConfigReadRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.cameraTimePointConfigRead(channel, request.getDeviceId(), request.getCameraIndex());
        }
    }

    /**
     * 摄像头周期配置读取全部请求
     *
     * @param request
     */
    public void handleRequest(CameraTimePointConfigReadAllRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.cameraTimePointConfigReadAll(channel, request.getDeviceId());
        }
    }

    /**
     * 摄像头周期配置写入请求
     *
     * @param request
     */
    public void handleRequest(CameraTimePointConfigWriteRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.cameraTimePointConfigWrite(channel, request.getDeviceId(), request.getCameraIndex(), request.getCameraTimePointConfig());
        }
    }

    /**
     * 摄像头周期配置删除请求
     *
     * @param request
     */
    public void handleRequest(CameraTimePointConfigDeleteRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.cameraTimePointConfigDelete(channel, request.getDeviceId(), request.getCameraIndex());
        }
    }

    /**
     * 摄像头周期配置删除全部请求
     *
     * @param request
     */
    public void handleRequest(CameraTimePointConfigDeleteAllRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.cameraTimePointConfigDeleteAll(channel, request.getDeviceId());
        }
    }

    /**
     * 读取历史数据请求
     *
     * @param request
     */
    public void handleRequest(HistoryDataReadRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.historyDataRead(channel, request.getDeviceId(), request.getOffset(), request.getLimit());
        }
    }

    /**
     * 删除历史数据请求
     *
     * @param request
     */
    public void handleRequest(HistoryDataDeleteRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.historyDataDelete(channel, request.getDeviceId());
        }
    }

    /**
     * 读取历史数据条数请求
     *
     * @param request
     */
    public void handleRequest(HistoryDataCountRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.historyDataDelete(channel, request.getDeviceId());
        }
    }

    // State
    public void handleRequest(InputStateReadRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.switchInputStateRead(channel, request.getDeviceId());
        }
    }

    public void handleRequest(OutputStateReadRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.switchOutputStateRead(channel, request.getDeviceId());
        }
    }

    public void handleRequest(TransformerStateReadRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.transformerStateRead(channel, request.getDeviceId());
        }
    }

    public void handleRequest(CoulometerReadRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.coulometerRead(channel, request.getDeviceId());
        }
    }

    // Work Model
    public void handleRequest(IOWorkModelReadRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.ioWorkModelRead(channel, request.getDeviceId());
        }
    }

    public void handleRequest(IOWorkModelWriteRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.ioWorkModelWrite(channel, request.getDeviceId(), request.getModel());
        }
    }

    // 485
    public void handleRequest(Rs485ConfigWriteRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.rs485ConfigWrite(channel, request.getDeviceId(), request.getIndex(), request.getConfig());
        }
    }

    public void handleRequest(Rs485ConfigReadRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.rs485ConfigRead(channel, request.getDeviceId(), request.getIndex());
        }
    }

    public void handleRequest(Rs485ConfigReadAllRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.rs485ConfigReadAll(channel, request.getDeviceId());
        }
    }

    public void handleRequest(Rs485ConfigDeleteRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.rs485ConfigDelete(channel, request.getDeviceId(), request.getIndex());
        }
    }

    public void handleRequest(Rs485ConfigDeleteAllRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.rs485ConfigDeleteAll(channel, request.getDeviceId());
        }
    }

    public void handleRequest(Rs485CollectRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.rs485Collect(channel, request.getDeviceId(), request.getIndex());
        }
    }

    // 透传
    public void handleRequest(TransmitRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.sendTransmitTransparently(channel, DeviceType.valueOf(request.getDeviceType()), request.getDeviceId(), request.getBaudrate(), request.getCheckMode(), request.getDc(),
                    request.getCommand());
        }
    }

    // InputConfig
    public void handleRequest(InputConfigWriteRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.switchInputConfigWrite(channel, request.getDeviceId(), request.getIndex(), request.getConfig());
        }
    }

    public void handleRequest(InputConfigReadRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.switchInputConfigRead(channel, request.getDeviceId(), request.getIndex());
        }
    }

    public void handleRequest(InputConfigReadAllRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.switchInputConfigReadAll(channel, request.getDeviceId());
        }
    }

    public void handleRequest(InputConfigDeleteRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.switchInputConfigDelete(channel, request.getDeviceId(), request.getIndex());
        }
    }

    public void handleRequest(InputConfigDeleteAllRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.switchInputConfigDeleteAll(channel, request.getDeviceId());
        }
    }

    // Output
    public void handleRequest(OutputConfigWriteRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.switchOutputConfigWrite(channel, request.getDeviceId(), request.getIndex(), request.getConfig());
        }
    }

    public void handleRequest(OutputConfigReadRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.switchOutputConfigRead(channel, request.getDeviceId(), request.getIndex());
        }
    }

    public void handleRequest(OutputConfigReadAllRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.switchOutputConfigReadAll(channel, request.getDeviceId());
        }
    }

    public void handleRequest(OutputConfigDeleteRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.switchOutputConfigDelete(channel, request.getDeviceId(), request.getIndex());
        }
    }

    public void handleRequest(OutputConfigDeleteAllRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.switchOutputConfigDeleteAll(channel, request.getDeviceId());
        }
    }

    public void handleRequest(OutputOpenRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.switchOutputOpen(channel, request.getDeviceId(), request.getIndex());
        }
    }

    public void handleRequest(OutputCloseRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.switchOutputClose(channel, request.getDeviceId(), request.getIndex());
        }
    }

    // Output Period Config
    public void handleRequest(OutputPeriodConfigWriteRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.switchOutputPeriodConfigWrite(channel, request.getDeviceId(), request.getIndex(), request.getConfig());
        }
    }

    public void handleRequest(OutputPeriodConfigReadRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.switchOutputPeriodConfigRead(channel, request.getDeviceId(), request.getIndex());
        }
    }

    public void handleRequest(OutputPeriodConfigReadAllRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.switchOutputPeriodConfigReadAll(channel, request.getDeviceId());
        }
    }

    public void handleRequest(OutputPeriodConfigDeleteRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.switchOutputPeriodConfigDelete(channel, request.getDeviceId(), request.getIndex());
        }
    }

    public void handleRequest(OutputPeriodConfigDeleteAllRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.switchOutputPeriodConfigDeleteAll(channel, request.getDeviceId());
        }
    }

    /**
     * 固件升级请求
     */
    public void handleRequest(UpgradeRequest request) {
        UgradeThreadManager.getInstance().upgrade(request);
    }

    /**
     * 取消固件升级
     *
     * @param request
     */
    public void handleRequest(UpgradeCancelRequest request) {
        UpgradeCancelResponse cancelResponse = new UpgradeCancelResponse(request.getDeviceType(), request.getDeviceId());
        transportResponseKafkaSender.sendMessage(cancelResponse);

        UpgradeThread upgradeThread = UgradeThreadManager.getInstance().getThread(request.getDeviceType(), request.getDeviceId());
        if (upgradeThread != null) {
            upgradeThread.cancelUpgrade();
            UgradeThreadManager.getInstance().removeUgradeThread(request.getDeviceType(), request.getDeviceId());
        }
    }



    // 资源连接
    public void handleRequest(ResourceConfigReadTenRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.resourceConfigReadTen(channel, request.getDeviceId(), request.getPage());
        }
    }

    public void handleRequest(ResourceConfigDeleteTenRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.resourceConfigDeleteTen(channel, request.getDeviceId(), request.getPage());
        }
    }

    public void handleRequest(ResourceConfigWriteTenRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.resourceConfigWriteTen(channel, request.getDeviceId(), request.getPage(), request.getConfigs());
        }
    }

    public void handleRequest(ResourceConfigReadRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.resourceConfigRead(channel, request.getDeviceId(), request.getIndex());
        }
    }

    public void handleRequest(ResourceConfigDeleteRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.resourceConfigDelete(channel, request.getDeviceId(), request.getIndex());
        }
    }

    public void handleRequest(ResourceConfigWriteRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.resourceConfigWrite(channel, request.getDeviceId(), request.getIndex(), request.getConfig());
        }
    }

    public void handleRequest(ResourceConfigTestDataRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.resourceConfigTest(channel, request.getDeviceId(), request.getIndex());
        }
    }

    public void handleRequest(ModbusIOControlRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.modbusIOControl(channel, request.getDeviceId(), request.getRegisterAddress(), request.isOpen());
        }
    }


    // 动力设备监测电流配置
    public void handleRequest(OverloadCurrentReadAllRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.overloadCurrentReadAll(channel, request.getDeviceId());
        }
    }

    public void handleRequest(OverloadCurrentDeleteRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.overloadCurrentDeleteOne(channel, request.getDeviceId(), request.getIndex());
        }
    }

    public void handleRequest(OverloadCurrentDeleteAllRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.overloadCurrentDeleteAll(channel, request.getDeviceId());
        }
    }

    public void handleRequest(OverloadCurrentReadRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.overloadCurrentReadOne(channel, request.getDeviceId(), request.getIndex());
        }
    }

    public void handleRequest(OverloadCurrentWriteRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.overloadCurrentWriteOne(channel, request.getDeviceId(), request.getIndex(), request.getConfig());
        }
    }


    // 动力设备监测电流状态
    public void handleRequest(OverloadWarningReadAllRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.overloadWarningReadAll(channel, request.getDeviceId());
        }
    }

    public void handleRequest(OverloadWarningDeleteRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.overloadWarningDeleteOne(channel, request.getDeviceId(), request.getIndex(), request.getType());
        }
    }

    public void handleRequest(OverloadWarningDeleteAllRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.overloadWarningDeleteAll(channel, request.getDeviceId());
        }
    }

    public void handleRequest(OverloadWarningReadRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.overloadWarningReadOne(channel, request.getDeviceId(), request.getIndex(), request.getType());
        }
    }



    public void handleRequest(WirelessSwitchOpenRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.wirelessSwitchOpen(channel, request.getDeviceId(), request.getIndex());
        }
    }

    public void handleRequest(WirelessSwitchOpenAllRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.wirelessSwitchOpenAll(channel, request.getDeviceId());
        }
    }

    public void handleRequest(WirelessSwitchCloseRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.wirelessSwitchClose(channel, request.getDeviceId(), request.getIndex());
        }
    }

    public void handleRequest(WirelessSwitchCloseAllRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.wirelessSwitchCloseAll(channel, request.getDeviceId());
        }
    }


    /**
     * Modbus 参数写入请求
     *
     * @param request
     */
    public void handleRequest(ModbusDataWriteRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.equipmentMonitoringParamWrite(channel, request.getDeviceId(), request.getConfigIndex(), request.getDataIndex(), request.getRegisterAddress(), request.getData());
            try {
                Thread.sleep(500L);
            } catch (Exception e) {

            }
            CommandUtil.modbusDataWrite(channel, request.getDeviceId(), request.getRegisterAddress(), request.getRegisterType(), request.getMethod(), request.getData());
        }
    }

    /**
     * 数据转发配置变更时， 关闭并清除 Channel 已有的转发器
     *
     * @param request
     */
    public void handleRequest(TransferChangedRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            channel.attr(AttrKey.HAS_TRANSFERRER).set(null);
            Transferrer transferrer = channel.attr(AttrKey.TRANSFERRER).get();
            if (transferrer != null) {
                transferrer.close();
                channel.attr(AttrKey.TRANSFERRER).set(null);
            }
        }
    }

    public void handleRequest(DecoderCompileRequest request) {
//        DecoderCompileResponse response = new DecoderCompileResponse(request.getSessionId());
//        try {
//            GroovyDecoderUtil.compileDecoder(request.getScript());
//            response.setCode(0);
//            response.setMessage("success");
//        } catch (Exception e) {
//            response.setCode(1);
//            response.setMessage(e.getMessage());
//        }
//        transportResponseSender.sendMessage(response);
    }

    // AT指令
    public void handleRequest(AtCommandRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.sendAtCommand(channel, DeviceType.valueOf(request.getDeviceType()), request.getDeviceId(), request.getCommand());
        }
    }

    // 新数据采集配置 - 设备配置
    public void handleRequest(EquipmentConfigReadRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.equipmentConfigRead(channel, request.getDeviceId(), request.getIndex());
        }
    }

    public void handleRequest(EquipmentConfigReadTenRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.equipmentConfigReadTen(channel, request.getDeviceId(), request.getPage());
        }
    }

    public void handleRequest(EquipmentConfigDeleteRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.equipmentConfigDelete(channel, request.getDeviceId(), request.getIndex());
        }
    }

    public void handleRequest(EquipmentConfigDeleteAllRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.equipmentConfigDeleteAll(channel, request.getDeviceId());
        }
    }

    public void handleRequest(EquipmentConfigWriteRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.equipmentConfigWrite(channel, request.getDeviceId(), request.getIndex(), request.getConfig());
        }
    }

    // 新数据采集配置 - 测点配置
    public void handleRequest(EquipmentMonitoringReadRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.equipmentMonitoringRead(channel, request.getDeviceId(), request.getIndex());
        }
    }

    public void handleRequest(EquipmentMonitoringReadTenRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.equipmentMonitoringReadTen(channel, request.getDeviceId(), request.getPage());
        }
    }

    public void handleRequest(EquipmentMonitoringDeleteRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.equipmentMonitoringDelete(channel, request.getDeviceId(), request.getIndex());
        }
    }

    public void handleRequest(EquipmentMonitoringDeleteAllRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.equipmentMonitoringDeleteAll(channel, request.getDeviceId());
        }
    }

    public void handleRequest(EquipmentMonitoringWriteRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.equipmentMonitoringWrite(channel, request.getDeviceId(), request.getIndex(), request.getConfig());
        }
    }

    // 新数据采集配置 - 报警配置
    public void handleRequest(WarningConfigReadRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.warningConfigRead(channel, request.getDeviceId(), request.getIndex());
        }
    }

    public void handleRequest(WarningConfigReadTenRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.warningConfigReadTen(channel, request.getDeviceId(), request.getPage());
        }
    }

    public void handleRequest(WarningConfigDeleteRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.warningConfigDelete(channel, request.getDeviceId(), request.getIndex());
        }
    }

    public void handleRequest(WarningConfigDeleteAllRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.warningConfigDeleteAll(channel, request.getDeviceId());
        }
    }

    public void handleRequest(WarningConfigWriteRequest request) {
        Channel channel = ChannelManager.getInstance().getByDevice(request.getDeviceType(), request.getDeviceId());
        if (channel != null) {
            CommandUtil.warningConfigWrite(channel, request.getDeviceId(), request.getIndex(), request.getConfig());
        }
    }
}
