package com.qhiot.transport.netty.handler;

import com.qhiot.core.util.StringUtil;
import com.qhiot.transport.codec.bean.AttrKey;
import com.qhiot.transport.message.define.Device;
import com.qhiot.transport.message.define.DeviceType;
import io.netty.channel.Channel;
import org.springframework.web.context.support.SpringBeanAutowiringSupport;

import java.util.Collection;
import java.util.Iterator;
import java.util.Map;
import java.util.Vector;
import java.util.concurrent.ConcurrentHashMap;

/**
 * Netty 通道管理器, 保存终端与 channel 之间的映射关系
 *
 * @author 金建强(ptma@163.com)
 */
public class ChannelManager extends SpringBeanAutowiringSupport {

    private static class ChannelManagerHolder {
        protected final static ChannelManager INSTANCE = new ChannelManager();
    }

    private static Map<String, Channel>    channelsMap     = new ConcurrentHashMap<String, Channel>();

    private ChannelManager() {
    }

    public static ChannelManager getInstance() {
        return ChannelManagerHolder.INSTANCE;
    }

    public Collection<Channel> getChannels() {
        return channelsMap.values();
    }

    /**
     * 添加终端与 channel 的映射关系
     *
     * @param deviceId
     * @param deviceType
     * @param channel
     */
    public void add(byte deviceType, String deviceId, Channel channel) {
        String key = StringUtil.format("{}_{}", deviceType, deviceId);
        channelsMap.put(key, channel);

        if (deviceType == DeviceType.GATEWAY.getValue() || deviceType == DeviceType.GATEWAY_4G.getValue()) {
            markAsMasterDevice(deviceType, deviceId, channel);
        } else {
            markAsSlaveDevice(deviceType, deviceId, channel);
        }
    }

    /**
     * 标记为主终端
     */
    private void markAsMasterDevice(byte deviceType, String deviceId, Channel channel) {
        Device device = channel.attr(AttrKey.MASTER_DEVICE).get();
        if (device == null) {
            device = new Device(deviceType, deviceId);
        } else {
            device.setDeviceType(deviceType);
            device.setDeviceId(deviceId);
        }
        channel.attr(AttrKey.MASTER_DEVICE).set(device);
    }

    /**
     * 标记为间接终端
     */
    private void markAsSlaveDevice(byte deviceType, String deviceId, Channel channel) {
        Vector<Device> slaveDevices = channel.attr(AttrKey.SLAVE_DEVICE).get();
        if (slaveDevices == null) {
            slaveDevices = new Vector<Device>();
            channel.attr(AttrKey.SLAVE_DEVICE).set(slaveDevices);
        }
        Device device = new Device(deviceType, deviceId);
        if (slaveDevices.contains(device)) {
            return;
        } else {
            slaveDevices.add(device);
            channel.attr(AttrKey.SLAVE_DEVICE).set(slaveDevices);
        }
    }

    /**
     * 移除终端与 channel 的映射关系
     *
     * @param deviceType
     * @param deviceId
     */
    public void remove(byte deviceType, String deviceId) {
        String key = StringUtil.format("{}_{}", deviceType, deviceId);
        Channel channel = channelsMap.get(key);
        if (channel != null) {
            try {
                channel.close();
            } catch (Exception e) {

            }
        }
        channelsMap.remove(key);
    }

    /**
     * 移除 channel
     *
     * @param channel
     */
    public void removeChannel(Channel channel) {
        String channelId = channel.id().asLongText();
        Iterator<Map.Entry<String, Channel>> itr = channelsMap.entrySet().iterator();
        while (itr.hasNext()) {
            Map.Entry<String, Channel> entry = itr.next();
            if (channelId.equals(entry.getValue().id().asLongText())) {
                itr.remove();
            }
        }
    }

    /**
     * 获取终端(主终端)对应的 Channel
     *
     * @param deviceId
     * @return
     */
    public Channel getByDevice(byte deviceType, String deviceId) {
        String key = StringUtil.format("{}_{}", deviceType, deviceId);
        return channelsMap.get(key);
    }

}
