package com.qhiot.transport.netty.handler;


import com.qhiot.cache.util.CacheUtil;
import com.qhiot.core.exception.ExceptionWapper;
import com.qhiot.core.util.ByteUtil;
import com.qhiot.core.util.DateUtil;
import com.qhiot.core.util.StringUtil;
import com.qhiot.core.util.ThreadUtil;
import com.qhiot.transport.codec.ChainContext;
import com.qhiot.transport.codec.bean.AttrKey;
import com.qhiot.transport.codec.bean.Online;
import com.qhiot.transport.codec.util.ByteBufUtil;
import com.qhiot.transport.codec.util.UgradeThreadManager;
import com.qhiot.transport.codec.util.UpgradeThread;
import com.qhiot.transport.db.model.DeviceNetworkTraffic;
import com.qhiot.transport.db.service.DeviceNetworkTrafficService;
import com.qhiot.transport.jms.kafka.CodecLogKafkaSender;
import com.qhiot.transport.jms.kafka.OnlineDataKafkaSender;
import com.qhiot.transport.jms.kafka.TrafficDataKafkaSender;
import com.qhiot.transport.message.CodecLogMessage;
import com.qhiot.transport.message.data.OnlineData;
import com.qhiot.transport.message.data.TrafficData;
import com.qhiot.transport.message.define.Device;
import com.qhiot.transport.netty.transfer.Transferrer;
import com.qhiot.transport.online.OnlineManager;
import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.net.SocketAddress;
import java.util.*;
import java.util.concurrent.ExecutorService;

/**
 * @author 金建强(ptma@163.com)
 * @version 2018/1/11 21:37
 */
@Component
public class ChannelUtil {

    private static final String         MAP_KEY_TEMPLATE      = "{}_{}";

    private static final Logger         LOGGER                = LoggerFactory.getLogger("transport.codec.ChannelUtil");

    private static ExecutorService       onlineUpdaterExecutor = ThreadUtil.newCachedThreadPool(10, 10000, 600L, "seem-transport-online-updater--%d");

//    @Autowired
//    private TransportDataTopicSender    transportDataTopicSender;
//    @Autowired
//    private InsightDataQueueSender      insightDataQueueSender;
//    @Autowired
//    private CodecLogSender              codecLogSender;


    /**---kafka---*/
    @Autowired
    private CodecLogKafkaSender codecLogSender;
    @Autowired
    private OnlineDataKafkaSender onlineDataKafkaSender;
    @Autowired
    private TrafficDataKafkaSender trafficDataKafkaSender;
    /**------*/

    @Autowired
    private DeviceNetworkTrafficService deviceNetworkTrafficService;
    @Autowired
    private OnlinePersistence           onlinePersistence;

    @Value("${app.node:0}")
    private int node;

    public void upstream(Channel channel, ChainContext context, ByteBuf msg) {
        Date inTime = new Date();
        byte[] rawBytes = ByteBufUtil.getBytesRemaining(msg);

        Integer upstreamSize = channel.attr(AttrKey.UPSTREAM_SIZE).get();
        if (upstreamSize == null) {
            upstreamSize = 0;
        }
        upstreamSize += rawBytes.length;
        channel.attr(AttrKey.UPSTREAM_SIZE).set(upstreamSize);

        Device device = channel.attr(AttrKey.MASTER_DEVICE).get();
        String deviceId;
        if (device != null) {
            deviceId = device.getDeviceId();
        } else {
            deviceId = "";
        }
        CodecLogMessage logMessage = CodecLogMessage.upstreamMessage(channel.id().asShortText(), deviceId, context.getId(), StringUtil.format("终端上发[{}] = {}", rawBytes.length, ByteUtil.toHexString(rawBytes, true)));

        LOGGER.debug(logMessage.getMessage());

        if (codecLogSender != null) {
            codecLogSender.sendMessage(logMessage);
        }

    }

    public void channelActive(Channel channel) throws Exception {
        CodecLogMessage logMessage = CodecLogMessage.normalMessage(channel.id().asShortText(), "", "", "连接建立, " + channelToString(channel));
        LOGGER.debug(logMessage.getMessage());
        if (codecLogSender != null) {
            codecLogSender.sendMessage(logMessage);
        }
    }

    public void channelInactive(Channel channel) throws Exception {
        ChannelManager.getInstance().removeChannel(channel);

        // 保存流量记录
//        saveTrafficStream(channel, true);

        // 清除在线数据
        removeOnlines(channel);

        // 主终端数据处理
        Device device = channel.attr(AttrKey.MASTER_DEVICE).get();
        if (device != null) {
            String onlineDataKey = StringUtil.format("{}_{}", device.getDeviceType(), device.getDeviceId());
            OnlineData onlineData =
                    (OnlineData) CacheUtil.get(OnlineManager.ONLINE_DATA_CACHE_NAME, onlineDataKey);
            if (onlineData != null && channel.id().asShortText().equals(onlineData.getChannelId())) {
                CacheUtil.del(OnlineManager.ONLINE_DATA_CACHE_NAME, onlineDataKey);
            }

            // 中断主终端升级线程
            UpgradeThread upgradeThread = UgradeThreadManager.getInstance().getThread(device.getDeviceType(), device.getDeviceId());
            if (upgradeThread != null) {
                upgradeThread.abortUpgrade();
            }
        }

        // 从终端数据处理
        Vector<Device> slaveDevices = channel.attr(AttrKey.SLAVE_DEVICE).get();
        if (slaveDevices != null && !slaveDevices.isEmpty()) {
            for (Device slaveDevice : slaveDevices) {
                String connectionDataKey = StringUtil.format("{}_{}", device.getDeviceType(), device.getDeviceId());
                OnlineData onlineData = (OnlineData) CacheUtil
                        .get(OnlineManager.ONLINE_DATA_CACHE_NAME, connectionDataKey);
                if (onlineData != null && channel.id().asShortText().equals(onlineData.getChannelId())) {
                    CacheUtil.del(OnlineManager.ONLINE_DATA_CACHE_NAME, connectionDataKey);
                }
                // 中断从终端升级线程
                UpgradeThread upgradeThread = UgradeThreadManager.getInstance().getThread(slaveDevice.getDeviceType(), slaveDevice.getDeviceId());
                if (upgradeThread != null) {
                    upgradeThread.abortUpgrade();
                }
            }
        }

        // 关闭转发客户端
        Transferrer transferrer = channel.attr(AttrKey.TRANSFERRER).get();
        if (transferrer != null) {
            LOGGER.debug("转发已经关闭,通道id={}", channel.id().asShortText());
            transferrer.close();
        }
    }

    public void exceptionCaught(Channel channel, Throwable cause) throws Exception {
        Device device = channel.attr(AttrKey.MASTER_DEVICE).get();
        String message = StringUtil.format("传输异常, {}, {}", ExceptionWapper.from(cause).toString(), channelToString(channel));
        CodecLogMessage logMessage = CodecLogMessage.normalMessage(channel.id().asShortText(), device == null ? "" : device.getDeviceId(), "", message);
        LOGGER.debug(logMessage.getMessage());
        if (codecLogSender != null) {
            codecLogSender.sendMessage(logMessage);
        }
    }

    /**
     * 更新在线状态
     *
     * @param channel
     * @param onlineData
     */
    public void updateOnline(final Channel channel, final OnlineData onlineData) {
        onlineUpdaterExecutor.submit(new Runnable() {
            @Override
            public void run() {
                try {
                    String onlineDataKey = StringUtil.format(MAP_KEY_TEMPLATE, onlineData.getDeviceType(), onlineData.getDeviceId());
                    // 同一通道内连接的终端信息保存在
                    Map<String, Online> onlineMap = channel.attr(AttrKey.ONLINE_MAP).get();
                    if (onlineMap == null) {
                        onlineMap = new HashMap<String, Online>();
                    }
                    Online online = onlineMap.get(onlineDataKey);
                    if (online == null) {
                        online = new Online(onlineData.getDeviceType(), onlineData.getDeviceId(), getChannelId(channel));
                    } else {
                        online.setLastActiveTime(DateUtil.now());
                    }
                    onlineMap.put(onlineDataKey, online);
                    channel.attr(AttrKey.ONLINE_MAP).set(onlineMap);

                    onlinePersistence.append(online);

                    CacheUtil.set(OnlineManager.ONLINE_DATA_CACHE_NAME, onlineDataKey, onlineData);
                } catch (Exception e) {
                    LOGGER.error(e.getMessage());
                }
            }
        });
    }

    /**
     * 重置连接
     */
    public void resetConnections() {
        // 保存终端连接记录
        onlinePersistence.reset();
    }

    /**
     * 重置传输流量
     * @param channel
     */
    public void resetTrafficStream(Channel channel) {
        // 保存流量记录
        saveTrafficStream(channel, false);
    }

    /**
     * 保存流量记录
     *
     * @param channel
     * @param disconnected 连接是否已断开
     */
    private void saveTrafficStream(Channel channel, boolean disconnected) {
        Date trafficTime = DateUtil.now();
        Integer upstreamSize = channel.attr(AttrKey.UPSTREAM_SIZE).get();
        if (upstreamSize == null) {
            upstreamSize = 0;
        }
        Integer downstreamSize = channel.attr(AttrKey.DOWNSTREAM_SIZE).get();
        if (downstreamSize == null) {
            downstreamSize = 0;
        }

        Device device = channel.attr(AttrKey.MASTER_DEVICE).get();

        String message;
        if (disconnected) {
            message = StringUtil.format("连接断开, 流量[上行={}, 下行={}], {}", upstreamSize, downstreamSize, channelToString(channel));
        } else {
            message = StringUtil.format("流量重置, 流量[上行={}, 下行={}], {}", upstreamSize, downstreamSize, channelToString(channel));
        }

        CodecLogMessage logMessage = CodecLogMessage.normalMessage(channel.id().asShortText(), device == null ? "" : device.getDeviceId(), "", message);
        LOGGER.debug(logMessage.getMessage());
        if (codecLogSender != null) {
            codecLogSender.sendMessage(logMessage);
        }

        if (device != null) {
            OnlineData onlineData = new OnlineData(device.getDeviceType(), device.getDeviceId());
            onlineData.setConnected(false);
//            if (transportDataTopicSender != null) {
//                transportDataTopicSender.sendMessage(onlineData);
//            }
            onlineDataKafkaSender.sendMessage(onlineData);


            if (upstreamSize > 0 || downstreamSize > 0) {
                try {
                    DeviceNetworkTraffic networkTraffic = new DeviceNetworkTraffic();
                    networkTraffic.setDeviceId(device.getDeviceId());
                    networkTraffic.setUpstream(upstreamSize);
                    networkTraffic.setDownstream(downstreamSize);
                    if (!disconnected) {
                        trafficTime = DateUtil.endTimeOfDay(DateUtil.offsetDay(DateUtil.now(), -1));
                        channel.attr(AttrKey.UPSTREAM_SIZE).set(0);
                        channel.attr(AttrKey.DOWNSTREAM_SIZE).set(0);
                    }
                    networkTraffic.setTrafficTime(trafficTime);

                    TrafficData trafficData = new TrafficData(device.getDeviceType(), device.getDeviceId());
                    trafficData.setUpstream(upstreamSize);
                    trafficData.setDownstream(downstreamSize);
                    trafficData.setTrafficTime(trafficTime);
//                    insightDataQueueSender.sendMessage(trafficData);
                    trafficDataKafkaSender.sendMessage(trafficData);
                    if (deviceNetworkTrafficService != null) {
                        deviceNetworkTrafficService.insertSelective(networkTraffic);
                    }
                } catch (Exception e) {

                }
            }
        }
    }

    /**
     * 保存终端连接记录
     *
     * @param channel
     */
    private void removeOnlines(Channel channel) {
        final Map<String, Online> onlineMap = channel.attr(AttrKey.ONLINE_MAP).get();
        ThreadUtil.submit(new Runnable() {
            @Override
            public void run() {
                if (onlineMap != null && !onlineMap.isEmpty()) {
                    Iterator<Online> iterator = onlineMap.values().iterator();
                    while (iterator.hasNext()) {
                        Online online = iterator.next();
                        onlinePersistence.remove(online);
                    }
                }
            }
        });
    }

    private String channelToString(Channel channel) {
        String strVal;
        SocketAddress remoteAddr = channel.remoteAddress();
        SocketAddress localAddr = channel.localAddress();
        if (remoteAddr != null) {
            StringBuilder buf =
                    new StringBuilder(96).append("[id: ").append(channel.id().asShortText()).append(", L:").append(localAddr).append(channel.isActive() ? " - " : " ! ").append("R:")
                            .append(remoteAddr).append(']');
            strVal = buf.toString();
        } else if (localAddr != null) {
            StringBuilder buf = new StringBuilder(64).append("[id: ").append(channel.id().asShortText()).append(", L:").append(localAddr).append(']');
            strVal = buf.toString();
        } else {
            StringBuilder buf = new StringBuilder(16).append("[id: ").append(channel.id().asShortText()).append(']');
            strVal = buf.toString();
        }

        return strVal;
    }

    private String getChannelId(Channel channel) {
        return channel.id().asShortText();
    }
}
