package com.qhiot.transport.netty.handler;

import com.qhiot.core.sql.generator.SnowflakeID;
import com.qhiot.core.util.DateUtil;
import com.qhiot.core.util.StringUtil;
import com.qhiot.core.util.ThreadUtil;
import com.qhiot.transport.codec.bean.Online;
import com.qhiot.transport.db.model.Device;
import com.qhiot.transport.db.model.DeviceConnection;
import com.qhiot.transport.db.service.DeviceConnectionService;
import com.qhiot.transport.db.service.DeviceService;
import com.qhiot.transport.influxdb.model.DeviceConnectStatusInflux;
import com.qhiot.transport.influxdb.service.DeviceConnectStatusInfluxService;
import com.qhiot.transport.jms.kafka.ConnectionDataKafkaSender;
import com.qhiot.transport.message.data.ConnectionData;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.ReentrantLock;

/**
 * 在线连接数据持久化工具， 间隔写入
 *
 * @author 金建强(ptma@163.com)
 * @version 2018/5/7 16:22
 */
public class OnlinePersistence {

    private static final Logger      LOGGER    = LoggerFactory.getLogger("transport.codec.OnlinePersistence");

    private Map<String, Online>      onlines   = new ConcurrentHashMap<String, Online>();

    private final ReentrantLock      lock      = new ReentrantLock(true);

    private ScheduledExecutorService service;

    private boolean                  closed    = false;

    private int                      flushSecs;

    private Date                     flushTime = DateUtil.now();

    @Autowired
    private DeviceService            deviceService;

    @Autowired
    private DeviceConnectionService  deviceConnectionService;

    @Autowired
    private DeviceConnectStatusInfluxService deviceConnectStatusInfluxService;

    @Autowired
    private SnowflakeID              snowflakeID;

//    @Autowired
//    private InsightDataQueueSender insightDataQueueSender;

    /**---kafka---*/
    @Autowired
    private ConnectionDataKafkaSender connectionDataKafkaSender;
    /**------*/

    public void init() {
        service = ThreadUtil.newSingleScheduledExecutor("seem-transport-online-flusher-%d");
        service.scheduleAtFixedRate(new Runnable() {
            @Override
            public void run() {
                flush();
            }
        }, flushSecs, flushSecs, TimeUnit.SECONDS);
    }

    public void append(Online online) {
        try {
            String key = StringUtil.format("{}_{}_{}", online.getDeviceType(), online.getDeviceId(), online.getChannelId());
            if (onlines.containsKey(key)) {
                Online chached = onlines.get(key);
                chached.setLastActiveTime(DateUtil.now());
            } else {
                onlines.put(key, online);
            }
        } catch (Exception e) {
            LOGGER.error(e.getMessage(), e);
        }
    }

    /**
     * 移除在线数据， 移除时保存
     *
     * @param online
     */
    public void remove(Online online) {
        lock.lock();
        try {
            String key = StringUtil.format("{}_{}_{}", online.getDeviceType(), online.getDeviceId(), online.getChannelId());
            if (onlines.containsKey(key)) {
                Online cachedOnline = onlines.get(key);

                DeviceConnection deviceConnection = onlineToConnection(cachedOnline);

                ConnectionData connectionData = new ConnectionData(cachedOnline.getDeviceType(), cachedOnline.getDeviceId());
                connectionData.setConnectedTime(deviceConnection.getConnectedTime());
                connectionData.setDisconnectedTime(deviceConnection.getDisconnectedTime());
                connectionData.setConnectionId(deviceConnection.getConnectionId());
                connectionData.setConnectionDuration(deviceConnection.getConnectionDuration());
//                insightDataQueueSender.sendMessage(connectionData);
                connectionDataKafkaSender.sendMessage(connectionData);


                deviceConnectionService.updateSelective(deviceConnection);

//                deviceService.updateLastActiveTime(cachedOnline.getDeviceId(), cachedOnline.getLastActiveTime());

                DeviceConnectStatusInflux status = new DeviceConnectStatusInflux();
                status.setDeviceId(online.getDeviceId());
                status.setStatus(DeviceConnectStatusInflux.OFF);
                deviceConnectStatusInfluxService.insert(status);

                onlines.remove(key);
            }
        } catch (Exception e) {
            LOGGER.error(e.getMessage(), e);
        } finally {
            lock.unlock();
        }
    }

    /**
     * 重置在线时间
     */
    public void reset() {
        flush();
        lock.lock();
        try {
            DeviceConnection deviceConnection;
            List<DeviceConnection> insertConnectionList = new ArrayList<>();
            Iterator<Online> iterator = onlines.values().iterator();
            Online online;
            while (iterator.hasNext()) {
                online = iterator.next();
                online.setCreateTime(DateUtil.now());
                online.setLastActiveTime(online.getCreateTime());

                long newId = snowflakeID.next();
                online.setConnectionId(newId);

                deviceConnection = onlineToConnection(online);
                deviceConnection.setConnectionId(newId);

                ConnectionData connectionData = new ConnectionData(online.getDeviceType(), online.getDeviceId());
                connectionData.setConnectedTime(deviceConnection.getConnectedTime());
                connectionData.setDisconnectedTime(deviceConnection.getDisconnectedTime());
                connectionData.setConnectionId(deviceConnection.getConnectionId());
                connectionData.setConnectionDuration(deviceConnection.getConnectionDuration());
//                insightDataQueueSender.sendMessage(connectionData);
                connectionDataKafkaSender.sendMessage(connectionData);
                insertConnectionList.add(deviceConnection);
            }
            deviceConnectionService.insertBatch(insertConnectionList);
        } catch (Throwable t) {
            LOGGER.error("保存在线记录出错: {}", t.getMessage(), t);
        } finally {
            lock.unlock();
        }
    }

    /**
     * 按计划写入
     */
    private void flush() {
        if (closed) {
            return;
        }
        lock.lock();
        try {
            long startTime = System.currentTimeMillis();
            if (onlines.isEmpty()) {
                return;
            }
            Device device;
            DeviceConnection deviceConnection;
            Online online;
            List<DeviceConnection> insertConnectionList = new ArrayList<>();
            List<DeviceConnection> updateConnectionList = new ArrayList<>();
            List<DeviceConnectStatusInflux> insertStatusInfluxList = new ArrayList<>();
            List<Device> updateDeviceList = new ArrayList<Device>();
            Iterator<Online> iterator = onlines.values().iterator();
            while (iterator.hasNext()) {
                online = iterator.next();
                if (online.getLastActiveTime().getTime() < flushTime.getTime()) {
                    continue;
                }

                deviceConnection = onlineToConnection(online);

                if (online.getConnectionId() == null) {
                    //新连接通道
                    long newId = snowflakeID.next();
                    online.setConnectionId(newId);
                    deviceConnection.setConnectionId(newId);
                    insertConnectionList.add(deviceConnection);

                    DeviceConnectStatusInflux status = new DeviceConnectStatusInflux();
                    status.setDeviceId(online.getDeviceId());
                    status.setStatus(DeviceConnectStatusInflux.ON);
                    status.setDate(online.getCreateTime());
                    insertStatusInfluxList.add(status);
                } else {
                    //旧连接通道
                    updateConnectionList.add(deviceConnection);
                }

                ConnectionData connectionData = new ConnectionData(online.getDeviceType(), online.getDeviceId());
                connectionData.setConnectedTime(deviceConnection.getConnectedTime());
                connectionData.setDisconnectedTime(deviceConnection.getDisconnectedTime());
                connectionData.setConnectionId(deviceConnection.getConnectionId());
                connectionData.setConnectionDuration(deviceConnection.getConnectionDuration());
//                insightDataQueueSender.sendMessage(connectionData);
                connectionDataKafkaSender.sendMessage(connectionData);
                // 终端最后连接时间
                device = new Device();
                device.setDeviceId(online.getDeviceId());
                device.setLastActiveTime(online.getLastActiveTime());
                updateDeviceList.add(device);
            }
            if (!insertConnectionList.isEmpty()) {
                deviceConnectionService.insertBatch(insertConnectionList);
            }
            if (!updateConnectionList.isEmpty()) {
                deviceConnectionService.updateBatch(updateConnectionList);
            }
            if (!insertStatusInfluxList.isEmpty()) {
                deviceConnectStatusInfluxService.batchInsert(insertStatusInfluxList);
            }
            if (!updateDeviceList.isEmpty()) {
                deviceService.batchUpdateLastActiveTime(updateDeviceList);
            }
            flushTime = DateUtil.now();
            LOGGER.debug("保存在线记录耗时：{}ms",flushTime.getTime() - startTime);
        } catch (Throwable t) {
            LOGGER.error("保存在线记录出错: {}", t.getMessage(), t);
        } finally {
            lock.unlock();
        }
    }

    /**
     * 关闭写入计划
     */
    public void close() {
        if (!closed) {
            service.shutdownNow();
            flush();
            closed = true;
        }
    }

    public void setFlushSecs(int flushSecs) {
        this.flushSecs = flushSecs;
    }

    private DeviceConnection onlineToConnection(Online online) {
        DeviceConnection deviceConnection = new DeviceConnection();
        deviceConnection.setConnectionId(online.getConnectionId());
        deviceConnection.setDeviceId(online.getDeviceId());
        deviceConnection.setConnectedTime(online.getCreateTime());
        deviceConnection.setDisconnectedTime(online.getLastActiveTime());
        int duration = (int) DateUtil.diffSeconds(online.getCreateTime(), online.getLastActiveTime());
        deviceConnection.setConnectionDuration(duration);

        return deviceConnection;
    }
}
