package com.qhiot.transport.netty.handler;

import com.qhiot.transport.codec.ChainContext;
import com.qhiot.transport.codec.FrameDecoder;
import com.qhiot.transport.codec.bean.AttrKey;
import io.netty.buffer.ByteBuf;
import io.netty.channel.ChannelHandler.Sharable;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.SimpleChannelInboundHandler;
import io.netty.handler.timeout.IdleStateEvent;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;

/**
 * <p>
 * ChannelInboundHandlerAdapter
 * </p>
 * <p>
 * channelRead(ctx, msg)中需要释放 msg， 如果msg是ByteBuf类型，则需要调用ByteBuf.release()方法
 * （因为Netty默认使用支持池的Buf，不release的会发生内存泄露，即这个Buf在JVM退出之前永远不会被GC回收）;
 * </p>
 *
 * <p>
 * SimpleChannelInboundHandler
 * </p>
 * <p>
 * it will release the response once the channelRead0(...) method completes. This is done by Netty
 * to handle all messages that are implement ReferenceCounted.
 * </p>
 *
 * @author 金建强(ptma@163.com)
 */
@Sharable
public class UdpChannelHandler extends SimpleChannelInboundHandler<ByteBuf> {

    private static final Logger LOGGER = LoggerFactory.getLogger("transport.codec.UdpChannelHandler");

    @Autowired
    private FrameDecoder        frameDecoder;
    @Autowired
    private ChannelUtil channelUtil;

    @Override
    public void channelRead0(ChannelHandlerContext ctx, ByteBuf msg) throws Exception {
        ChainContext context = new ChainContext();
//        channelUtil.upstream(ctx.channel(), context, msg);
        frameDecoder.decode(ctx.channel(), msg.copy(), context, null);
    }

    @Override
    public void channelReadComplete(ChannelHandlerContext ctx) {
        ctx.flush();
    }

    @Override
    public void channelActive(ChannelHandlerContext ctx) throws Exception {
        super.channelActive(ctx);
        ctx.channel().attr(AttrKey.UDP).set(Boolean.TRUE);
        channelUtil.channelActive(ctx.channel());
    }

    @Override
    public void channelInactive(ChannelHandlerContext ctx) throws Exception {
        channelUtil.channelInactive(ctx.channel());
        super.channelInactive(ctx);
    }

    @Override
    public void channelWritabilityChanged(ChannelHandlerContext ctx) throws Exception {
        super.channelWritabilityChanged(ctx);
    }

    @Override
    public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) throws Exception {
        //channelUtil.exceptionCaught(ctx.channel(), cause);
        ChannelManager.getInstance().removeChannel(ctx.channel());
        ctx.channel().close();
    }

    @Override
    public void userEventTriggered(ChannelHandlerContext ctx, Object evt) throws Exception {
        if (evt instanceof IdleStateEvent) {
            ctx.channel().close();
        } else {
            super.userEventTriggered(ctx, evt);
        }
    }

}
